Package: bibnets
Title: Importing, Constructing, and Exporting Bibliometric Networks
Version: 0.4.4
Authors@R: c(
    person("Mohammed", "Saqr", , "saqr@saqr.me", role = c("aut", "cre", "cph"),
           comment = c(ORCID = "0000-0001-5881-3109")),
    person("Sonsoles", "López-Pernas", role = "aut",
           comment = c(ORCID = "0000-0002-9621-1392")))
Description: Imports, constructs, and exports bibliometric networks from
    scholarly metadata. Reads 'Scopus', 'Web of Science', 'BibTeX',
    'RIS', 'OpenAlex', 'Lens.org', 'Dimensions', and 'Crossref'
    exports. Goes beyond standard co-networks with attention-weighted
    networks (lead, last, proximity, circular position weights),
    position-aware counting (harmonic, arithmetic, geometric,
    golden-ratio), similarity and dissimilarity normalisations,
    temporal networks with fixed, sliding, and cumulative windows,
    disparity-filter backbone extraction, historiograph construction,
    and local citation scoring. Methods described in López-Pernas,
    Saqr & Apiola (2023) <doi:10.1007/978-3-031-25336-2_5>.
License: MIT + file LICENSE
URL: https://github.com/mohsaqr/bibnets
BugReports: https://github.com/mohsaqr/bibnets/issues
Depends: R (>= 4.1.0)
Imports: Matrix, stats, utils
Suggests: cograph, igraph, knitr, openalexR, rcrossref, rmarkdown,
        testthat (>= 3.0.0), tidygraph
VignetteBuilder: knitr
Config/testthat/edition: 3
Encoding: UTF-8
LazyData: true
LazyDataCompression: xz
RoxygenNote: 7.3.3
NeedsCompilation: no
Packaged: 2026-05-16 07:39:30 UTC; mohammedsaqr
Author: Mohammed Saqr [aut, cre, cph] (ORCID:
    <https://orcid.org/0000-0001-5881-3109>),
  Sonsoles López-Pernas [aut] (ORCID:
    <https://orcid.org/0000-0002-9621-1392>)
Maintainer: Mohammed Saqr <saqr@saqr.me>
Repository: CRAN
Date/Publication: 2026-05-20 09:10:14 UTC

# Generated by roxygen2: do not edit by hand

S3method(print,bibnets_network)
S3method(summary,bibnets_network)
export(author_network)
export(backbone)
export(conetwork)
export(country_network)
export(document_network)
export(filter_top)
export(historiograph)
export(institution_network)
export(keyword_network)
export(local_citations)
export(normalize)
export(prune)
export(read_biblio)
export(read_bibtex)
export(read_crossref)
export(read_dimensions)
export(read_lens)
export(read_openalex)
export(read_openalex_csv)
export(read_ris)
export(read_scopus)
export(read_wos)
export(reference_network)
export(source_network)
export(split_field)
export(temporal_network)
export(to_cograph)
export(to_gephi)
export(to_graphml)
export(to_igraph)
export(to_matrix)
export(to_tbl_graph)
importFrom(utils,head)

#' Build an institution network
#'
#' Constructs a network between institutions (affiliations).
#'
#' @param data A data frame with `id` and `affiliations` (list-column of
#'   institution names). For coupling, also needs `references`.
#' @param type Character. `"collaboration"` (default), `"coupling"`, or
#'   `"equivalence"`.
#' @param counting Character. Counting method. Default `"full"`.
#' @param similarity Character. Similarity measure. Default `"none"`.
#' @param threshold Numeric. Minimum edge weight. Default 0.
#' @param min_occur Integer. Minimum papers per institution. Default 1.
#' @param top_n Integer or NULL. Return only the top n edges by weight.
#'   Default NULL (all edges).
#' @inheritParams author_network
#'
#' @return Depends on `format`: a `bibnets_network` data frame (default),
#'   a Gephi-ready data frame, an igraph graph, a cograph_network, or a
#'   sparse matrix.
#'
#' @export
#' @examples
#' data(open_alex_gold_open_access_learning_analytics)
#' institution_network(open_alex_gold_open_access_learning_analytics, "collaboration")
institution_network <- function(data,
                                type = "collaboration",
                                counting = "full",
                                similarity = "none",
                                threshold = 0,
                                min_occur = 1L,
                                attention = NULL,
                                top_n = NULL,
                                self_loops = FALSE,
                                deduplicate = TRUE,
                                format = "edgelist") {
  check_data(data, c("id", "affiliations"))
  check_choice(similarity, c("none", "association", "cosine", "jaccard",
                              "inclusion", "equivalence"), "similarity")
  check_format(format)

  if (!is.null(attention)) {
    check_choice(attention, all_attention_methods(), "attention")
    B <- build_author_bipartite(data, field = "affiliations",
                                counting = paste0("attention_", attention),
                                deduplicate = deduplicate)
    result <- multiply_bipartite(B, mode = "columns", similarity = similarity,
                                 threshold = threshold, top_n = top_n,
                                 self_loops = self_loops)
    return(as_bibnets_network(result,
      network_type = paste0("institution_attention_", attention),
      counting = attention, similarity = similarity, format = format))
  }

  check_choice(type, c("collaboration", "coupling", "equivalence"), "type")
  check_choice(counting, position_independent_counts(), "counting")

  result <- if (type == "collaboration") {
    B <- build_bipartite(data, field = "affiliations", min_freq = min_occur, deduplicate = deduplicate)
    B <- apply_counting(B, counting = counting, network_type = "symmetric")
    multiply_bipartite(B, mode = "columns", similarity = similarity,
                       threshold = threshold, top_n = top_n,
                       self_loops = self_loops)

  } else if (type == "coupling") {
    if (!"references" %in% names(data))
      stop("Column 'references' not found. Required for type = 'coupling'.", call. = FALSE)
    agg <- aggregate_by_entity(data, entity_field = "affiliations",
                                value_field = "references",
                                min_freq = min_occur)
    B <- build_bipartite(agg, field = "references")
    B <- apply_counting(B, counting = counting, network_type = "coupling")
    multiply_bipartite(B, mode = "rows", similarity = similarity,
                       threshold = threshold, top_n = top_n,
                       self_loops = self_loops)

  } else {
    B <- build_bipartite(data, field = "affiliations", min_freq = min_occur, deduplicate = deduplicate)
    multiply_bipartite(B, mode = "columns", similarity = "cosine",
                       threshold = threshold, top_n = top_n,
                       self_loops = self_loops)
  }

  as_bibnets_network(result, network_type = paste0("institution_", type),
                      counting = counting, similarity = similarity,
                      format = format)
}

## Tests for S3 print/summary methods and as_bibnets_network()

test_that("bibnets_network has correct class", {
  d <- make_test_data()
  edges <- keyword_network(d)
  expect_s3_class(edges, "bibnets_network")
  expect_s3_class(edges, "data.frame")
})

test_that("all network functions return bibnets_network", {
  d <- make_test_data()
  dext <- d
  dext$countries    <- list(c("USA","UK"), c("USA","DE"), c("UK","DE"))
  dext$affiliations <- list(c("MIT","Oxford"), c("MIT","TU Berlin"), c("Oxford","TU Berlin"))

  fns <- list(
    keyword_network(d),
    author_network(d, "collaboration"),
    reference_network(d, threshold = 0),
    document_network(d, "coupling"),
    source_network(d, "coupling"),
    country_network(dext, "collaboration"),
    institution_network(dext, "collaboration"),
    conetwork(d, "keywords")
  )

  for (e in fns) {
    expect_true(inherits(e, "bibnets_network"))
  }
})

test_that("network_type attribute is set correctly", {
  d <- make_test_data()
  expect_equal(attr(keyword_network(d), "network_type"), "keyword_co_occurrence")
  expect_equal(attr(author_network(d, "collaboration"), "network_type"), "author_collaboration")
  expect_equal(attr(author_network(d, "coupling"), "network_type"), "author_coupling")
  expect_equal(attr(reference_network(d, threshold=0), "network_type"), "reference_co_citation")
  expect_equal(attr(document_network(d, "coupling"), "network_type"), "document_coupling")
  expect_equal(attr(document_network(d, "citation"), "network_type"), "document_citation")
})

test_that("print.bibnets_network outputs correctly", {
  d <- make_test_data()
  edges <- keyword_network(d)
  out <- capture.output(print(edges))
  ## First line should mention nodes and edges
  expect_true(grepl("bibnets network", out[1]))
  expect_true(grepl("nodes", out[1]))
  expect_true(grepl("edges", out[1]))
  ## Should mention the type
  expect_true(any(grepl("keyword_co_occurrence", out)))
})

test_that("print.bibnets_network n= argument limits rows shown", {
  d <- make_test_data()
  edges <- keyword_network(d)
  ## Show only 2 rows
  out <- capture.output(print(edges, n = 2))
  ## Should mention "more edges" if there are more than 2
  if (nrow(edges) > 2) {
    expect_true(any(grepl("more edges", out)))
  }
})

test_that("summary.bibnets_network outputs correctly", {
  d <- make_test_data()
  edges <- keyword_network(d)
  out <- capture.output(summary(edges))
  expect_true(any(grepl("bibnets network", out)))
  expect_true(any(grepl("Nodes", out)))
  expect_true(any(grepl("Edges", out)))
  expect_true(any(grepl("Density", out)))
  expect_true(any(grepl("Top nodes", out)))
})

test_that("summary.bibnets_network returns object invisibly", {
  d <- make_test_data()
  edges <- keyword_network(d)
  ret <- capture.output(x <- summary(edges))
  expect_identical(x, edges)
})

test_that("print.bibnets_network returns object invisibly", {
  d <- make_test_data()
  edges <- keyword_network(d)
  ret <- capture.output(x <- print(edges))
  expect_identical(x, edges)
})

test_that("as_bibnets_network stamps correct attributes", {
  df <- data.frame(from = "A", to = "B", weight = 1, stringsAsFactors = FALSE)
  net <- bibnets:::as_bibnets_network(df, "test_type", "full", "cosine")
  expect_s3_class(net, "bibnets_network")
  expect_equal(attr(net, "network_type"), "test_type")
  expect_equal(attr(net, "counting"),     "full")
  expect_equal(attr(net, "similarity"),   "cosine")
})

## Tests for source_network(), country_network(), institution_network(), conetwork()
## All should return bibnets_network S3 objects.

## Extended test data with countries and affiliations
make_extended_data <- function() {
  d <- make_test_data()
  d$countries <- list(
    c("USA", "UK"),
    c("USA", "DE"),
    c("UK", "DE")
  )
  d$affiliations <- list(
    c("MIT", "Oxford"),
    c("MIT", "TU Berlin"),
    c("Oxford", "TU Berlin")
  )
  d
}

## ── source_network ──────────────────────────────────────────────────────────

test_that("source_network coupling returns bibnets_network", {
  d <- make_test_data()
  edges <- source_network(d, type = "coupling")
  expect_s3_class(edges, "bibnets_network")
  expect_true(all(c("from", "to", "weight") %in% names(edges)))
  expect_equal(attr(edges, "network_type"), "source_coupling")
})

test_that("source_network coupling produces journal nodes", {
  d <- make_test_data()
  edges <- source_network(d, type = "coupling")
  ## J1 and J2 both cite shared refs — should be linked
  all_nodes <- unique(c(edges$from, edges$to))
  expect_true(all(all_nodes %in% c("J1", "J2")))
})

test_that("source_network coupling applies min_occur to aggregated sources", {
  d <- make_test_data()
  edges <- source_network(d, type = "coupling", min_occur = 2)
  all_nodes <- unique(c(edges$from, edges$to))
  expect_true(all(all_nodes %in% "J1"))
  expect_equal(nrow(edges), 0L)
})

test_that("source_network equivalence returns bibnets_network", {
  d <- make_test_data()
  edges <- source_network(d, type = "equivalence")
  expect_s3_class(edges, "bibnets_network")
  expect_equal(attr(edges, "network_type"), "source_equivalence")
})

test_that("source_network co_citation stops without cited_journals", {
  d <- make_test_data()
  expect_error(source_network(d, type = "co_citation"))
})

test_that("source_network co_citation works with cited_journals column", {
  d <- make_test_data()
  d$cited_journals <- list(
    c("J1", "J2"),
    c("J1", "J3"),
    c("J2", "J3")
  )
  edges <- source_network(d, type = "co_citation")
  expect_s3_class(edges, "bibnets_network")
  expect_equal(attr(edges, "network_type"), "source_co_citation")
})

test_that("source_network coupling requires journal column", {
  d <- make_test_data()
  d$journal <- NULL
  expect_error(source_network(d, type = "coupling"))
})

## ── country_network ─────────────────────────────────────────────────────────

test_that("country_network collaboration returns bibnets_network", {
  d <- make_extended_data()
  edges <- country_network(d, type = "collaboration")
  expect_s3_class(edges, "bibnets_network")
  expect_equal(attr(edges, "network_type"), "country_collaboration")
})

test_that("country_network collaboration has correct country nodes", {
  d <- make_extended_data()
  edges <- country_network(d, type = "collaboration")
  all_nodes <- unique(c(edges$from, edges$to))
  expect_true(all(all_nodes %in% c("USA", "UK", "DE")))
})

test_that("country_network collaboration edge counts are correct", {
  d <- make_extended_data()
  edges <- country_network(d, type = "collaboration")

  get_w <- function(a, b) {
    row <- edges[(edges$from == a & edges$to == b) |
                   (edges$from == b & edges$to == a), ]
    if (nrow(row) == 0) return(0)
    row$weight[1]
  }
  ## USA-UK: W1; USA-DE: W2; UK-DE: W3
  expect_equal(get_w("USA", "UK"), 1)
  expect_equal(get_w("USA", "DE"), 1)
  expect_equal(get_w("UK",  "DE"), 1)
})

test_that("country_network coupling returns bibnets_network", {
  d <- make_extended_data()
  edges <- country_network(d, type = "coupling")
  expect_s3_class(edges, "bibnets_network")
  expect_equal(attr(edges, "network_type"), "country_coupling")
})

test_that("country_network equivalence returns bibnets_network", {
  d <- make_extended_data()
  edges <- country_network(d, type = "equivalence")
  expect_s3_class(edges, "bibnets_network")
  expect_equal(attr(edges, "network_type"), "country_equivalence")
})

## ── institution_network ─────────────────────────────────────────────────────

test_that("institution_network collaboration returns bibnets_network", {
  d <- make_extended_data()
  edges <- institution_network(d, type = "collaboration")
  expect_s3_class(edges, "bibnets_network")
  expect_equal(attr(edges, "network_type"), "institution_collaboration")
})

test_that("institution_network collaboration has correct nodes", {
  d <- make_extended_data()
  edges <- institution_network(d, type = "collaboration")
  all_nodes <- unique(c(edges$from, edges$to))
  ## build_bipartite uppercases all entity names
  expect_true(all(all_nodes %in% c("MIT", "OXFORD", "TU BERLIN")))
})

test_that("institution_network coupling returns bibnets_network", {
  d <- make_extended_data()
  edges <- institution_network(d, type = "coupling")
  expect_s3_class(edges, "bibnets_network")
  expect_equal(attr(edges, "network_type"), "institution_coupling")
})

test_that("institution_network equivalence returns bibnets_network", {
  d <- make_extended_data()
  edges <- institution_network(d, type = "equivalence")
  expect_s3_class(edges, "bibnets_network")
  expect_equal(attr(edges, "network_type"), "institution_equivalence")
})

## ── conetwork ───────────────────────────────────────────────────────────────

test_that("conetwork co-occurrence returns bibnets_network", {
  d <- make_test_data()
  edges <- conetwork(d, "keywords")
  expect_s3_class(edges, "bibnets_network")
  expect_equal(attr(edges, "network_type"), "keywords_co_occurrence")
})

test_that("conetwork co-occurrence has correct keyword nodes", {
  d <- make_test_data()
  edges <- conetwork(d, "keywords")
  all_nodes <- unique(c(edges$from, edges$to))
  ## build_bipartite uppercases all entity names
  expect_true(all(all_nodes %in% c("ML", "DL", "NLP", "CV")))
})

test_that("conetwork by= returns bibnets_network with by_ label", {
  d <- make_test_data()
  edges <- conetwork(d, "authors", by = "keywords")
  expect_s3_class(edges, "bibnets_network")
  expect_equal(attr(edges, "network_type"), "authors_by_keywords")
})

test_that("conetwork by= links entities through shared values", {
  d <- make_test_data()
  ## Authors sharing a keyword are linked
  edges <- conetwork(d, "authors", by = "keywords")
  ## Alice and Carol both use nlp (via W2)
  all_nodes <- unique(c(edges$from, edges$to))
  ## At least some authors should be linked
  expect_true(nrow(edges) > 0)
})

test_that("conetwork by= normalizes field labels before aggregation", {
  d <- data.frame(
    id = c("W1", "W2", "W3"),
    tags = c("ml; dl", "ML; cv", "dl; CV"),
    topics = c("x; y", "x", "y"),
    stringsAsFactors = FALSE
  )

  edges <- conetwork(d, "tags", by = "topics")
  all_nodes <- unique(c(edges$from, edges$to))

  expect_false(any(all_nodes %in% c("ml", "dl", "cv")))
  expect_true(all(all_nodes %in% c("ML", "DL", "CV")))
})

test_that("conetwork splits string columns on sep", {
  d <- data.frame(
    id   = c("W1", "W2", "W3"),
    tags = c("ml; dl", "ml; nlp", "dl; nlp"),
    stringsAsFactors = FALSE
  )
  edges <- conetwork(d, "tags")
  expect_true(is.data.frame(edges))
  expect_true(nrow(edges) > 0)
  all_nodes <- unique(c(edges$from, edges$to))
  ## build_bipartite uppercases entity names
  expect_true(any(grepl("ML|DL|NLP", all_nodes)))
})

test_that("conetwork counting attribute is preserved", {
  d <- make_test_data()
  edges <- conetwork(d, "keywords", counting = "fractional")
  expect_equal(attr(edges, "counting"), "fractional")
})

test_that("conetwork similarity attribute is preserved", {
  d <- make_test_data()
  edges <- conetwork(d, "keywords", similarity = "association")
  expect_equal(attr(edges, "similarity"), "association")
})

test_that("author_network collaboration with full counting", {
  d <- make_test_data()
  edges <- author_network(d, "collaboration")

  expect_true(is.data.frame(edges))
  expect_true(all(c("from", "to", "weight", "count") %in% names(edges)))

  get_w <- function(a, b) {
    row <- edges[
      (edges$from == a & edges$to == b) |
      (edges$from == b & edges$to == a), ]
    if (nrow(row) == 0) return(0)
    row$weight[1]
  }

  expect_equal(get_w("ALICE", "BOB"), 1)
  expect_equal(get_w("ALICE", "CAROL"), 1)
  expect_equal(get_w("BOB", "CAROL"), 1)
  expect_equal(get_w("ALICE", "DAN"), 0)
})

test_that("author_network collaboration with harmonic counting", {
  d <- make_test_data()
  edges <- author_network(d, "collaboration", counting = "harmonic")

  get_w <- function(a, b) {
    row <- edges[
      (edges$from == a & edges$to == b) |
      (edges$from == b & edges$to == a), ]
    if (nrow(row) == 0) return(0)
    row$weight[1]
  }

  ## W1: Alice (2/3), Bob (1/3) -> co-auth = 2/3 * 1/3 = 2/9
  expect_equal(get_w("ALICE", "BOB"), 2/9, tolerance = tol)

  ## W3: Bob (1/H3), Carol ((1/2)/H3), Dan ((1/3)/H3)
  h3 <- 1 + 1/2 + 1/3
  ## Bob-Carol from W3: (1/h3) * ((1/2)/h3)
  expect_equal(get_w("BOB", "CAROL"), (1/h3) * ((1/2)/h3), tolerance = tol)
})

test_that("author_network collaboration with geometric counting", {
  d <- make_test_data()
  edges <- author_network(d, "collaboration", counting = "geometric")

  get_w <- function(a, b) {
    row <- edges[
      (edges$from == a & edges$to == b) |
      (edges$from == b & edges$to == a), ]
    if (nrow(row) == 0) return(0)
    row$weight[1]
  }

  ## W2: Alice (pos 1), Carol (pos 2), n=2
  ## geometric: (1, 0.5) / 1.5 = (2/3, 1/3)
  expect_equal(get_w("ALICE", "CAROL"), (2/3) * (1/3), tolerance = tol)
})

test_that("author_network collaboration with first-author counting", {
  d <- make_test_data()
  edges <- author_network(d, "collaboration", counting = "first")

  get_w <- function(a, b) {
    row <- edges[
      (edges$from == a & edges$to == b) |
      (edges$from == b & edges$to == a), ]
    if (nrow(row) == 0) return(0)
    row$weight[1]
  }

  ## W1: Alice=1, Bob=0 -> link = 0
  ## W3: Bob=1, Carol=0, Dan=0 -> all links = 0
  ## No edges should exist with first-author counting in co-authorship
  ## because non-first authors have weight 0
  expect_equal(nrow(edges), 0)
})

test_that("author_network with normalization", {
  d <- make_test_data()
  edges <- author_network(d, "collaboration", similarity = "association")

  expect_true(all(edges$weight >= 0))
  expect_true(all(is.finite(edges$weight)))
})

test_that("author_network coupling type works", {
  d <- make_test_data()
  edges <- author_network(d, "coupling")

  expect_true(is.data.frame(edges))
  expect_true(nrow(edges) > 0)
})

## -- attention parameter tests -----------------------------------------------

test_that("attention: all four methods return valid bibnets_network", {
  d <- make_test_data()
  for (att in c("proximity", "lead", "last", "circular")) {
    edges <- author_network(d, attention = att)
    expect_true(inherits(edges, "bibnets_network"), info = att)
    expect_true(all(c("from", "to", "weight", "count") %in% names(edges)), info = att)
    expect_true(all(edges$weight >= 0), info = att)
    expect_true(all(is.finite(edges$weight)), info = att)
  }
})

test_that("attention lead: first author gets highest individual weight", {
  d <- make_test_data()
  # W1: Alice(p1) + Bob(p2), n=2; lead weights = (4,1)/5
  # W(Alice,Bob) = sqrt(4/5) * sqrt(1/5) ... no: B'B gives sum of w_i * w_j
  # W1: Alice weight = 4/5, Bob weight = 1/5 -> edge = 4/5 * 1/5 = 4/25
  # W2: Alice(p1) + Carol(p2) -> same: Alice=4/5, Carol=1/5 -> edge = 4/25
  # W3: Bob(p1)+Carol(p2)+Dan(p3), n=3; (n+1-pos)^2 = (9,4,1)/14
  #   Bob=9/14, Carol=4/14, Dan=1/14
  edges <- author_network(d, attention = "lead")

  get_w <- function(a, b) {
    row <- edges[(edges$from == a & edges$to == b) |
                   (edges$from == b & edges$to == a), ]
    if (nrow(row) == 0) return(0)
    row$weight[1]
  }

  expect_equal(get_w("ALICE", "BOB"),   (4/5) * (1/5),           tolerance = tol)
  expect_equal(get_w("ALICE", "CAROL"), (4/5) * (1/5),           tolerance = tol)
  expect_equal(get_w("BOB",   "CAROL"), (9/14) * (4/14),         tolerance = tol)
  expect_equal(get_w("BOB",   "DAN"),   (9/14) * (1/14),         tolerance = tol)
  expect_equal(get_w("CAROL", "DAN"),   (4/14) * (1/14),         tolerance = tol)
})

test_that("attention last: last author gets highest individual weight", {
  d <- make_test_data()
  # W1: Alice(p1)+Bob(p2), n=2; pos^2 = (1,4)/5; Alice=1/5, Bob=4/5
  # W2: Alice(p1)+Carol(p2); Alice=1/5, Carol=4/5
  # W3: Bob(p1)+Carol(p2)+Dan(p3), n=3; pos^2=(1,4,9)/14
  edges <- author_network(d, attention = "last")

  get_w <- function(a, b) {
    row <- edges[(edges$from == a & edges$to == b) |
                   (edges$from == b & edges$to == a), ]
    if (nrow(row) == 0) return(0)
    row$weight[1]
  }

  expect_equal(get_w("ALICE", "BOB"),   (1/5) * (4/5),   tolerance = tol)
  expect_equal(get_w("BOB",   "DAN"),   (1/14) * (9/14), tolerance = tol)
  expect_equal(get_w("CAROL", "DAN"),   (4/14) * (9/14), tolerance = tol)
  ## last author (Dan) gets more weight than lead (Bob) in W3
  expect_true(get_w("CAROL", "DAN") > get_w("BOB", "CAROL"))
})

test_that("attention proximity: center authors weighted more than edges", {
  d <- make_test_data()
  # W3: Bob(p1)+Carol(p2)+Dan(p3), n=3; pmin = (1,2,1)/4
  # Bob=1/4, Carol=2/4, Dan=1/4
  edges <- author_network(d, attention = "proximity")

  get_w <- function(a, b) {
    row <- edges[(edges$from == a & edges$to == b) |
                   (edges$from == b & edges$to == a), ]
    if (nrow(row) == 0) return(0)
    row$weight[1]
  }

  expect_equal(get_w("BOB",   "CAROL"), (1/4) * (2/4), tolerance = tol)
  expect_equal(get_w("CAROL", "DAN"),   (2/4) * (1/4), tolerance = tol)
  expect_equal(get_w("BOB",   "DAN"),   (1/4) * (1/4), tolerance = tol)
  ## center pair outweighs edge pair
  expect_true(get_w("BOB", "CAROL") > get_w("BOB", "DAN"))
})

test_that("attention circular: first and last outweigh middle", {
  d <- make_test_data()
  # W3: Bob(p1)+Carol(p2)+Dan(p3), n=3; pmax = (3,2,3)/8
  # Bob=3/8, Carol=2/8, Dan=3/8
  edges <- author_network(d, attention = "circular")

  get_w <- function(a, b) {
    row <- edges[(edges$from == a & edges$to == b) |
                   (edges$from == b & edges$to == a), ]
    if (nrow(row) == 0) return(0)
    row$weight[1]
  }

  expect_equal(get_w("BOB",   "DAN"),   (3/8) * (3/8), tolerance = tol)
  expect_equal(get_w("BOB",   "CAROL"), (3/8) * (2/8), tolerance = tol)
  ## first-last edge outweighs first-middle
  expect_true(get_w("BOB", "DAN") > get_w("BOB", "CAROL"))
})

test_that("attention: invalid method gives clear error", {
  d <- make_test_data()
  expect_error(author_network(d, attention = "bad"), "attention")
})

test_that("attention: works on keyword_network", {
  d <- make_test_data()
  edges <- keyword_network(d, attention = "lead")
  expect_s3_class(edges, "bibnets_network")
  expect_true(nrow(edges) > 0)
})

test_that("attention: works on country_network", {
  d <- make_test_data()
  d$countries <- list(c("FI", "SE"), c("FI", "DE"), c("SE", "DE", "FI"))
  edges <- country_network(d, attention = "circular")
  expect_s3_class(edges, "bibnets_network")
  expect_true(nrow(edges) > 0)
})

test_that("attention: works on institution_network", {
  d <- make_test_data()
  d$affiliations <- list(c("MIT", "Harvard"), c("MIT", "Stanford"),
                         c("Harvard", "Stanford", "MIT"))
  edges <- institution_network(d, attention = "proximity")
  expect_s3_class(edges, "bibnets_network")
  expect_true(nrow(edges) > 0)
})

#' Prune a weighted edge list
#'
#' Reduces a weighted edge list by removing weak or excess edges.
#'
#' @param edges A data frame with at least columns `from`, `to`, and `weight`.
#' @param threshold Numeric. Keep only edges with `weight >= threshold`.
#' @param top_n Integer. For each node, keep only its `top_n` strongest edges.
#'   An edge is kept if it is in the top `top_n` for *either* endpoint.
#'
#' @return The filtered edge data frame (same columns as input).
#'
#' @export
#' @examples
#' edges <- data.frame(
#'   from   = c("A","A","A","B","B","C"),
#'   to     = c("B","C","D","C","D","D"),
#'   weight = c(5,  1,  2,  4,  1,  3)
#' )
#'
#' # Keep only edges with weight >= 3
#' prune(edges, threshold = 3)
#'
#' # Keep the 2 strongest edges per node
#' prune(edges, top_n = 2)
prune <- function(edges, threshold = NULL, top_n = NULL) {
  check_edges(edges)
  if (is.null(threshold) && is.null(top_n))
    stop("One of 'threshold' or 'top_n' must be specified.", call. = FALSE)
  if (!is.null(threshold) && !is.null(top_n))
    stop("Only one of 'threshold' or 'top_n' may be specified.", call. = FALSE)

  if (nrow(edges) == 0L) return(edges)

  if (!is.null(threshold)) {
    if (!is.numeric(threshold) || length(threshold) != 1L || threshold < 0)
      stop("'threshold' must be a non-negative number", call. = FALSE)
    edges <- edges[edges$weight >= threshold, ]
    edges <- edges[order(-edges$weight), ]
    rownames(edges) <- NULL
    return(edges)
  }

  if (!is.numeric(top_n) || length(top_n) != 1L || top_n <= 0)
    stop("'top_n' must be a positive number", call. = FALSE)
  n <- as.integer(top_n)

  adj <- split(c(edges$weight, edges$weight), c(edges$from, edges$to))
  cutoff <- vapply(adj, function(w) {
    if (length(w) <= n) return(-Inf)
    sort(w, decreasing = TRUE)[n]
  }, numeric(1L))

  keep <- mapply(function(f, t, w) {
    w >= cutoff[f] | w >= cutoff[t]
  }, edges$from, edges$to, edges$weight)

  edges <- edges[keep, ]
  edges <- edges[order(-edges$weight), ]
  rownames(edges) <- NULL
  edges
}

## Tests for prune() — threshold and top_n pruning

make_prune_edges <- function() {
  data.frame(
    from   = c("A","A","A","B","B","C"),
    to     = c("B","C","D","C","D","D"),
    weight = c(5,  1,  2,  4,  1,  3),
    stringsAsFactors = FALSE
  )
}

## ── threshold ──────────────────────────────────────────────────────────────

test_that("prune threshold keeps edges at or above value", {
  edges <- make_prune_edges()
  p <- prune(edges, threshold = 3)
  expect_true(all(p$weight >= 3))
  expect_equal(nrow(p), 3)   # weights 5, 4, 3
})

test_that("prune threshold=0 keeps everything", {
  edges <- make_prune_edges()
  p <- prune(edges, threshold = 0)
  expect_equal(nrow(p), nrow(edges))
})

test_that("prune threshold=6 drops everything", {
  edges <- make_prune_edges()
  p <- prune(edges, threshold = 6)
  expect_equal(nrow(p), 0)
})

test_that("prune threshold result sorted descending by weight", {
  edges <- make_prune_edges()
  p <- prune(edges, threshold = 0)
  expect_true(all(diff(p$weight) <= 0))
})

## ── top_n ──────────────────────────────────────────────────────────────────

test_that("prune top_n keeps correct edges", {
  edges <- make_prune_edges()
  p <- prune(edges, top_n = 2)

  expect_equal(nrow(p), 4)
  expect_true(all(p$weight >= 2))

  has_edge <- function(f, t) {
    any((p$from == f & p$to == t) | (p$from == t & p$to == f))
  }
  expect_true(has_edge("A", "B"))
  expect_true(has_edge("A", "D"))
  expect_true(has_edge("B", "C"))
  expect_true(has_edge("C", "D"))
  expect_false(has_edge("A", "C"))
  expect_false(has_edge("B", "D"))
})

test_that("prune top_n with n >= max degree keeps everything", {
  edges <- make_prune_edges()
  p <- prune(edges, top_n = 10)
  expect_equal(nrow(p), nrow(edges))
})

test_that("prune top_n=1 keeps only the strongest edge per node", {
  edges <- make_prune_edges()
  p <- prune(edges, top_n = 1)

  has_edge <- function(f, t) {
    any((p$from == f & p$to == t) | (p$from == t & p$to == f))
  }
  expect_true(has_edge("A", "B"))
  expect_true(has_edge("B", "C"))
})

test_that("prune top_n result sorted descending by weight", {
  edges <- make_prune_edges()
  p <- prune(edges, top_n = 2)
  expect_true(all(diff(p$weight) <= 0))
})

## ── input preservation ─────────────────────────────────────────────────────

test_that("prune preserves extra columns", {
  edges <- make_prune_edges()
  edges$count <- 1L

  p <- prune(edges, threshold = 3)
  expect_true("count" %in% names(p))
})

test_that("prune returns empty data frame on empty input", {
  empty <- data.frame(from = character(0), to = character(0),
                       weight = numeric(0), stringsAsFactors = FALSE)
  p <- prune(empty, threshold = 1)
  expect_equal(nrow(p), 0)
})

test_that("prune errors when neither threshold nor top_n given", {
  edges <- make_prune_edges()
  expect_error(prune(edges), "must be specified")
})

test_that("prune errors when both threshold and top_n given", {
  edges <- make_prune_edges()
  expect_error(prune(edges, threshold = 2, top_n = 3), "Only one")
})

test_that("prune rejects top_n=0", {
  edges <- make_prune_edges()
  expect_error(prune(edges, top_n = 0), "positive")
})

## ----include = FALSE----------------------------------------------------------
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>"
)

## ----setup--------------------------------------------------------------------
library(bibnets)

## ----read-biblio-signature, eval = FALSE--------------------------------------
# data <- read_biblio("export.csv")          # auto-detect format
# data <- read_biblio("scopus_dir/")         # entire directory, rbind'd
# data <- read_biblio(c("a.csv", "b.csv"))   # multiple files, rbind'd
# data <- read_biblio("file.csv", format = "scopus")   # force a format

## ----openalex-csv-demo--------------------------------------------------------
f <- system.file("extdata", "openalex_works.csv", package = "bibnets")
oa <- read_openalex_csv(f)
str(oa, max.level = 1)
head(oa[, c("id", "title", "year", "journal", "type")], 5)

## ----openalex-csv-lists-------------------------------------------------------
oa$authors[[1]]
oa$affiliations[[1]]
oa$countries[[1]]

## ----openalex-csv-limits------------------------------------------------------
all(vapply(oa$references, function(x) length(x) == 0 || all(is.na(x)), logical(1)))
all(is.na(oa$abstract))

## ----openalex-csv-networks----------------------------------------------------
co <- country_network(oa, counting = "fractional")
head(co, 5)

## ----scopus-call, eval = FALSE------------------------------------------------
# sc <- read_scopus("scopus.csv")

## ----wos-call, eval = FALSE---------------------------------------------------
# wos1 <- read_wos("savedrecs.txt")                       # plaintext (default)
# wos2 <- read_wos("savedrecs.tsv", format = "tab")       # tab-delimited

## ----openalex-fetch, eval = FALSE---------------------------------------------
# library(openalexR)
# raw  <- oa_fetch(entity = "works", search = "learning analytics", per_page = 200)
# data <- read_openalex(raw)

## ----dimensions-call, eval = FALSE--------------------------------------------
# dm <- read_dimensions("dimensions_export.csv")

## ----lens-call, eval = FALSE--------------------------------------------------
# ln <- read_lens("lens_export.csv")

## ----bibtex-ris-call, eval = FALSE--------------------------------------------
# bt <- read_bibtex("library.bib")
# ri <- read_ris("savedrecs.ris")

## ----crossref-call, eval = FALSE----------------------------------------------
# library(rcrossref)
# raw  <- cr_works(query = "graph neural networks", limit = 100)
# data <- read_crossref(raw$data)

## ----generic-call, eval = FALSE-----------------------------------------------
# data <- read_biblio(
#   "my_data.csv",
#   format  = "generic",
#   id      = "doc_id",
#   actors  = c("Authors", "Keywords"),
#   sep     = ";"
# )

## ----generic-demo-------------------------------------------------------------
f <- system.file("extdata", "openalex_works.csv", package = "bibnets")
generic <- read_biblio(
  f,
  format = "generic",
  id     = "id",
  actors = c("authorships.author.display_name", "primary_topic.display_name"),
  sep    = "|"
)
names(generic)[1:6]
generic$authorships.author.display_name[[1]]

## ----manual-build-------------------------------------------------------------
df <- data.frame(
  id    = c("p1", "p2", "p3"),
  title = c("Paper A", "Paper B", "Paper C"),
  year  = c(2020L, 2021L, 2022L),
  stringsAsFactors = FALSE
)
df$authors <- list(
  c("ALICE", "BOB"),
  c("BOB", "CAROL"),
  c("ALICE", "CAROL", "DAVE")
)
df$references <- list(
  c("R1", "R2"),
  c("R1", "R3"),
  c("R2", "R3", "R4")
)
df$keywords <- list(
  c("graph", "network"),
  c("network", "embedding"),
  c("graph", "embedding", "neural")
)

author_network(df, "collaboration")
keyword_network(df)
reference_network(df)

## ----split-field-demo---------------------------------------------------------
split_field(c("Alice; Bob; Carol", "Dave; Eve"))
split_field(c("a|b|c", "d|e"), sep = "|")

## ----combine-sources----------------------------------------------------------
common <- c("id", "title", "year", "journal", "doi", "cited_by_count",
            "abstract", "type", "authors", "references", "keywords")

data(biblio_data)
b1 <- biblio_data
b2 <- biblio_data
b2$id <- paste0(b2$id, "_dup")

cols <- intersect(common, names(b1))
combined <- rbind(b1[, cols], b2[, cols])
nrow(combined)

## ----sanity-check-------------------------------------------------------------
data(scopus_quantum_cloud)
sc <- scopus_quantum_cloud

range(lengths(sc$authors))
range(lengths(sc$references))
range(lengths(sc$keywords))

head(sort(table(sc$journal), decreasing = TRUE), 5)
range(sc$year, na.rm = TRUE)
table(sc$type)

#' Build an author network
#'
#' Constructs a network between authors using one of four relationship types
#' and any of 13 counting methods, including 9 position-dependent methods
#' that respect author byline order.
#'
#' @param data A data frame with at least `id` and `authors` (list-column,
#'   order preserved). For coupling/co-citation, also needs `references`.
#' @param type Character. Relationship type:
#'   \describe{
#'     \item{`"collaboration"`}{Co-authorship: authors linked when they
#'       co-author a publication.}
#'     \item{`"coupling"`}{Bibliographic coupling aggregated at author level:
#'       authors linked when they cite the same references.}
#'     \item{`"co_citation"`}{Author co-citation: authors linked when they
#'       are cited together by the same paper. Requires a
#'       `cited_first_authors` list-column.}
#'     \item{`"equivalence"`}{Profile similarity: cosine similarity of
#'       authors' full collaboration/citation profiles.}
#'   }
#' @param counting Character. Counting method. Position-independent methods
#'   (`"full"`, `"fractional"`, `"paper"`, `"strength"`) work for all types.
#'   Position-dependent methods (`"harmonic"`, `"arithmetic"`, `"geometric"`,
#'   `"adaptive_geometric"`, `"golden"`, `"first"`, `"last"`,
#'   `"first_last"`, `"position_weighted"`) are available for
#'   `type = "collaboration"`.
#' @param similarity Character. Similarity measure: `"none"`, `"association"`,
#'   `"cosine"`, `"jaccard"`, `"inclusion"`, `"equivalence"`.
#' @param threshold Numeric. Minimum edge weight. Default 0.
#' @param min_occur Integer. Minimum number of papers for an author to be
#'   included. Default 1.
#' @param position_weights Numeric vector. Custom weights for
#'   `counting = "position_weighted"`. Default `c(1, 0.8, 0.6, 0.4)`.
#' @param first_last_weight Numeric. Multiplier for `counting = "first_last"`.
#'   Default 2.
#' @param attention Character or NULL. Attention-based weighting independent of
#'   `type` and `counting`. One of `"proximity"` (center authors weighted most),
#'   `"lead"` (first author dominates, quadratic drop), `"last"` (last author
#'   dominates, quadratic rise), `"circular"` (first and last both prominent).
#'   Default `NULL` (disabled).
#' @param top_n Integer or NULL. Return only the top n edges by weight.
#'   Default NULL (all edges).
#' @param self_loops Logical. If `TRUE`, include self-loops (an entity linked
#'   to itself). Default `FALSE`.
#' @param deduplicate Logical. If `TRUE` (default), each `(paper, entity)`
#'   pair is counted at most once — duplicate entries in the source data
#'   (e.g., the same author listed twice on a paper) are treated as one
#'   occurrence. Set to `FALSE` to count every raw occurrence.
#' @param format Character. Output format:
#'   \describe{
#'     \item{`"edgelist"`}{Default. A `bibnets_network` data frame with
#'       columns `from`, `to`, `weight`, `count`.}
#'     \item{`"gephi"`}{Gephi-ready data frame: `Source`, `Target`,
#'       `Weight`, `Count`, `Type`.}
#'     \item{`"igraph"`}{An igraph graph object (requires igraph).}
#'     \item{`"cograph"`}{A cograph_network object (requires cograph).}
#'     \item{`"matrix"`}{A sparse adjacency matrix.}
#'   }
#'
#' @return Depends on `format`: a `bibnets_network` data frame (default),
#'   a Gephi-ready data frame, an igraph graph, a cograph_network, or a
#'   sparse matrix.
#'
#' @export
#' @examples
#' data(biblio_data)
#' author_network(biblio_data, "collaboration")
#' author_network(biblio_data, "collaboration", counting = "harmonic")
#' author_network(biblio_data, "collaboration", counting = "geometric",
#'                similarity = "association")
author_network <- function(data,
                           type = "collaboration",
                           counting = "full",
                           similarity = "none",
                           threshold = 0,
                           min_occur = 1L,
                           position_weights = c(1, 0.8, 0.6, 0.4),
                           first_last_weight = 2,
                           attention = NULL,
                           top_n = NULL,
                           self_loops = FALSE,
                           deduplicate = TRUE,
                           format = "edgelist") {
  check_data(data, c("id", "authors"))
  check_choice(similarity, c("none", "association", "cosine", "jaccard",
                              "inclusion", "equivalence"), "similarity")
  check_format(format)

  if (!is.null(attention)) {
    check_choice(attention, all_attention_methods(), "attention")
    B <- build_author_bipartite(data, field = "authors",
                                counting = paste0("attention_", attention),
                                deduplicate = deduplicate)
    result <- multiply_bipartite(B, mode = "columns", similarity = similarity,
                                 threshold = threshold, top_n = top_n,
                                 self_loops = self_loops)
    return(as_bibnets_network(result,
      network_type = paste0("author_attention_", attention),
      counting = attention, similarity = similarity, format = format))
  }

  check_choice(type, c("collaboration", "coupling", "co_citation", "equivalence"), "type")
  check_choice(counting, all_counts(), "counting")

  is_positional <- counting %in% position_dependent_counts()

  result <- if (type == "collaboration") {
    if (is_positional) {
      B <- build_author_bipartite(
        data, counting = counting,
        position_weights = position_weights,
        first_last_weight = first_last_weight,
        deduplicate = deduplicate
      )
      m <- if (type == "equivalence") "cosine" else similarity
      multiply_bipartite(B, mode = "columns", similarity = m,
                         threshold = threshold, top_n = top_n,
                         self_loops = self_loops)
    } else {
      B <- build_bipartite(data, field = "authors", min_freq = min_occur, deduplicate = deduplicate)
      B <- apply_counting(B, counting = counting, network_type = "symmetric")
      multiply_bipartite(B, mode = "columns", similarity = similarity,
                         threshold = threshold, top_n = top_n,
                         self_loops = self_loops)
    }

  } else if (type == "coupling") {
    check_data(data, "references")
    agg <- aggregate_by_entity(data, entity_field = "authors",
                                value_field = "references",
                                min_freq = min_occur)
    B <- build_bipartite(agg, field = "references")
    ct <- if (is_positional) "full" else counting
    B <- apply_counting(B, counting = ct, network_type = "coupling")
    multiply_bipartite(B, mode = "rows", similarity = similarity,
                       threshold = threshold, top_n = top_n,
                       self_loops = self_loops)

  } else if (type == "co_citation") {
    field <- if ("cited_first_authors" %in% names(data)) {
      "cited_first_authors"
    } else {
      stop("Column 'cited_first_authors' not found. ",
           "Parse reference strings to extract cited authors first.",
           call. = FALSE)
    }
    B <- build_bipartite(data, field = field, min_freq = min_occur, deduplicate = deduplicate)
    ct <- if (is_positional) "full" else counting
    B <- apply_counting(B, counting = ct, network_type = "symmetric")
    multiply_bipartite(B, mode = "columns", similarity = similarity,
                       threshold = threshold, top_n = top_n)

  } else {
    B <- build_bipartite(data, field = "authors", min_freq = min_occur)
    multiply_bipartite(B, mode = "columns", similarity = "cosine",
                       threshold = threshold, top_n = top_n,
                       self_loops = self_loops)
  }

  as_bibnets_network(result,
    network_type = paste0("author_", type),
    counting = counting, similarity = similarity,
    format = format)
}

## Tests for filter_top()

test_that("filter_top keeps only edges among top-n nodes", {
  d <- make_test_data()
  edges <- author_network(d, "collaboration")

  ## 4 authors: Alice(2 papers), Bob(2 papers), Carol(2 papers), Dan(1 paper)
  ## Degree in the network: Alice touches 2 edges, Bob touches 2, Carol touches 2, Dan touches 1
  top3 <- filter_top(edges, 3)

  top_nodes <- unique(c(top3$from, top3$to))
  expect_true(length(top_nodes) <= 3)
})

test_that("filter_top returns all edges when n >= number of nodes", {
  d <- make_test_data()
  edges <- author_network(d, "collaboration")
  n_nodes <- length(unique(c(edges$from, edges$to)))

  all_edges <- filter_top(edges, n_nodes)
  expect_equal(nrow(all_edges), nrow(edges))
})

test_that("filter_top returns empty when n=0 edges result", {
  ## Single edge — 2 nodes. filter_top(n=1) can only include 1 node → no edge
  edges <- data.frame(from = "A", to = "B", weight = 1,
                       stringsAsFactors = FALSE)
  ## filter_top(1): top 1 node is A or B; the edge requires both → 0 edges
  top1 <- filter_top(edges, 1)
  expect_equal(nrow(top1), 0)
})

test_that("filter_top sorts result by weight descending", {
  d <- make_test_data()
  edges <- author_network(d, "collaboration", counting = "full")
  top <- filter_top(edges, 4)
  if (nrow(top) > 1) {
    expect_true(all(diff(top$weight) <= 0))
  }
})

test_that("filter_top validates inputs", {
  edges <- data.frame(from = "A", to = "B", weight = 1, stringsAsFactors = FALSE)
  expect_error(filter_top(edges, 0))
  expect_error(filter_top(edges, -1))
  expect_error(filter_top(list(), 2))
})

test_that("filter_top handles empty edge list", {
  empty <- data.frame(from = character(0), to = character(0),
                       weight = numeric(0), stringsAsFactors = FALSE)
  top <- filter_top(empty, 5)
  expect_equal(nrow(top), 0)
})

#' Read bibliometric data
#'
#' Universal reader that handles files, folders, format detection, and
#' generic CSV input. Accepts a single file, multiple files, or a directory.
#'
#' @param path Character. Path to a file, a vector of file paths, or a
#'   directory containing export files.
#' @param format Character. File format:
#'   \describe{
#'     \item{`"auto"`}{Default. Auto-detect from file content.}
#'     \item{`"scopus"`}{Scopus CSV.}
#'     \item{`"wos"`}{Web of Science plaintext.}
#'     \item{`"wos_tab"`}{Web of Science tab-delimited.}
#'     \item{`"bibtex"`}{BibTeX .bib file.}
#'     \item{`"ris"`}{RIS file.}
#'     \item{`"dimensions"`}{Dimensions CSV.}
#'     \item{`"lens"`}{Lens.org CSV.}
#'     \item{`"openalex_csv"`}{Flat OpenAlex CSV export (pipe-delimited fields).}
#'     \item{`"generic"`}{Any CSV. Use `id` and `actors` to specify columns.}
#'   }
#' @param id Character. Column name for document identifier. Only used
#'   when `format = "generic"`. Default `NULL` (uses row numbers).
#' @param actors Character vector. Column names to split into list-columns.
#'   Only used when `format = "generic"`.
#' @param sep Character. Delimiter for splitting actor columns. Default `";"`.
#' @param ... Additional arguments passed to the format-specific reader.
#'
#' @return A data frame.
#'
#' @export
#' @examples
#' # Auto-detect format from file content (here: a bundled OpenAlex CSV)
#' f <- system.file("extdata", "openalex_works.csv", package = "bibnets")
#' data <- read_biblio(f)
#' head(data[, c("id", "title", "year", "journal")])
#'
#' # Read multiple files at once; auto-detects each format
#' f_scopus <- system.file("extdata", "scopus_sample.csv", package = "bibnets")
#' f_wos    <- system.file("extdata", "wos_sample.txt",  package = "bibnets")
#' combined <- read_biblio(c(f_scopus, f_wos))
#' head(combined[, c("id", "title", "year", "journal")])
#'
#' # Read every supported export in a directory (here: the bundled extdata)
#' folder <- system.file("extdata", package = "bibnets")
#' all_data <- read_biblio(folder)
#' nrow(all_data)
#'
#' # Generic CSV: point read_biblio at any CSV and name the list-column fields
#' tmp <- tempfile(fileext = ".csv")
#' write.csv(data.frame(
#'   doc_id  = c("a", "b"),
#'   Authors = c("Smith J; Jones A", "Davis M"),
#'   Keywords = c("networks; bibliometrics", "analytics")
#' ), tmp, row.names = FALSE)
#' generic <- read_biblio(tmp, format = "generic",
#'                        id = "doc_id",
#'                        actors = c("Authors", "Keywords"),
#'                        sep = ";")
#' head(generic)
read_biblio <- function(path,
                        format = "auto",
                        id = NULL,
                        actors = NULL,
                        sep = ";",
                        ...) {
  ## Collect all file paths
  files <- resolve_paths(path)

  if (length(files) == 0) {
    stop("No files found at: ", paste(path, collapse = ", "), call. = FALSE)
  }

  ## Read each file
  dfs <- lapply(files, function(f) {
    read_single_biblio(f, format = format, id = id, actors = actors,
                        sep = sep, ...)
  })

  ## Combine
  dfs <- align_biblio_columns(dfs)
  result <- do.call(rbind, dfs)
  rownames(result) <- NULL

  if (length(files) > 1) {
    message(sprintf("Read %d files: %d rows total", length(files), nrow(result)))
  }

  result
}


#' Align columns before row-binding bibliographic files
#' @keywords internal
align_biblio_columns <- function(dfs) {
  all_cols <- unique(unlist(lapply(dfs, names), use.names = FALSE))
  list_cols <- all_cols[vapply(all_cols, function(col) {
    any(vapply(dfs, function(d) col %in% names(d) && is.list(d[[col]]),
               logical(1L)))
  }, logical(1L))]

  lapply(dfs, function(d) {
    missing <- setdiff(all_cols, names(d))
    for (col in missing) {
      if (col %in% list_cols) {
        d[[col]] <- replicate(nrow(d), character(0), simplify = FALSE)
      } else {
        d[[col]] <- rep(NA, nrow(d))
      }
    }
    d[, all_cols, drop = FALSE]
  })
}


#' Read a single bibliometric file
#' @keywords internal
read_single_biblio <- function(file, format, id, actors, sep, ...) {
  if (format == "generic") {
    return(read_generic(file, id = id, actors = actors, sep = sep))
  }

  if (format == "auto") {
    format <- detect_format(file)
  }

  switch(format,
    scopus       = read_scopus(file, ...),
    wos          = read_wos(file, ...),
    wos_tab      = read_wos(file, format = "tab", ...),
    bibtex       = read_bibtex(file, ...),
    ris          = read_ris(file, ...),
    dimensions   = read_dimensions(file, ...),
    lens         = read_lens(file, ...),
    openalex_csv = read_openalex_csv(file, ...),
    stop(
      "Could not detect file format for: ", file, "\n\n",
      "Supported file formats:\n",
      "  auto, scopus, wos, wos_tab, bibtex, ris, dimensions, lens,\n",
      "  openalex_csv, generic\n\n",
      "Note: Nested OpenAlex (openalexR::oa_fetch()) and Crossref data\n",
      "must be loaded into R first, then converted with\n",
      "read_openalex() or read_crossref().\n\n",
      "For generic CSV, use: read_biblio(file, format = 'generic', ",
      "actors = c('Authors', 'Keywords'), sep = ';')",
      call. = FALSE
    )
  )
}


#' Read a generic CSV with user-specified columns
#' @keywords internal
read_generic <- function(file, id = NULL, actors = NULL, sep = ";") {
  check_file(file)

  data <- utils::read.csv(file, stringsAsFactors = FALSE, fileEncoding = "UTF-8",
                           check.names = FALSE)

  ## Set ID column
  if (!is.null(id) && id %in% names(data)) {
    data[["id"]] <- as.character(data[[id]])
  } else if (!"id" %in% names(data)) {
    data[["id"]] <- as.character(seq_len(nrow(data)))
  }

  ## Split actor columns into list-columns
  if (!is.null(actors)) {
    cols <- intersect(actors, names(data))
    data[cols] <- lapply(data[cols], split_field, sep = sep)
  }

  data
}


#' Resolve file paths from a file, vector of files, or directory
#' @keywords internal
resolve_paths <- function(path) {
  unlist(lapply(path, function(p) {
    if (dir.exists(p)) {
      list.files(p, pattern = "\\.(csv|txt|bib|ris|xlsx?)$",
                 full.names = TRUE, ignore.case = TRUE)
    } else if (file.exists(p)) {
      p
    } else {
      character(0)
    }
  }), use.names = FALSE)
}


#' Detect bibliometric file format
#' @param file Path to file.
#' @return Character: format name or `"unknown"`.
#' @keywords internal
detect_format <- function(file) {
  lines <- readLines(file, n = 10, warn = FALSE, encoding = "UTF-8")
  lines <- lines[nchar(trimws(lines)) > 0]
  if (length(lines) == 0) return("unknown")

  first <- trimws(lines[1])

  ## BibTeX: starts with @
  if (grepl("^@", first)) return("bibtex")

  ## RIS: starts with TY  -
  if (grepl("^TY\\s+-", first)) return("ris")

  ## WoS plaintext: starts with FN or PT
  if (grepl("^(FN|PT)\\s", first)) return("wos")

  ## CSV-based: check header line — Dimensions prepends a metadata row so
  ## also check line 2 when line 1 looks like "About the data: ..."
  header <- tolower(first)
  if (grepl("^\"?about the data", header) && length(lines) >= 2) {
    header <- tolower(trimws(lines[2]))
  }

  ## Scopus: has EID column
  if (grepl("\\beid\\b", header)) return("scopus")

  ## Dimensions: has "publication id" or "dimensions url"
  if (grepl("publication id|dimensions url", header)) return("dimensions")

  ## Lens: has "lens id"
  if (grepl("lens id", header)) return("lens")

  ## OpenAlex flat CSV: has "authorships.author.display_name" column header
  if (grepl("authorships\\.author\\.display_name", header)) return("openalex_csv")

  "unknown"
}

## Tests for read_ris()
## testthat 3.0 — no network calls, all data via tempfile()

## Helper: write RIS text to a temp file and return the path
ris_tempfile <- function(text) {
  f <- tempfile(fileext = ".ris")
  writeLines(text, f)
  f
}

## ── Standard columns ─────────────────────────────────────────────────────────

test_that("read_ris returns the 11 standard bibnets columns", {
  f <- ris_tempfile(c(
    "TY  - JOUR",
    "AU  - Smith, J.",
    "TI  - A test title",
    "JO  - Test Journal",
    "PY  - 2021",
    "DO  - 10.1000/xyz",
    "AB  - An abstract.",
    "KW  - network",
    "ER  - "
  ))
  d <- read_ris(f)
  expected <- c("id", "title", "year", "journal", "doi",
                "cited_by_count", "abstract", "type",
                "authors", "references", "keywords")
  expect_true(all(expected %in% names(d)))
})

test_that("read_ris column order matches bibnets standard schema", {
  f <- ris_tempfile(c(
    "TY  - JOUR",
    "AU  - Smith, J.",
    "TI  - Title",
    "JO  - Journal",
    "PY  - 2020",
    "ER  - "
  ))
  d <- read_ris(f)
  scalar_cols <- c("id", "title", "year", "journal", "doi",
                   "cited_by_count", "abstract", "type")
  expect_equal(names(d)[seq_along(scalar_cols)], scalar_cols)
})

## ── Single record: values ────────────────────────────────────────────────────

test_that("read_ris single record: scalar fields parsed correctly", {
  f <- ris_tempfile(c(
    "TY  - JOUR",
    "AU  - Smith, Jane",
    "TI  - Bibliometric analysis",
    "JO  - Test Journal",
    "PY  - 2020",
    "DO  - 10.1000/test",
    "AB  - An abstract here.",
    "ER  - "
  ))
  d <- read_ris(f)
  expect_equal(nrow(d), 1L)
  expect_equal(d$title,   "Bibliometric analysis")
  expect_equal(d$journal, "Test Journal")
  expect_equal(d$year,    2020L)
  expect_equal(d$doi,     "10.1000/test")
  expect_equal(d$abstract, "An abstract here.")
  expect_equal(d$type,    "JOUR")
})

test_that("read_ris year is integer type", {
  f <- ris_tempfile(c(
    "TY  - JOUR",
    "TI  - Title",
    "PY  - 2019",
    "ER  - "
  ))
  d <- read_ris(f)
  expect_type(d$year, "integer")
})

test_that("read_ris cited_by_count is always NA_integer_", {
  f <- ris_tempfile(c(
    "TY  - JOUR",
    "TI  - Title",
    "PY  - 2020",
    "ER  - "
  ))
  d <- read_ris(f)
  expect_true(is.na(d$cited_by_count))
  expect_type(d$cited_by_count, "integer")
})

## ── Multi-record ─────────────────────────────────────────────────────────────

test_that("read_ris multi-record: correct row count", {
  f <- ris_tempfile(c(
    "TY  - JOUR",
    "TI  - Paper One",
    "PY  - 2020",
    "ER  - ",
    "TY  - CONF",
    "TI  - Paper Two",
    "PY  - 2021",
    "ER  - ",
    "TY  - BOOK",
    "TI  - Paper Three",
    "PY  - 2022",
    "ER  - "
  ))
  d <- read_ris(f)
  expect_equal(nrow(d), 3L)
})

test_that("read_ris multi-record: no field bleeding between records", {
  f <- ris_tempfile(c(
    "TY  - JOUR",
    "TI  - First Paper",
    "AU  - Alpha, A.",
    "PY  - 2000",
    "DO  - 10.1/first",
    "ER  - ",
    "TY  - JOUR",
    "TI  - Second Paper",
    "AU  - Beta, B.",
    "PY  - 2001",
    "DO  - 10.1/second",
    "ER  - "
  ))
  d <- read_ris(f)
  expect_equal(d$title[1],  "First Paper")
  expect_equal(d$title[2],  "Second Paper")
  expect_equal(d$doi[1],   "10.1/first")
  expect_equal(d$doi[2],   "10.1/second")
  expect_equal(d$year[1],  2000L)
  expect_equal(d$year[2],  2001L)
})

test_that("read_ris multi-record: each row has its own author list", {
  f <- ris_tempfile(c(
    "TY  - JOUR",
    "TI  - Paper One",
    "AU  - Alpha, A.",
    "AU  - Beta, B.",
    "ER  - ",
    "TY  - JOUR",
    "TI  - Paper Two",
    "AU  - Gamma, G.",
    "ER  - "
  ))
  d <- read_ris(f)
  expect_equal(length(d$authors[[1]]), 2L)
  expect_equal(length(d$authors[[2]]), 1L)
})

## ── Authors list-column ──────────────────────────────────────────────────────

test_that("read_ris authors list-column has correct length for multi-author record", {
  f <- ris_tempfile(c(
    "TY  - JOUR",
    "TI  - Collab paper",
    "AU  - Smith, John",
    "AU  - Jones, Kate",
    "AU  - Brown, Lee",
    "PY  - 2022",
    "ER  - "
  ))
  d <- read_ris(f)
  expect_equal(length(d$authors[[1]]), 3L)
})

test_that("read_ris authors are uppercased (standardize_authors)", {
  f <- ris_tempfile(c(
    "TY  - JOUR",
    "TI  - Title",
    "AU  - Smith, Jane",
    "ER  - "
  ))
  d <- read_ris(f)
  expect_equal(d$authors[[1]], toupper(d$authors[[1]]))
})

test_that("read_ris authors is always a list", {
  f <- ris_tempfile(c(
    "TY  - JOUR",
    "TI  - Title",
    "AU  - Smith, J.",
    "ER  - "
  ))
  d <- read_ris(f)
  expect_true(is.list(d$authors))
})

## ── Keywords list-column ─────────────────────────────────────────────────────

test_that("read_ris keywords list-column has correct length for multi-keyword record", {
  f <- ris_tempfile(c(
    "TY  - JOUR",
    "TI  - Paper",
    "KW  - networks",
    "KW  - bibliometrics",
    "KW  - co-citation",
    "ER  - "
  ))
  d <- read_ris(f)
  expect_equal(length(d$keywords[[1]]), 3L)
})

test_that("read_ris keywords values are trimmed", {
  f <- ris_tempfile(c(
    "TY  - JOUR",
    "TI  - Paper",
    "KW  -   spaced keyword  ",
    "ER  - "
  ))
  d <- read_ris(f)
  expect_equal(d$keywords[[1]], "spaced keyword")
})

test_that("read_ris keywords is always a list", {
  f <- ris_tempfile(c(
    "TY  - JOUR",
    "TI  - Title",
    "KW  - science",
    "ER  - "
  ))
  d <- read_ris(f)
  expect_true(is.list(d$keywords))
})

## ── References column ────────────────────────────────────────────────────────

test_that("read_ris references column is a list", {
  f <- ris_tempfile(c(
    "TY  - JOUR",
    "TI  - Title",
    "ER  - "
  ))
  d <- read_ris(f)
  expect_true(is.list(d$references))
})

test_that("read_ris references are empty for standard RIS (no references field)", {
  f <- ris_tempfile(c(
    "TY  - JOUR",
    "TI  - Title",
    "AU  - Smith, J.",
    "KW  - networks",
    "PY  - 2020",
    "ER  - "
  ))
  d <- read_ris(f)
  expect_equal(length(d$references[[1]]), 0L)
  expect_true(all(vapply(d$references, length, integer(1)) == 0L))
})

## ── Missing optional tags → NA / empty list ──────────────────────────────────

test_that("read_ris missing DOI yields NA", {
  f <- ris_tempfile(c(
    "TY  - JOUR",
    "TI  - No DOI paper",
    "PY  - 2018",
    "ER  - "
  ))
  d <- read_ris(f)
  expect_true(is.na(d$doi))
})

test_that("read_ris missing abstract yields NA", {
  f <- ris_tempfile(c(
    "TY  - JOUR",
    "TI  - No abstract",
    "PY  - 2018",
    "ER  - "
  ))
  d <- read_ris(f)
  expect_true(is.na(d$abstract))
})

test_that("read_ris missing keywords yields empty character vector in list", {
  f <- ris_tempfile(c(
    "TY  - JOUR",
    "TI  - No keywords",
    "PY  - 2019",
    "ER  - "
  ))
  d <- read_ris(f)
  expect_equal(d$keywords[[1]], character(0))
})

test_that("read_ris missing authors yields empty character vector in list", {
  f <- ris_tempfile(c(
    "TY  - JOUR",
    "TI  - No authors",
    "PY  - 2020",
    "ER  - "
  ))
  d <- read_ris(f)
  expect_equal(d$authors[[1]], character(0))
})

test_that("read_ris missing year yields NA_integer_", {
  f <- ris_tempfile(c(
    "TY  - JOUR",
    "TI  - No year",
    "ER  - "
  ))
  d <- read_ris(f)
  expect_true(is.na(d$year))
  expect_type(d$year, "integer")
})

test_that("read_ris missing title yields NA", {
  f <- ris_tempfile(c(
    "TY  - JOUR",
    "PY  - 2020",
    "ER  - "
  ))
  d <- read_ris(f)
  expect_true(is.na(d$title))
})

## ── Alternative tags ─────────────────────────────────────────────────────────

test_that("read_ris accepts Y1 as year fallback", {
  f <- ris_tempfile(c(
    "TY  - JOUR",
    "TI  - Y1 year",
    "Y1  - 2015/03/01",
    "ER  - "
  ))
  d <- read_ris(f)
  expect_equal(d$year, 2015L)
})

test_that("read_ris accepts T2 as journal fallback", {
  f <- ris_tempfile(c(
    "TY  - JOUR",
    "TI  - T2 journal",
    "T2  - Fallback Journal",
    "PY  - 2016",
    "ER  - "
  ))
  d <- read_ris(f)
  expect_equal(d$journal, "Fallback Journal")
})

test_that("read_ris accepts T1 as title fallback", {
  f <- ris_tempfile(c(
    "TY  - JOUR",
    "T1  - T1 title here",
    "PY  - 2016",
    "ER  - "
  ))
  d <- read_ris(f)
  expect_equal(d$title, "T1 title here")
})

test_that("read_ris accepts N2 as abstract fallback", {
  f <- ris_tempfile(c(
    "TY  - JOUR",
    "TI  - Paper",
    "N2  - Abstract from N2.",
    "PY  - 2018",
    "ER  - "
  ))
  d <- read_ris(f)
  expect_equal(d$abstract, "Abstract from N2.")
})

test_that("read_ris accepts A1 as author fallback when AU absent", {
  f <- ris_tempfile(c(
    "TY  - JOUR",
    "TI  - Paper",
    "A1  - Doe, John",
    "PY  - 2015",
    "ER  - "
  ))
  d <- read_ris(f)
  expect_equal(length(d$authors[[1]]), 1L)
  expect_equal(d$authors[[1]], "DOE, JOHN")
})

## ── ID assignment ────────────────────────────────────────────────────────────

test_that("read_ris uses DO as id when present", {
  f <- ris_tempfile(c(
    "TY  - JOUR",
    "TI  - Has DOI",
    "DO  - 10.99/test",
    "ER  - "
  ))
  d <- read_ris(f)
  expect_equal(d$id, "10.99/test")
})

test_that("read_ris generates RIS-prefixed id when DO absent", {
  f <- ris_tempfile(c(
    "TY  - JOUR",
    "TI  - No DOI",
    "PY  - 2020",
    "ER  - "
  ))
  d <- read_ris(f)
  expect_match(d$id, "^RIS")
})

## ── Empty file / empty records ───────────────────────────────────────────────

test_that("read_ris empty file returns zero-row data frame with correct columns", {
  f <- ris_tempfile(character(0))
  d <- read_ris(f)
  expect_equal(nrow(d), 0L)
  expect_true(all(c("id", "title", "year", "journal", "doi",
                    "cited_by_count", "abstract", "type",
                    "authors", "references", "keywords") %in% names(d)))
})

test_that("read_ris file with only blank lines returns zero-row data frame", {
  f <- ris_tempfile(c("", "  ", ""))
  d <- read_ris(f)
  expect_equal(nrow(d), 0L)
})

## ── Error path ───────────────────────────────────────────────────────────────

test_that("read_ris errors informatively on non-existent file", {
  expect_error(read_ris("/tmp/does-not-exist.ris"), "File not found")
})

## ── Non-ASCII / UTF-8 author names ───────────────────────────────────────────

test_that("read_ris UTF-8 author names round-trip cleanly", {
  f <- ris_tempfile(c(
    "TY  - JOUR",
    "TI  - UTF-8 test",
    "AU  - Échalas, Mélanie",
    "AU  - García, José",
    "PY  - 2023",
    "ER  - "
  ))
  d <- read_ris(f, encoding = "UTF-8")
  expect_equal(length(d$authors[[1]]), 2L)
  expect_true(is.character(d$authors[[1]]))
  ## Both names should survive (non-empty) after uppercasing
  expect_true(all(nchar(d$authors[[1]]) > 0L))
})

## ── read_biblio auto-detection ───────────────────────────────────────────────

test_that("read_biblio auto-detects RIS format and parses correctly", {
  f <- ris_tempfile(c(
    "TY  - JOUR",
    "AU  - Smith, J.",
    "TI  - Auto-detect test",
    "JO  - Some Journal",
    "PY  - 2020",
    "DO  - 10.1000/autodetect",
    "ER  - "
  ))
  d <- read_biblio(f)
  expect_equal(nrow(d), 1L)
  expect_equal(d$title, "Auto-detect test")
  expect_true(is.list(d$authors))
})

## tests/testthat/test-read-biblio.R
## Target: >= 85% line coverage of R/read-biblio.R
## No network calls; all fixtures via tempfile().

## ── helpers ───────────────────────────────────────────────────────────────────

## Minimal Scopus CSV (has EID → auto-detected as "scopus")
make_scopus_file <- function(n = 2L) {
  df <- data.frame(
    Title           = paste0("Paper ", seq_len(n)),
    Authors         = paste0("Author", seq_len(n), " A."),
    Year            = 2020L + seq_len(n) - 1L,
    `Source title`  = paste0("Journal ", seq_len(n)),
    `Cited by`      = seq_len(n),
    EID             = paste0("2-s2.0-00", seq_len(n)),
    DOI             = paste0("10.1/sc", seq_len(n)),
    check.names     = FALSE,
    stringsAsFactors = FALSE
  )
  f <- tempfile(fileext = ".csv")
  write.csv(df, f, row.names = FALSE)
  f
}

## Minimal Dimensions CSV (prepended "About the data: …" header line)
make_dimensions_file <- function() {
  lines <- c(
    '"About the data: Export created 2024-01-01"',
    '"Publication ID","Title","PubYear","Source title","DOI","Times cited","Abstract","Publication Type","Authors","Cited references","Authors Affiliations - Name of Research organization","Authors Affiliations - Country of Research organization","Keywords"',
    '"pub.111","A dimensions paper","2021","Dim Journal","10.1/dim","3","Some abstract.","article","Smith, John","Jones A 2019 Jour","Uni A","USA","network"'
  )
  f <- tempfile(fileext = ".csv")
  writeLines(lines, f)
  f
}

## Minimal Lens CSV (has "Lens ID" header → auto-detected)
make_lens_file <- function() {
  lines <- c(
    '"Lens ID","Title","Publication Year","Source Title","DOI","Citing Works Count","Abstract","Publication Type","Authors","References","Keywords"',
    '"000-001","A lens paper","2022","Lens Journal","10.1/lens","7","Lens abstract.","journal article","Smith J; Jones K","Ref A 2020","networks"'
  )
  f <- tempfile(fileext = ".csv")
  writeLines(lines, f)
  f
}

## Minimal WoS plaintext (starts with "FN" → auto-detected as "wos")
make_wos_file <- function() {
  f <- tempfile(fileext = ".txt")
  writeLines(c(
    "FN Web of Science",
    "VR 1.0",
    "UT WOS:000001",
    "TI  A wos paper",
    "AU  Smith, John",
    "PY  2021",
    "SO  WoS Journal",
    "TC  4",
    "ER",
    "EF"
  ), f)
  f
}

## Minimal BibTeX (starts with @ → auto-detected as "bibtex")
make_bibtex_file <- function() {
  f <- tempfile(fileext = ".bib")
  writeLines(c(
    "@article{key1,",
    "  title  = {A bibtex paper},",
    "  author = {Smith, John},",
    "  year   = {2023},",
    "  journal = {Bib Journal}",
    "}"
  ), f)
  f
}

## Minimal RIS (starts with "TY  -" → auto-detected as "ris")
make_ris_file <- function() {
  f <- tempfile(fileext = ".ris")
  writeLines(c(
    "TY  - JOUR",
    "TI  - A ris paper",
    "AU  - Smith, John",
    "PY  - 2022",
    "JO  - RIS Journal",
    "DO  - 10.1/ris1",
    "ER  - "
  ), f)
  f
}

## Generic CSV helper
make_generic_file <- function() {
  f <- tempfile(fileext = ".csv")
  df <- data.frame(
    doc_id   = c("D1", "D2", "D3"),
    headline = c("Paper One", "Paper Two", "Paper Three"),
    Authors  = c("Alice|Bob", "Carol", "Dave|Eve|Frank"),
    Tags     = c("network|graph", "statistics", "learning|AI"),
    year     = c(2020L, 2021L, 2022L),
    stringsAsFactors = FALSE
  )
  write.csv(df, f, row.names = FALSE)
  f
}


## ════════════════════════════════════════════════════════════════════════════
## 1.  detect_format() — uncovered branches
## ════════════════════════════════════════════════════════════════════════════

test_that("detect_format returns 'unknown' for an empty file", {
  f <- tempfile(fileext = ".csv")
  writeLines(character(0), f)
  ## bibnets internal function; access via :::
  result <- bibnets:::detect_format(f)
  expect_equal(result, "unknown")
})

test_that("detect_format returns 'wos' for a file starting with FN", {
  f <- tempfile(fileext = ".txt")
  writeLines(c("FN Web of Science", "VR 1.0"), f)
  expect_equal(bibnets:::detect_format(f), "wos")
})

test_that("detect_format returns 'wos' for a file starting with PT", {
  f <- tempfile(fileext = ".txt")
  writeLines(c("PT J", "AU Smith, J"), f)
  expect_equal(bibnets:::detect_format(f), "wos")
})

test_that("detect_format returns 'scopus' for CSV with EID header", {
  f <- make_scopus_file(1L)
  expect_equal(bibnets:::detect_format(f), "scopus")
})

test_that("detect_format returns 'lens' for CSV with Lens ID header", {
  f <- make_lens_file()
  expect_equal(bibnets:::detect_format(f), "lens")
})

test_that("detect_format returns 'unknown' for an unrecognised CSV", {
  f <- tempfile(fileext = ".csv")
  writeLines(c("col_a,col_b,col_c", "1,2,3"), f)
  expect_equal(bibnets:::detect_format(f), "unknown")
})


## ════════════════════════════════════════════════════════════════════════════
## 2.  resolve_paths() — directory and nonexistent path branches
## ════════════════════════════════════════════════════════════════════════════

test_that("resolve_paths returns files from a directory", {
  d <- tempdir()
  f1 <- file.path(d, paste0("test_rb_", Sys.getpid(), "_a.csv"))
  f2 <- file.path(d, paste0("test_rb_", Sys.getpid(), "_b.bib"))
  writeLines("col\n1", f1)
  writeLines("@misc{k}", f2)
  on.exit({ unlink(f1); unlink(f2) }, add = TRUE)

  result <- bibnets:::resolve_paths(d)
  expect_true(f1 %in% result)
  expect_true(f2 %in% result)
})

test_that("resolve_paths returns character(0) for nonexistent path", {
  result <- bibnets:::resolve_paths("/tmp/no_such_path_xyz_bibnets")
  expect_equal(length(result), 0L)
  expect_type(result, "character")
})

test_that("resolve_paths handles mix of existing file and nonexistent path", {
  f <- make_ris_file()
  result <- bibnets:::resolve_paths(c(f, "/tmp/definitely_missing_xyz"))
  expect_equal(result, f)
})


## ════════════════════════════════════════════════════════════════════════════
## 3.  read_biblio() — empty path stops with informative error
## ════════════════════════════════════════════════════════════════════════════

test_that("read_biblio stops with informative error when no files found", {
  expect_error(
    read_biblio("/tmp/absolutely_no_file_bibnets_xyz.csv"),
    regexp = "No files found"
  )
})

test_that("read_biblio stops when given a nonexistent directory", {
  expect_error(
    read_biblio("/tmp/nonexistent_dir_bibnets_xyz/"),
    regexp = "No files found"
  )
})


## ════════════════════════════════════════════════════════════════════════════
## 4.  read_single_biblio() — format = "generic" short-circuit path
## ════════════════════════════════════════════════════════════════════════════

test_that("read_biblio with format='generic' invokes read_generic correctly", {
  f <- make_generic_file()
  d <- read_biblio(f, format = "generic", id = "doc_id",
                   actors = c("Authors", "Tags"), sep = "|")
  expect_equal(nrow(d), 3L)
  expect_true(is.list(d$Authors))
  expect_true(is.list(d$Tags))
  ## First row: two Authors split on "|"
  expect_equal(length(d$Authors[[1]]), 2L)
  ## Third row: three Authors
  expect_equal(length(d$Authors[[3]]), 3L)
  ## ID column set from doc_id
  expect_equal(d$id, c("D1", "D2", "D3"))
})

test_that("read_biblio generic: actors not in file are silently skipped", {
  f <- make_generic_file()
  d <- read_biblio(f, format = "generic", actors = c("Authors", "NonExistent"))
  expect_true(is.list(d$Authors))
  ## "NonExistent" is absent — no error, not added as list col
  expect_false("NonExistent" %in% names(d))
})

test_that("read_biblio generic: NULL id uses row numbers as character id", {
  f <- make_generic_file()
  d <- read_biblio(f, format = "generic", id = NULL)
  expect_equal(d$id, as.character(1:3))
})

test_that("read_biblio generic: CSV already has an 'id' column → used as-is", {
  f <- tempfile(fileext = ".csv")
  writeLines(c("id,title", "A001,Title One", "A002,Title Two"), f)
  d <- read_biblio(f, format = "generic")
  ## id stays from the CSV column (character)
  expect_equal(d$id, c("A001", "A002"))
})


## ════════════════════════════════════════════════════════════════════════════
## 5.  read_single_biblio() — unknown format error path
## ════════════════════════════════════════════════════════════════════════════

test_that("read_biblio with undetectable format stops with actionable error", {
  f <- tempfile(fileext = ".csv")
  writeLines(c("col_x,col_y", "1,2"), f)
  ## auto-detect returns "unknown" → switch falls through to stop()
  expect_error(
    read_biblio(f),
    regexp = "Could not detect file format"
  )
})

test_that("read_single_biblio unknown format message names the file", {
  f <- tempfile(fileext = ".csv")
  writeLines(c("col_x,col_y", "1,2"), f)
  err <- tryCatch(read_biblio(f), error = function(e) conditionMessage(e))
  expect_true(grepl(basename(f), err, fixed = FALSE) ||
              grepl("Could not detect", err, fixed = FALSE))
})


## ════════════════════════════════════════════════════════════════════════════
## 6.  read_biblio() auto-detect dispatch — formats not yet covered
## ════════════════════════════════════════════════════════════════════════════

test_that("read_biblio auto-detects scopus format", {
  f <- make_scopus_file(2L)
  d <- suppressMessages(read_biblio(f))
  expect_equal(nrow(d), 2L)
  ## EID column used as id
  expect_true(all(grepl("^2-s2\\.0-", d$id)))
})

test_that("read_biblio auto-detects wos format (FN line)", {
  f <- make_wos_file()
  d <- suppressMessages(read_biblio(f))
  expect_equal(nrow(d), 1L)
  expect_equal(d$id, "WOS:000001")
})

test_that("read_biblio auto-detects lens format", {
  f <- make_lens_file()
  d <- suppressMessages(read_biblio(f))
  expect_equal(nrow(d), 1L)
  expect_equal(d$id, "000-001")
})

test_that("read_biblio auto-detects dimensions format", {
  f <- make_dimensions_file()
  d <- suppressMessages(read_biblio(f))
  expect_equal(nrow(d), 1L)
  expect_true(grepl("pub\\.111", d$id))
})

test_that("read_biblio explicit format='wos' bypasses auto-detection", {
  f <- make_wos_file()
  d <- suppressMessages(read_biblio(f, format = "wos"))
  expect_equal(nrow(d), 1L)
})

test_that("read_biblio explicit format='wos_tab' dispatches to tab parser", {
  ## Build a minimal WoS tab-delimited file
  f <- tempfile(fileext = ".txt")
  writeLines(c(
    paste(c("UT", "TI", "AU", "PY", "SO", "DI", "TC",
            "AB", "DT", "DE", "ID", "CR"), collapse = "\t"),
    paste(c("WOS:T001", "Tab paper", "Smith, J", "2021", "Tab Journal",
            "10.1/t", "7", "An abstract.", "Article",
            "networks", "deep learning", ""), collapse = "\t")
  ), f)
  d <- suppressMessages(read_biblio(f, format = "wos_tab"))
  expect_equal(nrow(d), 1L)
  expect_equal(d$id, "WOS:T001")
})

test_that("read_biblio explicit format='bibtex' dispatches correctly", {
  f <- make_bibtex_file()
  d <- suppressMessages(read_biblio(f, format = "bibtex"))
  expect_equal(nrow(d), 1L)
  expect_equal(d$title, "A bibtex paper")
})

test_that("read_biblio explicit format='ris' dispatches correctly", {
  f <- make_ris_file()
  d <- suppressMessages(read_biblio(f, format = "ris"))
  expect_equal(nrow(d), 1L)
  expect_equal(d$title, "A ris paper")
})

test_that("read_biblio explicit format='dimensions' dispatches correctly", {
  f <- make_dimensions_file()
  d <- suppressMessages(read_biblio(f, format = "dimensions"))
  expect_equal(nrow(d), 1L)
})

test_that("read_biblio explicit format='lens' dispatches correctly", {
  f <- make_lens_file()
  d <- suppressMessages(read_biblio(f, format = "lens"))
  expect_equal(nrow(d), 1L)
})


## ════════════════════════════════════════════════════════════════════════════
## 7.  Multi-file ingest — message and row-count assertions
## ════════════════════════════════════════════════════════════════════════════

test_that("read_biblio emits 'Read N files: M rows total' message for multiple files", {
  f1 <- make_ris_file()
  f2 <- make_ris_file()
  expect_message(
    read_biblio(c(f1, f2)),
    regexp = "Read 2 files: 2 rows total"
  )
})

test_that("read_biblio combines two RIS files into correct row count", {
  f1 <- make_ris_file()
  f2 <- make_ris_file()
  d <- suppressMessages(read_biblio(c(f1, f2)))
  expect_equal(nrow(d), 2L)
})

test_that("read_biblio directory input reads all matching files", {
  d_dir <- tempfile()
  dir.create(d_dir)
  on.exit(unlink(d_dir, recursive = TRUE), add = TRUE)

  ## Write three RIS files into the temp directory
  writeLines(c("TY  - JOUR", "TI  - Paper A", "PY  - 2020", "ER  - "),
             file.path(d_dir, "a.ris"))
  writeLines(c("TY  - JOUR", "TI  - Paper B", "PY  - 2021", "ER  - "),
             file.path(d_dir, "b.ris"))
  writeLines(c("TY  - JOUR", "TI  - Paper C", "PY  - 2022", "ER  - "),
             file.path(d_dir, "c.ris"))

  d <- suppressMessages(read_biblio(d_dir))
  expect_equal(nrow(d), 3L)
})

test_that("read_biblio directory: message fires when > 1 file found", {
  d_dir <- tempfile()
  dir.create(d_dir)
  on.exit(unlink(d_dir, recursive = TRUE), add = TRUE)

  writeLines(c("TY  - JOUR", "TI  - P1", "PY  - 2020", "ER  - "),
             file.path(d_dir, "r1.ris"))
  writeLines(c("TY  - JOUR", "TI  - P2", "PY  - 2021", "ER  - "),
             file.path(d_dir, "r2.ris"))

  expect_message(read_biblio(d_dir), "Read 2 files")
})

test_that("read_biblio vector of 3 files: combined row count and message", {
  f1 <- make_ris_file()   ## 1 row
  f2 <- make_ris_file()   ## 1 row
  f3 <- make_bibtex_file() ## 1 row
  expect_message(
    {d <- read_biblio(c(f1, f2, f3))},
    regexp = "Read 3 files: 3 rows total"
  )
  expect_equal(nrow(d), 3L)
})

test_that("read_biblio single file does NOT emit the multi-file message", {
  f <- make_ris_file()
  expect_no_message(read_biblio(f))
})


## ════════════════════════════════════════════════════════════════════════════
## 8.  align_biblio_columns() — missing list-column filled with empty vectors
## ════════════════════════════════════════════════════════════════════════════

test_that("align_biblio_columns fills missing list-columns with empty vectors", {
  ## RIS has no 'affiliations'; OA CSV does.  Combining them exercises the
  ## list-col fill branch of align_biblio_columns().
  oa_f <- tempfile(fileext = ".csv")
  writeLines(c(
    "id,display_name,publication_year,primary_location.source.display_name,doi,cited_by_count,type,authorships.author.display_name,authorships.institutions.display_name,authorships.countries,primary_topic.display_name",
    "https://openalex.org/W9,OA paper,2024,Journal OA,https://doi.org/10.1/oa,2,article,Alice|Bob,Uni A|Uni B,US|GB,Networks"
  ), oa_f)

  ris_f <- make_ris_file()

  d <- suppressMessages(read_biblio(c(oa_f, ris_f)))

  expect_equal(nrow(d), 2L)
  ## 'countries' and 'affiliations' exist only in the OA row; the RIS row
  ## should have them as empty list elements (not NA, not missing column)
  expect_true("countries" %in% names(d))
  expect_true(is.list(d$countries))
  ris_row <- d[is.na(d$abstract) | d$id != "W9", ]
  ## The RIS row's countries entry should be an empty character vector
  expect_equal(length(d$countries[[2]]), 0L)
})

test_that("align_biblio_columns fills missing scalar columns with NA", {
  ## Two Scopus files combined: both have same schema, no missing columns
  ## Use BibTeX + Scopus to get a scalar-col mismatch (e.g., 'language')
  bib_f <- make_bibtex_file()
  sc_f  <- make_scopus_file(1L)

  d <- suppressMessages(read_biblio(c(sc_f, bib_f)))
  ## Scopus adds 'language'; BibTeX doesn't — should be NA for BibTeX row
  if ("language" %in% names(d)) {
    expect_true(any(is.na(d$language)))
  }
  expect_equal(nrow(d), 2L)
})


## ════════════════════════════════════════════════════════════════════════════
## 9.  read_generic() — edge-cases
## ════════════════════════════════════════════════════════════════════════════

test_that("read_generic errors on a nonexistent file", {
  ## read_biblio checks resolve_paths first; it never reaches read_generic for
  ## a missing file path.  So we call the internal directly, or via a
  ## zero-length resolve that triggers "No files found" first.
  ## The "No files found" error is the correct user-facing error here.
  expect_error(
    read_biblio("/no/such/generic/file_bibnets_xyz.csv", format = "generic"),
    regexp = "No files found|File not found|not found|cannot open"
  )
})

test_that("read_generic sep parameter splits on custom delimiter", {
  f <- tempfile(fileext = ".csv")
  df <- data.frame(
    id      = c("X1", "X2"),
    authors = c("A::B::C", "D::E"),
    stringsAsFactors = FALSE
  )
  write.csv(df, f, row.names = FALSE)
  d <- read_biblio(f, format = "generic", actors = "authors", sep = "::")
  expect_equal(length(d$authors[[1]]), 3L)
  expect_equal(length(d$authors[[2]]), 2L)
})

test_that("read_generic with no actors argument leaves columns as-is", {
  f <- make_generic_file()
  d <- read_biblio(f, format = "generic")
  ## No actors specified: Authors column should remain character, not list
  expect_false(is.list(d$Authors))
})

# The bundled openalex_works.csv is a small 30-row fixture (a subset of
# the full OpenAlex "Works" export). It is read-only here, so parse it
# ONCE at file scope and share `oa` across the read_openalex_csv()
# assertions instead of re-parsing per test_that(). The read_biblio
# blocks deliberately keep their own calls — they exercise a different
# code path (format auto-detection) and must not reuse `oa`.
f  <- system.file("extdata", "openalex_works.csv", package = "bibnets")
oa <- read_openalex_csv(f)

test_that("read_openalex_csv returns standard columns", {
  expect_true(all(c("id", "title", "year", "journal", "doi",
                    "cited_by_count", "abstract", "type",
                    "authors", "references", "keywords",
                    "affiliations", "countries") %in% names(oa)))
})

test_that("read_openalex_csv returns 30 rows from bundled fixture", {
  expect_equal(nrow(oa), 30L)
})

test_that("read_openalex_csv strips OpenAlex URL prefix from id", {
  expect_false(any(grepl("https://openalex.org/", oa$id)))
  expect_true(all(grepl("^W[0-9]+$", oa$id)))
})

test_that("read_openalex_csv strips DOI URL prefix", {
  doi_present <- oa$doi[!is.na(oa$doi)]
  expect_false(any(grepl("^https://doi.org/", doi_present)))
})

test_that("read_openalex_csv produces list-columns for authors, references, keywords", {
  expect_true(is.list(oa$authors))
  expect_true(is.list(oa$references))
  expect_true(is.list(oa$keywords))
  expect_true(is.list(oa$affiliations))
  expect_true(is.list(oa$countries))
})

test_that("read_openalex_csv pipe-splits authors correctly", {
  multi_author <- Filter(function(x) length(x) > 1, oa$authors)
  expect_true(length(multi_author) > 0)
})

test_that("read_openalex_csv references column is always empty", {
  expect_true(all(vapply(oa$references, length, integer(1)) == 0L))
})

test_that("read_openalex_csv abstract column is all NA", {
  expect_true(all(is.na(oa$abstract)))
})

test_that("read_openalex_csv year is integer", {
  expect_type(oa$year, "integer")
})

test_that("read_openalex_csv cited_by_count is integer with no NAs", {
  expect_type(oa$cited_by_count, "integer")
  expect_false(any(is.na(oa$cited_by_count)))
})

test_that("read_biblio auto-detects openalex_csv format", {
  f <- system.file("extdata", "openalex_works.csv", package = "bibnets")
  d <- read_biblio(f)
  expect_equal(nrow(d), 30L)
  expect_true(is.list(d$authors))
})

test_that("read_biblio with format='openalex_csv' works explicitly", {
  f <- system.file("extdata", "openalex_works.csv", package = "bibnets")
  d <- read_biblio(f, format = "openalex_csv")
  expect_equal(nrow(d), 30L)
})

test_that("read_openalex_csv countries are pipe-split into character vectors", {
  multi_country <- Filter(function(x) length(x) > 1, oa$countries)
  expect_true(length(multi_country) > 0)
  all_codes <- unlist(oa$countries)
  expect_true(all(nchar(all_codes) == 2L))
})

test_that("read_openalex_csv keywords are single-element lists from primary_topic", {
  kw_lengths <- vapply(oa$keywords, length, integer(1))
  expect_true(all(kw_lengths %in% c(0L, 1L)))
  expect_true(any(kw_lengths == 1L))
})

test_that("read_openalex_csv errors on non-existent file", {
  expect_error(read_openalex_csv("no_such_file.csv"))
})

test_that("read_biblio row-binds files with source-specific columns", {
  oa <- tempfile(fileext = ".csv")
  bib <- tempfile(fileext = ".bib")

  writeLines(c(
    "id,display_name,publication_year,primary_location.source.display_name,doi,cited_by_count,type,authorships.author.display_name,authorships.institutions.display_name,authorships.countries,primary_topic.display_name",
    "https://openalex.org/W1,OpenAlex paper,2024,Journal A,https://doi.org/10.1/oa,2,article,Alice|Bob,Uni A|Uni B,US|GB,Networks"
  ), oa)
  writeLines(c(
    "@article{key1,",
    "  title = {BibTeX paper},",
    "  author = {Smith, Jane},",
    "  year = {2023},",
    "  journal = {Journal B}",
    "}"
  ), bib)

  d <- read_biblio(c(oa, bib))

  expect_equal(nrow(d), 2L)
  expect_true(all(c("countries", "affiliations", "authors") %in% names(d)))
  expect_true(is.list(d$countries))
  expect_true(is.list(d$authors))
})

test_that("read_bibtex extracts non-standard cited references", {
  bib <- tempfile(fileext = ".bib")
  writeLines(c(
    "@article{key1,",
    "  title = {BibTeX paper},",
    "  author = {Smith, Jane},",
    "  year = {2023},",
    "  cited-references = {Ref A, 2020; Ref B, 2021}",
    "}"
  ), bib)

  d <- read_bibtex(bib)

  expect_equal(d$references[[1]], c("REF A, 2020", "REF B, 2021"))
})

## ── helpers ──────────────────────────────────────────────────────────────────

make_dim_csv <- function(rows, extra_cols = character(0), file = tempfile(fileext = ".csv")) {
  ## Dimensions CSVs always begin with a one-line metadata header then the
  ## real column-name row.  The parser (read_dimensions.R line 26) detects
  ## lines starting with ^"?About the data and skips 1 row.
  header <- paste0(
    '"About the data: Export created ',
    format(Sys.Date(), "%Y-%m-%d"),
    '"'
  )
  col_names <- c(
    "Publication ID", "Title", "PubYear", "Source title", "DOI",
    "Times cited", "Abstract", "Publication Type",
    "Authors", "Cited references",
    "Authors Affiliations - Name of Research organization",
    "Authors Affiliations - Country of Research organization",
    "Keywords",
    extra_cols
  )

  write_row <- function(r) {
    paste(vapply(r, function(v) {
      if (is.na(v)) "" else paste0('"', gsub('"', '""', v), '"')
    }, character(1)), collapse = ",")
  }

  lines <- c(
    header,
    paste(vapply(col_names, function(n) paste0('"', n, '"'), character(1)),
          collapse = ","),
    vapply(rows, write_row, character(1))
  )
  writeLines(lines, file)
  file
}

## Three representative rows used in most tests
typical_rows <- list(
  ## 1: full record, two authors, two references
  c(
    "pub.1111111111", "Learning Networks", "2021",
    "Journal of Education", "10.1000/xyz001", "15",
    "This paper studies learning networks.",
    "article",
    "Smith, John; Jones, Mary",
    "Brown A, Title X, 2018, Journal Z; Green B, Title Y, 2019, Journal W",
    "University A; University B",
    "USA; UK",
    "network; learning"
  ),
  ## 2: multiple affiliations & countries; non-ASCII author name
  c(
    "pub.2222222222", "Réseaux éducatifs", "2022",
    "Revue Française", "10.1000/xyz002", "3",
    "Study of éducation réseaux.",
    "article",
    "Müller, Hans; Lefèvre, Claire; Tanaka, Yuki",
    "White C, Study Z, 2020, Jour Q",
    "Universität Berlin; Université Paris; Kyoto University",
    "Germany; France; Japan",
    "éducation; réseaux; multilingual"
  ),
  ## 3: missing references (empty string)
  c(
    "pub.3333333333", "Citation-free Article", "2023",
    "Open Access Journal", "10.1000/xyz003", "0",
    "An article with no references.",
    "conference paper",
    "Doe, Jane",
    "",
    "State College",
    "USA",
    "open access"
  )
)

## ── Standard columns ──────────────────────────────────────────────────────────

test_that("read_dimensions returns all required standard columns", {
  f <- make_dim_csv(typical_rows)
  d <- read_dimensions(f)

  expected <- c("id", "title", "year", "journal", "doi",
                "cited_by_count", "abstract", "type",
                "authors", "references", "keywords",
                "affiliations", "countries")
  expect_true(all(expected %in% names(d)))
})

test_that("read_dimensions returns columns in correct order", {
  f <- make_dim_csv(typical_rows)
  d <- read_dimensions(f)
  std_order <- c("id", "title", "year", "journal", "doi",
                 "cited_by_count", "abstract", "type",
                 "authors", "references", "keywords",
                 "affiliations", "countries")
  expect_equal(names(d)[seq_along(std_order)], std_order)
})

test_that("read_dimensions returns 3 rows from 3-record fixture", {
  f <- make_dim_csv(typical_rows)
  d <- read_dimensions(f)
  expect_equal(nrow(d), 3L)
})

## ── Scalar column types ───────────────────────────────────────────────────────

test_that("year is integer", {
  f <- make_dim_csv(typical_rows)
  d <- read_dimensions(f)
  expect_type(d$year, "integer")
  expect_equal(d$year, c(2021L, 2022L, 2023L))
})

test_that("cited_by_count is integer with no NAs", {
  f <- make_dim_csv(typical_rows)
  d <- read_dimensions(f)
  expect_type(d$cited_by_count, "integer")
  expect_false(any(is.na(d$cited_by_count)))
  expect_equal(d$cited_by_count, c(15L, 3L, 0L))
})

test_that("id column is populated from Publication ID", {
  f <- make_dim_csv(typical_rows)
  d <- read_dimensions(f)
  expect_equal(d$id, c("pub.1111111111", "pub.2222222222", "pub.3333333333"))
})

test_that("title, journal, doi, abstract, type are character", {
  f <- make_dim_csv(typical_rows)
  d <- read_dimensions(f)
  for (col in c("title", "journal", "doi", "abstract", "type")) {
    expect_type(d[[col]], "character")
  }
})

## ── List-columns ──────────────────────────────────────────────────────────────

test_that("authors, references, keywords, affiliations, countries are list-columns", {
  f <- make_dim_csv(typical_rows)
  d <- read_dimensions(f)
  for (col in c("authors", "references", "keywords", "affiliations", "countries")) {
    expect_true(is.list(d[[col]]))
  }
})

test_that("semicolons split authors into multiple elements", {
  f <- make_dim_csv(typical_rows)
  d <- read_dimensions(f)
  expect_equal(length(d$authors[[1]]), 2L)   ## "Smith, John; Jones, Mary"
  expect_equal(length(d$authors[[2]]), 3L)   ## three authors
  expect_equal(length(d$authors[[3]]), 1L)   ## "Doe, Jane"
})

test_that("authors are uppercased", {
  f <- make_dim_csv(typical_rows)
  d <- read_dimensions(f)
  all_authors <- unlist(d$authors)
  expect_true(all(all_authors == toupper(all_authors)))
})

test_that("semicolons split references correctly", {
  f <- make_dim_csv(typical_rows)
  d <- read_dimensions(f)
  expect_equal(length(d$references[[1]]), 2L)
  expect_equal(length(d$references[[2]]), 1L)
})

test_that("references are uppercased", {
  f <- make_dim_csv(typical_rows)
  d <- read_dimensions(f)
  present <- unlist(d$references)
  expect_true(all(present == toupper(present)))
})

test_that("empty references yield character(0)", {
  f <- make_dim_csv(typical_rows)
  d <- read_dimensions(f)
  expect_equal(d$references[[3]], character(0))
})

test_that("keywords are split by semicolon", {
  f <- make_dim_csv(typical_rows)
  d <- read_dimensions(f)
  expect_equal(length(d$keywords[[1]]), 2L)   ## "network; learning"
  expect_equal(length(d$keywords[[2]]), 3L)
  expect_equal(length(d$keywords[[3]]), 1L)
})

test_that("affiliations are split by semicolon", {
  f <- make_dim_csv(typical_rows)
  d <- read_dimensions(f)
  expect_equal(length(d$affiliations[[1]]), 2L)
  expect_equal(length(d$affiliations[[2]]), 3L)
  expect_equal(length(d$affiliations[[3]]), 1L)
})

test_that("countries are split by semicolon", {
  f <- make_dim_csv(typical_rows)
  d <- read_dimensions(f)
  expect_equal(length(d$countries[[1]]), 2L)  ## "USA; UK"
  expect_equal(length(d$countries[[2]]), 3L)  ## three countries
  expect_equal(length(d$countries[[3]]), 1L)
})

## ── Dimensions metadata header quirk ─────────────────────────────────────────

test_that("file with About-the-data header is parsed correctly", {
  ## Parser should detect and skip the metadata line; only data rows returned.
  f <- make_dim_csv(typical_rows)
  first <- readLines(f, n = 1L)
  expect_true(grepl('^"About the data', first))
  d <- read_dimensions(f)
  expect_equal(nrow(d), 3L)
  ## First row title must be the first actual record, not junk from the header
  expect_equal(d$title[1], "Learning Networks")
})

test_that("file without About-the-data header is also parsed (skip_rows=0)", {
  ## Build a plain CSV with no metadata line — parser falls back to skip=0.
  rows <- typical_rows[1:2]
  col_names <- c(
    "Publication ID", "Title", "PubYear", "Source title", "DOI",
    "Times cited", "Abstract", "Publication Type",
    "Authors", "Cited references",
    "Authors Affiliations - Name of Research organization",
    "Authors Affiliations - Country of Research organization",
    "Keywords"
  )
  f <- tempfile(fileext = ".csv")
  write_row <- function(r) {
    paste(vapply(r, function(v) {
      if (is.na(v)) "" else paste0('"', gsub('"', '""', v), '"')
    }, character(1)), collapse = ",")
  }
  lines <- c(
    paste(vapply(col_names, function(n) paste0('"', n, '"'), character(1)),
          collapse = ","),
    vapply(rows, write_row, character(1))
  )
  writeLines(lines, f)
  d <- read_dimensions(f)
  expect_equal(nrow(d), 2L)
})

## ── Column name aliases ───────────────────────────────────────────────────────

test_that("Publication Year alias is accepted for year column", {
  f <- tempfile(fileext = ".csv")
  writeLines(c(
    '"About the data: test"',
    '"Publication ID","Title","Publication Year","Source title","DOI","Times cited","Abstract","Publication Type","Authors","Cited references","Authors Affiliations - Name of Research organization","Authors Affiliations - Country of Research organization","Keywords"',
    '"pub.99","Alt Year Test","2020","Some Journal","10.0/x","5","Abstract here.","article","Author, A","","Org A","CountryX","kw1"'
  ), f)
  d <- read_dimensions(f)
  expect_equal(d$year, 2020L)
})

test_that("Dimensions URL alias is accepted for id column", {
  f <- tempfile(fileext = ".csv")
  writeLines(c(
    '"About the data: test"',
    '"Dimensions URL","Title","PubYear","Source title","DOI","Times cited","Abstract","Publication Type","Authors","Cited references","Authors Affiliations - Name of Research organization","Authors Affiliations - Country of Research organization","Keywords"',
    '"https://app.dimensions.ai/pub.555","URL ID Test","2021","Journal","10.0/u","2","Abs.","article","Auth, B","","Org B","CountryY","kw2"'
  ), f)
  d <- read_dimensions(f)
  expect_equal(d$id[1], "https://app.dimensions.ai/pub.555")
})

## ── Fallback ID generation ────────────────────────────────────────────────────

test_that("rows with empty Publication ID get DIMn fallback id", {
  f <- tempfile(fileext = ".csv")
  writeLines(c(
    '"About the data: test"',
    '"Publication ID","Title","PubYear","Source title","DOI","Times cited","Abstract","Publication Type","Authors","Cited references","Authors Affiliations - Name of Research organization","Authors Affiliations - Country of Research organization","Keywords"',
    '"","No ID Paper","2020","Journal","10.0/n","1","Abs.","article","Auth, C","","Org C","CountryZ","kw3"'
  ), f)
  d <- read_dimensions(f)
  expect_equal(d$id[1], "DIM1")
})

## ── Missing optional columns ──────────────────────────────────────────────────

test_that("missing Keywords column produces empty list-column", {
  f <- tempfile(fileext = ".csv")
  writeLines(c(
    '"About the data: test"',
    '"Publication ID","Title","PubYear","Source title","DOI","Times cited","Abstract","Publication Type","Authors","Cited references","Authors Affiliations - Name of Research organization","Authors Affiliations - Country of Research organization"',
    '"pub.1","No KW","2021","J","10.0/k","0","A.","article","Auth, D","","Org D","CountryA"'
  ), f)
  d <- read_dimensions(f)
  expect_true(is.list(d$keywords))
  expect_equal(d$keywords[[1]], character(0))
})

test_that("missing affiliations/countries columns produce empty list-columns", {
  f <- tempfile(fileext = ".csv")
  writeLines(c(
    '"About the data: test"',
    '"Publication ID","Title","PubYear","Source title","DOI","Times cited","Abstract","Publication Type","Authors","Cited references","Keywords"',
    '"pub.2","No Aff","2022","J","10.0/a","2","Abs.","article","Auth, E","ref1","kw"'
  ), f)
  d <- read_dimensions(f)
  expect_true(is.list(d$affiliations))
  expect_true(is.list(d$countries))
  expect_equal(d$affiliations[[1]], character(0))
  expect_equal(d$countries[[1]], character(0))
})

## ── Non-ASCII / special characters ───────────────────────────────────────────

test_that("non-ASCII characters in author names are preserved", {
  f <- make_dim_csv(typical_rows)
  d <- read_dimensions(f)
  ## Row 2 has Müller, Hans; Lefèvre, Claire; Tanaka, Yuki
  ## After uppercasing these become MÜLLER, HANS etc.
  author_str <- paste(d$authors[[2]], collapse = " ")
  expect_true(nchar(author_str) > 0)
  expect_equal(length(d$authors[[2]]), 3L)
})

## ── Error on bad file ─────────────────────────────────────────────────────────

test_that("read_dimensions errors on non-existent file", {
  expect_error(read_dimensions("no_such_file_xyz.csv"))
})

## ── read_biblio integration ───────────────────────────────────────────────────

test_that("read_biblio auto-detects Dimensions format", {
  f <- make_dim_csv(typical_rows)
  d <- read_biblio(f)
  expect_equal(nrow(d), 3L)
  expect_true(is.list(d$authors))
  expect_true(is.list(d$affiliations))
})

#' Convert a sparse square matrix to a tidy edge list
#'
#' Extracts non-zero entries directly from the sparse representation —
#' no dense matrix allocation. For undirected networks, only the upper
#' triangle is returned.
#'
#' @param A A square sparse or dense matrix.
#' @param directed Logical. If `FALSE` (default), returns only upper-triangle
#'   entries. If `TRUE`, returns all non-zero off-diagonal entries.
#'
#' @return A data frame with columns `from`, `to`, `weight`, sorted by
#'   descending weight.
#'
#' @keywords internal
mat_to_edgelist <- function(A, directed = FALSE) {
  stopifnot(nrow(A) == ncol(A))

  node_names <- rownames(A)
  if (is.null(node_names)) {
    node_names <- as.character(seq_len(nrow(A)))
  }

  ## Extract triplets directly from sparse matrix
  if (inherits(A, "Matrix")) {
    triplets <- Matrix::summary(A)
    i <- triplets$i
    j <- triplets$j
    x <- triplets$x
  } else {
    idx <- which(A != 0, arr.ind = TRUE)
    if (nrow(idx) == 0L) {
      return(data.frame(from = character(0), to = character(0),
                        weight = numeric(0), stringsAsFactors = FALSE))
    }
    i <- idx[, 1]
    j <- idx[, 2]
    x <- A[idx]
  }

  ## Remove diagonal
  keep <- i != j
  i <- i[keep]; j <- j[keep]; x <- x[keep]

  ## Upper triangle only for undirected
  if (!directed) {
    keep <- i < j
    i <- i[keep]; j <- j[keep]; x <- x[keep]
  }

  if (length(i) == 0L) {
    return(data.frame(from = character(0), to = character(0),
                      weight = numeric(0), stringsAsFactors = FALSE))
  }

  edges <- data.frame(
    from = node_names[i],
    to = node_names[j],
    weight = x,
    stringsAsFactors = FALSE
  )

  edges[order(-edges$weight), ]
}


#' Convert edge list to a square sparse matrix
#'
#' @param edges A data frame with columns `from`, `to`, `weight`.
#' @param nodes Optional character vector of node names. If `NULL`,
#'   derived from the edge list.
#' @param symmetric Logical. If `TRUE` (default), the matrix is made
#'   symmetric.
#'
#' @return A sparse `dgCMatrix`.
#' @keywords internal
edgelist_to_mat <- function(edges, nodes = NULL, symmetric = TRUE) {
  if (is.null(nodes)) {
    nodes <- sort(unique(c(edges$from, edges$to)))
  }

  i <- match(edges$from, nodes)
  j <- match(edges$to, nodes)
  n <- length(nodes)

  A <- Matrix::sparseMatrix(
    i = i, j = j, x = edges$weight,
    dims = c(n, n),
    dimnames = list(nodes, nodes)
  )

  if (symmetric) {
    A <- A + Matrix::t(A)
    Matrix::diag(A) <- Matrix::diag(A) / 2
  }

  A
}

## Tests for internal helpers in R/edgelist.R
## Functions: mat_to_edgelist(), edgelist_to_mat()
##
## Use bibnets:::name so file_coverage() can instrument the source file
## directly without depending on exported wrappers.

library(Matrix)

# ── helpers ────────────────────────────────────────────────────────────────────

make_sym_mat <- function() {
  ## 3 x 3 named symmetric sparse matrix
  A <- Matrix::sparseMatrix(
    i = c(1, 1, 2, 2, 3, 3),
    j = c(2, 3, 1, 3, 1, 2),
    x = c(5, 3, 5, 2, 3, 2),
    dims = c(3L, 3L),
    dimnames = list(c("A", "B", "C"), c("A", "B", "C"))
  )
  A
}

make_dir_mat <- function() {
  ## Asymmetric: A->B=5, B->C=2 (upper ≠ lower)
  A <- Matrix::sparseMatrix(
    i = c(1, 2),
    j = c(2, 3),
    x = c(5, 2),
    dims = c(3L, 3L),
    dimnames = list(c("A", "B", "C"), c("A", "B", "C"))
  )
  A
}

make_edges <- function() {
  data.frame(
    from   = c("A", "A", "B"),
    to     = c("B", "C", "C"),
    weight = c(5,   3,   2),
    stringsAsFactors = FALSE
  )
}

# ── mat_to_edgelist ────────────────────────────────────────────────────────────

test_that("mat_to_edgelist returns data.frame with from/to/weight columns", {
  A   <- make_sym_mat()
  out <- bibnets:::mat_to_edgelist(A)
  expect_true(is.data.frame(out))
  expect_true(all(c("from", "to", "weight") %in% names(out)))
})

test_that("mat_to_edgelist undirected returns only upper triangle", {
  A   <- make_sym_mat()
  out <- bibnets:::mat_to_edgelist(A, directed = FALSE)
  ## Upper triangle for 3 nodes has 3 pairs: A-B, A-C, B-C
  expect_equal(nrow(out), 3L)
  ## Every from should be alphabetically less than to (upper triangle)
  expect_true(all(out$from < out$to))
})

test_that("mat_to_edgelist directed returns all non-diagonal non-zero entries", {
  A   <- make_sym_mat()        # symmetric: 6 off-diagonal entries
  out <- bibnets:::mat_to_edgelist(A, directed = TRUE)
  expect_equal(nrow(out), 6L)
})

test_that("mat_to_edgelist weights are correct", {
  A   <- make_sym_mat()
  out <- bibnets:::mat_to_edgelist(A, directed = FALSE)
  w_AB <- out$weight[out$from == "A" & out$to == "B"]
  w_AC <- out$weight[out$from == "A" & out$to == "C"]
  w_BC <- out$weight[out$from == "B" & out$to == "C"]
  expect_equal(w_AB, 5)
  expect_equal(w_AC, 3)
  expect_equal(w_BC, 2)
})

test_that("mat_to_edgelist is sorted descending by weight", {
  A   <- make_sym_mat()
  out <- bibnets:::mat_to_edgelist(A, directed = FALSE)
  expect_equal(out$weight, sort(out$weight, decreasing = TRUE))
})

test_that("mat_to_edgelist preserves named nodes", {
  A   <- make_sym_mat()
  out <- bibnets:::mat_to_edgelist(A)
  expect_true(all(out$from %in% c("A", "B", "C")))
  expect_true(all(out$to   %in% c("A", "B", "C")))
})

test_that("mat_to_edgelist generates integer names when dimnames NULL", {
  A <- Matrix::sparseMatrix(
    i = c(1, 1), j = c(2, 3), x = c(4, 2),
    dims = c(3L, 3L)
    ## no dimnames
  )
  out <- bibnets:::mat_to_edgelist(A, directed = FALSE)
  expect_equal(nrow(out), 2L)
  expect_true(all(out$from %in% c("1", "2", "3")))
  expect_true(all(out$to   %in% c("1", "2", "3")))
})

test_that("mat_to_edgelist empty sparse matrix returns 0-row data frame", {
  A <- Matrix::sparseMatrix(i = integer(0), j = integer(0), x = numeric(0),
                            dims = c(3L, 3L),
                            dimnames = list(c("A","B","C"), c("A","B","C")))
  out <- bibnets:::mat_to_edgelist(A, directed = FALSE)
  expect_equal(nrow(out), 0L)
  expect_true(all(c("from", "to", "weight") %in% names(out)))
})

test_that("mat_to_edgelist diagonal-only sparse matrix returns 0-row data frame", {
  ## Only diagonal: after removing diagonal, nothing remains
  A <- Matrix::sparseMatrix(i = c(1, 2), j = c(1, 2), x = c(9, 7),
                            dims = c(3L, 3L),
                            dimnames = list(c("A","B","C"), c("A","B","C")))
  out <- bibnets:::mat_to_edgelist(A, directed = FALSE)
  expect_equal(nrow(out), 0L)
})

test_that("mat_to_edgelist single-edge sparse matrix works", {
  A <- Matrix::sparseMatrix(i = c(1, 2), j = c(2, 1), x = c(7, 7),
                            dims = c(2L, 2L),
                            dimnames = list(c("X", "Y"), c("X", "Y")))
  out <- bibnets:::mat_to_edgelist(A, directed = FALSE)
  expect_equal(nrow(out), 1L)
  expect_equal(out$weight, 7)
})

test_that("mat_to_edgelist directed single-edge matrix returns two rows", {
  A <- Matrix::sparseMatrix(i = c(1, 2), j = c(2, 1), x = c(7, 7),
                            dims = c(2L, 2L),
                            dimnames = list(c("X", "Y"), c("X", "Y")))
  out <- bibnets:::mat_to_edgelist(A, directed = TRUE)
  expect_equal(nrow(out), 2L)
})

test_that("mat_to_edgelist works on dense base matrix (non-Matrix branch)", {
  ## Dense path: inherits(A, "Matrix") is FALSE
  A <- matrix(c(0, 5, 3,
                5, 0, 2,
                3, 2, 0), nrow = 3, ncol = 3,
              dimnames = list(c("A", "B", "C"), c("A", "B", "C")))
  out <- bibnets:::mat_to_edgelist(A, directed = FALSE)
  expect_equal(nrow(out), 3L)
  expect_equal(out$weight[out$from == "A" & out$to == "B"], 5)
})

test_that("mat_to_edgelist dense all-zero matrix returns 0-row data frame", {
  A <- matrix(0, nrow = 3, ncol = 3,
              dimnames = list(c("A","B","C"), c("A","B","C")))
  out <- bibnets:::mat_to_edgelist(A, directed = FALSE)
  expect_equal(nrow(out), 0L)
  expect_true(all(c("from", "to", "weight") %in% names(out)))
})

test_that("mat_to_edgelist stopifnot fires on non-square matrix", {
  A <- matrix(1:6, nrow = 2, ncol = 3)
  expect_error(bibnets:::mat_to_edgelist(A))
})

test_that("mat_to_edgelist directed asymmetric matrix returns only A->B (not B->A)", {
  A <- make_dir_mat()   # only A[1,2]=5, A[2,3]=2
  out_dir  <- bibnets:::mat_to_edgelist(A, directed = TRUE)
  out_undir <- bibnets:::mat_to_edgelist(A, directed = FALSE)
  ## directed: 2 entries
  expect_equal(nrow(out_dir), 2L)
  ## undirected: same 2 entries (already upper triangle)
  expect_equal(nrow(out_undir), 2L)
  expect_true(any(out_dir$from == "A" & out_dir$to == "B"))
})

# ── edgelist_to_mat ────────────────────────────────────────────────────────────

test_that("edgelist_to_mat returns dgCMatrix", {
  edges <- make_edges()
  A <- bibnets:::edgelist_to_mat(edges)
  expect_true(inherits(A, "dgCMatrix"))
})

test_that("edgelist_to_mat is square with correct dimensions", {
  edges <- make_edges()
  A <- bibnets:::edgelist_to_mat(edges)
  expect_equal(nrow(A), 3L)
  expect_equal(ncol(A), 3L)
})

test_that("edgelist_to_mat preserves dimnames derived from edges", {
  edges <- make_edges()
  A <- bibnets:::edgelist_to_mat(edges)
  expect_equal(sort(rownames(A)), c("A", "B", "C"))
  expect_equal(sort(colnames(A)), c("A", "B", "C"))
})

test_that("edgelist_to_mat symmetric=TRUE makes matrix symmetric", {
  edges <- make_edges()
  A <- bibnets:::edgelist_to_mat(edges, symmetric = TRUE)
  expect_equal(as.matrix(A), t(as.matrix(A)))
})

test_that("edgelist_to_mat symmetric=FALSE gives asymmetric matrix", {
  edges <- make_edges()   # upper-triangle edges only
  A <- bibnets:::edgelist_to_mat(edges, symmetric = FALSE)
  ## lower triangle is zero
  expect_equal(as.matrix(A)["B", "A"], 0)
  expect_equal(as.matrix(A)["A", "B"], 5)
})

test_that("edgelist_to_mat places weights at correct positions", {
  edges <- make_edges()
  A <- bibnets:::edgelist_to_mat(edges, symmetric = TRUE)
  m <- as.matrix(A)
  expect_equal(m["A", "B"], 5)
  expect_equal(m["A", "C"], 3)
  expect_equal(m["B", "C"], 2)
})

test_that("edgelist_to_mat diagonal is zero after symmetrization", {
  edges <- make_edges()
  A <- bibnets:::edgelist_to_mat(edges, symmetric = TRUE)
  m <- as.matrix(A)
  expect_equal(unname(diag(m)), c(0, 0, 0))
})

test_that("edgelist_to_mat accepts custom nodes vector", {
  edges <- make_edges()
  nodes <- c("A", "B", "C", "D")   # extra node not in edges
  A <- bibnets:::edgelist_to_mat(edges, nodes = nodes, symmetric = TRUE)
  expect_equal(nrow(A), 4L)
  expect_equal(ncol(A), 4L)
  expect_true("D" %in% rownames(A))
  ## D row and column should be all zero
  expect_equal(sum(as.matrix(A)["D", ]), 0)
})

test_that("edgelist_to_mat round-trip: edges -> mat -> edges matches original", {
  edges_in <- make_edges()
  A        <- bibnets:::edgelist_to_mat(edges_in, symmetric = TRUE)
  edges_out <- bibnets:::mat_to_edgelist(A, directed = FALSE)

  ## Same edge pairs (order may differ)
  pairs_in  <- paste(sort(edges_in$from), sort(edges_in$to),  sep = "-")
  pairs_out <- paste(sort(edges_out$from), sort(edges_out$to), sep = "-")
  expect_equal(sort(pairs_in), sort(pairs_out))

  ## Weights preserved for each pair
  for (i in seq_len(nrow(edges_in))) {
    f <- edges_in$from[i]; t_node <- edges_in$to[i]
    expected_w <- edges_in$weight[i]
    actual_w   <- edges_out$weight[edges_out$from == f & edges_out$to == t_node]
    if (length(actual_w) == 0)
      actual_w <- edges_out$weight[edges_out$from == t_node & edges_out$to == f]
    expect_equal(actual_w, expected_w)
  }
})

test_that("edgelist_to_mat single-edge round-trip", {
  edges <- data.frame(from = "X", to = "Y", weight = 3.5,
                      stringsAsFactors = FALSE)
  A   <- bibnets:::edgelist_to_mat(edges, symmetric = TRUE)
  out <- bibnets:::mat_to_edgelist(A, directed = FALSE)
  expect_equal(nrow(out), 1L)
  expect_equal(out$weight, 3.5)
  expect_true(all(sort(c(out$from, out$to)) == c("X", "Y")))
})

## Cap BLAS/OpenMP threads so check-farm machines with multi-threaded
## BLAS (OpenBLAS, MKL) don't make CPU time exceed elapsed by >2x. CRAN
## permits at most 2 cores in tests. The package uses no explicit
## parallelism; the only multi-threaded path is Matrix's
## crossprod/tcrossprod inheriting BLAS threading on tiny test matrices.
## (The data.table/biblionetwork equivalence tests, the previous source
## of >2x CPU, now live outside the package in
## local_testing_and_equivalence/ and are not part of R CMD check.)
Sys.setenv(OMP_THREAD_LIMIT = "2", OMP_NUM_THREADS = "2")

library(testthat)
library(bibnets)

test_check("bibnets")

#' Read an RIS file
#'
#' Parses a `.ris` file into a standardized bibliometric data frame.
#' Like BibTeX, standard RIS does not include cited references.
#'
#' @param file Path to a `.ris` file.
#' @param encoding Character. File encoding. Default `"UTF-8"`.
#'
#' @return A data frame in the standard bibnets format: `id`, `title`,
#'   `year`, `journal`, `doi`, `cited_by_count`, `abstract`, `type`,
#'   plus list-columns `authors`, `references` (typically empty for
#'   RIS), and `keywords`.
#'
#' @export
#' @examples
#' # Write a minimal RIS record to a temp file, then read it
#' ris <- "TY  - JOUR
#' AU  - Smith, J.
#' AU  - Jones, K.
#' TI  - Bibliometric networks
#' JO  - Test Journal
#' PY  - 2020
#' DO  - 10.1000/test
#' ER  - "
#' f <- tempfile(fileext = ".ris")
#' writeLines(ris, f)
#' data <- read_ris(f)
#' data[, c("id", "title", "year", "journal", "doi")]
#' unlink(f)
read_ris <- function(file, encoding = "UTF-8") {
  check_file(file)

  lines <- readLines(file, encoding = encoding, warn = FALSE)

  records <- list()
  current <- list()

  for (line in lines) {
    line <- trimws(line)
    if (nchar(line) == 0) next

    ## End of record
    if (grepl("^ER\\s*-", line)) {
      if (length(current) > 0) {
        records <- c(records, list(current))
      }
      current <- list()
      next
    }

    ## Tag line: "TY  - JOUR"
    if (grepl("^[A-Z][A-Z0-9]\\s+-\\s+", line)) {
      tag <- sub("\\s+-.*$", "", line)
      tag <- trimws(tag)
      value <- sub("^[A-Z][A-Z0-9]\\s+-\\s+", "", line)
      value <- trimws(value)
      current[[tag]] <- c(current[[tag]], value)
    }
  }

  if (length(current) > 0) {
    records <- c(records, list(current))
  }

  n <- length(records)
  if (n == 0) return(empty_biblio_df())

  get_ris <- function(rec, tags, collapse = NULL) {
    for (tag in tags) {
      val <- rec[[tag]]
      if (!is.null(val)) {
        if (!is.null(collapse)) return(paste(val, collapse = collapse))
        return(val[1])
      }
    }
    NA_character_
  }

  result <- data.frame(
    id = vapply(records, function(r) {
      id_val <- get_ris(r, c("DO", "AN", "ID"))
      if (is.na(id_val)) paste0("RIS", which(
        vapply(records, identical, logical(1), r)
      ))
      else id_val
    }, character(1)),
    title = vapply(records, function(r) get_ris(r, c("TI", "T1", "CT")),
                   character(1)),
    year = vapply(records, function(r) {
      y <- get_ris(r, c("PY", "Y1", "DA"))
      if (is.na(y)) return(NA_integer_)
      ## Extract 4-digit year from date strings
      m <- regmatches(y, regexpr("\\d{4}", y))
      if (length(m) > 0) as.integer(m[1]) else NA_integer_
    }, integer(1)),
    journal = vapply(records, function(r) {
      get_ris(r, c("JO", "JF", "T2", "JA"))
    }, character(1)),
    doi = vapply(records, function(r) get_ris(r, "DO"), character(1)),
    cited_by_count = NA_integer_,
    abstract = vapply(records, function(r) get_ris(r, c("AB", "N2")),
                      character(1)),
    type = vapply(records, function(r) get_ris(r, "TY"), character(1)),
    stringsAsFactors = FALSE
  )

  ## Authors (AU tag, repeatable)
  result$authors <- lapply(records, function(r) {
    au <- r[["AU"]]
    if (is.null(au)) au <- r[["A1"]]
    if (is.null(au)) return(character(0))
    standardize_authors(au)
  })

  ## No references in standard RIS
  result$references <- replicate(n, character(0), simplify = FALSE)

  ## Keywords (KW tag, repeatable)
  result$keywords <- lapply(records, function(r) {
    kw <- r[["KW"]]
    if (is.null(kw)) return(character(0))
    trimws(kw)
  })

  result
}

#' Construct a co-occurrence network via two-mode multiplication
#'
#' The unified engine for all bibliometric networks. Operates entirely in
#' sparse representation — never allocates a dense n x n matrix.
#'
#' @param B A sparse bipartite matrix (works x entities), already weighted
#'   by the counting method.
#' @param mode Character. `"columns"` for column-mode co-occurrence
#'   (e.g., co-citation), `"rows"` for row-mode (e.g., coupling).
#' @param similarity Character. Normalization method (see [normalize()]).
#' @param threshold Numeric. Minimum edge weight to retain.
#' @param top_n Integer or `NULL`. If specified, keep only the top `n`
#'   most frequent nodes and return all edges among them.
#'
#' @return A data frame with columns `from`, `to`, `weight`, `count`.
#' @keywords internal
multiply_bipartite <- function(B, mode = "columns",
                               similarity = "none",
                               threshold = 0,
                               top_n = NULL,
                               self_loops = FALSE) {
  ## Coupling-strength: deferred row-normalization from apply_counting.
  ## This denominator is defined for row-mode projections (e.g., document
  ## coupling). In column-mode projections, edge indices refer to column nodes,
  ## so applying a row-level denominator would be dimensionally wrong.
  row_scale <- attr(B, "row_scale")

  ## Weighted co-occurrence matrix (sparse)
  if (mode == "columns") {
    A <- Matrix::crossprod(B)
  } else {
    A <- Matrix::tcrossprod(B)
  }

  ## Raw binary co-occurrence for the count column
  B_bin <- (B > 0) * 1
  if (mode == "columns") {
    A_raw <- Matrix::crossprod(B_bin)
  } else {
    A_raw <- Matrix::tcrossprod(B_bin)
  }

  ## Top-n node filter: keep only the most frequent entities
  if (!is.null(top_n)) {
    freq <- Matrix::diag(A)
    if (length(freq) > top_n) {
      keep_idx <- order(-freq)[seq_len(top_n)]
      A <- A[keep_idx, keep_idx]
      A_raw <- A_raw[keep_idx, keep_idx]
      if (!is.null(row_scale) && mode == "rows") {
        row_scale <- row_scale[keep_idx]
      }
    }
  }

  ## Normalize weighted matrix (or just zero the diagonal)
  if (similarity != "none") {
    A_norm <- normalize(A, method = similarity)
  } else {
    if (!self_loops) Matrix::diag(A) <- 0
    A_norm <- A
  }

  ## Extract upper-triangle edges directly from sparse (no dense conversion)
  triplets <- Matrix::summary(A_norm)
  i <- triplets$i
  j <- triplets$j
  x <- triplets$x

  ## Remove lower triangle and zeros (diagonal kept when self_loops = TRUE)
  keep <- (if (self_loops) i <= j else i < j) & x != 0
  i <- i[keep]; j <- j[keep]; x <- x[keep]

  if (length(i) == 0L) {
    return(data.frame(
      from = character(0), to = character(0),
      weight = numeric(0), count = integer(0),
      stringsAsFactors = FALSE
    ))
  }

  ## Coupling-strength: divide by n_refs_i × n_refs_j
  if (!is.null(row_scale) && mode == "rows" && similarity == "none") {
    x <- x / (row_scale[i] * row_scale[j])
  }

  ## Threshold filter early (before name lookup)
  if (threshold > 0) {
    keep <- x >= threshold
    i <- i[keep]; j <- j[keep]; x <- x[keep]
  }

  if (length(i) == 0L) {
    return(data.frame(
      from = character(0), to = character(0),
      weight = numeric(0), count = integer(0),
      stringsAsFactors = FALSE
    ))
  }

  node_names <- rownames(A_norm)
  if (is.null(node_names)) node_names <- as.character(seq_len(nrow(A_norm)))

  ## Look up raw counts for matched edges (vectorized sparse access)
  raw_counts <- A_raw[cbind(i, j)]

  edges <- data.frame(
    from = node_names[i],
    to = node_names[j],
    weight = x,
    count = as.integer(raw_counts),
    stringsAsFactors = FALSE
  )

  ## Sort by weight descending
  edges <- edges[order(-edges$weight), ]
  rownames(edges) <- NULL
  edges
}

#' Read Web of Science plaintext or tab-delimited export
#'
#' Parses a Web of Science export file (plaintext or tab-delimited) into a
#' standardized bibliometric data frame.
#'
#' @param file Path to a WoS export file (.txt).
#' @param format Character. `"plaintext"` (default) for WoS tagged format,
#'   or `"tab"` for tab-delimited export.
#'
#' @return A data frame in the standard bibnets format: `id`, `title`,
#'   `year`, `journal`, `doi`, `cited_by_count`, `abstract`, `type`,
#'   plus list-columns `authors`, `references`, and `keywords`.
#'   WoS-specific extra: `keywords_plus` (list-column).
#'
#' @export
#' @examples
#' f <- system.file("extdata", "wos_sample.txt", package = "bibnets")
#' data <- read_wos(f)
#' head(data[, c("id", "title", "year", "journal")])
read_wos <- function(file, format = "plaintext") {
  check_file(file)
  check_choice(format, c("plaintext", "tab"), "format")

  if (format == "tab") {
    return(read_wos_tab(file))
  }

  lines <- readLines(file, encoding = "UTF-8", warn = FALSE)

  ## Parse tagged format: each record starts with field tags
  ## Records are delimited by "ER" lines
  records <- list()
  current <- list()
  current_tag <- ""

  for (line in lines) {
    if (grepl("^ER\\s*$", line)) {
      if (length(current) > 0) {
        records <- c(records, list(current))
      }
      current <- list()
      current_tag <- ""
      next
    }

    ## Skip file headers
    if (grepl("^(FN |VR |EF\\s*$)", line)) next

    ## Tag line: first two characters are the tag
    tag <- substr(line, 1, 2)
    value <- trimws(substr(line, 4, nchar(line)))

    if (tag != "  ") {
      current_tag <- tag
      current[[current_tag]] <- c(current[[current_tag]], value)
    } else if (nchar(current_tag) > 0) {
      ## Continuation line
      current[[current_tag]] <- c(current[[current_tag]], value)
    }
  }

  ## Convert records to data frame
  n <- length(records)
  if (n == 0) {
    return(empty_biblio_df())
  }

  get_field <- function(rec, tag, collapse = NULL) {
    val <- rec[[tag]]
    if (is.null(val)) return(NA_character_)
    if (!is.null(collapse)) paste(val, collapse = collapse) else val[1]
  }

  id <- vapply(records, function(r) {
    ut <- get_field(r, "UT")
    if (is.na(ut)) paste0("WOS", which(vapply(records, identical, logical(1), r)))
    else ut
  }, character(1))

  title <- vapply(records, function(r) get_field(r, "TI", collapse = " "),
                  character(1))

  authors_raw <- lapply(records, function(r) {
    au <- r[["AU"]]
    if (is.null(au)) character(0) else standardize_authors(au)
  })

  year <- vapply(records, function(r) {
    py <- get_field(r, "PY")
    if (is.na(py)) NA_integer_ else as.integer(py)
  }, integer(1))

  journal <- vapply(records, function(r) get_field(r, "SO"), character(1))

  doi <- vapply(records, function(r) get_field(r, "DI"), character(1))

  refs_raw <- lapply(records, function(r) {
    cr <- r[["CR"]]
    if (is.null(cr)) return(character(0))
    ## CR field may have multiple refs per line, semicolon-delimited
    refs <- unlist(strsplit(cr, ";"), use.names = FALSE)
    standardize_refs(refs)
  })

  keywords <- lapply(records, function(r) {
    de <- r[["DE"]]
    if (is.null(de)) return(character(0))
    kw <- unlist(strsplit(paste(de, collapse = ";"), ";"), use.names = FALSE)
    trimws(kw)
  })

  keywords_plus <- lapply(records, function(r) {
    id_field <- r[["ID"]]
    if (is.null(id_field)) return(character(0))
    kw <- unlist(strsplit(paste(id_field, collapse = ";"), ";"),
                  use.names = FALSE)
    trimws(kw)
  })

  cited_by <- vapply(records, function(r) {
    tc <- get_field(r, "TC")
    if (is.na(tc)) 0L else as.integer(tc)
  }, integer(1))

  abstract <- vapply(records, function(r) get_field(r, "AB", collapse = " "),
                     character(1))

  type <- vapply(records, function(r) get_field(r, "DT"), character(1))

  result <- data.frame(
    id = id,
    title = title,
    year = year,
    journal = journal,
    doi = doi,
    cited_by_count = cited_by,
    abstract = abstract,
    type = type,
    stringsAsFactors = FALSE
  )

  result$authors <- authors_raw
  result$references <- refs_raw
  result$keywords <- keywords
  result$keywords_plus <- keywords_plus

  result
}


#' @keywords internal
read_wos_tab <- function(file) {
  raw <- utils::read.delim(file, stringsAsFactors = FALSE, quote = "",
                            fileEncoding = "UTF-8", check.names = FALSE)

  get_col <- function(names_vec, default = NA_character_) {
    for (nm in names_vec) {
      if (nm %in% names(raw)) return(raw[[nm]])
    }
    rep(default, nrow(raw))
  }

  id <- get_col(c("UT", "Accession Number"), paste0("WOS", seq_len(nrow(raw))))

  result <- data.frame(
    id = id,
    title = get_col(c("TI", "Title")),
    year = as.integer(get_col(c("PY", "Publication Year"), NA_integer_)),
    journal = get_col(c("SO", "Source Title")),
    doi = get_col(c("DI", "DOI")),
    cited_by_count = as.integer(get_col(c("TC", "Times Cited"), 0L)),
    abstract = get_col(c("AB", "Abstract")),
    type = get_col(c("DT", "Document Type")),
    stringsAsFactors = FALSE
  )

  result$authors <- split_field(get_col(c("AU", "Authors")), sep = ";")
  result$authors <- lapply(result$authors, standardize_authors)

  result$references <- split_field(get_col(c("CR", "Cited References")),
                                    sep = ";")
  result$references <- lapply(result$references, standardize_refs)

  result$keywords <- split_field(get_col(c("DE", "Author Keywords")), sep = ";")
  result$keywords <- lapply(result$keywords, trimws)

  result$keywords_plus <- split_field(get_col(c("ID", "Keywords Plus")),
                                       sep = ";")
  result$keywords_plus <- lapply(result$keywords_plus, trimws)

  result
}


#' @keywords internal
empty_biblio_df <- function() {
  result <- data.frame(
    id = character(0),
    title = character(0),
    year = integer(0),
    journal = character(0),
    doi = character(0),
    cited_by_count = integer(0),
    abstract = character(0),
    type = character(0),
    stringsAsFactors = FALSE
  )
  result$authors <- list()
  result$references <- list()
  result$keywords <- list()
  result$keywords_plus <- list()
  result
}

## Tests for read_wos() — plaintext (tagged) and tab-delimited formats
## Synthetic files only; no network/API calls.

## ---------------------------------------------------------------------------
## Helpers
## ---------------------------------------------------------------------------

## Write a minimal WoS plaintext file and return its path
wos_pt_file <- function(records_text) {
  f <- tempfile(fileext = ".txt")
  writeLines(records_text, f)
  f
}

## Build a single WoS plaintext record from a named list of tag->value pairs.
## Each value may be a character vector (multiple continuation lines).
make_record <- function(fields) {
  lines <- character(0)
  for (nm in names(fields)) {
    vals <- fields[[nm]]
    lines <- c(lines, sprintf("%-2s %s", nm, vals[1]))
    if (length(vals) > 1) {
      lines <- c(lines, vapply(vals[-1], function(v) sprintf("   %s", v),
                               character(1)))
    }
  }
  c(lines, "ER")
}

## ---------------------------------------------------------------------------
## Plaintext: standard columns are returned
## ---------------------------------------------------------------------------

test_that("read_wos plaintext returns standard bibnets columns", {
  rec <- make_record(list(
    UT = "WOS:000001",
    TI = "A great paper",
    AU = c("Smith, John", "Doe, Jane"),
    PY = "2020",
    SO = "Journal of Testing",
    DI = "10.1000/test.001",
    TC = "5",
    AB = "This is an abstract.",
    DT = "Article",
    DE = "network analysis; bibliometrics",
    ID = "machine learning; deep learning",
    CR = "Ref A, 2010; Ref B, 2011"
  ))
  f <- wos_pt_file(rec)
  d <- read_wos(f, format = "plaintext")

  expected_cols <- c("id", "title", "year", "journal", "doi",
                     "cited_by_count", "abstract", "type",
                     "authors", "references", "keywords", "keywords_plus")
  expect_true(all(expected_cols %in% names(d)))
})

test_that("read_wos plaintext column order matches standard schema", {
  rec <- make_record(list(
    UT = "WOS:000001",
    TI = "Title",
    AU = "Smith, J",
    PY = "2020",
    SO = "Journal",
    DI = "10.1/x",
    TC = "1",
    AB = "Abstract.",
    DT = "Article",
    DE = "kw1",
    ID = "kw2"
  ))
  f <- wos_pt_file(rec)
  d <- read_wos(f, format = "plaintext")
  # first 8 scalar columns in order
  expect_equal(names(d)[1:8],
               c("id", "title", "year", "journal", "doi",
                 "cited_by_count", "abstract", "type"))
})

## ---------------------------------------------------------------------------
## Plaintext: scalar field values
## ---------------------------------------------------------------------------

test_that("read_wos plaintext parses a full record correctly", {
  rec <- make_record(list(
    UT = "WOS:000001",
    TI = "A great paper",
    AU = c("Smith, John", "Doe, Jane"),
    PY = "2020",
    SO = "Journal of Testing",
    DI = "10.1000/test.001",
    TC = "5",
    AB = "This is an abstract.",
    DT = "Article",
    DE = "network analysis; bibliometrics",
    ID = "machine learning; deep learning",
    CR = "Ref A, 2010; Ref B, 2011"
  ))
  f <- wos_pt_file(rec)
  d <- read_wos(f, format = "plaintext")

  expect_equal(nrow(d), 1L)
  expect_equal(d$id,             "WOS:000001")
  expect_equal(d$title,          "A great paper")
  expect_equal(d$year,           2020L)
  expect_equal(d$journal,        "Journal of Testing")
  expect_equal(d$doi,            "10.1000/test.001")
  expect_equal(d$cited_by_count, 5L)
  expect_equal(d$abstract,       "This is an abstract.")
  expect_equal(d$type,           "Article")
})

test_that("read_wos plaintext year is integer", {
  rec <- make_record(list(UT = "WOS:1", TI = "T", AU = "Smith, J",
                          PY = "2018", SO = "J"))
  f <- wos_pt_file(rec)
  d <- read_wos(f, format = "plaintext")
  expect_type(d$year, "integer")
})

test_that("read_wos plaintext cited_by_count is integer", {
  rec <- make_record(list(UT = "WOS:1", TI = "T", AU = "Smith, J",
                          PY = "2018", SO = "J", TC = "42"))
  f <- wos_pt_file(rec)
  d <- read_wos(f, format = "plaintext")
  expect_type(d$cited_by_count, "integer")
  expect_equal(d$cited_by_count, 42L)
})

## ---------------------------------------------------------------------------
## Plaintext: list-columns
## ---------------------------------------------------------------------------

test_that("read_wos plaintext authors is a list-column", {
  rec <- make_record(list(UT = "WOS:1", TI = "T", AU = c("Smith, J", "Doe, A"),
                          PY = "2020", SO = "J"))
  f <- wos_pt_file(rec)
  d <- read_wos(f, format = "plaintext")
  expect_true(is.list(d$authors))
})

test_that("read_wos plaintext author names are uppercased", {
  rec <- make_record(list(UT = "WOS:1", TI = "T",
                          AU = c("Smith, John", "Doe, Jane"),
                          PY = "2020", SO = "J"))
  f <- wos_pt_file(rec)
  d <- read_wos(f, format = "plaintext")
  expect_true(all(d$authors[[1]] == toupper(d$authors[[1]])))
})

test_that("read_wos plaintext multi-line author list collects all authors", {
  ## AU tag with continuation lines (each author on its own line)
  rec <- make_record(list(
    UT = "WOS:1",
    TI = "T",
    AU = c("Alpha, A", "Beta, B", "Gamma, G"),
    PY = "2020",
    SO = "J"
  ))
  f <- wos_pt_file(rec)
  d <- read_wos(f, format = "plaintext")
  expect_equal(length(d$authors[[1]]), 3L)
})

test_that("read_wos plaintext references is a list-column", {
  rec <- make_record(list(UT = "WOS:1", TI = "T", AU = "Smith, J",
                          PY = "2020", SO = "J",
                          CR = "Ref A, 2010; Ref B, 2011"))
  f <- wos_pt_file(rec)
  d <- read_wos(f, format = "plaintext")
  expect_true(is.list(d$references))
})

test_that("read_wos plaintext references are split on semicolon", {
  rec <- make_record(list(UT = "WOS:1", TI = "T", AU = "Smith, J",
                          PY = "2020", SO = "J",
                          CR = "Ref A, 2010; Ref B, 2011"))
  f <- wos_pt_file(rec)
  d <- read_wos(f, format = "plaintext")
  expect_equal(length(d$references[[1]]), 2L)
})

test_that("read_wos plaintext references are uppercased", {
  rec <- make_record(list(UT = "WOS:1", TI = "T", AU = "Smith, J",
                          PY = "2020", SO = "J", CR = "ref a, 2010"))
  f <- wos_pt_file(rec)
  d <- read_wos(f, format = "plaintext")
  expect_true(all(d$references[[1]] == toupper(d$references[[1]])))
})

test_that("read_wos plaintext keywords is a list-column split on semicolon", {
  rec <- make_record(list(UT = "WOS:1", TI = "T", AU = "Smith, J",
                          PY = "2020", SO = "J",
                          DE = "network analysis; bibliometrics"))
  f <- wos_pt_file(rec)
  d <- read_wos(f, format = "plaintext")
  expect_true(is.list(d$keywords))
  expect_equal(length(d$keywords[[1]]), 2L)
  expect_true("network analysis" %in% d$keywords[[1]])
})

test_that("read_wos plaintext keywords_plus is a list-column split on semicolon", {
  rec <- make_record(list(UT = "WOS:1", TI = "T", AU = "Smith, J",
                          PY = "2020", SO = "J",
                          ID = "machine learning; deep learning"))
  f <- wos_pt_file(rec)
  d <- read_wos(f, format = "plaintext")
  expect_true(is.list(d$keywords_plus))
  expect_equal(length(d$keywords_plus[[1]]), 2L)
})

## ---------------------------------------------------------------------------
## Plaintext: missing / optional tags → NA / empty list
## ---------------------------------------------------------------------------

test_that("read_wos plaintext missing DOI returns NA", {
  rec <- make_record(list(UT = "WOS:1", TI = "T", AU = "Smith, J",
                          PY = "2020", SO = "J"))
  f <- wos_pt_file(rec)
  d <- read_wos(f, format = "plaintext")
  expect_true(is.na(d$doi))
})

test_that("read_wos plaintext missing TC defaults to 0L", {
  rec <- make_record(list(UT = "WOS:1", TI = "T", AU = "Smith, J",
                          PY = "2020", SO = "J"))
  f <- wos_pt_file(rec)
  d <- read_wos(f, format = "plaintext")
  expect_equal(d$cited_by_count, 0L)
})

test_that("read_wos plaintext missing CR returns empty list element", {
  rec <- make_record(list(UT = "WOS:1", TI = "T", AU = "Smith, J",
                          PY = "2020", SO = "J"))
  f <- wos_pt_file(rec)
  d <- read_wos(f, format = "plaintext")
  expect_equal(length(d$references[[1]]), 0L)
})

test_that("read_wos plaintext missing DE returns empty keywords list element", {
  rec <- make_record(list(UT = "WOS:1", TI = "T", AU = "Smith, J",
                          PY = "2020", SO = "J"))
  f <- wos_pt_file(rec)
  d <- read_wos(f, format = "plaintext")
  expect_equal(length(d$keywords[[1]]), 0L)
})

test_that("read_wos plaintext missing ID returns empty keywords_plus list element", {
  rec <- make_record(list(UT = "WOS:1", TI = "T", AU = "Smith, J",
                          PY = "2020", SO = "J"))
  f <- wos_pt_file(rec)
  d <- read_wos(f, format = "plaintext")
  expect_equal(length(d$keywords_plus[[1]]), 0L)
})

test_that("read_wos plaintext missing PY returns NA_integer_", {
  rec <- make_record(list(UT = "WOS:1", TI = "T", AU = "Smith, J", SO = "J"))
  f <- wos_pt_file(rec)
  d <- read_wos(f, format = "plaintext")
  expect_true(is.na(d$year))
  expect_type(d$year, "integer")
})

test_that("read_wos plaintext missing UT falls back to generated id", {
  rec <- make_record(list(TI = "T", AU = "Smith, J", PY = "2020", SO = "J"))
  f <- wos_pt_file(rec)
  d <- read_wos(f, format = "plaintext")
  expect_false(is.na(d$id))
  expect_true(nchar(d$id) > 0)
})

## ---------------------------------------------------------------------------
## Plaintext: multi-record file
## ---------------------------------------------------------------------------

test_that("read_wos plaintext parses multiple records", {
  rec1 <- make_record(list(UT = "WOS:001", TI = "First paper",
                           AU = "Alpha, A", PY = "2020", SO = "Journal A"))
  rec2 <- make_record(list(UT = "WOS:002", TI = "Second paper",
                           AU = "Beta, B", PY = "2021", SO = "Journal B",
                           DI = "10.99/x", TC = "3"))
  rec3 <- make_record(list(UT = "WOS:003", TI = "Third paper",
                           AU = "Gamma, G", PY = "2022", SO = "Journal C",
                           CR = "Some ref, 2000", DE = "keyword"))
  f <- wos_pt_file(c(rec1, "", rec2, "", rec3))
  d <- read_wos(f, format = "plaintext")
  expect_equal(nrow(d), 3L)
  expect_equal(d$id,    c("WOS:001", "WOS:002", "WOS:003"))
  expect_equal(d$year,  c(2020L, 2021L, 2022L))
  expect_equal(d$cited_by_count[2], 3L)
})

## ---------------------------------------------------------------------------
## Plaintext: FN/VR/EF header lines are skipped
## ---------------------------------------------------------------------------

test_that("read_wos plaintext ignores FN/VR/EF file-header lines", {
  rec <- make_record(list(UT = "WOS:1", TI = "T", AU = "Smith, J",
                          PY = "2020", SO = "J"))
  lines <- c(
    "FN Web of Science",
    "VR 1.0",
    rec,
    "EF"
  )
  f <- wos_pt_file(lines)
  d <- read_wos(f, format = "plaintext")
  expect_equal(nrow(d), 1L)
})

## ---------------------------------------------------------------------------
## Plaintext: CRLF line endings
## ---------------------------------------------------------------------------

test_that("read_wos plaintext handles Windows CRLF line endings", {
  rec <- make_record(list(UT = "WOS:CRLF", TI = "CRLF paper",
                          AU = "Smith, J", PY = "2019", SO = "J",
                          TC = "2"))
  f <- tempfile(fileext = ".txt")
  ## Write with explicit CRLF line endings (Windows-style)
  ## Encode each line as UTF-8 bytes followed by CR+LF
  crlf_bytes <- do.call(c, lapply(rec, function(ln) {
    c(chartr("", "", iconv(ln, to = "UTF-8")), as.raw(c(0x0d, 0x0a)))
  }))
  ## Use paste+rawToChar approach: build the full text with \r\n
  full_text <- paste(rec, collapse = "\r\n")
  writeBin(chartr("", "", full_text), f)
  ## Actually write raw bytes: convert string to raw with CRLF
  raw_content <- iconv(paste(rec, collapse = "\r\n"), to = "UTF-8")
  con <- file(f, open = "wb")
  writeBin(raw_content, con)
  close(con)
  d <- read_wos(f, format = "plaintext")
  expect_equal(nrow(d), 1L)
  expect_equal(d$year, 2019L)
})

## ---------------------------------------------------------------------------
## Plaintext: non-ASCII characters in title/abstract
## ---------------------------------------------------------------------------

test_that("read_wos plaintext handles non-ASCII characters", {
  rec <- make_record(list(
    UT = "WOS:NONASCII",
    TI = "Über die Netzwerkanalyse",
    AU = "Müller, Hans",
    PY = "2021",
    SO = "Zeitschrift",
    AB = "Résumé: données bibliométriques"
  ))
  f <- wos_pt_file(rec)
  d <- read_wos(f, format = "plaintext")
  expect_equal(nrow(d), 1L)
  expect_true(grepl("Netzwerkanalyse", d$title))
})

## ---------------------------------------------------------------------------
## Plaintext: DOI with trailing DOI strip in references
## ---------------------------------------------------------------------------

test_that("read_wos plaintext CR DOI suffix is stripped from references", {
  rec <- make_record(list(
    UT = "WOS:DOI",
    TI = "DOI test",
    AU = "Smith, J",
    PY = "2020",
    SO = "J",
    CR = "Author A, 2010, DOI 10.1000/abc; Author B, 2011, DOI 10.9999/xyz"
  ))
  f <- wos_pt_file(rec)
  d <- read_wos(f, format = "plaintext")
  refs <- d$references[[1]]
  expect_equal(length(refs), 2L)
  expect_false(any(grepl("DOI", refs, ignore.case = FALSE)))
})

## ---------------------------------------------------------------------------
## Plaintext: empty file → empty data frame
## ---------------------------------------------------------------------------

test_that("read_wos plaintext empty file returns empty data frame", {
  f <- wos_pt_file(character(0))
  d <- read_wos(f, format = "plaintext")
  expect_equal(nrow(d), 0L)
  expect_true(all(c("id", "title", "year", "journal", "doi",
                    "cited_by_count", "abstract", "type") %in% names(d)))
  ## Regression: empty result must include keywords_plus to match non-empty schema
  expect_true("keywords_plus" %in% names(d))
  expect_true(is.list(d$keywords_plus))
})

## ---------------------------------------------------------------------------
## Plaintext: trailing whitespace in field values
## ---------------------------------------------------------------------------

test_that("read_wos plaintext trims trailing whitespace from field values", {
  lines <- c(
    "UT WOS:TRIM   ",
    "TI  Title with trailing spaces   ",
    "AU  Smith, J   ",
    "PY  2020   ",
    "SO  Journal   ",
    "ER"
  )
  f <- wos_pt_file(lines)
  d <- read_wos(f, format = "plaintext")
  expect_false(grepl("\\s+$", d$title))
  expect_false(grepl("\\s+$", d$journal))
})

## ---------------------------------------------------------------------------
## Error handling
## ---------------------------------------------------------------------------

test_that("read_wos errors on non-existent file", {
  expect_error(read_wos("no_such_file.txt"), "not found")
})

test_that("read_wos errors on invalid format argument", {
  f <- wos_pt_file(character(0))
  expect_error(read_wos(f, format = "invalid"), "format")
})

## ---------------------------------------------------------------------------
## Tab-delimited format
## ---------------------------------------------------------------------------

## Build a tab-delimited WoS export file from a header + rows list
wos_tab_file <- function(header, rows) {
  f <- tempfile(fileext = ".txt")
  lines <- c(
    paste(header, collapse = "\t"),
    vapply(rows, function(r) paste(r, collapse = "\t"), character(1))
  )
  writeLines(lines, f)
  f
}

## Canonical WoS tab header (abbreviated to what the parser actually uses)
wos_tab_header <- c("UT", "TI", "AU", "PY", "SO", "DI", "TC",
                    "AB", "DT", "DE", "ID", "CR")

test_that("read_wos tab returns standard bibnets columns", {
  f <- wos_tab_file(
    wos_tab_header,
    list(c("WOS:T001", "Tab paper", "Smith, J; Doe, A",
           "2021", "Tab Journal", "10.1/t", "7",
           "An abstract.", "Article",
           "networks; analysis", "deep learning",
           "Ref A, 2010; Ref B, 2011"))
  )
  d <- read_wos(f, format = "tab")
  expected_cols <- c("id", "title", "year", "journal", "doi",
                     "cited_by_count", "abstract", "type",
                     "authors", "references", "keywords", "keywords_plus")
  expect_true(all(expected_cols %in% names(d)))
})

test_that("read_wos tab parses scalar fields correctly", {
  f <- wos_tab_file(
    wos_tab_header,
    list(c("WOS:T001", "Tab paper", "Smith, J",
           "2021", "Tab Journal", "10.1/t", "7",
           "An abstract.", "Article", "networks", "deep learning", ""))
  )
  d <- read_wos(f, format = "tab")
  expect_equal(nrow(d), 1L)
  expect_equal(d$id,             "WOS:T001")
  expect_equal(d$title,          "Tab paper")
  expect_equal(d$year,           2021L)
  expect_equal(d$journal,        "Tab Journal")
  expect_equal(d$doi,            "10.1/t")
  expect_equal(d$cited_by_count, 7L)
  expect_equal(d$abstract,       "An abstract.")
  expect_equal(d$type,           "Article")
})

test_that("read_wos tab year is integer", {
  f <- wos_tab_file(
    wos_tab_header,
    list(c("WOS:T001", "T", "S, J", "2019", "J", "", "0", "", "A", "", "", ""))
  )
  d <- read_wos(f, format = "tab")
  expect_type(d$year, "integer")
})

test_that("read_wos tab cited_by_count is integer", {
  f <- wos_tab_file(
    wos_tab_header,
    list(c("WOS:T001", "T", "S, J", "2019", "J", "", "15", "", "A", "", "", ""))
  )
  d <- read_wos(f, format = "tab")
  expect_type(d$cited_by_count, "integer")
  expect_equal(d$cited_by_count, 15L)
})

test_that("read_wos tab authors are split on semicolon and uppercased", {
  f <- wos_tab_file(
    wos_tab_header,
    list(c("WOS:T001", "T", "Smith, J; Doe, A", "2020", "J",
           "", "0", "", "A", "", "", ""))
  )
  d <- read_wos(f, format = "tab")
  expect_true(is.list(d$authors))
  expect_equal(length(d$authors[[1]]), 2L)
  expect_true(all(d$authors[[1]] == toupper(d$authors[[1]])))
})

test_that("read_wos tab references are split on semicolon", {
  f <- wos_tab_file(
    wos_tab_header,
    list(c("WOS:T001", "T", "S, J", "2020", "J", "", "0", "", "A",
           "", "", "Ref A, 2010; Ref B, 2011; Ref C, 2012"))
  )
  d <- read_wos(f, format = "tab")
  expect_true(is.list(d$references))
  expect_equal(length(d$references[[1]]), 3L)
})

test_that("read_wos tab keywords split on semicolon", {
  f <- wos_tab_file(
    wos_tab_header,
    list(c("WOS:T001", "T", "S, J", "2020", "J", "", "0", "", "A",
           "networks; analysis; visualization", "", ""))
  )
  d <- read_wos(f, format = "tab")
  expect_true(is.list(d$keywords))
  expect_equal(length(d$keywords[[1]]), 3L)
})

test_that("read_wos tab keywords_plus split on semicolon", {
  f <- wos_tab_file(
    wos_tab_header,
    list(c("WOS:T001", "T", "S, J", "2020", "J", "", "0", "", "A",
           "", "machine learning; NLP", ""))
  )
  d <- read_wos(f, format = "tab")
  expect_true(is.list(d$keywords_plus))
  expect_equal(length(d$keywords_plus[[1]]), 2L)
})

test_that("read_wos tab multiple records parsed correctly", {
  rows <- list(
    c("WOS:T001", "Paper One", "Alpha, A", "2019", "J1",
      "10.1/a", "3", "Abs1", "Article", "kw1", "kp1", "r1; r2"),
    c("WOS:T002", "Paper Two", "Beta, B; Gamma, G", "2020", "J2",
      "10.1/b", "8", "Abs2", "Review", "kw2; kw3", "", ""),
    c("WOS:T003", "Paper Three", "Delta, D", "2021", "J3",
      "", "0", "", "Article", "", "", "")
  )
  f <- wos_tab_file(wos_tab_header, rows)
  d <- read_wos(f, format = "tab")
  expect_equal(nrow(d), 3L)
  expect_equal(d$id,   c("WOS:T001", "WOS:T002", "WOS:T003"))
  expect_equal(d$year, c(2019L, 2020L, 2021L))
  expect_equal(length(d$references[[1]]), 2L)
  expect_equal(length(d$authors[[2]]), 2L)
})

test_that("read_wos tab with alternative 'Title' / 'Source Title' headers", {
  alt_header <- c("Accession Number", "Title", "Authors",
                  "Publication Year", "Source Title", "DOI",
                  "Times Cited", "Abstract", "Document Type",
                  "Author Keywords", "Keywords Plus", "Cited References")
  f <- wos_tab_file(
    alt_header,
    list(c("WOS:ALT1", "Alt Title", "Smith, J", "2022", "Alt Journal",
           "10.99/alt", "12", "Alt abstract.", "Article",
           "alt kw", "alt kp", ""))
  )
  d <- read_wos(f, format = "tab")
  expect_equal(nrow(d), 1L)
  expect_equal(d$title,   "Alt Title")
  expect_equal(d$journal, "Alt Journal")
})

test_that("read_wos tab empty AU cell returns empty authors list element", {
  f <- wos_tab_file(
    wos_tab_header,
    list(c("WOS:T001", "T", "", "2020", "J", "", "0", "", "A", "", "", ""))
  )
  d <- read_wos(f, format = "tab")
  expect_equal(length(d$authors[[1]]), 0L)
})

test_that("read_wos tab errors on non-existent file", {
  expect_error(read_wos("no_such_file.txt", format = "tab"), "not found")
})

## ---------------------------------------------------------------------------
## Coverage: line 85 — plaintext record with no AU tag at all
## ---------------------------------------------------------------------------

test_that("read_wos plaintext record without AU tag returns empty authors list", {
  ## Deliberately omit the AU field entirely
  lines <- c(
    "UT WOS:NOAU",
    "TI  Paper Without Author",
    "PY  2022",
    "SO  Journal",
    "ER"
  )
  f <- wos_pt_file(lines)
  d <- read_wos(f, format = "plaintext")
  expect_equal(nrow(d), 1L)
  expect_equal(length(d$authors[[1]]), 0L)
})

## ---------------------------------------------------------------------------
## Coverage: line 160 — tab get_col fallback when column name absent
## ---------------------------------------------------------------------------

test_that("read_wos tab get_col returns NA default when column absent", {
  ## Use a header that has none of the expected DOI column names
  header_no_doi <- c("UT", "TI", "AU", "PY", "SO", "TC",
                     "AB", "DT", "DE", "ID", "CR")
  f <- wos_tab_file(
    header_no_doi,
    list(c("WOS:NODOI", "No DOI paper", "Smith, J",
           "2023", "Journal", "0", "Abstract.", "Article", "kw", "kp", ""))
  )
  d <- read_wos(f, format = "tab")
  expect_equal(nrow(d), 1L)
  expect_true(is.na(d$doi))
})

#' Aggregate multi-valued fields by an entity
#'
#' Groups documents by a single-valued or list-column entity (e.g., author,
#' journal) and pools all values from another list-column (e.g., references,
#' keywords) across documents belonging to that entity.
#'
#' @param data A data frame with `id` and the specified columns.
#' @param entity_field Character. Name of the entity column. If it is a scalar
#'   column (e.g., `"journal"`), each document belongs to one entity. If it
#'   is a list-column (e.g., `"authors"`), each document may belong to
#'   multiple entities.
#' @param value_field Character. Name of the list-column to aggregate
#'   (e.g., `"references"`).
#' @param min_freq Integer. Minimum number of papers per entity. Default 1.
#'
#' @return A data frame with columns `id` (entity name) and `value_field`
#'   (list-column of pooled values, with duplicates preserved).
#'
#' @keywords internal
aggregate_by_entity <- function(data, entity_field, value_field, min_freq = 1L) {
  stopifnot(
    is.data.frame(data),
    entity_field %in% names(data),
    value_field %in% names(data)
  )

  entities_col <- data[[entity_field]]
  values_col <- data[[value_field]]

  if (is.list(entities_col)) {
    ## Multi-valued entity: expand
    entity_names <- unlist(entities_col, use.names = FALSE)
    doc_idx <- rep(seq_len(nrow(data)), lengths(entities_col))
  } else {
    ## Scalar entity
    entity_names <- as.character(entities_col)
    doc_idx <- seq_len(nrow(data))
  }

  entity_names <- toupper(trimws(as.character(entity_names)))
  keep <- !is.na(entity_names) & nchar(entity_names) > 0
  entity_names <- entity_names[keep]
  doc_idx <- doc_idx[keep]

  ## Count papers per entity, not raw duplicated mentions within a paper.
  entity_docs <- unique(data.frame(entity = entity_names, doc = doc_idx,
                                   stringsAsFactors = FALSE))
  if (min_freq > 1L) {
    freq <- table(entity_docs$entity)
    keep_entities <- names(freq)[freq >= min_freq]
    entity_docs <- entity_docs[entity_docs$entity %in% keep_entities, , drop = FALSE]
  }

  if (nrow(entity_docs) == 0L) {
    result <- data.frame(id = character(0), stringsAsFactors = FALSE)
    result[[value_field]] <- list()
    return(result)
  }

  ## Pool values by entity
  unique_entities <- sort(unique(entity_docs$entity))
  pooled <- lapply(unique_entities, function(e) {
    rows <- entity_docs$doc[entity_docs$entity == e]
    if (is.list(values_col)) {
      unlist(values_col[rows], use.names = FALSE)
    } else {
      values_col[rows]
    }
  })

  result <- data.frame(id = unique_entities, stringsAsFactors = FALSE)
  result[[value_field]] <- pooled
  result
}


#' Parse semicolon-delimited strings into list-column
#'
#' Splits semicolon-separated strings (common in Scopus/WoS exports) into
#' character vectors, trimming whitespace.
#'
#' @param x A character vector of semicolon-delimited strings.
#' @param sep Character. Delimiter. Default `";"`.
#'
#' @return A list of character vectors.
#'
#' @export
#' @examples
#' split_field(c("Alice; Bob; Carol", "Dave; Eve"))
split_field <- function(x, sep = ";") {
  lapply(x, function(s) {
    if (is.na(s) || nchar(trimws(s)) == 0) return(character(0))
    parts <- trimws(strsplit(s, sep, fixed = TRUE)[[1]])
    parts[nchar(parts) > 0]
  })
}


#' Standardize author names
#'
#' Uppercase, whitespace normalisation, and dot removal from initials
#' (`F.J.` → `FJ`). Name order and format are preserved — consistent with
#' how bibliometrix handles multi-source data.
#'
#' @param x Character vector of author names.
#' @param flip_names Logical. If `TRUE`, names in `Last, First` format are
#'   reordered to `First Last`. Off by default — enable only when all names
#'   in `x` reliably follow the `Last, First` convention.
#' @return Character vector, uppercased and cleaned.
#' @keywords internal
standardize_authors <- function(x, flip_names = FALSE) {
  x <- trimws(gsub("\\s+", " ", x))
  x <- x[nchar(x) > 0]
  if (length(x) == 0) return(character(0))
  x <- gsub(".", "", x, fixed = TRUE)
  if (flip_names) {
    comma <- grepl(",", x, fixed = TRUE)
    x[comma] <- vapply(strsplit(x[comma], ",\\s*"), function(parts) {
      paste(rev(trimws(parts)), collapse = " ")
    }, character(1))
  }
  toupper(x)
}


## ── Input validation helpers ────────────────────────────────────────────────

#' @keywords internal
check_data <- function(data, required) {
  if (!is.data.frame(data))
    stop("'data' must be a data frame, not ", class(data)[1], call. = FALSE)
  missing <- setdiff(required, names(data))
  if (length(missing) > 0)
    stop("Required column(s) not found in data: ",
         paste0("'", missing, "'", collapse = ", "), call. = FALSE)
}

#' @keywords internal
check_edges <- function(edges) {
  if (!is.data.frame(edges))
    stop("'edges' must be a data frame, not ", class(edges)[1], call. = FALSE)
  missing <- setdiff(c("from", "to", "weight"), names(edges))
  if (length(missing) > 0)
    stop("Required column(s) not found in edges: ",
         paste0("'", missing, "'", collapse = ", "),
         "\nExpected columns: from, to, weight", call. = FALSE)
}

#' @keywords internal
check_choice <- function(value, choices, name) {
  if (!value %in% choices)
    stop(sprintf("'%s' is not a valid %s. Choose one of: %s",
                 value, name, paste0("'", choices, "'", collapse = ", ")),
         call. = FALSE)
}

#' @keywords internal
check_format <- function(format) {
  valid <- c("edgelist", "gephi", "igraph", "cograph", "matrix")
  check_choice(format, valid, "format")
}

#' @keywords internal
check_file <- function(file) {
  if (!file.exists(file))
    stop("File not found: ", file, call. = FALSE)
}


`%||%` <- function(a, b) if (!is.null(a)) a else b

#' @keywords internal
standardize_refs <- function(x) {
  x <- trimws(x)
  x <- gsub("\\s+", " ", x)
  ## Remove trailing DOI if present (for WoS CR field)
  x <- sub(",\\s*DOI\\s+.*$", "", x, ignore.case = TRUE)
  toupper(x)
}

## Tests for read_crossref() — uses synthetic data frames only; no API calls.
##
## Crossref column notes (from parser source):
##   doi, title, container.title, issued (string "YYYY-MM-DD" or "YYYY"),
##   is.referenced.by.count, abstract, type,
##   author  (list-col of data.frames with columns 'given', 'family'),
##   reference (list-col of data.frames with 'DOI', 'author', 'year',
##              'journal-title', or 'unstructured'),
##   subject  (list-col of character vectors)

## ── helpers ──────────────────────────────────────────────────────────────────

make_author_df <- function(given, family) {
  data.frame(given = given, family = family, stringsAsFactors = FALSE)
}

make_ref_df_doi <- function(dois) {
  data.frame(DOI = dois, stringsAsFactors = FALSE)
}

make_ref_df_unstructured <- function(strings) {
  data.frame(unstructured = strings, stringsAsFactors = FALSE)
}

## Minimal valid synthetic cr_works()$data with 3 records
make_synthetic <- function() {
  data.frame(
    doi   = c("10.1000/test1", "10.1000/test2", "10.1000/test3"),
    title = c("Title One", "Title Two: A Study", "Título Tres"),
    `container.title` = c("Journal A", "Journal B", "Journal C"),
    issued = c("2021-06-01", "2019", "2023-12-31"),
    `is.referenced.by.count` = c(10L, 0L, 5L),
    abstract = c("Abstract one.", NA_character_, "Resumen tres."),
    type  = c("journal-article", "book-chapter", "journal-article"),
    stringsAsFactors = FALSE,
    check.names = FALSE
  )
}

## ── Column presence & order ──────────────────────────────────────────────────

test_that("read_crossref returns all standard columns", {
  d <- make_synthetic()
  out <- read_crossref(d)
  expected <- c("id", "title", "year", "journal", "doi",
                "cited_by_count", "abstract", "type",
                "authors", "references", "keywords")
  expect_true(all(expected %in% names(out)))
})

test_that("standard columns appear in the correct order", {
  d <- make_synthetic()
  out <- read_crossref(d)
  col_positions <- match(
    c("id", "title", "year", "journal", "doi",
      "cited_by_count", "abstract", "type",
      "authors", "references", "keywords"),
    names(out)
  )
  expect_true(all(!is.na(col_positions)))
  expect_equal(col_positions, sort(col_positions))
})

test_that("read_crossref returns the correct number of rows", {
  d <- make_synthetic()
  expect_equal(nrow(read_crossref(d)), 3L)
})

## ── Types ─────────────────────────────────────────────────────────────────────

test_that("year column is integer", {
  d <- make_synthetic()
  out <- read_crossref(d)
  expect_type(out$year, "integer")
})

test_that("cited_by_count is integer", {
  d <- make_synthetic()
  out <- read_crossref(d)
  expect_type(out$cited_by_count, "integer")
})

test_that("authors, references, keywords are list-columns", {
  d <- make_synthetic()
  out <- read_crossref(d)
  expect_true(is.list(out$authors))
  expect_true(is.list(out$references))
  expect_true(is.list(out$keywords))
})

## ── ID / DOI logic ───────────────────────────────────────────────────────────

test_that("id equals doi when doi is present", {
  d <- make_synthetic()
  out <- read_crossref(d)
  expect_equal(out$id, d$doi)
})

test_that("id falls back to CR<n> when doi is NA", {
  d <- make_synthetic()
  d$doi[2] <- NA_character_
  out <- read_crossref(d)
  expect_equal(out$id[2], "CR2")
})

test_that("id falls back to CR<n> when doi is empty string", {
  d <- make_synthetic()
  d$doi[1] <- ""
  out <- read_crossref(d)
  expect_equal(out$id[1], "CR1")
})

## ── Year extraction ──────────────────────────────────────────────────────────

test_that("year is extracted from full date string", {
  d <- make_synthetic()
  out <- read_crossref(d)
  expect_equal(out$year[1], 2021L)
})

test_that("year is extracted from year-only string", {
  d <- make_synthetic()
  out <- read_crossref(d)
  expect_equal(out$year[2], 2019L)
})

test_that("year is NA_integer_ when issued contains no 4-digit sequence", {
  d <- data.frame(
    doi    = "10.1000/noyr",
    title  = "No Year",
    issued = "no-date-here",
    `is.referenced.by.count` = 0L,
    stringsAsFactors = FALSE, check.names = FALSE
  )
  out <- read_crossref(d)
  expect_true(is.na(out$year[1]))
})

test_that("year is NA_integer_ when issued is NA (regression)", {
  ## Regression: vapply over the issued vector previously inherited NA names,
  ## which made data.frame() throw 'row names contain missing values'.
  d <- data.frame(
    doi    = "10.1000/naissue",
    title  = "NA Issued",
    issued = NA_character_,
    `is.referenced.by.count` = 0L,
    stringsAsFactors = FALSE, check.names = FALSE
  )
  out <- read_crossref(d)
  expect_true(is.na(out$year[1]))
})

## ── cited_by_count ───────────────────────────────────────────────────────────

test_that("cited_by_count maps is.referenced.by.count correctly", {
  d <- make_synthetic()
  out <- read_crossref(d)
  expect_equal(out$cited_by_count, c(10L, 0L, 5L))
})

test_that("cited_by_count defaults to 0L when column absent", {
  d <- make_synthetic()
  d[["is.referenced.by.count"]] <- NULL
  out <- read_crossref(d)
  expect_true(all(out$cited_by_count == 0L))
})

## ── Authors ──────────────────────────────────────────────────────────────────

test_that("authors are extracted from nested author data.frame", {
  d <- make_synthetic()
  d$author <- list(
    make_author_df(c("Alice", "Bob"), c("Smith", "Jones")),
    make_author_df("Carlos", "García"),
    make_author_df("", "Solo")
  )
  out <- read_crossref(d)
  ## Parser pastes family + given then calls standardize_authors (toupper, trim)
  expect_true(length(out$authors[[1]]) == 2L)
  expect_equal(out$authors[[1]][1], "SMITH ALICE")
  expect_equal(out$authors[[1]][2], "JONES BOB")
})

test_that("non-ASCII author names are uppercased correctly", {
  d <- make_synthetic()
  d$author <- list(
    make_author_df("Carlos", "García"),
    NULL,
    NULL
  )
  out <- read_crossref(d)
  expect_equal(out$authors[[1]], toupper("GARCÍA CARLOS"))
})

test_that("NULL author cell yields empty character vector", {
  d <- make_synthetic()
  d$author <- list(NULL, NULL, NULL)
  out <- read_crossref(d)
  expect_equal(out$authors[[1]], character(0))
})

test_that("non-data.frame author cell yields empty character vector", {
  d <- make_synthetic()
  d$author <- list("some string", 42L, NULL)
  out <- read_crossref(d)
  expect_equal(out$authors[[1]], character(0))
  expect_equal(out$authors[[2]], character(0))
})

test_that("missing author column yields list of empty character vectors", {
  d <- make_synthetic()
  ## no 'author' column at all
  out <- read_crossref(d)
  expect_true(all(vapply(out$authors, length, integer(1)) == 0L))
})

## ── References ───────────────────────────────────────────────────────────────

test_that("references are extracted from DOI column in nested ref data.frame", {
  d <- make_synthetic()
  d$reference <- list(
    make_ref_df_doi(c("10.1000/ref1", "10.1000/ref2")),
    make_ref_df_doi(character(0)),
    NULL
  )
  out <- read_crossref(d)
  expect_equal(out$references[[1]], c("10.1000/REF1", "10.1000/REF2"))
})

test_that("references fall back to unstructured when DOI column absent", {
  d <- make_synthetic()
  d$reference <- list(
    make_ref_df_unstructured(c("Smith 2020 Journal X", "Jones 2019 Journal Y")),
    NULL,
    NULL
  )
  out <- read_crossref(d)
  expect_equal(
    out$references[[1]],
    c("SMITH 2020 JOURNAL X", "JONES 2019 JOURNAL Y")
  )
})

test_that("NULL reference cell yields empty character vector", {
  d <- make_synthetic()
  d$reference <- list(NULL, NULL, NULL)
  out <- read_crossref(d)
  expect_true(all(vapply(out$references, length, integer(1)) == 0L))
})

test_that("references with mixed DOI/NA fall back for NA rows", {
  d <- make_synthetic()
  ref_df <- data.frame(
    DOI    = c("10.1000/ref1", NA_character_),
    author = c("Smith A", "Jones B"),
    year   = c("2020", "2019"),
    `journal-title` = c("J A", "J B"),
    stringsAsFactors = FALSE,
    check.names = FALSE
  )
  d$reference <- list(ref_df, NULL, NULL)
  out <- read_crossref(d)
  ## First ref has DOI; second falls back to "Jones B 2019 J B"
  expect_equal(out$references[[1]][1], "10.1000/REF1")
  expect_true(grepl("JONES B", out$references[[1]][2]))
})

test_that("missing reference column yields list of empty character vectors", {
  d <- make_synthetic()
  out <- read_crossref(d)
  expect_true(all(vapply(out$references, length, integer(1)) == 0L))
})

test_that("references with no DOI and no unstructured use author+year fallback", {
  ## Covers lines 95-97: ref data.frame has neither 'DOI' nor 'unstructured'
  d <- data.frame(
    doi    = "10.1000/abc",
    title  = "Test",
    issued = "2020",
    `is.referenced.by.count` = 0L,
    stringsAsFactors = FALSE, check.names = FALSE
  )
  ref_no_doi <- data.frame(
    author = c("Smith A", "Jones B"),
    year   = c("2015", "2016"),
    stringsAsFactors = FALSE
  )
  d$reference <- list(ref_no_doi)
  out <- read_crossref(d)
  expect_equal(length(out$references[[1]]), 2L)
  expect_true(grepl("SMITH A", out$references[[1]][1]))
  expect_true(grepl("2015", out$references[[1]][1]))
})

test_that("references with no DOI, no unstructured, no author use year-only fallback", {
  ## Covers the 'author' absent branch inside lines 95-97
  d <- data.frame(
    doi    = "10.1000/abc",
    title  = "Test",
    issued = "2020",
    `is.referenced.by.count` = 0L,
    stringsAsFactors = FALSE, check.names = FALSE
  )
  ref_year_only <- data.frame(
    year = c("2015", "2016"),
    stringsAsFactors = FALSE
  )
  d$reference <- list(ref_year_only)
  out <- read_crossref(d)
  expect_equal(length(out$references[[1]]), 2L)
  expect_true(grepl("2015", out$references[[1]][1]))
})

## ── Keywords (subject) ───────────────────────────────────────────────────────

test_that("keywords are extracted from subject list-column", {
  d <- make_synthetic()
  d$subject <- list(
    c("Bibliometrics", "Networks"),
    c("Education"),
    NULL
  )
  out <- read_crossref(d)
  expect_equal(out$keywords[[1]], c("Bibliometrics", "Networks"))
  expect_equal(out$keywords[[2]], c("Education"))
  expect_equal(out$keywords[[3]], character(0))
})

test_that("NULL subject cell yields empty character vector", {
  d <- make_synthetic()
  d$subject <- list(NULL, NULL, NULL)
  out <- read_crossref(d)
  expect_true(all(vapply(out$keywords, length, integer(1)) == 0L))
})

test_that("missing subject column yields empty keyword lists", {
  d <- make_synthetic()
  out <- read_crossref(d)
  expect_true(all(vapply(out$keywords, length, integer(1)) == 0L))
})

## ── Dot-hyphen column name aliasing ──────────────────────────────────────────

test_that("container-title (hyphen form) is accepted as journal", {
  d <- make_synthetic()
  ## rename container.title to container-title
  names(d)[names(d) == "container.title"] <- "container-title"
  out <- read_crossref(d)
  expect_equal(out$journal[1], "Journal A")
})

test_that("is-referenced-by-count (hyphen form) is accepted", {
  d <- make_synthetic()
  names(d)[names(d) == "is.referenced.by.count"] <- "is-referenced-by-count"
  out <- read_crossref(d)
  expect_equal(out$cited_by_count, c(10L, 0L, 5L))
})

## ── Edge cases ────────────────────────────────────────────────────────────────

test_that("empty input data frame returns zero-row result", {
  d <- make_synthetic()[0, ]
  out <- read_crossref(d)
  expect_equal(nrow(out), 0L)
  expect_true("id" %in% names(out))
})

test_that("non-data.frame input raises an error", {
  expect_error(read_crossref(list(a = 1, b = 2)))
  expect_error(read_crossref("not a data frame"))
})

test_that("all-NA doi column generates CR<n> ids for every row", {
  d <- make_synthetic()
  d$doi <- NA_character_
  out <- read_crossref(d)
  expect_equal(out$id, c("CR1", "CR2", "CR3"))
})

test_that("abstract column passes through NA values", {
  d <- make_synthetic()
  out <- read_crossref(d)
  expect_true(is.na(out$abstract[2]))
})

test_that("type column is preserved as-is", {
  d <- make_synthetic()
  out <- read_crossref(d)
  expect_equal(out$type, c("journal-article", "book-chapter", "journal-article"))
})

## Tests for read_scopus() — synthetic CSV fixtures only (no network calls)

## ---------------------------------------------------------------------------
## Helper: build a proper quoted CSV file from a data frame
## ---------------------------------------------------------------------------

write_scopus_csv <- function(df) {
  f <- tempfile(fileext = ".csv")
  write.csv(df, f, row.names = FALSE)
  f
}

## Full-featured 3-record Scopus data frame
make_full_df <- function() {
  data.frame(
    Authors              = c("Smith J.; Doe A.", "Garcia M.", "Müller K.; Çelik O."),
    `Author full names`  = c("Smith, John; Doe, Alice", "Garcia, Maria", "Müller, Klaus; Çelik, Osman"),
    Title                = c("Network Analysis Study", "Survey Paper", "European Study"),
    Year                 = c(2020L, 2021L, 2022L),
    `Source title`       = c("Journal of Networks", "Review Journal", "European Journal"),
    `Cited by`           = c(15L, 5L, 0L),
    DOI                  = c("10.1000/test001", "10.1000/test002", ""),
    `Author Keywords`    = c("network analysis; machine learning", "survey", ""),
    `Index Keywords`     = c("graph theory; clustering", "review methodology", ""),
    References           = c("Ref A, 2019; Ref B, 2018", "", "Ref C, 2020"),
    Affiliations         = c("Uni A", "Uni B", "Uni C"),
    Abstract             = c("Abstract text here.", "Survey abstract.", "European abstract."),
    `Document Type`      = c("Article", "Review", "Conference Paper"),
    `Language of Original Document` = c("English", "Spanish", "German"),
    EID                  = c("2-s2.0-001", "2-s2.0-002", "2-s2.0-003"),
    check.names          = FALSE,
    stringsAsFactors     = FALSE
  )
}

## ---------------------------------------------------------------------------
## 1. Standard columns present in correct order
## ---------------------------------------------------------------------------

test_that("read_scopus returns all standard columns in correct order", {
  f <- write_scopus_csv(make_full_df())
  d <- read_scopus(f)
  expected_cols <- c("id", "title", "year", "journal", "doi",
                     "cited_by_count", "abstract", "type",
                     "authors", "references", "keywords",
                     "index_keywords", "affiliations", "language")
  expect_true(all(expected_cols %in% names(d)))
  ## The columns must appear in the expected order
  present <- names(d)[names(d) %in% expected_cols]
  expect_equal(present, expected_cols)
})

## ---------------------------------------------------------------------------
## 2. Row count
## ---------------------------------------------------------------------------

test_that("read_scopus returns correct number of rows", {
  f <- write_scopus_csv(make_full_df())
  d <- read_scopus(f)
  expect_equal(nrow(d), 3L)
})

## ---------------------------------------------------------------------------
## 3. Column types
## ---------------------------------------------------------------------------

test_that("read_scopus: character columns have correct types", {
  f <- write_scopus_csv(make_full_df())
  d <- read_scopus(f)
  expect_type(d$id,           "character")
  expect_type(d$title,        "character")
  expect_type(d$journal,      "character")
  expect_type(d$doi,          "character")
  expect_type(d$abstract,     "character")
  expect_type(d$type,         "character")
  expect_type(d$affiliations, "character")
  expect_type(d$language,     "character")
})

test_that("read_scopus: year is integer", {
  f <- write_scopus_csv(make_full_df())
  d <- read_scopus(f)
  expect_type(d$year, "integer")
})

test_that("read_scopus: cited_by_count is integer", {
  f <- write_scopus_csv(make_full_df())
  d <- read_scopus(f)
  expect_type(d$cited_by_count, "integer")
})

test_that("read_scopus: list-columns are lists", {
  f <- write_scopus_csv(make_full_df())
  d <- read_scopus(f)
  expect_true(is.list(d$authors))
  expect_true(is.list(d$references))
  expect_true(is.list(d$keywords))
  expect_true(is.list(d$index_keywords))
})

## ---------------------------------------------------------------------------
## 4. EID used as id when present
## ---------------------------------------------------------------------------

test_that("read_scopus uses EID column as id", {
  f <- write_scopus_csv(make_full_df())
  d <- read_scopus(f)
  expect_equal(d$id, c("2-s2.0-001", "2-s2.0-002", "2-s2.0-003"))
})

## ---------------------------------------------------------------------------
## 5. EID fallback: no EID column → sequential S1, S2, ...
## ---------------------------------------------------------------------------

test_that("read_scopus falls back to sequential id when no EID column", {
  df <- data.frame(
    Title = c("Paper One", "Paper Two"),
    Authors = c("Smith J.", "Doe A."),
    Year = c(2020L, 2021L),
    `Source title` = c("Some Journal", "Other Journal"),
    `Cited by` = c(3L, 0L),
    check.names = FALSE, stringsAsFactors = FALSE
  )
  d <- read_scopus(write_scopus_csv(df))
  expect_equal(d$id, c("S1", "S2"))
})

## ---------------------------------------------------------------------------
## 6. EID fallback: EID column present but empty → sequential id
## ---------------------------------------------------------------------------

test_that("read_scopus falls back to sequential id when EID is empty string", {
  df <- data.frame(
    Title = c("Paper One", "Paper Two"),
    Authors = c("Smith J.", "Doe A."),
    Year = c(2020L, 2021L),
    `Source title` = c("Some Journal", "Other Journal"),
    `Cited by` = c(3L, 0L),
    EID = c("", ""),
    check.names = FALSE, stringsAsFactors = FALSE
  )
  d <- read_scopus(write_scopus_csv(df))
  expect_equal(d$id, c("S1", "S2"))
})

## ---------------------------------------------------------------------------
## 7. Author splitting: semicolon-delimited, uppercased, dots removed
## ---------------------------------------------------------------------------

test_that("read_scopus splits multiple authors into list elements", {
  f <- write_scopus_csv(make_full_df())
  d <- read_scopus(f)
  ## Record 1: "Smith J.; Doe A." → 2 elements
  expect_equal(length(d$authors[[1]]), 2L)
})

test_that("read_scopus single author yields one-element list", {
  f <- write_scopus_csv(make_full_df())
  d <- read_scopus(f)
  ## Record 2: "Garcia M." → 1 element
  expect_equal(length(d$authors[[2]]), 1L)
})

test_that("read_scopus uppercases author names via standardize_authors", {
  f <- write_scopus_csv(make_full_df())
  d <- read_scopus(f)
  ## All author tokens should already be uppercase
  for (i in seq_along(d$authors)) {
    if (length(d$authors[[i]]) > 0)
      expect_equal(d$authors[[i]], toupper(d$authors[[i]]))
  }
})

test_that("read_scopus removes dots from author initials", {
  df <- data.frame(
    Authors = "Jones, F.J.; Brown, A.M.",
    Title = "Dots Test", Year = 2020L,
    `Source title` = "J", `Cited by` = 0L,
    check.names = FALSE, stringsAsFactors = FALSE
  )
  d <- read_scopus(write_scopus_csv(df))
  expect_false(any(grepl("\\.", d$authors[[1]])))
})

## ---------------------------------------------------------------------------
## 8. Reference splitting: semicolons, uppercased via standardize_refs
## ---------------------------------------------------------------------------

test_that("read_scopus splits references on semicolon", {
  f <- write_scopus_csv(make_full_df())
  d <- read_scopus(f)
  ## Record 1: "Ref A, 2019; Ref B, 2018" → 2 references
  expect_equal(length(d$references[[1]]), 2L)
})

test_that("read_scopus returns empty character vector for blank references", {
  f <- write_scopus_csv(make_full_df())
  d <- read_scopus(f)
  ## Record 2 has empty References field
  expect_equal(length(d$references[[2]]), 0L)
  expect_type(d$references[[2]], "character")
})

test_that("read_scopus uppercases references via standardize_refs", {
  f <- write_scopus_csv(make_full_df())
  d <- read_scopus(f)
  expect_equal(d$references[[1]], c("REF A, 2019", "REF B, 2018"))
})

## ---------------------------------------------------------------------------
## 9. Keyword splitting
## ---------------------------------------------------------------------------

test_that("read_scopus splits author keywords on semicolon", {
  f <- write_scopus_csv(make_full_df())
  d <- read_scopus(f)
  ## Record 1: "network analysis; machine learning" → 2
  expect_equal(length(d$keywords[[1]]), 2L)
  expect_equal(d$keywords[[1]], c("network analysis", "machine learning"))
})

test_that("read_scopus returns empty vector for missing author keywords", {
  f <- write_scopus_csv(make_full_df())
  d <- read_scopus(f)
  ## Record 3 has empty Author Keywords
  expect_equal(length(d$keywords[[3]]), 0L)
})

test_that("read_scopus splits index keywords on semicolon", {
  f <- write_scopus_csv(make_full_df())
  d <- read_scopus(f)
  expect_equal(length(d$index_keywords[[1]]), 2L)
  expect_equal(d$index_keywords[[1]], c("graph theory", "clustering"))
})

test_that("read_scopus returns empty vector for missing index keywords", {
  f <- write_scopus_csv(make_full_df())
  d <- read_scopus(f)
  expect_equal(length(d$index_keywords[[3]]), 0L)
})

## ---------------------------------------------------------------------------
## 10. Scalar field values
## ---------------------------------------------------------------------------

test_that("read_scopus parses cited_by_count correctly", {
  f <- write_scopus_csv(make_full_df())
  d <- read_scopus(f)
  expect_equal(d$cited_by_count, c(15L, 5L, 0L))
})

test_that("read_scopus parses year correctly", {
  f <- write_scopus_csv(make_full_df())
  d <- read_scopus(f)
  expect_equal(d$year, c(2020L, 2021L, 2022L))
})

test_that("read_scopus parses DOI correctly", {
  f <- write_scopus_csv(make_full_df())
  d <- read_scopus(f)
  expect_equal(d$doi[1], "10.1000/test001")
  expect_equal(d$doi[2], "10.1000/test002")
})

test_that("read_scopus parses language field", {
  f <- write_scopus_csv(make_full_df())
  d <- read_scopus(f)
  expect_equal(d$language, c("English", "Spanish", "German"))
})

test_that("read_scopus parses document type field", {
  f <- write_scopus_csv(make_full_df())
  d <- read_scopus(f)
  expect_equal(d$type, c("Article", "Review", "Conference Paper"))
})

## ---------------------------------------------------------------------------
## 11. Missing optional columns → NA defaults (no error)
## ---------------------------------------------------------------------------

test_that("read_scopus handles missing optional columns gracefully", {
  df <- data.frame(
    Title = "Minimal Paper", Year = 2023L,
    `Source title` = "Minimal Journal",
    check.names = FALSE, stringsAsFactors = FALSE
  )
  expect_no_error({
    d <- read_scopus(write_scopus_csv(df))
  })
  expect_equal(d$id, "S1")
  expect_true(is.na(d$doi))
  expect_true(is.na(d$abstract))
  expect_true(is.na(d$affiliations))
  expect_true(is.na(d$language))
  expect_equal(length(d$authors[[1]]),        0L)
  expect_equal(length(d$references[[1]]),     0L)
  expect_equal(length(d$keywords[[1]]),       0L)
  expect_equal(length(d$index_keywords[[1]]), 0L)
})

## ---------------------------------------------------------------------------
## 12. cited_by_count defaults to 0 (not NA) when column is absent
## ---------------------------------------------------------------------------

test_that("read_scopus defaults cited_by_count to 0 when column absent", {
  df <- data.frame(
    Title = "No Cited By", Year = 2023L,
    `Source title` = "Some Journal",
    check.names = FALSE, stringsAsFactors = FALSE
  )
  d <- read_scopus(write_scopus_csv(df))
  expect_equal(d$cited_by_count, 0L)
  expect_false(is.na(d$cited_by_count))
})

## ---------------------------------------------------------------------------
## 13. UTF-8 / non-ASCII characters in author names
## ---------------------------------------------------------------------------

test_that("read_scopus handles non-ASCII author names (UTF-8)", {
  f <- write_scopus_csv(make_full_df())
  d <- read_scopus(f)
  ## Record 3: Müller K. and Çelik O. → 2 authors, non-empty after uppercasing
  expect_equal(length(d$authors[[3]]), 2L)
  expect_true(all(nchar(d$authors[[3]]) > 0))
})

## ---------------------------------------------------------------------------
## 14. Case-insensitive alternate column spellings
## ---------------------------------------------------------------------------

test_that("read_scopus accepts 'Document Title' as alternate title column", {
  df <- data.frame(
    `Document Title` = "Alternate Title Paper", Year = 2023L,
    `Source title` = "Alt Journal", `Cited by` = 2L,
    check.names = FALSE, stringsAsFactors = FALSE
  )
  d <- read_scopus(write_scopus_csv(df))
  expect_equal(d$title, "Alternate Title Paper")
})

test_that("read_scopus accepts 'Author full names' as alternate authors column", {
  df <- data.frame(
    Title = "Some Paper",
    `Author full names` = "Jones A.; Brown B.",
    Year = 2022L,
    `Source title` = "Some Journal",
    `Cited by` = 1L,
    check.names = FALSE, stringsAsFactors = FALSE
  )
  d <- read_scopus(write_scopus_csv(df))
  expect_equal(length(d$authors[[1]]), 2L)
})

test_that("read_scopus accepts 'Authors with affiliations' as alternate affiliations column", {
  df <- data.frame(
    Title = "Affil Paper", Authors = "Smith J.", Year = 2021L,
    `Source title` = "Journal X", `Cited by` = 0L,
    `Authors with affiliations` = "Smith J., Uni D",
    check.names = FALSE, stringsAsFactors = FALSE
  )
  d <- read_scopus(write_scopus_csv(df))
  expect_false(is.na(d$affiliations))
  expect_equal(d$affiliations, "Smith J., Uni D")
})

## ---------------------------------------------------------------------------
## 15. File not found → informative error from check_file()
## ---------------------------------------------------------------------------

test_that("read_scopus errors informatively on missing file", {
  expect_error(read_scopus("no_such_file_xyz.csv"),
               regexp = "File not found")
})

## ---------------------------------------------------------------------------
## 16. Single-record file
## ---------------------------------------------------------------------------

test_that("read_scopus handles a single-row CSV correctly", {
  df <- data.frame(
    Authors = "Adams P.", Title = "Single Record Paper", Year = 2019L,
    `Source title` = "Journal One", `Cited by` = 10L,
    DOI = "10.1/single", EID = "2-s2.0-999",
    check.names = FALSE, stringsAsFactors = FALSE
  )
  d <- read_scopus(write_scopus_csv(df))
  expect_equal(nrow(d), 1L)
  expect_equal(d$id, "2-s2.0-999")
  expect_equal(d$year, 2019L)
  expect_equal(d$cited_by_count, 10L)
})

test_that("read_scopus normalizes empty-string DOI to NA (regression)", {
  ## Regression: empty DOI cells were previously stored as "" rather than NA,
  ## breaking is.na(doi) deduplication checks on downstream code.
  df <- data.frame(
    Title  = c("Has DOI", "Empty DOI"),
    Year   = c(2020L, 2021L),
    `Source title` = c("J1", "J2"),
    DOI    = c("10.1/x", ""),
    EID    = c("E1", "E2"),
    check.names = FALSE, stringsAsFactors = FALSE
  )
  d <- read_scopus(write_scopus_csv(df))
  expect_equal(d$doi[1], "10.1/x")
  expect_true(is.na(d$doi[2]))
})

#' Read a BibTeX file
#'
#' Parses a `.bib` file into a standardized bibliometric data frame.
#' Note: standard BibTeX does not contain cited references, so the
#' `references` column will be empty unless the file includes a
#' non-standard `cited-references` or `note` field with reference data.
#'
#' @param file Path to a `.bib` file.
#' @param encoding Character. File encoding. Default `"UTF-8"`.
#'
#' @return A data frame in the standard bibnets format: `id`, `title`,
#'   `year`, `journal`, `doi`, `cited_by_count`, `abstract`, `type`,
#'   plus list-columns `authors`, `references` (typically empty for
#'   BibTeX), and `keywords`.
#'
#' @export
#' @examples
#' # Write a minimal BibTeX entry to a temp file, then read it
#' bib <- '@article{smith2020,
#'   title  = {Bibliometric networks},
#'   author = {Smith, J. and Jones, K.},
#'   journal = {Test Journal},
#'   year   = {2020},
#'   doi    = {10.1000/test}
#' }'
#' f <- tempfile(fileext = ".bib")
#' writeLines(bib, f)
#' data <- read_bibtex(f)
#' data[, c("id", "title", "year", "journal", "doi")]
#' unlink(f)
read_bibtex <- function(file, encoding = "UTF-8") {
  check_file(file)

  lines <- readLines(file, encoding = encoding, warn = FALSE)

  ## Parse BibTeX entries
  entries <- list()
  current <- NULL
  brace_depth <- 0
  field_name <- ""
  field_value <- ""

  for (line in lines) {
    ## Start of new entry
    if (grepl("^@\\w+\\{", line)) {
      if (!is.null(current) && length(current) > 0) {
        entries <- c(entries, list(current))
      }
      ## Extract entry type and key
      m <- regmatches(line, regexec("^@(\\w+)\\{\\s*([^,]*)", line))[[1]]
      current <- list(.type = tolower(m[2]), .key = trimws(m[3]))
      brace_depth <- nchar(gsub("[^{]", "", line)) -
        nchar(gsub("[^}]", "", line))
      ## Check for inline fields
      rest <- sub("^@\\w+\\{[^,]*,?", "", line)
      if (nchar(trimws(rest)) > 0) line <- rest else next
    }

    if (is.null(current)) next

    ## Parse field = {value} or field = "value"
    if (grepl("^\\s*[[:alnum:]_-]+\\s*=", line)) {
      ## New field
      m <- regmatches(line, regexec("^\\s*([[:alnum:]_-]+)\\s*=\\s*(.*)", line))[[1]]
      field_name <- tolower(trimws(m[2]))
      raw_val <- m[3]

      ## Strip braces/quotes
      val <- clean_bibtex_value(raw_val)

      if (grepl("[{}]", raw_val)) {
        open <- nchar(gsub("[^{]", "", raw_val))
        close <- nchar(gsub("[^}]", "", raw_val))
        if (open > close) {
          field_value <- val
          next  # continuation on next lines
        }
      }

      current[[field_name]] <- val
      field_name <- ""

    } else if (nchar(field_name) > 0) {
      ## Continuation of previous field
      val <- clean_bibtex_value(line)
      field_value <- paste(field_value, val)

      open <- nchar(gsub("[^{]", "", line))
      close <- nchar(gsub("[^}]", "", line))
      brace_depth <- brace_depth + open - close

      if (brace_depth <= 1 || grepl("},?\\s*$", line)) {
        current[[field_name]] <- trimws(field_value)
        field_name <- ""
        field_value <- ""
      }
    }
  }

  ## Don't forget last entry

  if (!is.null(current) && length(current) > 0) {
    entries <- c(entries, list(current))
  }

  n <- length(entries)
  if (n == 0) return(empty_biblio_df())

  get_bib <- function(entry, field, default = NA_character_) {
    val <- entry[[field]]
    if (is.null(val) || nchar(trimws(val)) == 0) default else trimws(val)
  }

  result <- data.frame(
    id = vapply(entries, function(e) {
      key <- e[[".key"]]
      if (is.null(key) || nchar(key) == 0) NA_character_ else key
    }, character(1)),
    title = vapply(entries, function(e) get_bib(e, "title"), character(1)),
    year = vapply(entries, function(e) {
      y <- get_bib(e, "year", NA_character_)
      if (is.na(y)) NA_integer_ else as.integer(y)
    }, integer(1)),
    journal = vapply(entries, function(e) {
      get_bib(e, "journal", get_bib(e, "booktitle"))
    }, character(1)),
    doi = vapply(entries, function(e) get_bib(e, "doi"), character(1)),
    cited_by_count = NA_integer_,
    abstract = vapply(entries, function(e) get_bib(e, "abstract"), character(1)),
    type = vapply(entries, function(e) {
      get_bib(e, ".type", NA_character_)
    }, character(1)),
    stringsAsFactors = FALSE
  )

  ## Authors: "LastA, FirstA and LastB, FirstB"
  result$authors <- lapply(entries, function(e) {
    au <- get_bib(e, "author")
    if (is.na(au)) return(character(0))
    parts <- strsplit(au, "\\s+and\\s+", perl = TRUE)[[1]]
    standardize_authors(parts)
  })

  ## References: not standard in BibTeX, but some exports include them in
  ## cited-references/references or, less formally, in note.
  result$references <- lapply(entries, function(e) {
    ref_field <- NA_character_
    for (field in c("cited-references", "citedreferences", "references")) {
      ref_field <- get_bib(e, field)
      if (!is.na(ref_field)) break
    }
    if (is.na(ref_field)) {
      note <- get_bib(e, "note")
      if (!is.na(note) && grepl(";", note, fixed = TRUE)) {
        ref_field <- note
      }
    }
    if (is.na(ref_field)) return(character(0))
    refs <- unlist(strsplit(ref_field, ";"), use.names = FALSE)
    standardize_refs(refs)
  })

  ## Keywords
  result$keywords <- lapply(entries, function(e) {
    kw <- get_bib(e, "keywords")
    if (is.na(kw)) return(character(0))
    parts <- strsplit(kw, "[;,]")[[1]]
    trimws(parts)
  })

  result
}


#' @keywords internal
clean_bibtex_value <- function(x) {
  x <- trimws(x)
  x <- gsub("^[{\"]+|[}\",]+$", "", x)
  x <- gsub("\\{|\\}", "", x)
  trimws(x)
}

#' Print a bibnets network edge list
#'
#' @param x A `bibnets_network` data frame.
#' @param n Integer. Number of rows to show. Default 10.
#' @param ... Ignored.
#' @return Invisibly returns `x`.
#' @examples
#' data(biblio_data)
#' edges <- author_network(biblio_data, "collaboration")
#' print(edges)
#' @export
print.bibnets_network <- function(x, n = 10L, ...) {
  nodes  <- length(unique(c(x$from, x$to)))
  edges  <- nrow(x)
  type   <- attr(x, "network_type")
  method <- attr(x, "counting")
  sim    <- attr(x, "similarity")

  header <- paste0("# bibnets network: ", type %||% "unknown",
                   " | ", format(nodes, big.mark = ","), " nodes",
                   " \u00b7 ", format(edges, big.mark = ","), " edges")
  if (!is.null(method)) header <- paste0(header, " | counting: ", method)
  if (!is.null(sim) && sim != "none") header <- paste0(header, " | similarity: ", sim)
  cat(header, "\n")

  show <- min(n, edges)
  if (show == 0L) return(invisible(x))

  d <- x[seq_len(show), , drop = FALSE]

  trunc_chr <- function(s, w) {
    long <- nchar(s) > w
    s[long] <- paste0(substr(s[long], 1L, w - 1L), "\u2026")
    s
  }

  max_w    <- 30L
  from_fmt <- trunc_chr(d$from, max_w)
  to_fmt   <- trunc_chr(d$to,   max_w)
  wt_fmt   <- formatC(d$weight, format = "fg", digits = 4L)
  ct_fmt   <- as.character(d$count)

  w_idx <- nchar(as.character(show))
  max0  <- function(x, floor) if (length(x)) max(floor, max(nchar(x))) else floor
  w_f   <- max0(from_fmt, 4L)
  w_t   <- max0(to_fmt,   2L)
  w_w   <- max0(wt_fmt,   6L)
  w_c   <- max0(ct_fmt,   5L)

  cat(sprintf("%*s  %-*s  %-*s  %*s  %*s\n",
              w_idx, "", w_f, "from", w_t, "to", w_w, "weight", w_c, "count"))

  for (i in seq_len(show)) {
    cat(sprintf("%*d  %-*s  %-*s  %*s  %*s\n",
                w_idx, i,
                w_f, from_fmt[i],
                w_t, to_fmt[i],
                w_w, wt_fmt[i],
                w_c, ct_fmt[i]))
  }

  if (edges > show)
    cat(sprintf("# ... %s more edges\n", format(edges - show, big.mark = ",")))

  invisible(x)
}


#' Summarise a bibnets network
#'
#' @param object A `bibnets_network` data frame.
#' @param ... Ignored.
#' @return Invisibly returns `object`.
#' @examples
#' data(biblio_data)
#' edges <- author_network(biblio_data, "collaboration")
#' summary(edges)
#' @importFrom utils head
#' @export
summary.bibnets_network <- function(object, ...) {
  nodes   <- unique(c(object$from, object$to))
  n_nodes <- length(nodes)
  n_edges <- nrow(object)
  max_possible <- n_nodes * (n_nodes - 1L) / 2L
  density <- if (max_possible > 0) round(n_edges / max_possible, 4) else NA_real_

  type   <- attr(object, "network_type")
  method <- attr(object, "counting")
  sim    <- attr(object, "similarity")

  cat("bibnets network\n")
  cat(rep("-", 30L), "\n", sep = "")
  if (!is.null(type))   cat(sprintf("Type       : %s\n", type))
  if (!is.null(method)) cat(sprintf("Counting   : %s\n", method))
  if (!is.null(sim))    cat(sprintf("Similarity : %s\n", sim))
  cat(sprintf("Nodes      : %d\n", n_nodes))
  cat(sprintf("Edges      : %d\n", n_edges))
  cat(sprintf("Density    : %.4f\n", density))

  if (n_edges == 0L) {
    cat("Weight     : (no edges)\n")
    invisible(object)
    return(invisible(object))
  }

  cat(sprintf("Weight     : min %.3g  median %.3g  max %.3g\n",
              min(object$weight), stats::median(object$weight),
              max(object$weight)))

  ## Top 5 nodes by degree
  deg <- sort(table(c(object$from, object$to)), decreasing = TRUE)
  top <- head(deg, 5L)
  cat(sprintf("Top nodes  : %s\n",
              paste(sprintf("%s(%d)", names(top), as.integer(top)),
                    collapse = ", ")))
  invisible(object)
}


as_bibnets_network <- function(edges, network_type = NULL,
                                counting = NULL, similarity = NULL,
                                format = "edgelist", directed = FALSE) {
  if (format == "gephi") {
    out <- edges
    names(out)[names(out) == "from"]   <- "Source"
    names(out)[names(out) == "to"]     <- "Target"
    names(out)[names(out) == "weight"] <- "Weight"
    names(out)[names(out) == "count"]  <- "Count"
    out[["Type"]] <- rep(if (directed) "Directed" else "Undirected", nrow(out))
    return(out)
  }

  if (format == "igraph") {
    if (!requireNamespace("igraph", quietly = TRUE)) {
      stop("Package 'igraph' is required for format = 'igraph'. ",
           "Install it with: install.packages('igraph')", call. = FALSE)
    }
    return(igraph::graph_from_data_frame(edges, directed = directed))
  }

  if (format == "cograph") {
    if (!requireNamespace("cograph", quietly = TRUE)) {
      stop("Package 'cograph' is required for format = 'cograph'. ",
           'Install it with: install.packages("cograph", ',
           'repos = "https://mohsaqr.r-universe.dev")', call. = FALSE)
    }
    return(cograph::as_cograph(edges, directed = directed))
  }

  if (format == "matrix") {
    return(edgelist_to_mat(edges, symmetric = !directed))
  }

  class(edges) <- c("bibnets_network", "data.frame")
  attr(edges, "network_type") <- network_type
  attr(edges, "counting")     <- counting
  attr(edges, "similarity")   <- similarity
  edges
}

#' Compute positional author weights for a single paper
#'
#' Given the number of authors and their positions, returns a weight vector
#' for positional counting methods. All methods normalize weights to sum to 1
#' per paper. Position-independent methods (fractional, paper, strength) are
#' handled by [apply_counting()] instead.
#'
#' @param n Integer. Number of authors.
#' @param counting Character. Counting method.
#' @param position_weights Numeric vector. Custom weights for
#'   `counting = "position_weighted"`. Default `c(1, 0.8, 0.6, 0.4)`.
#' @param first_last_weight Numeric. Multiplier for first/last authors
#'   when `counting = "first_last"`. Default 2.
#'
#' @return Numeric vector of length `n`, summing to 1.
#' @keywords internal
author_weights <- function(n, counting = "fractional",
                           position_weights = c(1, 0.8, 0.6, 0.4),
                           first_last_weight = 2) {
  if (n <= 0) return(numeric(0))
  if (n == 1) return(1)

  pos <- seq_len(n)

  w <- switch(counting,

    full = rep(1, n),

    fractional = rep(1 / n, n),

    ## --- Position-dependent ---
    harmonic = {
      ## Harmonic allocation: weight_i = (1/i) / H_n
      raw <- 1 / pos
      raw / sum(raw)
    },

    arithmetic = {
      ## Linear decrease: weight_i = (n - i + 1) / sum(1:n)
      raw <- n - pos + 1
      raw / sum(raw)
    },

    geometric = {
      ## Geometric decay: weight_i = 0.5^(i-1), normalized
      raw <- 0.5^(pos - 1)
      raw / sum(raw)
    },

    adaptive_geometric = {
      ## Adaptive geometric: ratio of first to last weight equals n.
      ## r = n^(1/(n-1)), weight_i = r^(n-i) / sum(r^(n-j))
      if (n == 2) {
        c(2/3, 1/3)
      } else {
        r <- n^(1 / (n - 1))
        raw <- r^(n - pos)
        raw / sum(raw)
      }
    },

    golden = {
      ## Golden ratio decay: phi = (1+sqrt(5))/2
      phi <- (1 + sqrt(5)) / 2
      raw <- phi^(n - pos)
      raw / sum(raw)
    },

    first = {
      ## Straight first-author counting
      w <- rep(0, n)
      w[1] <- 1
      w
    },

    last = {
      ## Last-author counting
      w <- rep(0, n)
      w[n] <- 1
      w
    },

    first_last = {
      ## First and last elevated, middle share rest
      w <- rep(0, n)
      if (n == 2) {
        w <- c(0.5, 0.5)
      } else {
        elevated <- first_last_weight / (2 * first_last_weight + (n - 2))
        middle <- 1 / (2 * first_last_weight + (n - 2))
        w[1] <- elevated
        w[n] <- elevated
        w[2:(n - 1)] <- middle
      }
      w
    },

    position_weighted = {
      ## Custom weights vector, extended if needed
      raw <- vapply(pos, function(i) {
        if (i <= length(position_weights)) position_weights[i]
        else position_weights[length(position_weights)]
      }, numeric(1))
      raw / sum(raw)
    },

    ## --- Attention weights ---
    attention_proximity = {
      ## Pyramid: center entities get most weight, edges get least
      raw <- pmin(pos, n + 1 - pos)
      raw / sum(raw)
    },

    attention_lead = {
      ## Quadratic drop from first: lead author dominates
      raw <- (n + 1 - pos)^2
      raw / sum(raw)
    },

    attention_last = {
      ## Quadratic rise to last: last author (supervisor/PI) dominates
      raw <- pos^2
      raw / sum(raw)
    },

    attention_circular = {
      ## Edges high, center low: first + last are equally prominent
      raw <- pmax(pos, n + 1 - pos)
      raw / sum(raw)
    },

    stop(sprintf("Unknown counting method: '%s'", counting), call. = FALSE)
  )

  w
}


#' Build a weighted bipartite matrix using positional counting
#'
#' For position-dependent counting methods, this replaces the binary bipartite
#' matrix entries with positional weights derived from author order.
#'
#' @param data A data frame with `id` and `authors` (list-column where
#'   author order is preserved).
#' @param counting Character. Counting method.
#' @param position_weights Numeric vector for `counting = "position_weighted"`.
#' @param first_last_weight Numeric for `counting = "first_last"`.
#'
#' @return A sparse weighted bipartite matrix (works x authors).
#' @keywords internal
build_author_bipartite <- function(data, field = "authors",
                                   counting = "full",
                                   position_weights = c(1, 0.8, 0.6, 0.4),
                                   first_last_weight = 2,
                                   deduplicate = TRUE) {
  ids <- as.character(data[["id"]])
  authors_list <- data[[field]]

  ## Expand to long form with positions
  n_per_paper <- lengths(authors_list)
  work_idx <- rep(seq_along(ids), n_per_paper)
  author_names <- unlist(authors_list, use.names = FALSE)
  positions <- unlist(lapply(n_per_paper, seq_len), use.names = FALSE)

  ## Basic normalization: trim whitespace, case-fold to uppercase
  author_names <- toupper(trimws(author_names))

  ## Drop NAs, empty strings, whitespace-only
  keep <- !is.na(author_names) & nchar(author_names) > 0
  work_idx <- work_idx[keep]
  author_names <- author_names[keep]
  positions <- positions[keep]

  ## Deduplicate: keep first occurrence of each (paper, author) pair
  if (deduplicate) {
    dup <- duplicated(paste(work_idx, author_names))
    work_idx     <- work_idx[!dup]
    author_names <- author_names[!dup]
    positions    <- positions[!dup]
  }

  n_per <- n_per_paper[work_idx]

  ## Compute weights
  weights <- mapply(
    function(pos, n) {
      author_weights(n, counting = counting,
                     position_weights = position_weights,
                     first_last_weight = first_last_weight)[pos]
    },
    positions, n_per
  )

  ## Drop zero-weight entries (e.g., "first" or "last" counting)
  nonzero <- weights > 0
  work_idx <- work_idx[nonzero]
  author_names <- author_names[nonzero]
  weights <- weights[nonzero]

  ## Map to column indices
  unique_authors <- sort(unique(author_names))
  col_idx <- match(author_names, unique_authors)

  Matrix::sparseMatrix(
    i = work_idx,
    j = col_idx,
    x = weights,
    dims = c(length(ids), length(unique_authors)),
    dimnames = list(ids, unique_authors)
  )
}


#' Apply counting weights to a generic bipartite matrix
#'
#' For position-independent counting of non-author fields (references,
#' keywords, etc.). Modifies the bipartite matrix row weights.
#'
#' @param B A sparse binary bipartite matrix (works x entities).
#' @param counting Character. One of `"full"`, `"fractional"`, `"paper"`,
#'   `"strength"`.
#' @param network_type Character. `"symmetric"` or `"coupling"`.
#'
#' @return A weighted sparse matrix.
#' @keywords internal
apply_counting <- function(B, counting = "full",
                           network_type = "symmetric") {
  if (counting == "full") return(B)

  dn <- dimnames(B)

  n_per_work <- Matrix::rowSums(B > 0)
  n_per_work[n_per_work == 0] <- 1

  if (counting == "fractional") {
    if (network_type == "symmetric") {
      w <- ifelse(n_per_work > 1, 1 / (n_per_work - 1), 1)
    } else {
      w <- 1 / n_per_work
    }
    B <- Matrix::Diagonal(x = sqrt(w)) %*% B

  } else if (counting == "paper") {
    if (network_type == "symmetric") {
      w <- ifelse(n_per_work > 1, 2 / (n_per_work * (n_per_work - 1)), 1)
    } else {
      w <- 1 / n_per_work
    }
    B <- Matrix::Diagonal(x = sqrt(w)) %*% B

  } else if (counting == "strength") {
    n_works <- nrow(B)
    entity_freq <- Matrix::colSums(B > 0)
    entity_freq[entity_freq == 0] <- 1
    idf <- log(n_works / entity_freq)
    ## IDF weights columns only; row normalization (1/n_i × 1/n_j) is deferred
    ## to multiply_bipartite so the denominator is the product n_i × n_j, not
    ## the geometric mean √(n_i × n_j).
    B <- B %*% Matrix::Diagonal(x = sqrt(idf))
    attr(B, "row_scale") <- n_per_work
  }

  dimnames(B) <- dn
  B
}


#' List of all available counting methods
#'
#' @return Character vector of method names.
#' @keywords internal
position_independent_counts <- function() {
  c("full", "fractional", "paper", "strength")
}

#' @keywords internal
position_dependent_counts <- function() {
  c("harmonic", "arithmetic", "geometric", "adaptive_geometric",
    "golden", "first", "last", "first_last", "position_weighted",
    "attention_proximity", "attention_lead", "attention_last",
    "attention_circular")
}

#' @keywords internal
all_counts <- function() {
  c(position_independent_counts(), position_dependent_counts())
}

#' @keywords internal
all_attention_methods <- function() {
  c("proximity", "lead", "last", "circular")
}

test_that("author_weights returns correct values for all 13 methods", {
  ## 3 authors
  n <- 3

  ## Fractional: equal share
  w <- author_weights(n, "fractional")
  expect_equal(w, c(1/3, 1/3, 1/3), tolerance = tol)
  expect_equal(sum(w), 1, tolerance = tol)

  ## Harmonic: (1/1, 1/2, 1/3) / H_3
  w <- author_weights(n, "harmonic")
  h3 <- 1 + 1/2 + 1/3
  expect_equal(w, c(1/h3, (1/2)/h3, (1/3)/h3), tolerance = tol)
  expect_equal(sum(w), 1, tolerance = tol)

  ## Arithmetic: (3, 2, 1) / 6
  w <- author_weights(n, "arithmetic")
  expect_equal(w, c(3/6, 2/6, 1/6), tolerance = tol)

  ## Geometric: (0.25, 0.5, 1) normalized... wait (0.5^0, 0.5^1, 0.5^2) = (1, 0.5, 0.25)
  w <- author_weights(n, "geometric")
  raw <- c(1, 0.5, 0.25)
  expect_equal(w, raw / sum(raw), tolerance = tol)

  ## Adaptive geometric: ratio first/last = 3
  w <- author_weights(n, "adaptive_geometric")
  expect_equal(w[1] / w[3], 3, tolerance = 0.01)
  expect_equal(sum(w), 1, tolerance = tol)

  ## Golden: phi-based
  w <- author_weights(n, "golden")
  phi <- (1 + sqrt(5)) / 2
  raw <- c(phi^2, phi^1, phi^0)
  expect_equal(w, raw / sum(raw), tolerance = tol)

  ## First author only
  w <- author_weights(n, "first")
  expect_equal(w, c(1, 0, 0))

  ## Last author only
  w <- author_weights(n, "last")
  expect_equal(w, c(0, 0, 1))

  ## First-last elevated
  w <- author_weights(n, "first_last")
  expect_true(w[1] == w[3])  # first == last
  expect_true(w[1] > w[2])   # elevated > middle
  expect_equal(sum(w), 1, tolerance = tol)

  ## Position weighted
  w <- author_weights(n, "position_weighted")
  expect_equal(sum(w), 1, tolerance = tol)

  ## Full: all ones (NOT normalized to 1)
  w <- author_weights(n, "full")
  expect_equal(w, c(1, 1, 1))
})

test_that("author_weights handles single author", {
  for (m in all_counts()) {
    w <- author_weights(1, m)
    expect_equal(w, 1)
  }
})

test_that("all position-dependent methods sum to 1", {
  for (n in 2:10) {
    for (m in position_dependent_counts()) {
      w <- author_weights(n, m)
      expect_equal(sum(w), 1, tolerance = tol,
                   info = sprintf("%s with n=%d", m, n))
    }
  }
})

test_that("harmonic matches crisstats formula", {
  ## From crisstats: weight_i = (1/position) / sum(1/1:n)
  for (n in c(2, 5, 10)) {
    w <- author_weights(n, "harmonic")
    h_n <- sum(1 / seq_len(n))
    expected <- (1 / seq_len(n)) / h_n
    expect_equal(w, expected, tolerance = tol)
  }
})

test_that("geometric matches crisstats formula", {
  ## From crisstats: weight_i = 0.5^(i-1), normalized
  for (n in c(2, 5, 10)) {
    w <- author_weights(n, "geometric")
    raw <- 0.5^(seq_len(n) - 1)
    expected <- raw / sum(raw)
    expect_equal(w, expected, tolerance = tol)
  }
})

test_that("build_author_bipartite produces correct positional weights", {
  d <- make_test_data()

  ## Harmonic weighting
  B <- build_author_bipartite(d, counting = "harmonic")

  ## W1 has Alice (pos 1) and Bob (pos 2), n=2
  ## Harmonic: H_2 = 1 + 0.5 = 1.5
  ## Alice: (1/1)/1.5 = 2/3, Bob: (1/2)/1.5 = 1/3
  expect_equal(B["W1", "ALICE"], 2/3, tolerance = tol)
  expect_equal(B["W1", "BOB"], 1/3, tolerance = tol)

  ## W3 has Bob (pos 1), Carol (pos 2), Dan (pos 3), n=3
  h3 <- 1 + 1/2 + 1/3
  expect_equal(B["W3", "BOB"], 1/h3, tolerance = tol)
  expect_equal(B["W3", "CAROL"], (1/2)/h3, tolerance = tol)
  expect_equal(B["W3", "DAN"], (1/3)/h3, tolerance = tol)
})

#' Build a keyword co-occurrence network
#'
#' Constructs a network where two keywords are linked when they appear
#' together in the same document.
#'
#' @param data A data frame with `id` and a keyword list-column.
#' @param field Character. Name of the keyword list-column. Default
#'   `"keywords"`. Alternatives: `"author_keywords"`, `"index_keywords"`,
#'   `"keywords_plus"`.
#' @param counting Character. Counting method. Default `"full"`.
#' @param similarity Character. Similarity measure. Default `"none"`.
#' @param threshold Numeric. Minimum edge weight. Default 0.
#' @param min_occur Integer. Minimum keyword frequency. Default 1.
#' @param top_n Integer or NULL. Return only the top n edges by weight.
#'   Default NULL (all edges).
#' @inheritParams author_network
#'
#' @return Depends on `format`: a `bibnets_network` data frame (default),
#'   a Gephi-ready data frame, an igraph graph, a cograph_network, or a
#'   sparse matrix.
#'
#' @export
#' @examples
#' data(biblio_data)
#' keyword_network(biblio_data)
#' keyword_network(biblio_data, similarity = "association")
keyword_network <- function(data,
                            field = "keywords",
                            counting = "full",
                            similarity = "none",
                            threshold = 0,
                            min_occur = 1L,
                            attention = NULL,
                            top_n = NULL,
                            self_loops = FALSE,
                            deduplicate = TRUE,
                            format = "edgelist") {
  check_data(data, c("id", field))
  data <- ensure_list_column(data, field)
  check_choice(similarity, c("none", "association", "cosine", "jaccard",
                              "inclusion", "equivalence"), "similarity")
  check_format(format)

  if (!is.null(attention)) {
    check_choice(attention, all_attention_methods(), "attention")
    B <- build_author_bipartite(data, field = field,
                                counting = paste0("attention_", attention),
                                deduplicate = deduplicate)
    result <- multiply_bipartite(B, mode = "columns", similarity = similarity,
                                 threshold = threshold, top_n = top_n,
                                 self_loops = self_loops)
    return(as_bibnets_network(result,
      network_type = paste0("keyword_attention_", attention),
      counting = attention, similarity = similarity, format = format))
  }

  check_choice(counting, position_independent_counts(), "counting")
  B <- build_bipartite(data, field = field, min_freq = min_occur, deduplicate = deduplicate)
  B <- apply_counting(B, counting = counting, network_type = "symmetric")
  result <- multiply_bipartite(B, mode = "columns", similarity = similarity,
                               threshold = threshold, top_n = top_n,
                               self_loops = self_loops)
  as_bibnets_network(result, network_type = "keyword_co_occurrence",
                      counting = counting, similarity = similarity,
                      format = format)
}

test_that("keyword_network returns correct co-occurrence", {
  d <- make_test_data()
  edges <- keyword_network(d, threshold = 0)

  get_w <- function(a, b) {
    row <- edges[
      (edges$from == a & edges$to == b) |
      (edges$from == b & edges$to == a), ]
    if (nrow(row) == 0) return(0)
    row$weight[1]
  }

  expect_equal(get_w("ML", "DL"), 1)
  expect_equal(get_w("ML", "NLP"), 1)
  expect_equal(get_w("DL", "NLP"), 1)
  expect_equal(get_w("ML", "CV"), 0)
})

#' Build a reference network
#'
#' Constructs a co-citation or equivalence network among cited references.
#' Two references are linked when they are cited together by the same paper.
#'
#' @param data A data frame with `id` and `references` (list-column).
#' @param type Character. `"co_citation"` (default) or `"equivalence"`.
#' @param counting Character. Counting method. Default `"full"`.
#' @param similarity Character. Similarity measure. Default `"none"`.
#' @param threshold Numeric. Minimum edge weight. Default 0.
#' @param min_occur Integer. Minimum times a reference must be cited. Default 1.
#' @param top_n Integer or NULL. Return only the top n edges by weight.
#'   Default NULL (all edges).
#' @inheritParams author_network
#'
#' @return Depends on `format`: a `bibnets_network` data frame (default),
#'   a Gephi-ready data frame, an igraph graph, a cograph_network, or a
#'   sparse matrix.
#'
#' @export
#' @examples
#' data(biblio_data)
#' reference_network(biblio_data)
#' reference_network(biblio_data, similarity = "association")
reference_network <- function(data,
                              type = "co_citation",
                              counting = "full",
                              similarity = "none",
                              threshold = 0,
                              min_occur = 1L,
                              top_n = NULL,
                              self_loops = FALSE,
                              deduplicate = TRUE,
                              format = "edgelist") {
  check_data(data, c("id", "references"))
  check_choice(type, c("co_citation", "equivalence"), "type")
  check_choice(counting, position_independent_counts(), "counting")
  check_choice(similarity, c("none", "association", "cosine", "jaccard",
                              "inclusion", "equivalence"), "similarity")
  check_format(format)

  B <- build_bipartite(data, field = "references", min_freq = min_occur, deduplicate = deduplicate)

  result <- if (type == "equivalence") {
    multiply_bipartite(B, mode = "columns", similarity = "cosine",
                       threshold = threshold, top_n = top_n,
                       self_loops = self_loops)
  } else {
    B <- apply_counting(B, counting = counting, network_type = "symmetric")
    multiply_bipartite(B, mode = "columns", similarity = similarity,
                       threshold = threshold, top_n = top_n,
                       self_loops = self_loops)
  }
  as_bibnets_network(result, network_type = paste0("reference_", type),
                      counting = counting, similarity = similarity,
                      format = format)
}

test_that("reference_network co_citation with full counting", {
  d <- make_test_data()
  edges <- reference_network(d, min_occur = 1, threshold = 0)

  get_w <- function(a, b) {
    row <- edges[
      (edges$from == a & edges$to == b) |
      (edges$from == b & edges$to == a), ]
    if (nrow(row) == 0) return(0)
    row$weight[1]
  }

  ## R1-R2: cited together in W1, W2 -> 2
  ## R2-R3: cited together in W1, W3 -> 2
  ## R2-R4: cited together in W2, W3 -> 2
  expect_equal(get_w("R1", "R2"), 2)
  expect_equal(get_w("R2", "R3"), 2)
  expect_equal(get_w("R2", "R4"), 2)
  expect_equal(get_w("R1", "R3"), 1)
})

test_that("reference_network with association strength", {
  d <- make_test_data()
  edges <- reference_network(d, similarity = "association", threshold = 0)

  expect_true(all(edges$weight > 0))
  expect_true(all(is.finite(edges$weight)))
})

test_that("reference_network strength counting is finite in column mode", {
  d <- data.frame(id = paste0("W", 1:4), stringsAsFactors = FALSE)
  d$references <- list(
    c("R1", "R2", "R3", "R4", "R5"),
    c("R1", "R2"),
    c("R1", "R3"),
    c("R2", "R3")
  )

  edges <- reference_network(d, counting = "strength", threshold = 0)

  expect_gt(nrow(edges), 0L)
  expect_false(anyNA(edges$weight))
  expect_true(all(is.finite(edges$weight)))
})

#' Read Scopus CSV export
#'
#' Parses a CSV file exported from Scopus into a standardized bibliometric
#' data frame with list-columns for multi-valued fields.
#'
#' @param file Path to a Scopus CSV export file.
#' @param encoding Character. File encoding. Default `"UTF-8"`.
#'
#' @return A data frame in the standard bibnets format: `id`, `title`,
#'   `year`, `journal`, `doi`, `cited_by_count`, `abstract`, `type`,
#'   plus list-columns `authors`, `references`, and `keywords`.
#'   Scopus-specific extras: `index_keywords` (list-column),
#'   `affiliations` (character), `language` (character).
#'
#' @export
#' @examples
#' f <- system.file("extdata", "scopus_sample.csv", package = "bibnets")
#' data <- read_scopus(f)
#' head(data[, c("id", "title", "year", "journal")])
read_scopus <- function(file, encoding = "UTF-8") {
  check_file(file)

  raw <- utils::read.csv(file, stringsAsFactors = FALSE, fileEncoding = encoding,
                          check.names = FALSE)

  ## Scopus column name mapping (case-insensitive matching)
  col_map <- list(
    title    = c("Title", "Document Title"),
    authors  = c("Authors", "Author full names"),
    year     = c("Year"),
    journal  = c("Source title", "Source Title"),
    doi      = c("DOI"),
    refs     = c("References"),
    keywords = c("Author Keywords"),
    index_kw = c("Index Keywords"),
    cited_by = c("Cited by"),
    abstract = c("Abstract"),
    eid      = c("EID"),
    affiliations = c("Affiliations", "Authors with affiliations"),
    type     = c("Document Type"),
    language = c("Language of Original Document")
  )

  find_col <- function(candidates) {
    for (cand in candidates) {
      idx <- match(tolower(cand), tolower(names(raw)))
      if (!is.na(idx)) return(names(raw)[idx])
    }
    NA_character_
  }

  get_col <- function(candidates, default = NA_character_) {
    col_name <- find_col(candidates)
    if (is.na(col_name)) return(rep(default, nrow(raw)))
    raw[[col_name]]
  }

  ## Build ID
  eid <- get_col(col_map$eid, NA_character_)
  id <- ifelse(is.na(eid) | nchar(eid) == 0,
               paste0("S", seq_len(nrow(raw))),
               eid)

  doi_col <- get_col(col_map$doi)
  doi_col[!is.na(doi_col) & nchar(doi_col) == 0] <- NA_character_

  result <- data.frame(
    id = id,
    title = get_col(col_map$title),
    year = as.integer(get_col(col_map$year, NA_integer_)),
    journal = get_col(col_map$journal),
    doi = doi_col,
    cited_by_count = as.integer(get_col(col_map$cited_by, 0L)),
    abstract = get_col(col_map$abstract),
    type = get_col(col_map$type),
    stringsAsFactors = FALSE
  )

  result$authors <- split_field(get_col(col_map$authors), sep = ";")
  result$authors <- lapply(result$authors, standardize_authors)

  result$references <- split_field(get_col(col_map$refs), sep = ";")
  result$references <- lapply(result$references, standardize_refs)

  result$keywords <- split_field(get_col(col_map$keywords), sep = ";")
  result$keywords <- lapply(result$keywords, trimws)

  ## Source-specific extras
  result$index_keywords <- split_field(get_col(col_map$index_kw), sep = ";")
  result$index_keywords <- lapply(result$index_keywords, trimws)

  result$affiliations <- get_col(col_map$affiliations)
  result$language <- get_col(col_map$language)

  result
}

#' Convert OpenAlex data to bibnets format
#'
#' Takes the output of [openalexR::oa_fetch()] (a tibble/data frame of works)
#' and converts it to the standardized bibnets format with list-columns.
#'
#' @param data A data frame from `oa_fetch(entity = "works", ...)`. Must
#'   contain at least an `id` column. Common columns include `display_name`,
#'   `publication_year`, `so`, `doi`, `cited_by_count`, `referenced_works`,
#'   `ab`, and `author` (nested).
#'
#' @return A data frame in the standard bibnets format: `id`, `title`,
#'   `year`, `journal`, `doi`, `cited_by_count`, `abstract`, `type`,
#'   plus list-columns `authors`, `references`, and `keywords`.
#'
#' @export
#' @examples
#' # Construct a minimal data frame matching the structure returned by
#' # openalexR::oa_fetch(entity = "works", ...). In practice, pass the
#' # result of oa_fetch() directly.
#' raw <- data.frame(
#'   id = c("W123", "W456"),
#'   display_name = c("First paper", "Second paper"),
#'   publication_year = c(2022L, 2021L),
#'   so = c("Journal A", "Journal B"),
#'   doi = c("https://doi.org/10.1/a", "https://doi.org/10.2/b"),
#'   cited_by_count = c(5L, 12L),
#'   stringsAsFactors = FALSE
#' )
#' raw$author <- list(
#'   data.frame(au_display_name = c("Smith J", "Jones A"),
#'              stringsAsFactors = FALSE),
#'   data.frame(au_display_name = "Davis M", stringsAsFactors = FALSE)
#' )
#' raw$referenced_works <- list(c("W100", "W200"), "W123")
#' data <- read_openalex(raw)
#' head(data[, c("id", "title", "year", "journal", "doi")])
read_openalex <- function(data) {
  stopifnot(is.data.frame(data))

  n <- nrow(data)

  ## Helper: safely get column
  safe_col <- function(name, default = NA_character_) {
    if (name %in% names(data)) return(data[[name]])
    if (length(default) == n) default else rep(default, length.out = n)
  }

  ## ID: OpenAlex work ID
  id <- safe_col("id", paste0("OA", seq_len(n)))

  ## Title
  title <- safe_col("display_name")

  ## Year
  year <- as.integer(safe_col("publication_year", NA_integer_))

  ## Journal
  journal <- safe_col("so")

  ## DOI
  doi <- safe_col("doi")
  ## OpenAlex prefixes DOIs with "https://doi.org/"
  doi <- sub("^https://doi\\.org/", "", doi)

  ## Cited by count
  cited_by <- as.integer(safe_col("cited_by_count", 0L))

  ## Abstract
  abstract <- safe_col("ab")

  ## Type
  type <- safe_col("type")

  ## Authors: openalexR stores in nested `author` column
  if ("author" %in% names(data) && is.list(data[["author"]])) {
    authors <- lapply(data[["author"]], function(au) {
      if (is.null(au) || !is.data.frame(au)) return(character(0))
      name_col <- intersect(c("au_display_name", "display_name", "au_name"),
                             names(au))
      if (length(name_col) == 0) return(character(0))
      standardize_authors(au[[name_col[1]]])
    })
  } else {
    authors <- replicate(n, character(0), simplify = FALSE)
  }

  ## References: openalexR stores as `referenced_works` (character vector of IDs)
  if ("referenced_works" %in% names(data)) {
    references <- if (is.list(data[["referenced_works"]])) {
      lapply(data[["referenced_works"]], function(r) {
        if (is.null(r)) character(0) else as.character(r)
      })
    } else {
      split_field(as.character(data[["referenced_works"]]), sep = ",")
    }
  } else {
    references <- replicate(n, character(0), simplify = FALSE)
  }

  ## Keywords: openalexR may have `concepts` or `keywords` column
  keywords <- if ("concepts" %in% names(data) && is.list(data[["concepts"]])) {
    lapply(data[["concepts"]], function(c) {
      if (is.null(c) || !is.data.frame(c)) return(character(0))
      name_col <- intersect(c("display_name", "concept_name"), names(c))
      if (length(name_col) == 0) return(character(0))
      as.character(c[[name_col[1]]])
    })
  } else if ("keywords" %in% names(data) && is.list(data[["keywords"]])) {
    lapply(data[["keywords"]], function(k) {
      if (is.null(k)) return(character(0))
      if (is.data.frame(k)) {
        name_col <- intersect(c("display_name", "keyword"), names(k))
        if (length(name_col) > 0) as.character(k[[name_col[1]]])
        else character(0)
      } else {
        as.character(k)
      }
    })
  } else {
    replicate(n, character(0), simplify = FALSE)
  }

  result <- data.frame(
    id = id,
    title = title,
    year = year,
    journal = journal,
    doi = doi,
    cited_by_count = cited_by,
    abstract = abstract,
    type = type,
    stringsAsFactors = FALSE
  )

  result$authors <- authors
  result$references <- references
  result$keywords <- keywords

  result
}


#' Read a flat OpenAlex CSV export
#'
#' Reads the flat CSV format downloaded directly from the OpenAlex website
#' (`openalex.org/works` exports). Multi-value fields are pipe-delimited (`|`).
#' This is distinct from the nested tibble produced by [openalexR::oa_fetch()],
#' which is handled by [read_openalex()].
#'
#' @param file Path to the CSV file.
#' @param sep Character. Delimiter for multi-value fields. Default `"|"`.
#'
#' @return A data frame in the standard bibnets format: `id`, `title`,
#'   `year`, `journal`, `doi`, `cited_by_count`, `abstract`, `type`,
#'   plus list-columns `authors`, `references`, `keywords`, `affiliations`,
#'   `countries`. `abstract` and `references` are always `NA` / empty
#'   (not available in the flat export).
#'
#' @export
#' @examples
#' f <- system.file("extdata", "openalex_works.csv", package = "bibnets")
#' data <- read_openalex_csv(f)
read_openalex_csv <- function(file, sep = "|") {
  check_file(file)

  raw <- utils::read.csv(file, stringsAsFactors = FALSE, fileEncoding = "UTF-8",
                          check.names = FALSE)

  n <- nrow(raw)

  safe_col <- function(col, default = NA_character_) {
    if (col %in% names(raw)) as.character(raw[[col]])
    else rep(default, n)
  }

  id          <- sub("^https://openalex\\.org/", "", safe_col("id"))
  title       <- safe_col("display_name")
  year        <- suppressWarnings(as.integer(safe_col("publication_year")))
  journal     <- safe_col("primary_location.source.display_name")
  doi         <- sub("^https://doi\\.org/", "", safe_col("doi"))
  doi[doi == ""] <- NA_character_
  cited_by    <- suppressWarnings(as.integer(safe_col("cited_by_count")))
  cited_by[is.na(cited_by)] <- 0L
  abstract    <- rep(NA_character_, n)
  type        <- safe_col("type")
  authors     <- split_field(safe_col("authorships.author.display_name", ""), sep = sep)
  affiliations <- split_field(safe_col("authorships.institutions.display_name", ""), sep = sep)
  countries   <- split_field(safe_col("authorships.countries", ""), sep = sep)
  keywords    <- split_field(safe_col("primary_topic.display_name", ""), sep = sep)
  references  <- vector("list", n)

  result <- data.frame(
    id            = id,
    title         = title,
    year          = year,
    journal       = journal,
    doi           = doi,
    cited_by_count = cited_by,
    abstract      = abstract,
    type          = type,
    stringsAsFactors = FALSE
  )

  result$authors      <- authors
  result$references   <- references
  result$keywords     <- keywords
  result$affiliations <- affiliations
  result$countries    <- countries

  result
}

## Tests for read_openalex() — uses synthetic data frames only; NO network calls.
##
## read_openalex() takes the output of openalexR::oa_fetch(entity="works") —
## a nested tibble/data frame — and returns the standard bibnets schema.
##
## Key openalexR column names (confirmed from parser source):
##   id                (OpenAlex work URL, e.g. "https://openalex.org/W123")
##   display_name      (title)
##   publication_year  (integer or character)
##   so                (source/journal name)
##   doi               (may be prefixed with "https://doi.org/")
##   cited_by_count    (integer)
##   ab                (abstract string)
##   type              (e.g. "article", "book-chapter")
##   author            (list-col of data.frames: au_display_name, au_id, ...)
##   referenced_works  (list-col of character vectors of OpenAlex IDs)
##   concepts          (list-col of data.frames: display_name, ...)
##   keywords          (list-col of data.frames or char vectors: display_name, keyword)

## ── helpers ──────────────────────────────────────────────────────────────────

make_author_df <- function(names) {
  data.frame(au_display_name = names, stringsAsFactors = FALSE)
}

make_concept_df <- function(names) {
  data.frame(display_name = names, stringsAsFactors = FALSE)
}

make_keyword_df <- function(names) {
  data.frame(display_name = names, stringsAsFactors = FALSE)
}

## Minimal valid synthetic oa_fetch() output with 3 records
make_synthetic <- function() {
  data.frame(
    id               = c("https://openalex.org/W1001",
                         "https://openalex.org/W1002",
                         "https://openalex.org/W1003"),
    display_name     = c("Title One", "Title Two", "Title Three"),
    publication_year = c(2021L, 2019L, 2023L),
    so               = c("Journal A", "Journal B", "Journal C"),
    doi              = c("https://doi.org/10.1000/aaa",
                         "https://doi.org/10.1000/bbb",
                         NA_character_),
    cited_by_count   = c(10L, 0L, 5L),
    ab               = c("Abstract one.", NA_character_, "Abstract three."),
    type             = c("article", "book-chapter", "article"),
    stringsAsFactors = FALSE
  )
}

## ── Column presence ───────────────────────────────────────────────────────────

test_that("read_openalex returns all standard columns", {
  d   <- make_synthetic()
  out <- read_openalex(d)
  expected <- c("id", "title", "year", "journal", "doi",
                "cited_by_count", "abstract", "type",
                "authors", "references", "keywords")
  expect_true(all(expected %in% names(out)))
})

test_that("standard columns appear in the correct order", {
  d   <- make_synthetic()
  out <- read_openalex(d)
  positions <- match(
    c("id", "title", "year", "journal", "doi",
      "cited_by_count", "abstract", "type",
      "authors", "references", "keywords"),
    names(out)
  )
  expect_true(all(!is.na(positions)))
  expect_equal(positions, sort(positions))
})

test_that("read_openalex returns the correct number of rows", {
  d <- make_synthetic()
  expect_equal(nrow(read_openalex(d)), 3L)
})

## ── Types ─────────────────────────────────────────────────────────────────────

test_that("year column is integer", {
  out <- read_openalex(make_synthetic())
  expect_type(out$year, "integer")
})

test_that("cited_by_count is integer", {
  out <- read_openalex(make_synthetic())
  expect_type(out$cited_by_count, "integer")
})

test_that("authors, references, keywords are list-columns", {
  d   <- make_synthetic()
  out <- read_openalex(d)
  expect_true(is.list(out$authors))
  expect_true(is.list(out$references))
  expect_true(is.list(out$keywords))
})

## ── ID handling ───────────────────────────────────────────────────────────────

test_that("id column is preserved verbatim from oa_fetch (no URL stripping)", {
  ## The parser passes the `id` column through unchanged; OpenAlex IDs keep
  ## their full URL form (stripping is not done in read_openalex).
  d   <- make_synthetic()
  out <- read_openalex(d)
  expect_equal(out$id, d$id)
})

test_that("id falls back to OA-prefixed sequence when id column is absent", {
  ## Regression: previously, safe_col() passed the length-n default through
  ## rep(default, n), inflating output to n*n rows. Fixed so a length-n
  ## default is used as-is.
  d    <- make_synthetic()
  d$id <- NULL
  out  <- read_openalex(d)
  expect_equal(nrow(out), 3L)
  expect_equal(out$id, paste0("OA", 1:3))
})

## ── DOI handling ──────────────────────────────────────────────────────────────

test_that("DOI URL prefix is stripped", {
  d   <- make_synthetic()
  out <- read_openalex(d)
  expect_equal(out$doi[1], "10.1000/aaa")
  expect_equal(out$doi[2], "10.1000/bbb")
})

test_that("NA doi passes through as NA", {
  d   <- make_synthetic()
  out <- read_openalex(d)
  expect_true(is.na(out$doi[3]))
})

test_that("doi without URL prefix is preserved unchanged", {
  d     <- make_synthetic()
  d$doi <- c("10.1000/plain", NA_character_, "10.5555/xyz")
  out   <- read_openalex(d)
  expect_equal(out$doi[1], "10.1000/plain")
  expect_equal(out$doi[3], "10.5555/xyz")
})

## ── Other scalar columns ──────────────────────────────────────────────────────

test_that("title maps from display_name", {
  d   <- make_synthetic()
  out <- read_openalex(d)
  expect_equal(out$title, d$display_name)
})

test_that("journal maps from so column", {
  d   <- make_synthetic()
  out <- read_openalex(d)
  expect_equal(out$journal, d$so)
})

test_that("year values are correct", {
  d   <- make_synthetic()
  out <- read_openalex(d)
  expect_equal(out$year, c(2021L, 2019L, 2023L))
})

test_that("cited_by_count maps correctly", {
  d   <- make_synthetic()
  out <- read_openalex(d)
  expect_equal(out$cited_by_count, c(10L, 0L, 5L))
})

test_that("abstract passes through correctly including NA", {
  d   <- make_synthetic()
  out <- read_openalex(d)
  expect_equal(out$abstract[1], "Abstract one.")
  expect_true(is.na(out$abstract[2]))
  expect_equal(out$abstract[3], "Abstract three.")
})

test_that("type column is preserved as-is", {
  d   <- make_synthetic()
  out <- read_openalex(d)
  expect_equal(out$type, c("article", "book-chapter", "article"))
})

## ── Missing optional columns ─────────────────────────────────────────────────

test_that("missing display_name yields NA titles", {
  d            <- make_synthetic()
  d$display_name <- NULL
  out          <- read_openalex(d)
  expect_true(all(is.na(out$title)))
})

test_that("missing so column yields NA journals", {
  d    <- make_synthetic()
  d$so <- NULL
  out  <- read_openalex(d)
  expect_true(all(is.na(out$journal)))
})

test_that("missing cited_by_count column defaults to 0L", {
  d                 <- make_synthetic()
  d$cited_by_count  <- NULL
  out               <- read_openalex(d)
  expect_true(all(out$cited_by_count == 0L))
  expect_type(out$cited_by_count, "integer")
})

test_that("missing type column yields NA types", {
  d      <- make_synthetic()
  d$type <- NULL
  out    <- read_openalex(d)
  expect_true(all(is.na(out$type)))
})

test_that("missing ab column yields NA abstracts", {
  d     <- make_synthetic()
  d$ab  <- NULL
  out   <- read_openalex(d)
  expect_true(all(is.na(out$abstract)))
})

## ── Authors ───────────────────────────────────────────────────────────────────

test_that("authors are extracted from nested author data.frame (au_display_name)", {
  d        <- make_synthetic()
  d$author <- list(
    make_author_df(c("Alice Smith", "Bob Jones")),
    make_author_df("Carlos García"),
    make_author_df("Dana Lee")
  )
  out <- read_openalex(d)
  expect_equal(length(out$authors[[1]]), 2L)
  ## standardize_authors uppercases names
  expect_equal(out$authors[[1]], c("ALICE SMITH", "BOB JONES"))
  expect_equal(out$authors[[2]], "CARLOS GARCÍA")
})

test_that("author display_name column also accepted when au_display_name absent", {
  d        <- make_synthetic()
  d$author <- list(
    data.frame(display_name = c("Eve White", "Frank Black"),
               stringsAsFactors = FALSE),
    NULL,
    NULL
  )
  out <- read_openalex(d)
  expect_equal(out$authors[[1]], c("EVE WHITE", "FRANK BLACK"))
})

test_that("au_name column also accepted as fallback for author names", {
  d        <- make_synthetic()
  d$author <- list(
    data.frame(au_name = c("Grace Hopper"),
               stringsAsFactors = FALSE),
    NULL,
    NULL
  )
  out <- read_openalex(d)
  expect_equal(out$authors[[1]], "GRACE HOPPER")
})

test_that("NULL author cell yields empty character vector", {
  d        <- make_synthetic()
  d$author <- list(NULL, NULL, NULL)
  out      <- read_openalex(d)
  expect_true(all(vapply(out$authors, length, integer(1)) == 0L))
})

test_that("non-data.frame author cell yields empty character vector", {
  d        <- make_synthetic()
  d$author <- list("not a data frame", NULL, NULL)
  out      <- read_openalex(d)
  expect_equal(out$authors[[1]], character(0))
})

test_that("author data.frame with no recognized name column yields empty vector", {
  d        <- make_synthetic()
  d$author <- list(
    data.frame(some_other_col = "x", stringsAsFactors = FALSE),
    NULL,
    NULL
  )
  out <- read_openalex(d)
  expect_equal(out$authors[[1]], character(0))
})

test_that("missing author column yields list of empty character vectors", {
  d   <- make_synthetic()
  out <- read_openalex(d)
  expect_true(all(vapply(out$authors, length, integer(1)) == 0L))
})

test_that("non-ASCII author names are uppercased correctly", {
  d        <- make_synthetic()
  d$author <- list(
    make_author_df("José Martínez"),
    make_author_df("Li Wei"),
    NULL
  )
  out <- read_openalex(d)
  expect_equal(out$authors[[1]], toupper("José Martínez"))
  expect_equal(out$authors[[2]], "LI WEI")
})

## ── References ────────────────────────────────────────────────────────────────

test_that("referenced_works list-col is flattened to character vectors", {
  d                  <- make_synthetic()
  d$referenced_works <- list(
    c("https://openalex.org/W999", "https://openalex.org/W888"),
    character(0),
    NULL
  )
  out <- read_openalex(d)
  expect_equal(out$references[[1]],
               c("https://openalex.org/W999", "https://openalex.org/W888"))
  expect_equal(out$references[[2]], character(0))
  expect_equal(out$references[[3]], character(0))
})

test_that("NULL referenced_works cell yields empty character vector", {
  d                  <- make_synthetic()
  d$referenced_works <- list(NULL, NULL, NULL)
  out                <- read_openalex(d)
  expect_true(all(vapply(out$references, length, integer(1)) == 0L))
})

test_that("missing referenced_works column yields empty reference lists", {
  d   <- make_synthetic()
  out <- read_openalex(d)
  expect_true(all(vapply(out$references, length, integer(1)) == 0L))
})

test_that("referenced_works as scalar character (non-list) is split by comma", {
  ## Parser uses split_field(as.character(...), sep=",") for non-list column
  d                  <- make_synthetic()
  d$referenced_works <- c("W1,W2", "W3", NA_character_)
  ## Make it NOT a list so the else branch is taken
  class(d$referenced_works) <- "character"
  out <- read_openalex(d)
  expect_equal(out$references[[1]], c("W1", "W2"))
  expect_equal(out$references[[2]], "W3")
})

## ── Keywords (concepts branch) ────────────────────────────────────────────────

test_that("keywords extracted from concepts column (display_name)", {
  d           <- make_synthetic()
  d$concepts  <- list(
    make_concept_df(c("Bibliometrics", "Network Analysis")),
    make_concept_df("Education"),
    NULL
  )
  out <- read_openalex(d)
  expect_equal(out$keywords[[1]], c("Bibliometrics", "Network Analysis"))
  expect_equal(out$keywords[[2]], "Education")
  expect_equal(out$keywords[[3]], character(0))
})

test_that("concepts column: concept_name accepted as fallback", {
  d          <- make_synthetic()
  d$concepts <- list(
    data.frame(concept_name = c("Topic A", "Topic B"),
               stringsAsFactors = FALSE),
    NULL,
    NULL
  )
  out <- read_openalex(d)
  expect_equal(out$keywords[[1]], c("Topic A", "Topic B"))
})

test_that("concepts column: NULL cell yields empty character vector", {
  d          <- make_synthetic()
  d$concepts <- list(NULL, NULL, NULL)
  out        <- read_openalex(d)
  expect_true(all(vapply(out$keywords, length, integer(1)) == 0L))
})

test_that("concepts column: non-data.frame cell yields empty character vector", {
  d          <- make_synthetic()
  d$concepts <- list("not a df", NULL, NULL)
  out        <- read_openalex(d)
  expect_equal(out$keywords[[1]], character(0))
})

test_that("concepts column: data.frame with no recognized name column yields empty", {
  d          <- make_synthetic()
  d$concepts <- list(
    data.frame(score = 0.9, stringsAsFactors = FALSE),
    NULL,
    NULL
  )
  out <- read_openalex(d)
  expect_equal(out$keywords[[1]], character(0))
})

## ── Keywords (keywords branch, no concepts column) ───────────────────────────

test_that("keywords extracted from keywords data.frame column (display_name)", {
  d           <- make_synthetic()
  d$keywords  <- list(
    make_keyword_df(c("Machine Learning", "Citation Analysis")),
    make_keyword_df("Open Access"),
    NULL
  )
  out <- read_openalex(d)
  expect_equal(out$keywords[[1]], c("Machine Learning", "Citation Analysis"))
  expect_equal(out$keywords[[2]], "Open Access")
  expect_equal(out$keywords[[3]], character(0))
})

test_that("keywords data.frame: keyword column accepted as fallback", {
  d          <- make_synthetic()
  d$keywords <- list(
    data.frame(keyword = c("scientometrics", "altmetrics"),
               stringsAsFactors = FALSE),
    NULL,
    NULL
  )
  out <- read_openalex(d)
  expect_equal(out$keywords[[1]], c("scientometrics", "altmetrics"))
})

test_that("keywords as plain character vector (non-data.frame) is accepted", {
  d          <- make_synthetic()
  d$keywords <- list(
    c("keyword one", "keyword two"),
    NULL,
    NULL
  )
  out <- read_openalex(d)
  expect_equal(out$keywords[[1]], c("keyword one", "keyword two"))
})

test_that("keywords column: NULL cell yields empty character vector", {
  d          <- make_synthetic()
  d$keywords <- list(NULL, NULL, NULL)
  out        <- read_openalex(d)
  expect_true(all(vapply(out$keywords, length, integer(1)) == 0L))
})

test_that("keywords column: data.frame with no recognized name yields empty", {
  d          <- make_synthetic()
  d$keywords <- list(
    data.frame(score = 0.5, stringsAsFactors = FALSE),
    NULL,
    NULL
  )
  out <- read_openalex(d)
  expect_equal(out$keywords[[1]], character(0))
})

test_that("concepts takes precedence over keywords when both columns present", {
  d          <- make_synthetic()
  d$concepts <- list(
    make_concept_df("From Concepts"),
    NULL,
    NULL
  )
  d$keywords <- list(
    make_keyword_df("From Keywords"),
    NULL,
    NULL
  )
  out <- read_openalex(d)
  ## Parser checks concepts first
  expect_equal(out$keywords[[1]], "From Concepts")
})

test_that("missing both concepts and keywords yields empty keyword lists", {
  d   <- make_synthetic()
  out <- read_openalex(d)
  expect_true(all(vapply(out$keywords, length, integer(1)) == 0L))
})

## ── Edge cases ────────────────────────────────────────────────────────────────

test_that("empty input data frame returns zero-row result with correct columns", {
  d   <- make_synthetic()[0, ]
  out <- read_openalex(d)
  expect_equal(nrow(out), 0L)
  expect_true(all(c("id", "title", "year", "journal", "doi",
                    "cited_by_count", "abstract", "type",
                    "authors", "references", "keywords") %in% names(out)))
})

test_that("non-data.frame input raises an error", {
  expect_error(read_openalex(list(a = 1, b = 2)))
  expect_error(read_openalex("not a data frame"))
  expect_error(read_openalex(42L))
})

test_that("single-row input works correctly", {
  d <- data.frame(
    id               = "https://openalex.org/W9999",
    display_name     = "Solo Paper",
    publication_year = 2020L,
    so               = "Solo Journal",
    doi              = "https://doi.org/10.9999/solo",
    cited_by_count   = 3L,
    ab               = "Solo abstract.",
    type             = "article",
    stringsAsFactors = FALSE
  )
  out <- read_openalex(d)
  expect_equal(nrow(out), 1L)
  expect_equal(out$doi, "10.9999/solo")
  expect_equal(out$year, 2020L)
})

test_that("publication_year as character is coerced to integer", {
  d                  <- make_synthetic()
  d$publication_year <- c("2021", "2019", "2023")
  out                <- read_openalex(d)
  expect_type(out$year, "integer")
  expect_equal(out$year, c(2021L, 2019L, 2023L))
})

test_that("all columns may contain NA without error", {
  d <- data.frame(
    id               = NA_character_,
    display_name     = NA_character_,
    publication_year = NA_integer_,
    so               = NA_character_,
    doi              = NA_character_,
    cited_by_count   = NA_integer_,
    ab               = NA_character_,
    type             = NA_character_,
    stringsAsFactors = FALSE
  )
  expect_no_error(read_openalex(d))
  out <- read_openalex(d)
  expect_equal(nrow(out), 1L)
})

test_that("result is a plain data.frame (not tibble or other subclass)", {
  d   <- make_synthetic()
  out <- read_openalex(d)
  expect_true(is.data.frame(out))
})

#' Build time-windowed networks
#'
#' Splits data by time windows and builds a separate network for each
#' window using any network function.
#'
#' @param data A data frame with a numeric time column.
#' @param network_fun Function or character string naming a network function
#'   (e.g., `author_network`, `"reference_network"`, `conetwork`).
#' @param ... Additional arguments passed to `network_fun` (e.g., `type`,
#'   `counting`, `similarity`, `threshold`, `top_n`).
#' @param window Integer. Width of each time window in units of the time
#'   column (years, months, quarters, etc.). Default 3.
#' @param step Integer or `NULL`. Step size between windows. Default `NULL`
#'   (equals `window` for fixed, 1 for sliding).
#' @param strategy Character. Time window strategy:
#'   \describe{
#'     \item{`"fixed"`}{Disjoint non-overlapping windows (default).}
#'     \item{`"sliding"`}{Overlapping windows advancing by `step` units.}
#'     \item{`"cumulative"`}{Each window starts at the earliest value and
#'       extends further.}
#'   }
#' @param time_col Character. Name of the column containing the time variable.
#'   Default `"year"`. Works with any numeric time unit: years, months,
#'   quarters, semesters, weeks, etc. (e.g., `"month"`, `"quarter"`, `"time"`).
#'
#' @return A named list of data frames (edge lists). Names are window
#'   labels like `"2018-2020"`.
#'
#' @export
#' @examples
#' data(biblio_data)
#'
#' # Fixed 3-year windows
#' temporal_network(biblio_data, author_network, "collaboration")
#'
#' # Sliding window
#' temporal_network(biblio_data, author_network, "collaboration",
#'                  window = 2, strategy = "sliding")
#'
#' # Cumulative
#' temporal_network(biblio_data, reference_network,
#'                  threshold = 0, strategy = "cumulative", window = 2)
#'
#' # With string name
#' temporal_network(biblio_data, "keyword_network", window = 3)
temporal_network <- function(data,
                             network_fun,
                             ...,
                             window = 3,
                             step = NULL,
                             strategy = "fixed",
                             time_col = "year") {
  check_data(data, time_col)
  check_choice(strategy, c("fixed", "sliding", "cumulative"), "strategy")
  if (!is.numeric(window) || window < 1)
    stop("'window' must be a number >= 1", call. = FALSE)

  if (is.character(network_fun)) {
    network_fun <- match.fun(network_fun)
  }

  time_vals <- as.integer(data[[time_col]])
  valid_time <- time_vals[!is.na(time_vals)]
  if (length(valid_time) == 0) return(list())

  windows <- build_windows(min(valid_time), max(valid_time), window, step, strategy)

  window_list <- lapply(seq_len(nrow(windows)), function(i) {
    subset_data <- data[!is.na(time_vals) &
                          time_vals >= windows$start[i] &
                          time_vals <= windows$end[i], ]
    if (nrow(subset_data) < 2) return(NULL)
    edges <- tryCatch(network_fun(subset_data, ...), error = function(e) {
      warning("Network construction failed for window '", windows$label[i],
              "': ", conditionMessage(e), call. = FALSE)
      NULL
    })
    if (!is.null(edges) && nrow(edges) > 0) {
      edges$window <- windows$label[i]
      edges
    } else NULL
  })

  Filter(Negate(is.null), stats::setNames(window_list, windows$label))
}


#' Build time window definitions
#' @keywords internal
build_windows <- function(min_time, max_time, window, step, strategy) {
  window <- as.integer(window)

  if (strategy == "fixed") {
    if (is.null(step)) step <- window
    starts <- seq(min_time, max_time, by = step)
    ends <- pmin(starts + window - 1L, max_time)

  } else if (strategy == "sliding") {
    if (is.null(step)) step <- 1L
    starts <- seq(min_time, max_time - window + 1L, by = step)
    ends <- starts + window - 1L

  } else if (strategy == "cumulative") {
    if (is.null(step)) step <- 1L
    ends <- seq(min_time + window - 1L, max_time, by = step)
    starts <- rep(min_time, length(ends))
  }

  labels <- paste0(starts, "-", ends)

  data.frame(start = starts, end = ends, label = labels,
             stringsAsFactors = FALSE)
}

## Tests for temporal_network() and build_windows()

## Data spanning 5 years, with enough papers per year to form networks
make_temporal_data <- function() {
  d <- data.frame(
    id   = paste0("W", 1:10),
    year = c(2018L, 2018L, 2019L, 2019L, 2020L,
              2020L, 2021L, 2021L, 2022L, 2022L),
    stringsAsFactors = FALSE
  )
  d$keywords <- list(
    c("ml", "dl"),     # W1 2018
    c("ml", "nlp"),    # W2 2018
    c("dl", "cv"),     # W3 2019
    c("ml", "cv"),     # W4 2019
    c("nlp", "bert"),  # W5 2020
    c("dl", "bert"),   # W6 2020
    c("bert", "gpt"),  # W7 2021
    c("ml", "gpt"),    # W8 2021
    c("cv", "yolo"),   # W9 2022
    c("dl", "yolo")    # W10 2022
  )
  d$references <- lapply(seq_len(10), function(i) character(0))
  d$authors    <- lapply(paste0("A", 1:10), function(x) x)
  d
}

## ── build_windows ───────────────────────────────────────────────────────────

test_that("build_windows fixed produces disjoint non-overlapping windows", {
  w <- bibnets:::build_windows(2018, 2022, window = 2, step = NULL, strategy = "fixed")
  expect_equal(w$start, c(2018, 2020, 2022))
  expect_equal(w$end,   c(2019, 2021, 2022))
  expect_equal(w$label, c("2018-2019", "2020-2021", "2022-2022"))
})

test_that("build_windows sliding produces overlapping windows", {
  w <- bibnets:::build_windows(2018, 2020, window = 2, step = 1, strategy = "sliding")
  expect_equal(w$start, c(2018, 2019))
  expect_equal(w$end,   c(2019, 2020))
})

test_that("build_windows cumulative grows from first year", {
  w <- bibnets:::build_windows(2018, 2021, window = 2, step = 1, strategy = "cumulative")
  expect_true(all(w$start == 2018))
  expect_equal(w$end, c(2019, 2020, 2021))
})

## ── temporal_network ────────────────────────────────────────────────────────

test_that("temporal_network fixed returns named list", {
  d <- make_temporal_data()
  tn <- temporal_network(d, keyword_network, window = 2, strategy = "fixed",
                          threshold = 0)
  expect_true(is.list(tn))
  expect_true(length(tn) > 0)
  expect_true(all(grepl("^\\d{4}-\\d{4}$", names(tn))))
})

test_that("temporal_network fixed windows have correct year ranges", {
  d <- make_temporal_data()
  tn <- temporal_network(d, keyword_network, window = 2, strategy = "fixed",
                          threshold = 0)
  ## All edges in each window should come from papers in that year range
  for (nm in names(tn)) {
    years_in_window <- as.integer(strsplit(nm, "-")[[1]])
    start_y <- years_in_window[1]
    end_y   <- years_in_window[2]
    ## The 'window' column exists in the edge data
    expect_equal(unique(tn[[nm]]$window), nm)
  }
})

test_that("temporal_network sliding produces more windows than fixed", {
  d <- make_temporal_data()
  tn_fixed  <- temporal_network(d, keyword_network, window = 2,
                                 strategy = "fixed",  threshold = 0)
  tn_slide  <- temporal_network(d, keyword_network, window = 2,
                                 strategy = "sliding", threshold = 0)
  ## Sliding (step=1) always >= fixed (step=window)
  expect_true(length(tn_slide) >= length(tn_fixed))
})

test_that("temporal_network cumulative windows grow", {
  d <- make_temporal_data()
  tn <- temporal_network(d, keyword_network, window = 2, strategy = "cumulative",
                          threshold = 0)
  ## Each successive window starts at 2018; end years grow
  end_years <- vapply(names(tn), function(nm) {
    as.integer(strsplit(nm, "-")[[1]][2])
  }, integer(1L))
  expect_true(all(diff(end_years) > 0))
})

test_that("temporal_network accepts function name as string", {
  d <- make_temporal_data()
  tn <- temporal_network(d, "keyword_network", window = 3, threshold = 0)
  expect_true(is.list(tn))
})

test_that("temporal_network each window contains a data frame", {
  d <- make_temporal_data()
  tn <- temporal_network(d, keyword_network, window = 2, threshold = 0)
  for (nm in names(tn)) {
    expect_true(is.data.frame(tn[[nm]]))
    expect_true(all(c("from", "to", "weight") %in% names(tn[[nm]])))
  }
})

test_that("temporal_network skips windows with fewer than 2 papers", {
  ## Single-paper years cannot form co-occurrence networks
  d <- data.frame(
    id = c("W1", "W2"),
    year = c(2018L, 2020L),   # gap year 2019 has no data
    stringsAsFactors = FALSE
  )
  d$keywords <- list(c("ml", "dl"), c("ml", "cv"))
  tn <- temporal_network(d, keyword_network, window = 1, threshold = 0)
  ## Each year only has 1 paper → no edges → no windows returned
  expect_equal(length(tn), 0)
})

test_that("temporal_network passes ... args to network function", {
  d <- make_temporal_data()
  ## With threshold=100, no edges should survive in any window → empty list
  tn <- temporal_network(d, keyword_network, window = 5, threshold = 100)
  ## All windows produce 0 edges, so they're filtered out → empty list
  expect_equal(length(tn), 0)
})

test_that("temporal_network warns when a window fails", {
  d <- make_temporal_data()
  bad_fun <- function(data, ...) stop("boom", call. = FALSE)

  expect_warning(
    tn <- temporal_network(d, bad_fun, window = 5, threshold = 0),
    "Network construction failed for window"
  )
  expect_equal(length(tn), 0)
})

test_that("temporal_network requires year column", {
  d <- make_temporal_data()
  d$year <- NULL
  expect_error(temporal_network(d, keyword_network))
})

#' Build a bipartite incidence matrix from bibliometric data
#'
#' Constructs a sparse works x entities two-mode matrix from a data frame
#' with a list-column. This is the core engine behind all network construction
#' functions.
#'
#' @param data A data frame with at least columns `id` and the field specified.
#' @param field Character. Name of the list-column containing entities
#'   (e.g., `"authors"`, `"references"`, `"keywords"`).
#' @param min_freq Integer. Minimum number of occurrences for an entity to be
#'   included. Default 1 (no filtering).
#'
#' @return A sparse `dgCMatrix` with rows = works (named by `id`) and
#'   columns = unique entities.
#'
#' @keywords internal
build_bipartite <- function(data, field, min_freq = 1L, deduplicate = TRUE) {
  check_data(data, c("id", field))
  data <- ensure_list_column(data, field)

  ids <- as.character(data[["id"]])
  entities_list <- data[[field]]

  ## Expand to long form: (work_index, entity_name)
  work_idx <- rep(seq_along(ids), lengths(entities_list))
  entity_names <- unlist(entities_list, use.names = FALSE)

  ## Basic normalization: trim whitespace, case-fold to uppercase
  entity_names <- toupper(trimws(entity_names))

  ## Drop NAs, empty strings, whitespace-only
  keep <- !is.na(entity_names) & nchar(entity_names) > 0
  work_idx <- work_idx[keep]
  entity_names <- entity_names[keep]

  ## Frequency filter
  if (min_freq > 1L) {
    freq <- table(entity_names)
    keep_entities <- names(freq)[freq >= min_freq]
    keep <- entity_names %in% keep_entities
    work_idx <- work_idx[keep]
    entity_names <- entity_names[keep]
  }

  ## Optionally deduplicate: each (paper, entity) pair counts at most once
  if (deduplicate) {
    dup <- duplicated(paste(work_idx, entity_names))
    work_idx     <- work_idx[!dup]
    entity_names <- entity_names[!dup]
  }

  ## Map entity names to integer indices
  unique_entities <- sort(unique(entity_names))
  entity_idx <- match(entity_names, unique_entities)

  ## Build sparse matrix
  B <- Matrix::sparseMatrix(
    i = work_idx,
    j = entity_idx,
    x = rep(1, length(work_idx)),
    dims = c(length(ids), length(unique_entities)),
    dimnames = list(ids, unique_entities)
  )

  B
}


#' Build bipartite matrix from a long-format edge table
#'
#' Alternative constructor when data is already in long form (e.g., a two-column
#' data frame of document-reference pairs).
#'
#' @param edges A data frame with columns `source` (work id) and `target`
#'   (entity id).
#' @param min_freq Integer. Minimum entity frequency. Default 1.
#'
#' @return A sparse `dgCMatrix`.
#' @keywords internal
build_bipartite_long <- function(edges, min_freq = 1L) {
  stopifnot(
    is.data.frame(edges),
    all(c("source", "target") %in% names(edges))
  )

  sources <- toupper(trimws(as.character(edges[["source"]])))
  targets <- toupper(trimws(as.character(edges[["target"]])))

  keep <- !is.na(sources) & !is.na(targets) &
    nchar(sources) > 0 & nchar(targets) > 0
  sources <- sources[keep]
  targets <- targets[keep]

  if (min_freq > 1L) {
    freq <- table(targets)
    keep_targets <- names(freq)[freq >= min_freq]
    keep <- targets %in% keep_targets
    sources <- sources[keep]
    targets <- targets[keep]
  }

  unique_sources <- sort(unique(sources))
  unique_targets <- sort(unique(targets))

  i <- match(sources, unique_sources)
  j <- match(targets, unique_targets)

  Matrix::sparseMatrix(
    i = i, j = j,
    x = rep(1, length(i)),
    dims = c(length(unique_sources), length(unique_targets)),
    dimnames = list(unique_sources, unique_targets)
  )
}

test_that("build_bipartite creates correct sparse matrix", {
  d <- make_test_data()
  B <- build_bipartite(d, "references")

  expect_true(inherits(B, "dgCMatrix"))
  expect_equal(nrow(B), 3)  # 3 works
  expect_equal(ncol(B), 4)  # 4 unique refs: R1, R2, R3, R4
  expect_equal(rownames(B), c("W1", "W2", "W3"))
  expect_equal(colnames(B), c("R1", "R2", "R3", "R4"))

  ## W1 cites R1, R2, R3 (not R4)
  expect_equal(as.numeric(B["W1", ]), c(1, 1, 1, 0))
  ## W2 cites R1, R2, R4 (not R3)
  expect_equal(as.numeric(B["W2", ]), c(1, 1, 0, 1))
  ## W3 cites R2, R3, R4 (not R1)
  expect_equal(as.numeric(B["W3", ]), c(0, 1, 1, 1))
})

test_that("build_bipartite handles min_freq filtering", {
  d <- make_test_data()
  B <- build_bipartite(d, "references", min_freq = 2L)

  ## R1 appears in 2 papers, R2 in 3, R3 in 2, R4 in 2 -> all kept at min_freq=2
  expect_equal(ncol(B), 4)

  B3 <- build_bipartite(d, "references", min_freq = 3L)
  ## Only R2 appears in all 3 papers

  expect_equal(ncol(B3), 1)
  expect_equal(colnames(B3), "R2")
})

test_that("build_bipartite works with authors", {
  d <- make_test_data()
  B <- build_bipartite(d, "authors")

  expect_equal(nrow(B), 3)
  expect_equal(ncol(B), 4)  # ALICE, BOB, CAROL, DAN
  expect_equal(sum(B["W1", ]), 2)  # ALICE + BOB
  expect_equal(sum(B["W3", ]), 3)  # BOB + CAROL + DAN
})

test_that("build_bipartite drops NA, empty, whitespace", {
  d <- data.frame(id = c("W1", "W2"), stringsAsFactors = FALSE)
  d$refs <- list(c("R1", "R2"), c(NA_character_, "", "   "))
  B <- build_bipartite(d, "refs")

  expect_equal(ncol(B), 2)  # only R1, R2
  expect_equal(sum(B["W2", ]), 0)  # nothing valid
})

## Extra tests for R/bipartite.R — targets build_bipartite_long() and
## edge-case branches of build_bipartite() to raise coverage from 53.57% → ≥85%.
##
## All tests run under devtools::test() which exposes internal functions.

# ── build_bipartite_long() — basic usage ────────────────────────────────────

test_that("build_bipartite_long returns a dgCMatrix with correct dims", {
  edges <- data.frame(
    source = c("W1", "W1", "W2", "W3"),
    target = c("R1", "R2", "R1", "R3"),
    stringsAsFactors = FALSE
  )
  B <- build_bipartite_long(edges)

  expect_true(is(B, "dgCMatrix"))
  expect_equal(nrow(B), 3L)   # W1, W2, W3
  expect_equal(ncol(B), 3L)   # R1, R2, R3
})

test_that("build_bipartite_long uppercases and trims row/col names", {
  edges <- data.frame(
    source = c("  w1 ", "w2"),
    target = c("  r1 ", "r2"),
    stringsAsFactors = FALSE
  )
  B <- build_bipartite_long(edges)

  expect_equal(rownames(B), c("W1", "W2"))
  expect_equal(colnames(B), c("R1", "R2"))
})

test_that("build_bipartite_long assigns 1s in correct cells", {
  edges <- data.frame(
    source = c("A", "A", "B"),
    target = c("X", "Y", "X"),
    stringsAsFactors = FALSE
  )
  B <- build_bipartite_long(edges)

  # rows sorted: A, B; cols sorted: X, Y
  expect_equal(as.numeric(B["A", ]), c(1, 1))
  expect_equal(as.numeric(B["B", ]), c(1, 0))
})

test_that("build_bipartite_long sorts row and column names lexicographically", {
  edges <- data.frame(
    source = c("W3", "W1", "W2"),
    target = c("RC", "RA", "RB"),
    stringsAsFactors = FALSE
  )
  B <- build_bipartite_long(edges)

  expect_equal(rownames(B), c("W1", "W2", "W3"))
  expect_equal(colnames(B), c("RA", "RB", "RC"))
})

# ── build_bipartite_long() — min_freq filtering ─────────────────────────────

test_that("build_bipartite_long filters targets by min_freq", {
  edges <- data.frame(
    source = c("W1", "W2", "W3", "W4"),
    target = c("R1", "R1", "R2", "R3"),
    stringsAsFactors = FALSE
  )
  # R1 appears 2×, R2 1×, R3 1×
  B2 <- build_bipartite_long(edges, min_freq = 2L)

  expect_equal(ncol(B2), 1L)
  expect_equal(colnames(B2), "R1")
})

test_that("build_bipartite_long min_freq=1 keeps all targets", {
  edges <- data.frame(
    source = c("W1", "W2", "W3"),
    target = c("R1", "R2", "R3"),
    stringsAsFactors = FALSE
  )
  B <- build_bipartite_long(edges, min_freq = 1L)
  expect_equal(ncol(B), 3L)
})

test_that("build_bipartite_long min_freq=3 keeps only targets appearing 3+ times", {
  edges <- data.frame(
    source = c("W1", "W2", "W3", "W4", "W5"),
    target = c("R1", "R1", "R1", "R2", "R2"),
    stringsAsFactors = FALSE
  )
  B3 <- build_bipartite_long(edges, min_freq = 3L)
  expect_equal(ncol(B3), 1L)
  expect_equal(colnames(B3), "R1")
})

# ── build_bipartite_long() — NA / empty handling ────────────────────────────

test_that("build_bipartite_long drops rows where source or target is NA", {
  edges <- data.frame(
    source = c("W1", NA_character_,  "W3"),
    target = c("R1", "R2",           NA_character_),
    stringsAsFactors = FALSE
  )
  B <- build_bipartite_long(edges)

  # Only W1→R1 survives
  expect_equal(nrow(B), 1L)
  expect_equal(ncol(B), 1L)
  expect_equal(rownames(B), "W1")
  expect_equal(colnames(B), "R1")
})

test_that("build_bipartite_long drops rows with empty-string source or target", {
  edges <- data.frame(
    source = c("W1", "",  "W3"),
    target = c("",   "R2", "R3"),
    stringsAsFactors = FALSE
  )
  B <- build_bipartite_long(edges)

  # W1→"" and ""→R2 are both dropped; only W3→R3 survives
  expect_equal(nrow(B), 1L)
  expect_equal(rownames(B), "W3")
  expect_equal(colnames(B), "R3")
})

# ── build_bipartite_long() — error paths ────────────────────────────────────

test_that("build_bipartite_long errors when 'source' column is missing", {
  bad <- data.frame(from = "W1", target = "R1", stringsAsFactors = FALSE)
  expect_error(build_bipartite_long(bad))
})

test_that("build_bipartite_long errors when 'target' column is missing", {
  bad <- data.frame(source = "W1", to = "R1", stringsAsFactors = FALSE)
  expect_error(build_bipartite_long(bad))
})

test_that("build_bipartite_long errors when input is not a data frame", {
  expect_error(build_bipartite_long(list(source = "W1", target = "R1")))
})

# ── build_bipartite_long() — single-row edge ────────────────────────────────

test_that("build_bipartite_long handles a single-row input", {
  edges <- data.frame(source = "W1", target = "R1", stringsAsFactors = FALSE)
  B <- build_bipartite_long(edges)

  expect_equal(dim(B), c(1L, 1L))
  expect_equal(B[1L, 1L], 1)
})

# ── build_bipartite_long() — many-to-one / one-to-many ──────────────────────

test_that("build_bipartite_long handles multiple sources sharing same target", {
  edges <- data.frame(
    source = c("W1", "W2", "W3"),
    target = c("R1", "R1", "R1"),
    stringsAsFactors = FALSE
  )
  B <- build_bipartite_long(edges)

  expect_equal(ncol(B), 1L)
  expect_equal(sum(B), 3)   # each source has one 1
})

test_that("build_bipartite_long handles one source pointing to many targets", {
  edges <- data.frame(
    source = c("W1", "W1", "W1"),
    target = c("R1", "R2", "R3"),
    stringsAsFactors = FALSE
  )
  B <- build_bipartite_long(edges)

  expect_equal(nrow(B), 1L)
  expect_equal(sum(B["W1", ]), 3)
})

# ── build_bipartite() — deduplicate = FALSE ──────────────────────────────────
# (the TRUE branch is already covered 135× by existing tests; FALSE is new)

test_that("build_bipartite deduplicate=FALSE keeps repeated (paper, entity) pairs", {
  d <- data.frame(id = "W1", stringsAsFactors = FALSE)
  d$kw <- list(c("ML", "ML", "DL"))  # ML appears twice

  B_dup   <- build_bipartite(d, "kw", deduplicate = FALSE)
  B_dedup <- build_bipartite(d, "kw", deduplicate = TRUE)

  # With dedup TRUE: ML=1, DL=1 (binary)
  expect_equal(as.numeric(B_dedup["W1", "ML"]), 1)
  # With dedup FALSE: ML=2, DL=1 (counts raw occurrences)
  expect_equal(as.numeric(B_dup["W1", "ML"]), 2)
  expect_equal(as.numeric(B_dup["W1", "DL"]), 1)
})

test_that("build_bipartite deduplicate=FALSE with no repeats matches deduplicate=TRUE", {
  d <- data.frame(id = c("W1", "W2"), stringsAsFactors = FALSE)
  d$kw <- list(c("A", "B"), c("B", "C"))

  B_t <- build_bipartite(d, "kw", deduplicate = TRUE)
  B_f <- build_bipartite(d, "kw", deduplicate = FALSE)

  expect_equal(as.matrix(B_t), as.matrix(B_f))
})

# ── build_bipartite() — ensure_list_column auto-split ───────────────────────

test_that("build_bipartite auto-splits semicolon-delimited character column", {
  d <- data.frame(
    id  = c("W1", "W2"),
    kw  = c("ML; DL", "NLP; CV; ML"),
    stringsAsFactors = FALSE
  )
  # kw is a plain character vector, not a list
  B <- build_bipartite(d, "kw")

  expect_true(is(B, "dgCMatrix"))
  # Uppercased entities: CV, DL, ML, NLP
  expect_true("ML" %in% colnames(B))
  expect_true("DL" %in% colnames(B))
  expect_equal(ncol(B), 4L)
  expect_equal(as.numeric(B["W1", "ML"]), 1)
  expect_equal(as.numeric(B["W2", "NLP"]), 1)
})

# ── build_bipartite() — edge cases ──────────────────────────────────────────

test_that("build_bipartite with min_freq=2 excludes singletons", {
  d <- data.frame(id = c("W1", "W2", "W3"), stringsAsFactors = FALSE)
  d$kw <- list(c("A", "B"), c("A", "C"), c("B", "D"))
  # A: 2, B: 2, C: 1, D: 1 -> keep A and B only

  B <- build_bipartite(d, "kw", min_freq = 2L)
  expect_equal(sort(colnames(B)), c("A", "B"))
})

test_that("build_bipartite with min_freq=3 keeps only universal entities", {
  d <- make_test_data()
  B <- build_bipartite(d, "references", min_freq = 3L)

  expect_equal(colnames(B), "R2")   # R2 is in all 3 papers
  expect_equal(nrow(B), 3L)
})

test_that("build_bipartite entity labels are always uppercased", {
  d <- data.frame(id = "W1", stringsAsFactors = FALSE)
  d$auth <- list(c("alice", "Bob", " CAROL "))

  B <- build_bipartite(d, "auth")
  expect_equal(sort(colnames(B)), c("ALICE", "BOB", "CAROL"))
})

test_that("build_bipartite errors when required column is missing", {
  d <- data.frame(id = "W1", stringsAsFactors = FALSE)
  expect_error(
    build_bipartite(d, "authors"),
    regexp = "authors"
  )
})

test_that("build_bipartite errors when 'id' column is missing", {
  d <- data.frame(authors = I(list(c("Alice"))), stringsAsFactors = FALSE)
  expect_error(
    build_bipartite(d, "authors"),
    regexp = "id"
  )
})

test_that("build_bipartite errors when data is not a data frame", {
  expect_error(
    build_bipartite(list(id = "W1", authors = list("Alice")), "authors"),
    regexp = "data frame"
  )
})

test_that("build_bipartite handles all-empty list-column entries gracefully", {
  d <- data.frame(id = c("W1", "W2"), stringsAsFactors = FALSE)
  d$kw <- list(character(0), character(0))

  B <- build_bipartite(d, "kw")
  expect_equal(ncol(B), 0L)
  expect_equal(nrow(B), 2L)
})

test_that("build_bipartite handles single-record input", {
  d <- data.frame(id = "W1", stringsAsFactors = FALSE)
  d$kw <- list(c("A", "B", "C"))

  B <- build_bipartite(d, "kw")
  expect_equal(nrow(B), 1L)
  expect_equal(ncol(B), 3L)
  expect_equal(sum(B), 3)
})

test_that("build_bipartite handles single entity per record", {
  d <- data.frame(id = c("W1", "W2"), stringsAsFactors = FALSE)
  d$kw <- list("X", "Y")

  B <- build_bipartite(d, "kw")
  expect_equal(nrow(B), 2L)
  expect_equal(ncol(B), 2L)
  expect_equal(sum(diag(as.matrix(B))), 2)
})

#' Normalize a co-occurrence matrix
#'
#' Applies a similarity normalization to a square co-occurrence matrix.
#' The diagonal of the input matrix is used as the total occurrence count
#' for each item. Operates entirely in sparse representation.
#'
#' @param A A square symmetric matrix (dense or sparse) representing
#'   co-occurrence counts.
#' @param method Character. Normalization method:
#'   \describe{
#'     \item{`"none"`}{No normalization. Returns raw co-occurrence counts.}
#'     \item{`"association"`}{Association strength (probabilistic affinity
#'       index). \eqn{s_{ij} = c_{ij} / (w_i \cdot w_j)}.
#'       Often recommended as the best normalization for co-occurrence data.}
#'     \item{`"cosine"`}{Salton's cosine.
#'       \eqn{s_{ij} = c_{ij} / \sqrt{w_i \cdot w_j}}.}
#'     \item{`"jaccard"`}{Jaccard index.
#'       \eqn{s_{ij} = c_{ij} / (w_i + w_j - c_{ij})}.}
#'     \item{`"inclusion"`}{Inclusion index (Simpson coefficient).
#'       \eqn{s_{ij} = c_{ij} / \min(w_i, w_j)}.}
#'     \item{`"equivalence"`}{Equivalence index (Salton's cosine squared).
#'       \eqn{s_{ij} = c_{ij}^2 / (w_i \cdot w_j)}.}
#'   }
#'
#' @return A normalized sparse matrix of the same dimensions.
#'
#' @export
#' @examples
#' # Create a small co-occurrence matrix
#' A <- matrix(c(10, 3, 1, 3, 8, 2, 1, 2, 5), nrow = 3,
#'             dimnames = list(c("a", "b", "c"), c("a", "b", "c")))
#' normalize(A, "association")
#' normalize(A, "cosine")
#' normalize(A, "jaccard")
normalize <- function(A, method = "none") {
  if (!is.matrix(A) && !inherits(A, "Matrix"))
    stop("'A' must be a matrix or Matrix object", call. = FALSE)
  if (nrow(A) != ncol(A))
    stop("'A' must be square (got ", nrow(A), " x ", ncol(A), ")", call. = FALSE)
  check_choice(method, c("none", "association", "cosine", "jaccard",
                          "inclusion", "equivalence"), "method")

  if (method == "none") return(A)

  ## Diagonal = total occurrence count per item
  D <- Matrix::diag(A)
  D[D == 0] <- 1

  ## For small matrices (< 500), use dense path (simpler, fast enough)
  if (nrow(A) < 500) {
    return(normalize_dense(A, method, D))
  }

  ## Sparse path: operate on triplets, never allocate dense outer product
  normalize_sparse(A, method, D)
}


#' @keywords internal
normalize_dense <- function(A, method, D) {
  A <- as.matrix(A)

  S <- switch(method,
    association = {
      A / outer(D, D, `*`)
    },
    cosine = {
      A / outer(D, D, function(a, b) sqrt(a * b))
    },
    jaccard = {
      denom <- outer(D, D, `+`) - A
      denom[denom == 0] <- 1
      A / denom
    },
    inclusion = {
      denom <- outer(D, D, pmin)
      denom[denom == 0] <- 1
      A / denom
    },
    equivalence = {
      A^2 / outer(D, D, `*`)
    }
  )

  S[is.nan(S)] <- 0
  S[is.infinite(S)] <- 0
  diag(S) <- 0
  Matrix::Matrix(S, sparse = TRUE)
}


#' @keywords internal
normalize_sparse <- function(A, method, D) {
  ## Extract non-zero triplets
  triplets <- Matrix::summary(A)
  i <- triplets$i
  j <- triplets$j
  x <- triplets$x

  ## Remove diagonal
  off <- i != j
  i <- i[off]; j <- j[off]; x <- x[off]

  ## Upper triangle only (symmetric)
  upper <- i < j
  i <- i[upper]; j <- j[upper]; x <- x[upper]

  Di <- D[i]
  Dj <- D[j]

  new_x <- switch(method,
    association = x / (Di * Dj),
    cosine = x / sqrt(Di * Dj),
    jaccard = x / (Di + Dj - x),
    inclusion = x / pmin(Di, Dj),
    equivalence = x^2 / (Di * Dj)
  )

  new_x[is.nan(new_x)] <- 0
  new_x[is.infinite(new_x)] <- 0

  ## Build symmetric sparse matrix from upper triangle
  n <- nrow(A)
  nms <- rownames(A)
  S <- Matrix::sparseMatrix(
    i = c(i, j), j = c(j, i), x = c(new_x, new_x),
    dims = c(n, n), dimnames = list(nms, nms)
  )

  S
}

test_that("normalize with 'none' returns input with zeroed diagonal", {
  A <- matrix(c(10, 3, 1, 3, 8, 2, 1, 2, 5), nrow = 3,
              dimnames = list(c("a", "b", "c"), c("a", "b", "c")))
  result <- normalize(A, "none")
  expect_equal(as.matrix(result), A)
})

test_that("association strength: s_ij = c_ij / (w_i * w_j)", {
  A <- matrix(c(10, 3, 1, 3, 8, 2, 1, 2, 5), nrow = 3,
              dimnames = list(c("a", "b", "c"), c("a", "b", "c")))
  S <- as.matrix(normalize(A, "association"))

  ## s_ab = 3 / (10 * 8) = 0.0375
  expect_equal(S["a", "b"], 3 / (10 * 8), tolerance = tol)
  ## s_ac = 1 / (10 * 5) = 0.02
  expect_equal(S["a", "c"], 1 / (10 * 5), tolerance = tol)
  ## s_bc = 2 / (8 * 5) = 0.05
  expect_equal(S["b", "c"], 2 / (8 * 5), tolerance = tol)
  ## Diagonal should be zero
  expect_equal(S["a", "a"], 0)
})

test_that("cosine normalization: s_ij = c_ij / sqrt(w_i * w_j)", {
  A <- matrix(c(10, 3, 1, 3, 8, 2, 1, 2, 5), nrow = 3,
              dimnames = list(c("a", "b", "c"), c("a", "b", "c")))
  S <- as.matrix(normalize(A, "cosine"))

  expect_equal(S["a", "b"], 3 / sqrt(10 * 8), tolerance = tol)
  expect_equal(S["a", "c"], 1 / sqrt(10 * 5), tolerance = tol)
  expect_equal(S["b", "c"], 2 / sqrt(8 * 5), tolerance = tol)
})

test_that("jaccard normalization: s_ij = c_ij / (w_i + w_j - c_ij)", {
  A <- matrix(c(10, 3, 1, 3, 8, 2, 1, 2, 5), nrow = 3,
              dimnames = list(c("a", "b", "c"), c("a", "b", "c")))
  S <- as.matrix(normalize(A, "jaccard"))

  expect_equal(S["a", "b"], 3 / (10 + 8 - 3), tolerance = tol)
  expect_equal(S["a", "c"], 1 / (10 + 5 - 1), tolerance = tol)
  expect_equal(S["b", "c"], 2 / (8 + 5 - 2), tolerance = tol)
})

test_that("inclusion normalization: s_ij = c_ij / min(w_i, w_j)", {
  A <- matrix(c(10, 3, 1, 3, 8, 2, 1, 2, 5), nrow = 3,
              dimnames = list(c("a", "b", "c"), c("a", "b", "c")))
  S <- as.matrix(normalize(A, "inclusion"))

  expect_equal(S["a", "b"], 3 / min(10, 8), tolerance = tol)
  expect_equal(S["a", "c"], 1 / min(10, 5), tolerance = tol)
  expect_equal(S["b", "c"], 2 / min(8, 5), tolerance = tol)
})

test_that("equivalence normalization: s_ij = c_ij^2 / (w_i * w_j)", {
  A <- matrix(c(10, 3, 1, 3, 8, 2, 1, 2, 5), nrow = 3,
              dimnames = list(c("a", "b", "c"), c("a", "b", "c")))
  S <- as.matrix(normalize(A, "equivalence"))

  expect_equal(S["a", "b"], 3^2 / (10 * 8), tolerance = tol)
  expect_equal(S["a", "c"], 1^2 / (10 * 5), tolerance = tol)
  expect_equal(S["b", "c"], 2^2 / (8 * 5), tolerance = tol)
})

test_that("normalize handles zero diagonal gracefully", {
  A <- matrix(c(0, 2, 2, 0), nrow = 2,
              dimnames = list(c("x", "y"), c("x", "y")))
  S <- as.matrix(normalize(A, "association"))
  ## With zero diagonal, all entries should be 0 (no division by zero errors)
  expect_true(all(is.finite(S)))
})

#' Build a source (journal) network
#'
#' Constructs a network between publication sources (journals, book series).
#'
#' @param data A data frame with `id` and `journal` (character column).
#'   For coupling, also needs `references`. For co-citation, needs a
#'   `cited_journals` list-column.
#' @param type Character. `"coupling"` (default), `"co_citation"`, or
#'   `"equivalence"`.
#' @param counting Character. Counting method. Default `"full"`.
#' @param similarity Character. Similarity measure. Default `"none"`.
#' @param threshold Numeric. Minimum edge weight. Default 0.
#' @param min_occur Integer. Minimum papers per source. Default 1.
#' @param top_n Integer or NULL. Return only the top n edges by weight.
#'   Default NULL (all edges).
#' @inheritParams author_network
#'
#' @return Depends on `format`: a `bibnets_network` data frame (default),
#'   a Gephi-ready data frame, an igraph graph, a cograph_network, or a
#'   sparse matrix.
#'
#' @export
#' @examples
#' data(biblio_data)
#' source_network(biblio_data, "coupling")
source_network <- function(data,
                           type = "coupling",
                           counting = "full",
                           similarity = "none",
                           threshold = 0,
                           min_occur = 1L,
                           top_n = NULL,
                           self_loops = FALSE,
                           deduplicate = TRUE,
                           format = "edgelist") {
  check_data(data, "id")
  check_choice(type, c("coupling", "co_citation", "equivalence"), "type")
  check_choice(counting, position_independent_counts(), "counting")
  check_choice(similarity, c("none", "association", "cosine", "jaccard",
                              "inclusion", "equivalence"), "similarity")
  check_format(format)

  result <- if (type == "coupling") {
    check_data(data, c("journal", "references"))
    agg <- aggregate_by_entity(data, entity_field = "journal",
                                value_field = "references",
                                min_freq = min_occur)
    B <- build_bipartite(agg, field = "references")
    B <- apply_counting(B, counting = counting, network_type = "coupling")
    multiply_bipartite(B, mode = "rows", similarity = similarity,
                       threshold = threshold, top_n = top_n,
                       self_loops = self_loops)

  } else if (type == "co_citation") {
    field <- if ("cited_journals" %in% names(data)) {
      "cited_journals"
    } else {
      stop("Column 'cited_journals' not found. ",
           "Parse reference strings to extract cited journals first.",
           call. = FALSE)
    }
    B <- build_bipartite(data, field = field, min_freq = min_occur, deduplicate = deduplicate)
    B <- apply_counting(B, counting = counting, network_type = "symmetric")
    multiply_bipartite(B, mode = "columns", similarity = similarity,
                       threshold = threshold, top_n = top_n,
                       self_loops = self_loops)

  } else {
    check_data(data, c("journal", "references"))
    agg <- aggregate_by_entity(data, entity_field = "journal",
                                value_field = "references",
                                min_freq = min_occur)
    B <- build_bipartite(agg, field = "references")
    multiply_bipartite(B, mode = "rows", similarity = "cosine",
                       threshold = threshold, top_n = top_n,
                       self_loops = self_loops)
  }

  as_bibnets_network(result, network_type = paste0("source_", type),
                      counting = counting, similarity = similarity,
                      format = format)
}

#' Example bibliometric dataset
#'
#' A small synthetic dataset of 10 scholarly papers with overlapping authors,
#' references, and keywords. Designed for testing and demonstrating all
#' network construction functions in bibnets.
#'
#' @format A data frame with 10 rows and 9 columns:
#' \describe{
#'   \item{id}{Unique document identifier (W1--W10).}
#'   \item{title}{Document title.}
#'   \item{year}{Publication year (2018--2022).}
#'   \item{journal}{Source journal (Scientometrics, Journal of Informetrics,
#'     JASIST, Quantitative Science Studies).}
#'   \item{doi}{DOI string.}
#'   \item{cited_by_count}{Times cited.}
#'   \item{authors}{List-column of author name strings (6 unique authors).}
#'   \item{references}{List-column of cited reference IDs (10 unique refs,
#'     R1--R10). Each paper cites exactly 4 references.}
#'   \item{keywords}{List-column of keyword strings (24 unique keywords).
#'     Each paper has 3 keywords.}
#' }
#'
#' @examples
#' data(biblio_data)
#' reference_network(biblio_data)
#' document_network(biblio_data, "coupling")
#' author_network(biblio_data, "collaboration")
"biblio_data"


#' OpenAlex Gold Open Access Learning Analytics dataset
#'
#' A corpus of 1,508 gold open-access scholarly works on learning analytics,
#' retrieved from OpenAlex (CC0 licence). All records have a verified title,
#' publication year, and at least one author. Journal names are present for
#' works published in a named source; preprints and book chapters may have
#' `NA` in `journal`.
#'
#' @format A data frame with 1,508 rows and 11 columns:
#' \describe{
#'   \item{id}{OpenAlex work ID (e.g. \code{"W2769342982"}).}
#'   \item{title}{Work title.}
#'   \item{year}{Publication year (integer).}
#'   \item{journal}{Source name, or \code{NA} if not available.}
#'   \item{doi}{DOI string without the \code{https://doi.org/} prefix,
#'     or \code{NA}.}
#'   \item{cited_by_count}{Number of citing works as recorded in OpenAlex.}
#'   \item{type}{Work type (\code{"article"}, \code{"review"},
#'     \code{"preprint"}, \code{"book-chapter"}, etc.).}
#'   \item{authors}{List-column of author display names (pipe-split from
#'     the OpenAlex flat export; one name per authorship slot).}
#'   \item{keywords}{List-column with one element: the primary OpenAlex
#'     topic for the work (e.g. \code{"Online Learning and Analytics"}).}
#'   \item{affiliations}{List-column of institution display names (one
#'     entry per authorship–institution pair).}
#'   \item{countries}{List-column of two-letter ISO country codes (one
#'     entry per authorship–institution pair).}
#' }
#'
#' @source OpenAlex \url{https://openalex.org}, CC0 licence.
#'
#' @examples
#' data(open_alex_gold_open_access_learning_analytics)
#' d <- open_alex_gold_open_access_learning_analytics
#' author_network(d, "collaboration")
#' country_network(d, "collaboration")
"open_alex_gold_open_access_learning_analytics"


#' Scopus dataset — Green Cloud Computing and Quantization (2020–2025)
#'
#' First 500 records from a Scopus bibliometric export on the intersection of
#' green cloud computing and quantization, covering 2020–2025. Includes full
#' references, author keywords, index keywords, and affiliations.
#'
#' @format A data frame with 499 rows and 12 columns:
#' \describe{
#'   \item{id}{Scopus EID.}
#'   \item{title}{Work title.}
#'   \item{year}{Publication year (integer).}
#'   \item{journal}{Source title.}
#'   \item{doi}{DOI string without the \code{https://doi.org/} prefix.}
#'   \item{cited_by_count}{Times cited in Scopus.}
#'   \item{abstract}{Abstract text.}
#'   \item{type}{Document type (\code{"Article"}, \code{"Review"}, etc.).}
#'   \item{authors}{List-column of author name strings.}
#'   \item{references}{List-column of cited reference strings.}
#'   \item{keywords}{List-column of author keywords.}
#'   \item{affiliations}{List-column of affiliation strings.}
#' }
#'
#' @source Scopus bibliometric export. Dataset archived at
#'   \doi{10.5281/zenodo.17142636} (CC BY 4.0).
#'
#' @examples
#' data(scopus_quantum_cloud)
#' author_network(scopus_quantum_cloud, "collaboration")
#' keyword_network(scopus_quantum_cloud)
#' document_network(scopus_quantum_cloud, "coupling", similarity = "cosine")
"scopus_quantum_cloud"

#' Extract network backbone using the disparity filter
#'
#' Applies the disparity filter to a weighted edge list. For each edge, it
#' computes an alpha (p-value) from both endpoints and keeps the edge if it
#' is statistically significant from at least one endpoint.
#'
#' The null model asks: given that node \eqn{i} has total strength \eqn{s_i}
#' distributed uniformly across \eqn{k_i} edges, what is the probability that
#' a single edge weight is as large as \eqn{w_{ij}}? The answer is
#' \deqn{\alpha_{ij} = \left(1 - \frac{w_{ij}}{s_i}\right)^{k_i - 1}}
#'
#' An edge is retained if \eqn{\min(\alpha_{ij}, \alpha_{ji}) < \alpha}.
#' Nodes with only one edge always have \eqn{\alpha = 0} and are always kept.
#'
#' @param edges A data frame with at least columns `from`, `to`, and `weight`.
#'   Must be an undirected edge list (each pair appears once).
#' @param alpha Numeric. Significance threshold in (0, 1). Default `0.05`.
#'
#' @return The filtered edge data frame with an added `alpha` column (the
#'   minimum alpha from the two endpoints).
#'
#' @export
#' @examples
#' edges <- data.frame(
#'   from   = c("A", "A", "A", "B", "C"),
#'   to     = c("B", "C", "D", "C", "D"),
#'   weight = c(10,   1,   1,   8,   1)
#' )
#' backbone(edges, alpha = 0.05)
backbone <- function(edges, alpha = 0.05) {
  check_edges(edges)
  if (!is.numeric(alpha) || length(alpha) != 1L || alpha <= 0 || alpha >= 1)
    stop("'alpha' must be a single number between 0 and 1 (exclusive), got: ",
         alpha, call. = FALSE)

  if (nrow(edges) == 0L) {
    edges$alpha <- numeric(0)
    return(edges)
  }

  ## Strength and degree per node in one O(m) pass: double the edge list
  ## so every node appears once as "from", then tapply aggregates.
  node_both   <- c(edges$from, edges$to)
  weight_both <- c(edges$weight, edges$weight)
  strength <- tapply(weight_both, node_both, sum)
  degree   <- tapply(rep(1L, length(node_both)), node_both, sum)

  ## Alpha from node i for edge (i,j): (1 - w/s_i)^(k_i - 1)
  ## Nodes with degree 1 always get alpha = 0 (always kept)
  compute_alpha <- function(node, w) {
    s <- strength[node]
    k <- degree[node]
    if (k <= 1L || s == 0) return(0)
    (1 - w / s) ^ (k - 1L)
  }

  edge_alpha <- pmin(
    mapply(compute_alpha, edges$from, edges$weight),
    mapply(compute_alpha, edges$to,   edges$weight)
  )

  edges$alpha <- edge_alpha
  result <- edges[edge_alpha < alpha, ]
  result <- result[order(-result$weight), ]
  rownames(result) <- NULL
  result
}

## Tests for backbone() — Serrano disparity filter

## Hub graph: A is a hub with 3 weak edges and 1 strong one
## A-B(10), A-C(1), A-D(1), B-C(8), C-D(1)
make_backbone_edges <- function() {
  data.frame(
    from   = c("A", "A", "A", "B", "C"),
    to     = c("B", "C", "D", "C", "D"),
    weight = c(10,   1,   1,   8,   1),
    stringsAsFactors = FALSE
  )
}

test_that("backbone returns correct edges for hub graph at alpha 0.05", {
  edges <- make_backbone_edges()
  b <- backbone(edges, alpha = 0.05)

  ## Expected: A-B (alpha from A ≈ 0.028 < 0.05) and B-C (alpha from C = 0.04 < 0.05)
  ## All other edges have min-alpha above 0.05
  expect_equal(nrow(b), 2)
  expect_true(all(c("from", "to", "weight", "alpha") %in% names(b)))

  ## A-B must be present
  ab <- b[(b$from == "A" & b$to == "B") | (b$from == "B" & b$to == "A"), ]
  expect_equal(nrow(ab), 1)
  expect_equal(ab$weight, 10)

  ## B-C must be present
  bc <- b[(b$from == "B" & b$to == "C") | (b$from == "C" & b$to == "B"), ]
  expect_equal(nrow(bc), 1)
  expect_equal(bc$weight, 8)
})

test_that("backbone alpha values are correct", {
  edges <- make_backbone_edges()
  b <- backbone(edges, alpha = 0.9999)   # keep everything

  expect_equal(nrow(b), nrow(edges))

  ## For A-B edge: from A: s=12, k=3, alpha=(1-10/12)^2 = (1/6)^2 ≈ 0.0278
  ##               from B: s=18, k=2, alpha=(1-10/18)^1 = 8/18 ≈ 0.444
  ##               min = 0.0278
  ab <- b[(b$from == "A" & b$to == "B") | (b$from == "B" & b$to == "A"), ]
  expect_equal(ab$alpha, (1 - 10/12)^2, tolerance = 1e-10)
})

test_that("backbone returns empty data frame when no edges pass filter", {
  ## All weak, tiny graph — nothing passes strict alpha
  edges <- data.frame(
    from   = c("A", "B"),
    to     = c("B", "C"),
    weight = c(1,   1),
    stringsAsFactors = FALSE
  )
  ## Nodes with degree 1 always get alpha=0, so both edges are kept
  b <- backbone(edges, alpha = 0.01)
  expect_equal(nrow(b), 2)
})

test_that("backbone always keeps edges from degree-1 nodes", {
  ## A is a leaf (degree 1): its alpha = 0, so A-B is always kept
  edges <- data.frame(
    from   = c("A", "B", "B"),
    to     = c("B", "C", "D"),
    weight = c(1,   100,  100),
    stringsAsFactors = FALSE
  )
  b <- backbone(edges, alpha = 0.001)
  ## A-B should be kept (A has degree 1 → alpha=0 from A's side)
  ab <- b[(b$from == "A" & b$to == "B") | (b$from == "B" & b$to == "A"), ]
  expect_equal(nrow(ab), 1)
})

test_that("backbone returns input columns unchanged plus alpha", {
  edges <- make_backbone_edges()
  edges$count <- 1L
  b <- backbone(edges, alpha = 0.9999)  # keep all
  expect_true("count" %in% names(b))
  expect_true("alpha" %in% names(b))
})

test_that("backbone handles empty edge list", {
  empty <- data.frame(from = character(0), to = character(0),
                       weight = numeric(0), stringsAsFactors = FALSE)
  b <- backbone(empty, alpha = 0.05)
  expect_equal(nrow(b), 0)
  expect_true("alpha" %in% names(b))
})

test_that("backbone result is sorted by weight descending", {
  edges <- make_backbone_edges()
  b <- backbone(edges, alpha = 0.9999)
  expect_true(all(diff(b$weight) <= 0))
})

test_that("backbone rejects bad alpha values", {
  edges <- make_backbone_edges()
  expect_error(backbone(edges, alpha = 0))
  expect_error(backbone(edges, alpha = 1))
  expect_error(backbone(edges, alpha = -0.1))
  expect_error(backbone(edges, alpha = 1.1))
})

#' Convert edge data frame to igraph
#'
#' @param edges A data frame with at least `from`, `to`, `weight` columns,
#'   as returned by any network function in bibnets.
#' @param directed Logical. Default `FALSE`.
#'
#' @return An igraph graph object.
#'
#' @export
#' @examplesIf requireNamespace("igraph", quietly = TRUE)
#' data(biblio_data)
#' edges <- author_network(biblio_data, "collaboration")
#' g <- to_igraph(edges)
to_igraph <- function(edges, directed = FALSE) {
  if (!requireNamespace("igraph", quietly = TRUE)) {
    stop("Package 'igraph' is required. Install it with: ",
         "install.packages('igraph')", call. = FALSE)
  }
  check_edges(edges)
  igraph::graph_from_data_frame(edges, directed = directed)
}


#' Convert edge data frame to tbl_graph
#'
#' @param edges A data frame with at least `from`, `to`, `weight` columns.
#' @param directed Logical. Default `FALSE`.
#'
#' @return A tbl_graph object (tidygraph).
#'
#' @export
#' @examplesIf requireNamespace("tidygraph", quietly = TRUE)
#' data(biblio_data)
#' edges <- keyword_network(biblio_data)
#' tg <- to_tbl_graph(edges)
to_tbl_graph <- function(edges, directed = FALSE) {
  if (!requireNamespace("tidygraph", quietly = TRUE)) {
    stop("Package 'tidygraph' is required. Install it with: ",
         "install.packages('tidygraph')", call. = FALSE)
  }
  check_edges(edges)
  tidygraph::as_tbl_graph(to_igraph(edges, directed = directed))
}


#' Convert edge data frame to adjacency matrix
#'
#' @param edges A data frame with `from`, `to`, `weight` columns.
#' @param symmetric Logical. If `TRUE` (default), produces a symmetric matrix.
#'
#' @return A sparse Matrix.
#'
#' @export
#' @examples
#' data(biblio_data)
#' edges <- reference_network(biblio_data, min_occur = 2)
#' to_matrix(edges)
to_matrix <- function(edges, symmetric = TRUE) {
  check_edges(edges)
  edgelist_to_mat(edges, symmetric = symmetric)
}


#' Export to Gephi node and edge tables
#'
#' Converts a bibnets edge list (and optional node table) to the CSV format
#' expected by Gephi's Data Laboratory. Column names are remapped to Gephi
#' conventions (`Source`, `Target`, `Weight`, `Id`, `Label`).
#'
#' @param edges A data frame with at least `from`, `to`, `weight` columns.
#' @param nodes Optional data frame of node attributes. Must contain an `id`
#'   column. All other columns are included as Gephi node attributes.
#' @param file Optional directory path. If supplied, writes `nodes.csv` and
#'   `edges.csv` into that directory. If `NULL` (default), returns a list.
#' @param directed Logical. Sets the `Type` column. Default `FALSE`.
#'
#' @return If `file = NULL`: a list with `$nodes` and `$edges` data frames.
#'   If `file` is a directory path: writes two CSV files invisibly and returns
#'   the file paths.
#'
#' @export
#' @examples
#' data(biblio_data)
#' edges <- author_network(biblio_data, "collaboration")
#' gephi <- to_gephi(edges)
#' head(gephi$edges)
to_gephi <- function(edges, nodes = NULL, file = NULL, directed = FALSE) {
  check_edges(edges)

  type_label <- if (directed) "Directed" else "Undirected"

  ## Edge table — Gephi expects Source, Target, Weight, Type.
  ## Strip bibnets_network class and attributes so the returned data frame
  ## prints as a plain table (the renamed columns no longer match the
  ## bibnets print method's expected `from`/`to`/`weight`).
  edge_out <- as.data.frame(edges, stringsAsFactors = FALSE)
  class(edge_out) <- "data.frame"
  for (a in c("network_type", "counting", "similarity"))
    attr(edge_out, a) <- NULL
  names(edge_out)[names(edge_out) == "from"]   <- "Source"
  names(edge_out)[names(edge_out) == "to"]     <- "Target"
  names(edge_out)[names(edge_out) == "weight"] <- "Weight"
  edge_out$Type <- type_label

  ## Move standard columns to the front
  first <- intersect(c("Source", "Target", "Weight", "Type"), names(edge_out))
  edge_out <- edge_out[, c(first, setdiff(names(edge_out), first))]
  row.names(edge_out) <- NULL

  ## Node table — derive from edge list if not supplied
  if (is.null(nodes)) {
    all_ids <- unique(c(edges$from, edges$to))
    node_out <- data.frame(Id = all_ids, Label = all_ids,
                           stringsAsFactors = FALSE)
  } else {
    if (!is.data.frame(nodes) || !"id" %in% names(nodes))
      stop("'nodes' must be a data frame with an 'id' column.", call. = FALSE)
    node_out <- nodes
    names(node_out)[names(node_out) == "id"] <- "Id"
    node_out$Label <- node_out$Id
    first_n <- intersect(c("Id", "Label"), names(node_out))
    node_out <- node_out[, c(first_n, setdiff(names(node_out), first_n))]
  }

  if (is.null(file)) {
    return(list(nodes = node_out, edges = edge_out))
  }

  if (!dir.exists(file))
    stop("'file' must be an existing directory when writing CSVs. Got: '", file, "'", call. = FALSE)
  node_path <- file.path(file, "nodes.csv")
  edge_path <- file.path(file, "edges.csv")
  utils::write.csv(node_out, node_path, row.names = FALSE)
  utils::write.csv(edge_out, edge_path, row.names = FALSE)
  message("Written: ", node_path, "\n        ", edge_path)
  invisible(c(node_path, edge_path))
}


#' Export to GraphML
#'
#' Writes a bibnets edge list (and optional node attributes) to a GraphML
#' file using pure base R — no XML package required.
#'
#' @param edges A data frame with at least `from`, `to`, `weight` columns.
#' @param nodes Optional data frame of node attributes with an `id` column.
#' @param file File path to write. If `NULL` (default), returns the GraphML
#'   as a character string.
#' @param directed Logical. Default `FALSE`.
#'
#' @return If `file = NULL`: GraphML as a character string. Otherwise writes
#'   the file and returns the path invisibly.
#'
#' @export
#' @examples
#' data(biblio_data)
#' edges <- keyword_network(biblio_data)
#' xml <- to_graphml(edges)
#' cat(substr(xml, 1, 300))
to_graphml <- function(edges, nodes = NULL, file = NULL, directed = FALSE) {
  check_edges(edges)

  r_to_graphml_type <- function(x) {
    if (is.integer(x))   return("int")
    if (is.numeric(x))   return("double")
    if (is.logical(x))   return("boolean")
    "string"
  }

  xml_escape <- function(x) {
    x <- gsub("&",  "&amp;",  as.character(x))
    x <- gsub("<",  "&lt;",   x)
    x <- gsub(">",  "&gt;",   x)
    x <- gsub("\"", "&quot;", x)
    x
  }

  edge_default <- if (directed) "directed" else "undirected"

  ## Key declarations
  edge_attr_cols <- setdiff(names(edges), c("from", "to"))
  edge_keys <- vapply(edge_attr_cols, function(col) {
    sprintf('  <key id="%s" for="edge" attr.name="%s" attr.type="%s"/>',
            col, col, r_to_graphml_type(edges[[col]]))
  }, character(1L))

  node_attr_cols <- character(0)
  node_keys <- character(0)
  if (!is.null(nodes)) {
    if (!is.data.frame(nodes) || !"id" %in% names(nodes))
      stop("'nodes' must be a data frame with an 'id' column.", call. = FALSE)
    node_attr_cols <- setdiff(names(nodes), "id")
    node_keys <- vapply(node_attr_cols, function(col) {
      sprintf('  <key id="%s" for="node" attr.name="%s" attr.type="%s"/>',
              col, col, r_to_graphml_type(nodes[[col]]))
    }, character(1L))
  }

  ## Build a <data> tag, skipping NA values entirely (GraphML treats absent
  ## data as the type's default; emitting "NA" pollutes downstream tools).
  data_tag <- function(col, value) {
    if (is.na(value)) return(NA_character_)
    sprintf('      <data key="%s">%s</data>', col, xml_escape(value))
  }

  drop_na <- function(x) x[!is.na(x)]

  ## Node elements
  all_ids <- unique(c(edges$from, edges$to))
  node_elems <- vapply(all_ids, function(v) {
    attrs <- ""
    if (!is.null(nodes)) {
      row <- nodes[nodes$id == v, , drop = FALSE]
      if (nrow(row) > 0) {
        tags <- drop_na(vapply(node_attr_cols, function(col) {
          data_tag(col, row[[col]][1])
        }, character(1L)))
        if (length(tags))
          attrs <- paste0("\n", paste(tags, collapse = "\n"), "\n    ")
      }
    }
    sprintf('    <node id="%s">%s</node>', xml_escape(v), attrs)
  }, character(1L))

  ## Edge elements
  edge_elems <- vapply(seq_len(nrow(edges)), function(i) {
    tags <- drop_na(vapply(edge_attr_cols, function(col) {
      data_tag(col, edges[[col]][i])
    }, character(1L)))
    body <- if (length(tags)) paste0("\n", paste(tags, collapse = "\n"), "\n    ") else ""
    sprintf('    <edge source="%s" target="%s">%s</edge>',
            xml_escape(edges$from[i]), xml_escape(edges$to[i]), body)
  }, character(1L))

  ## Assemble
  lines <- c(
    '<?xml version="1.0" encoding="UTF-8"?>',
    '<graphml xmlns="http://graphml.graphdrawing.org/graphml"',
    '         xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance"',
    '         xsi:schemaLocation="http://graphml.graphdrawing.org/graphml',
    '         http://graphml.graphdrawing.org/graphml/1.0/graphml.xsd">',
    node_keys,
    edge_keys,
    sprintf('  <graph id="G" edgedefault="%s">', edge_default),
    node_elems,
    edge_elems,
    '  </graph>',
    '</graphml>'
  )

  xml <- paste(lines, collapse = "\n")

  if (is.null(file)) return(xml)

  writeLines(xml, file, useBytes = FALSE)
  message("Written: ", file)
  invisible(file)
}


#' Prepare network for cograph::splot()
#'
#' Converts a bibnets edge list to a `cograph_network` object by calling
#' `cograph::as_cograph()`. Optionally merges node metadata (e.g., from
#' [local_citations()]) into the network's node table so attributes like
#' `lcs` or `year` can be used directly in `splot()` aesthetic parameters
#' (e.g., `node_size = "lcs"`).
#'
#' Note: bibnets edge lists (`from`, `to`, `weight`) are accepted directly
#' by `cograph::splot()` without conversion. This function is only needed
#' when you want to attach node-level metadata.
#'
#' @param edges A data frame with at least `from`, `to`, `weight` columns.
#' @param nodes Optional data frame of node attributes with an `id` column
#'   (e.g., output of [local_citations()]). All columns are merged into the
#'   `cograph_network$nodes` table and become available as aesthetic mappings.
#' @param directed Logical. Default `FALSE`.
#'
#' @return A `cograph_network` object (S3 list with `$nodes` and `$edges`).
#'
#' @export
#' @examplesIf requireNamespace("cograph", quietly = TRUE)
#' data(biblio_data)
#'
#' # Without metadata: splot() accepts bibnets edges directly
#' edges <- author_network(biblio_data, "collaboration")
#'
#' # With metadata: document network + local citation scores as node size
#' edges <- document_network(biblio_data, type = "coupling")
#' nodes <- local_citations(biblio_data)   # keyed by document id
#' net   <- to_cograph(edges, nodes = nodes)
to_cograph <- function(edges, nodes = NULL, directed = FALSE) {
  if (!requireNamespace("cograph", quietly = TRUE)) {
    stop("Package 'cograph' is required. Install it with: ",
         'install.packages("cograph", repos = "https://mohsaqr.r-universe.dev")',
         call. = FALSE)
  }
  check_edges(edges)

  net <- cograph::as_cograph(edges, directed = directed)

  if (!is.null(nodes)) {
    if (!is.data.frame(nodes) || !"id" %in% names(nodes))
      stop("'nodes' must be a data frame with an 'id' column.", call. = FALSE)
    ## Match node metadata by label (cograph stores node names in $nodes$label)
    attr_cols <- setdiff(names(nodes), "id")
    idx <- match(net$nodes$label, nodes$id)
    for (col in attr_cols) {
      net$nodes[[col]] <- nodes[[col]][idx]
    }
  }

  net
}

## Tests for to_gephi(), to_graphml(), to_matrix()

make_small_edges <- function() {
  data.frame(
    from   = c("A", "A", "B"),
    to     = c("B", "C", "C"),
    weight = c(5,   3,   2),
    count  = c(2L,  1L,  1L),
    stringsAsFactors = FALSE
  )
}

## ── to_gephi ────────────────────────────────────────────────────────────────

test_that("to_gephi returns list with nodes and edges", {
  edges <- make_small_edges()
  g <- to_gephi(edges)

  expect_true(is.list(g))
  expect_true(all(c("nodes", "edges") %in% names(g)))
})

test_that("to_gephi edge table has Gephi column names", {
  edges <- make_small_edges()
  g <- to_gephi(edges)
  expect_true(all(c("Source", "Target", "Weight", "Type") %in% names(g$edges)))
})

test_that("to_gephi node table has Id and Label", {
  edges <- make_small_edges()
  g <- to_gephi(edges)
  expect_true(all(c("Id", "Label") %in% names(g$nodes)))
  expect_equal(sort(g$nodes$Id), c("A", "B", "C"))
})

test_that("to_gephi Type is Undirected by default", {
  edges <- make_small_edges()
  g <- to_gephi(edges)
  expect_true(all(g$edges$Type == "Undirected"))
})

test_that("to_gephi Type is Directed when directed=TRUE", {
  edges <- make_small_edges()
  g <- to_gephi(edges, directed = TRUE)
  expect_true(all(g$edges$Type == "Directed"))
})

test_that("to_gephi custom nodes table merges correctly", {
  edges <- make_small_edges()
  nodes <- data.frame(id = c("A","B","C"), group = c("g1","g1","g2"),
                       stringsAsFactors = FALSE)
  g <- to_gephi(edges, nodes = nodes)
  expect_true("group" %in% names(g$nodes))
  expect_equal(nrow(g$nodes), 3)
})

test_that("to_gephi $edges is a plain data.frame, not bibnets_network", {
  ## Regression: previously the renamed Source/Target/Weight columns kept
  ## the bibnets_network class, causing the S3 print method to look up
  ## the now-missing $from / $to / $weight and display NA NA NA.
  edges <- structure(make_small_edges(), class = c("bibnets_network", "data.frame"),
                     network_type = "test", counting = "full")
  g <- to_gephi(edges)
  expect_identical(class(g$edges), "data.frame")
  expect_null(attr(g$edges, "network_type"))
  expect_null(attr(g$edges, "counting"))
  ## Print must not produce any NA tokens for from/to/weight
  out <- capture.output(print(g$edges))
  expect_false(any(grepl("\\bNA\\b", out)))
})

test_that("to_gephi writes CSV files to disk", {
  edges <- make_small_edges()
  tmp <- tempdir()
  paths <- to_gephi(edges, file = tmp)
  expect_true(file.exists(file.path(tmp, "nodes.csv")))
  expect_true(file.exists(file.path(tmp, "edges.csv")))
  ## Clean up
  file.remove(file.path(tmp, "nodes.csv"), file.path(tmp, "edges.csv"))
})

## ── to_graphml ──────────────────────────────────────────────────────────────

test_that("to_graphml returns a character string", {
  edges <- make_small_edges()
  xml <- to_graphml(edges)
  expect_true(is.character(xml))
  expect_equal(length(xml), 1)
})

test_that("to_graphml output starts with XML declaration", {
  edges <- make_small_edges()
  xml <- to_graphml(edges)
  expect_true(startsWith(xml, "<?xml"))
})

test_that("to_graphml contains node and edge elements", {
  edges <- make_small_edges()
  xml <- to_graphml(edges)
  expect_true(grepl('<node id="A"', xml, fixed = TRUE))
  expect_true(grepl('<node id="B"', xml, fixed = TRUE))
  expect_true(grepl('<edge source="A" target="B"', xml, fixed = TRUE))
})

test_that("to_graphml includes edge weight", {
  edges <- make_small_edges()
  xml <- to_graphml(edges)
  expect_true(grepl('key="weight"', xml, fixed = TRUE))
})

test_that("to_graphml undirected by default", {
  edges <- make_small_edges()
  xml <- to_graphml(edges)
  expect_true(grepl('edgedefault="undirected"', xml, fixed = TRUE))
})

test_that("to_graphml directed when directed=TRUE", {
  edges <- make_small_edges()
  xml <- to_graphml(edges, directed = TRUE)
  expect_true(grepl('edgedefault="directed"', xml, fixed = TRUE))
})

test_that("to_graphml includes node attributes when nodes supplied", {
  edges <- make_small_edges()
  nodes <- data.frame(id = c("A","B","C"), community = c(1L, 1L, 2L),
                       stringsAsFactors = FALSE)
  xml <- to_graphml(edges, nodes = nodes)
  expect_true(grepl('attr.name="community"', xml, fixed = TRUE))
})

test_that("to_graphml writes to file", {
  edges <- make_small_edges()
  tmp <- tempfile(fileext = ".graphml")
  path <- to_graphml(edges, file = tmp)
  expect_true(file.exists(tmp))
  content <- readLines(tmp)
  expect_true(any(grepl("graphml", content)))
  file.remove(tmp)
})

test_that("to_graphml never emits literal NA in <data> tags", {
  ## Regression: previously NA edge attribute values produced
  ## <data key="weight">NA</data>, which downstream tools treat as a string.
  edges <- data.frame(
    from = c("A", "B"), to = c("B", "C"),
    weight = c(NA_real_, 2),
    count  = c(1L, NA_integer_),
    stringsAsFactors = FALSE
  )
  xml <- to_graphml(edges)
  expect_false(grepl(">NA<",     xml, fixed = TRUE))
  expect_false(grepl(">NA_real", xml, fixed = TRUE))
  ## Non-NA values still emitted
  expect_true(grepl('<data key="weight">2</data>', xml, fixed = TRUE))
  expect_true(grepl('<data key="count">1</data>',  xml, fixed = TRUE))
})

test_that("to_graphml skips NA node attributes when nodes table has them", {
  edges <- make_small_edges()
  nodes <- data.frame(id = c("A","B","C"),
                      community = c(1L, NA_integer_, 2L),
                      stringsAsFactors = FALSE)
  xml <- to_graphml(edges, nodes = nodes)
  ## Node B has NA community → no <data key="community"> tag for that node
  expect_false(grepl(">NA<", xml, fixed = TRUE))
})

test_that("to_graphml escapes special characters in node IDs", {
  edges <- data.frame(
    from   = c('Smith & Jones', 'Smith & Jones'),
    to     = c('Lee <2020>', 'Other'),
    weight = c(1, 1),
    stringsAsFactors = FALSE
  )
  xml <- to_graphml(edges)
  expect_true(grepl("&amp;", xml, fixed = TRUE))
  expect_true(grepl("&lt;",  xml, fixed = TRUE))
})

## ── to_matrix ───────────────────────────────────────────────────────────────

test_that("to_matrix returns a matrix or Matrix", {
  edges <- make_small_edges()
  m <- to_matrix(edges)
  ## Works with either base matrix or Matrix class
  expect_true(is.matrix(m) || inherits(m, "Matrix"))
})

test_that("to_matrix symmetric by default", {
  edges <- make_small_edges()
  m <- to_matrix(edges)
  m_dense <- as.matrix(m)
  expect_equal(m_dense, t(m_dense))
})

test_that("to_matrix has correct dimensions and values", {
  edges <- make_small_edges()
  m <- to_matrix(edges)
  m_dense <- as.matrix(m)
  expect_equal(nrow(m_dense), 3)
  expect_equal(ncol(m_dense), 3)
  expect_equal(m_dense["A", "B"], 5)
  expect_equal(m_dense["A", "C"], 3)
  expect_equal(m_dense["B", "C"], 2)
})

#' Build a document network
#'
#' Constructs a network between documents (papers) in the dataset.
#'
#' @param data A data frame with `id` and `references` (list-column).
#' @param type Character. Relationship type:
#'   \describe{
#'     \item{`"coupling"`}{Bibliographic coupling: documents linked when they
#'       share cited references.}
#'     \item{`"citation"`}{Direct citation: directed edges from citing to
#'       cited documents (internal citations only).}
#'     \item{`"co_citation"`}{Co-citation: documents linked when they are
#'       cited together by other documents in the dataset.}
#'     \item{`"equivalence"`}{Profile similarity of reference vectors.}
#'   }
#' @param counting Character. Counting method. Default `"full"`.
#'   Position-dependent methods are not applicable to document networks.
#' @param similarity Character. Similarity measure. Default `"none"`.
#' @param threshold Numeric. Minimum edge weight. Default 0.
#' @param min_occur Integer. Minimum reference frequency. Default 1.
#' @param top_n Integer or NULL. Return only the top n edges by weight.
#'   Default NULL (all edges).
#' @inheritParams author_network
#'
#' @return Depends on `format`. For `type = "citation"`, edges are directed
#'   (from = citing, to = cited) with `weight` and `count` both 1.
#'
#' @export
#' @examples
#' data(biblio_data)
#' document_network(biblio_data, "coupling")
#' document_network(biblio_data, "coupling", counting = "strength")
document_network <- function(data,
                             type = "coupling",
                             counting = "full",
                             similarity = "none",
                             threshold = 0,
                             min_occur = 1L,
                             top_n = NULL,
                             self_loops = FALSE,
                             deduplicate = TRUE,
                             format = "edgelist") {
  check_data(data, c("id", "references"))
  check_choice(type, c("coupling", "citation", "co_citation", "equivalence"), "type")
  check_choice(counting, position_independent_counts(), "counting")
  check_choice(similarity, c("none", "association", "cosine", "jaccard",
                              "inclusion", "equivalence"), "similarity")
  check_format(format)

  result <- if (type == "citation") {
    edges <- build_direct_citation(data)
    if (!is.null(top_n) && nrow(edges) > 0) {
      freq <- sort(table(c(edges$from, edges$to)), decreasing = TRUE)
      top_nodes <- names(freq)[seq_len(min(top_n, length(freq)))]
      edges <- edges[edges$from %in% top_nodes & edges$to %in% top_nodes, ]
    }
    edges
  } else {
    B <- build_bipartite(data, field = "references", min_freq = min_occur, deduplicate = deduplicate)
    if (type == "coupling") {
      B <- apply_counting(B, counting = counting, network_type = "coupling")
      multiply_bipartite(B, mode = "rows", similarity = similarity,
                         threshold = threshold, top_n = top_n,
                         self_loops = self_loops)
    } else if (type == "co_citation") {
      B <- apply_counting(B, counting = counting, network_type = "symmetric")
      multiply_bipartite(B, mode = "columns", similarity = similarity,
                         threshold = threshold, top_n = top_n,
                         self_loops = self_loops)
    } else {
      multiply_bipartite(B, mode = "rows", similarity = "cosine",
                         threshold = threshold, top_n = top_n,
                         self_loops = self_loops)
    }
  }

  directed <- (type == "citation")
  as_bibnets_network(result, network_type = paste0("document_", type),
                      counting = counting, similarity = similarity,
                      format = format, directed = directed)
}


#' @keywords internal
build_direct_citation <- function(data) {
  ids <- as.character(data[["id"]])
  refs_list <- data[["references"]]

  citing <- rep(ids, lengths(refs_list))
  cited <- unlist(refs_list, use.names = FALSE)

  keep <- cited %in% ids & !is.na(cited)
  citing <- citing[keep]
  cited <- cited[keep]

  if (length(citing) == 0L) {
    return(data.frame(
      from = character(0), to = character(0),
      weight = numeric(0), count = integer(0),
      stringsAsFactors = FALSE
    ))
  }

  data.frame(
    from = citing, to = cited,
    weight = rep(1, length(citing)),
    count = rep(1L, length(citing)),
    stringsAsFactors = FALSE
  )
}

test_that("document_network coupling returns correct edges", {
  d <- make_test_data()
  edges <- document_network(d, "coupling")

  get_w <- function(a, b) {
    row <- edges[
      (edges$from == a & edges$to == b) |
      (edges$from == b & edges$to == a), ]
    if (nrow(row) == 0) return(0)
    row$weight[1]
  }

  ## All pairs share 2 refs
  expect_equal(get_w("W1", "W2"), 2)
  expect_equal(get_w("W1", "W3"), 2)
  expect_equal(get_w("W2", "W3"), 2)
})

test_that("document_network citation returns directed internal citations", {
  d <- data.frame(id = c("W1", "W2", "W3"), stringsAsFactors = FALSE)
  d$references <- list(c("W2", "W3", "R1"), c("W3"), c("R2"))

  edges <- document_network(d, "citation")
  expect_true(any(edges$from == "W1" & edges$to == "W2"))
  expect_true(any(edges$from == "W1" & edges$to == "W3"))
  expect_false("R1" %in% c(edges$from, edges$to))
})

test_that("document_network with cosine gives coupling angle", {
  d <- make_test_data()
  edges <- document_network(d, "coupling", similarity = "cosine")

  get_w <- function(a, b) {
    row <- edges[
      (edges$from == a & edges$to == b) |
      (edges$from == b & edges$to == a), ]
    if (nrow(row) == 0) return(0)
    row$weight[1]
  }

  ## W1-W2: 2 shared out of 3 each -> 2/sqrt(3*3) = 2/3
  expect_equal(get_w("W1", "W2"), 2/3, tolerance = tol)
})

#' Build a co-occurrence network from any field
#'
#' With one field, entities are linked when they co-occur in the same
#' document. With `by`, entities are linked when they share values of the
#' `by` field across documents.
#'
#' Fields can be list-columns (already split) or character columns with
#' delimiters (auto-split via `sep`).
#'
#' @param data A data frame with column `id` and the specified field(s).
#' @param field Character. The entity field — determines what the nodes are.
#' @param by Character or `NULL`. What links the nodes. If `NULL` (default),
#'   entities are linked by co-occurring in the same document. If specified,
#'   entities are linked when they share values from the `by` field.
#' @param sep Character or `NULL`. Delimiter for splitting character columns.
#'   Default `";"`. Set to `NULL` if columns are already list-columns.
#' @param counting Character. Counting method. Default `"full"`.
#' @param similarity Character. Normalization method. Default `"none"`.
#' @param threshold Numeric. Minimum edge weight. Default 0.
#' @param min_occur Integer. Minimum entity frequency. Default 1.
#' @param top_n Integer or NULL. Return only the top n edges by weight.
#'   Default NULL (all edges).
#' @inheritParams author_network
#'
#' @return Depends on `format`: a `bibnets_network` data frame (default),
#'   a Gephi-ready data frame, an igraph graph, a cograph_network, or a
#'   sparse matrix.
#'
#' @export
#' @examples
#' data(biblio_data)
#'
#' # Co-occurrence: keywords appearing in the same document
#' conetwork(biblio_data, "keywords")
#'
#' # Authors linked by shared keywords
#' conetwork(biblio_data, "authors", by = "keywords")
#'
#' # Keywords linked by shared authors
#' conetwork(biblio_data, "keywords", by = "authors")
#'
#' # Journals linked by shared references (= journal coupling)
#' conetwork(biblio_data, "journal", by = "references", similarity = "cosine")
#'
#' # Auto-splits semicolon-delimited string columns
#' d <- data.frame(id = 1:3, tags = c("ml; dl; nlp", "ml; cv", "dl; cv"))
#' conetwork(d, "tags")
conetwork <- function(data,
                       field,
                       by = NULL,
                       sep = ";",
                       counting = "full",
                       similarity = "none",
                       threshold = 0,
                       min_occur = 1L,
                       top_n = NULL,
                       self_loops = FALSE,
                       deduplicate = TRUE,
                       format = "edgelist") {
  check_data(data, c("id", field))
  check_choice(counting, position_independent_counts(), "counting")
  check_choice(similarity, c("none", "association", "cosine", "jaccard",
                              "inclusion", "equivalence"), "similarity")
  check_format(format)

  data <- ensure_list_column(data, field, sep)

  result <- if (is.null(by)) {
    ## Co-occurrence within one field (same document)
    B <- build_bipartite(data, field = field, min_freq = min_occur, deduplicate = deduplicate)
    B <- apply_counting(B, counting = counting, network_type = "symmetric")
    multiply_bipartite(B, mode = "columns", similarity = similarity,
                       threshold = threshold, top_n = top_n,
                       self_loops = self_loops)
  } else {
    ## Entities linked by shared values from `by` field
    if (!by %in% names(data))
      stop("Column '", by, "' not found in data.", call. = FALSE)
    data <- ensure_list_column(data, by, sep)
    build_by_network(data, field = field, by = by,
                     counting = counting, similarity = similarity,
                     threshold = threshold, min_occur = min_occur,
                     top_n = top_n, self_loops = self_loops,
                     deduplicate = deduplicate)
  }

  net_type <- if (is.null(by)) {
    paste0(field, "_co_occurrence")
  } else {
    paste0(field, "_by_", by)
  }
  as_bibnets_network(result, network_type = net_type,
                      counting = counting, similarity = similarity,
                      format = format)
}


#' Build a network where entities share values from another field
#' @keywords internal
build_by_network <- function(data, field, by, counting, similarity,
                              threshold, min_occur, top_n = NULL,
                              self_loops = FALSE, deduplicate = TRUE) {
  field_col <- data[[field]]
  by_col <- data[[by]]

  ## Expand field → (doc_idx, entity)
  if (is.list(field_col)) {
    f_doc <- rep(seq_len(nrow(data)), lengths(field_col))
    f_val <- unlist(field_col, use.names = FALSE)
  } else {
    f_doc <- seq_len(nrow(data))
    f_val <- as.character(field_col)
  }

  ## Expand by → (doc_idx, by_value)
  if (is.list(by_col)) {
    b_doc <- rep(seq_len(nrow(data)), lengths(by_col))
    b_val <- unlist(by_col, use.names = FALSE)
  } else {
    b_doc <- seq_len(nrow(data))
    b_val <- as.character(by_col)
  }

  ## Clean
  f_val <- toupper(trimws(as.character(f_val)))
  b_val <- toupper(trimws(as.character(b_val)))
  keep_f <- !is.na(f_val) & nchar(f_val) > 0
  f_doc <- f_doc[keep_f]; f_val <- f_val[keep_f]
  keep_b <- !is.na(b_val) & nchar(b_val) > 0
  b_doc <- b_doc[keep_b]; b_val <- b_val[keep_b]

  ## Frequency filter on field entities
  if (min_occur > 1L) {
    freq <- table(f_val)
    keep <- f_val %in% names(freq[freq >= min_occur])
    f_doc <- f_doc[keep]; f_val <- f_val[keep]
  }

  ## Join: for each entity, collect all by-values through shared docs
  f_df <- data.frame(doc = f_doc, entity = f_val, stringsAsFactors = FALSE)
  b_df <- data.frame(doc = b_doc, by_val = b_val, stringsAsFactors = FALSE)
  pairs <- merge(f_df, b_df, by = "doc")

  if (nrow(pairs) == 0L) {
    return(data.frame(from = character(0), to = character(0),
                      weight = numeric(0), count = integer(0),
                      stringsAsFactors = FALSE))
  }

  ## Unique entity × by_value pairs
  pairs <- unique(pairs[, c("entity", "by_val")])

  ## Aggregate per entity: which by-values does each entity have?
  agg <- stats::aggregate(by_val ~ entity, data = pairs,
                           FUN = function(x) list(unique(x)))
  agg_df <- data.frame(id = agg$entity, stringsAsFactors = FALSE)
  agg_df[["values"]] <- agg$by_val

  ## Build bipartite: entities × by_values, then project to entity × entity
  B <- build_bipartite(agg_df, field = "values", min_freq = 1L, deduplicate = deduplicate)
  B <- apply_counting(B, counting = counting, network_type = "symmetric")
  multiply_bipartite(B, mode = "rows", similarity = similarity,
                     threshold = threshold, top_n = top_n,
                     self_loops = self_loops)
}


#' Ensure a column is a list-column, splitting if needed
#' @keywords internal
ensure_list_column <- function(data, field, sep = ";") {
  col <- data[[field]]
  if (!is.list(col) && !is.null(sep)) {
    data[[field]] <- split_field(col, sep = sep)
  } else if (!is.list(col)) {
    data[[field]] <- as.list(as.character(col))
  }
  data
}

#' Filter edges to top-n nodes
#'
#' Keeps only edges between the most frequent nodes. Node frequency is
#' determined by how many edges each node participates in.
#'
#' @param edges A data frame with at least `from`, `to`, `weight` columns.
#' @param n Integer. Number of top nodes to keep.
#'
#' @return A filtered data frame with edges among the top `n` nodes.
#'
#' @export
#' @examples
#' data(biblio_data)
#' edges <- author_network(biblio_data, "collaboration")
#'
#' # Keep only edges among the top 3 most connected authors
#' filter_top(edges, 3)
filter_top <- function(edges, n) {
  check_edges(edges)
  if (!is.numeric(n) || length(n) != 1 || n <= 0)
    stop("'n' must be a positive number", call. = FALSE)

  if (nrow(edges) == 0L) return(edges)

  n <- as.integer(n)

  ## Node frequency = number of edges per node
  all_nodes <- c(edges$from, edges$to)
  freq <- sort(table(all_nodes), decreasing = TRUE)
  top_nodes <- names(freq)[seq_len(min(n, length(freq)))]

  edges <- edges[edges$from %in% top_nodes & edges$to %in% top_nodes, ]
  edges <- edges[order(-edges$weight), ]
  rownames(edges) <- NULL
  edges
}

## Test helpers for bibnets

## Small test data: 3 papers, 3 authors, 4 references
make_test_data <- function() {
  d <- data.frame(
    id = c("W1", "W2", "W3"),
    title = c("Paper A", "Paper B", "Paper C"),
    year = c(2020L, 2021L, 2022L),
    journal = c("J1", "J1", "J2"),
    stringsAsFactors = FALSE
  )
  d$authors <- list(
    c("Alice", "Bob"),       # W1: 2 authors
    c("Alice", "Carol"),     # W2: 2 authors
    c("Bob", "Carol", "Dan") # W3: 3 authors
  )
  d$references <- list(
    c("R1", "R2", "R3"),     # W1: 3 refs
    c("R1", "R2", "R4"),     # W2: 3 refs
    c("R2", "R3", "R4")      # W3: 3 refs
  )
  d$keywords <- list(
    c("ml", "dl"),
    c("ml", "nlp"),
    c("dl", "nlp", "cv")
  )
  d
}

## Tolerance for floating point comparisons
tol <- 1e-10

#' Convert Crossref API data to bibnets format
#'
#' Takes the output of `rcrossref::cr_works()` (the `$data` tibble/data frame)
#' and converts it to the standardized bibnets format.
#'
#' @param data A data frame from `cr_works(...)$data`.
#'
#' @return A data frame in the standard bibnets format: `id`, `title`,
#'   `year`, `journal`, `doi`, `cited_by_count`, `abstract`, `type`,
#'   plus list-columns `authors`, `references`, and `keywords`.
#'
#' @export
#' @examples
#' # Construct a minimal data frame matching the structure of
#' # rcrossref::cr_works(...)$data. In practice, pass that data frame directly.
#' raw <- data.frame(
#'   doi = c("10.1/a", "10.2/b"),
#'   title = c("First paper", "Second paper"),
#'   issued = c("2022-01-01", "2021-06-15"),
#'   container.title = c("Journal A", "Journal B"),
#'   is.referenced.by.count = c("3", "9"),
#'   type = c("journal-article", "journal-article"),
#'   stringsAsFactors = FALSE
#' )
#' raw$author <- list(
#'   data.frame(given = c("Jane", "Anne"),
#'              family = c("Smith", "Jones"),
#'              stringsAsFactors = FALSE),
#'   data.frame(given = "Mark", family = "Davis", stringsAsFactors = FALSE)
#' )
#' data <- read_crossref(raw)
#' head(data[, c("id", "title", "year", "journal")])
read_crossref <- function(data) {
  stopifnot(is.data.frame(data))

  n <- nrow(data)

  safe_col <- function(name, default = NA_character_) {
    ## Crossref uses both dot and hyphen notation
    candidates <- c(name, gsub("\\.", "-", name), gsub("-", ".", name))
    for (cand in candidates) {
      if (cand %in% names(data)) return(data[[cand]])
    }
    rep(default, n)
  }

  ## ID: DOI is the primary identifier
  doi <- safe_col("doi")
  id <- ifelse(is.na(doi) | nchar(doi) == 0,
               paste0("CR", seq_len(n)),
               doi)

  ## Year: from issued field (may be nested or string)
  issued <- safe_col("issued")
  year <- unname(vapply(issued, function(x) {
    if (is.na(x)) return(NA_integer_)
    m <- regmatches(as.character(x), regexpr("\\d{4}", as.character(x)))
    if (length(m) > 0) as.integer(m[1]) else NA_integer_
  }, integer(1)))

  result <- data.frame(
    id = id,
    title = safe_col("title"),
    year = year,
    journal = safe_col("container.title"),
    doi = doi,
    cited_by_count = as.integer(safe_col("is.referenced.by.count", 0L)),
    abstract = safe_col("abstract"),
    type = safe_col("type"),
    stringsAsFactors = FALSE
  )

  ## Authors: nested data.frame with 'given' and 'family'
  if ("author" %in% names(data) && is.list(data[["author"]])) {
    result$authors <- lapply(data[["author"]], function(au) {
      if (is.null(au) || !is.data.frame(au)) return(character(0))
      family <- if ("family" %in% names(au)) au$family else character(0)
      given <- if ("given" %in% names(au)) au$given else ""
      ## Format as "FAMILY GIVEN"
      standardize_authors(paste(trimws(family), trimws(given)))
    })
  } else {
    result$authors <- replicate(n, character(0), simplify = FALSE)
  }

  ## References: nested data.frame with DOI, article-title, author, year etc.
  if ("reference" %in% names(data) && is.list(data[["reference"]])) {
    result$references <- lapply(data[["reference"]], function(ref) {
      if (is.null(ref) || !is.data.frame(ref)) return(character(0))
      ## Prefer DOI as identifier, fall back to constructed string
      if ("DOI" %in% names(ref)) {
        doi_refs <- ref$DOI
        ## For refs without DOI, construct a string
        no_doi <- is.na(doi_refs) | nchar(doi_refs) == 0
        if (any(no_doi)) {
          au <- if ("author" %in% names(ref)) ref$author[no_doi] else ""
          yr <- if ("year" %in% names(ref)) ref$year[no_doi] else ""
          jt <- if ("journal-title" %in% names(ref)) {
            ref[["journal-title"]][no_doi]
          } else ""
          doi_refs[no_doi] <- paste(au, yr, jt)
        }
        standardize_refs(doi_refs)
      } else {
        ## No DOI column — use unstructured or construct
        if ("unstructured" %in% names(ref)) {
          standardize_refs(ref$unstructured)
        } else {
          au <- if ("author" %in% names(ref)) ref$author else ""
          yr <- if ("year" %in% names(ref)) ref$year else ""
          standardize_refs(paste(au, yr))
        }
      }
    })
  } else {
    result$references <- replicate(n, character(0), simplify = FALSE)
  }

  ## Keywords: Crossref has 'subject' field
  if ("subject" %in% names(data) && is.list(data[["subject"]])) {
    result$keywords <- lapply(data[["subject"]], function(s) {
      if (is.null(s)) character(0) else as.character(s)
    })
  } else {
    result$keywords <- replicate(n, character(0), simplify = FALSE)
  }

  result
}

#' Compute local citation scores
#'
#' Counts how many times each document is cited by other documents
#' within the dataset.
#'
#' @param data A data frame with `id` and `references` (list-column).
#'   Optionally `year`, `title`, `journal`, `doi`, `cited_by_count`.
#'
#' @return A data frame with columns:
#'   \describe{
#'     \item{`id`}{Document identifier.}
#'     \item{`lcs`}{Local Citation Score: times cited within the dataset.}
#'     \item{`gcs`}{Global Citation Score: `cited_by_count` if available.}
#'   }
#'   Plus any metadata columns present in the input (`title`, `year`,
#'   `journal`, `doi`).
#'
#' @export
#' @examples
#' data(biblio_data)
#' local_citations(biblio_data)
local_citations <- function(data) {
  check_data(data, c("id", "references"))

  ids <- as.character(data[["id"]])

  ## Expand all references
  all_refs <- unlist(data[["references"]], use.names = FALSE)
  all_refs <- all_refs[!is.na(all_refs)]

  ## Count how many times each dataset ID appears as a reference
  internal_refs <- all_refs[all_refs %in% ids]
  lcs_table <- table(internal_refs)

  lcs <- integer(length(ids))
  names(lcs) <- ids
  matched <- intersect(names(lcs_table), ids)
  lcs[matched] <- as.integer(lcs_table[matched])

  ## Build result
  result <- data.frame(
    id = ids,
    lcs = as.integer(lcs),
    stringsAsFactors = FALSE
  )

  ## Add metadata columns in canonical order: gcs, year, title, journal, doi
  if ("cited_by_count" %in% names(data)) {
    result$gcs <- as.integer(data[["cited_by_count"]])
  }
  for (col in c("year", "title", "journal", "doi")) {
    if (col %in% names(data)) {
      result[[col]] <- if (col == "year") as.integer(data[[col]]) else data[[col]]
    }
  }

  result <- result[order(-result$lcs), ]
  rownames(result) <- NULL
  result
}


#' Build a historiograph (chronological citation network)
#'
#' Constructs a Garfield-style historiograph: a directed citation network
#' among the most locally cited documents, laid out chronologically.
#'
#' @param data A data frame with `id`, `references` (list-column), and
#'   `year`. Optionally `title`, `journal`, `doi`, `cited_by_count`.
#' @param n Integer. Number of top locally cited documents to include.
#'   Default 30.
#' @param min_lcs Integer. Minimum local citation score for inclusion.
#'   Default 1.
#'
#' @return A list with:
#'   \describe{
#'     \item{`$nodes`}{Data frame of included documents with `id`, `lcs`,
#'       `gcs`, `year`, `title`, `journal`, `doi`.}
#'     \item{`$edges`}{Data frame of directed citation edges with `from`
#'       (citing), `to` (cited), `year_from`, `year_to`.}
#'   }
#'
#' @export
#' @examples
#' data(biblio_data)
#' h <- historiograph(biblio_data, n = 5)
#' h$nodes
#' h$edges
historiograph <- function(data, n = 30, min_lcs = 1) {
  check_data(data, c("id", "references", "year"))

  ## Compute local citations
  lcs_df <- local_citations(data)

  ## Filter by min_lcs
  lcs_df <- lcs_df[lcs_df$lcs >= min_lcs, ]

  if (nrow(lcs_df) == 0) {
    empty_nodes <- data.frame(
      id = character(0), lcs = integer(0), gcs = integer(0),
      year = integer(0), title = character(0),
      journal = character(0), doi = character(0),
      stringsAsFactors = FALSE
    )
    ## Drop columns absent from the input
    keep_cols <- c("id", "lcs",
                   if ("cited_by_count" %in% names(data)) "gcs",
                   "year",
                   intersect(c("title", "journal", "doi"), names(data)))
    return(list(
      nodes = empty_nodes[, keep_cols, drop = FALSE],
      edges = data.frame(from = character(0), to = character(0),
                          year_from = integer(0), year_to = integer(0),
                          stringsAsFactors = FALSE)
    ))
  }

  ## Top n by LCS
  if (nrow(lcs_df) > n) {
    lcs_df <- lcs_df[seq_len(n), ]
  }

  nodes <- lcs_df
  node_ids <- nodes$id

  ## Build directed citation edges among included nodes
  ids <- as.character(data[["id"]])
  refs_list <- data[["references"]]
  years <- data[["year"]]

  ## Year lookup
  year_lookup <- stats::setNames(years, ids)

  ## Expand references for ALL papers (a non-top paper can cite a top paper)
  citing <- rep(ids, lengths(refs_list))
  cited <- unlist(refs_list, use.names = FALSE)

  keep <- !is.na(cited) & cited %in% node_ids & citing %in% ids
  citing <- citing[keep]
  cited  <- cited[keep]

  if (length(citing) == 0) {
    edges <- data.frame(from = character(0), to = character(0),
                         year_from = integer(0), year_to = integer(0),
                         stringsAsFactors = FALSE)
  } else {
    edges <- data.frame(
      from = citing,
      to = cited,
      year_from = as.integer(year_lookup[citing]),
      year_to = as.integer(year_lookup[cited]),
      stringsAsFactors = FALSE
    )
    ## Remove duplicates
    edges <- unique(edges)
    edges <- edges[order(edges$year_from, edges$year_to), ]
    rownames(edges) <- NULL
  }

  list(nodes = nodes, edges = edges)
}

## Tests for local_citations() and historiograph()

## Data where papers cite each other:
## W1 (2018) ← W2 (2019), W3 (2020)
## W2 (2019) ← W3 (2020), W4 (2021)
## W3 (2020) ← W4 (2021)
## W4 (2021) ← nobody
make_citation_data <- function() {
  d <- data.frame(
    id    = c("W1", "W2", "W3", "W4"),
    year  = c(2018L, 2019L, 2020L, 2021L),
    title = c("T1", "T2", "T3", "T4"),
    stringsAsFactors = FALSE
  )
  d$references <- list(
    character(0),        # W1 cites nobody
    c("W1"),             # W2 cites W1
    c("W1", "W2"),       # W3 cites W1 and W2
    c("W2", "W3")        # W4 cites W2 and W3
  )
  d
}

## ── local_citations ────────────────────────────────────────────────────────

test_that("local_citations returns correct LCS counts", {
  d <- make_citation_data()
  lcs <- local_citations(d)

  expect_true(is.data.frame(lcs))
  expect_true(all(c("id", "lcs") %in% names(lcs)))

  get_lcs <- function(id) lcs$lcs[lcs$id == id]

  expect_equal(get_lcs("W1"), 2L)  # cited by W2, W3
  expect_equal(get_lcs("W2"), 2L)  # cited by W3, W4
  expect_equal(get_lcs("W3"), 1L)  # cited by W4
  expect_equal(get_lcs("W4"), 0L)  # not cited
})

test_that("local_citations is sorted by lcs descending", {
  d <- make_citation_data()
  lcs <- local_citations(d)
  expect_true(all(diff(lcs$lcs) <= 0))
})

test_that("local_citations includes year when present", {
  d <- make_citation_data()
  lcs <- local_citations(d)
  expect_true("year" %in% names(lcs))
  expect_true(all(is.integer(lcs$year)))
})

test_that("local_citations includes gcs when cited_by_count present", {
  d <- make_citation_data()
  d$cited_by_count <- c(100L, 50L, 20L, 5L)
  lcs <- local_citations(d)
  expect_true("gcs" %in% names(lcs))
})

test_that("local_citations works when no internal citations exist", {
  d <- make_test_data()  # W1-W3 only cite external refs R1-R4
  lcs <- local_citations(d)
  expect_equal(nrow(lcs), 3)
  expect_true(all(lcs$lcs == 0L))
})

test_that("local_citations handles missing references gracefully", {
  d <- make_citation_data()
  d$references <- lapply(d$references, function(x) c(x, NA_character_))
  lcs <- local_citations(d)
  expect_equal(nrow(lcs), 4)
})

## ── historiograph ──────────────────────────────────────────────────────────

test_that("historiograph returns list with nodes and edges", {
  d <- make_citation_data()
  h <- historiograph(d, n = 3)

  expect_true(is.list(h))
  expect_true(all(c("nodes", "edges") %in% names(h)))
})

test_that("historiograph nodes are top-n by LCS", {
  d <- make_citation_data()
  h <- historiograph(d, n = 3, min_lcs = 1)

  ## Top 3 by LCS: W1(2), W2(2), W3(1); W4 has lcs=0
  expect_equal(nrow(h$nodes), 3)
  expect_false("W4" %in% h$nodes$id)
})

test_that("historiograph edges have correct structure", {
  d <- make_citation_data()
  h <- historiograph(d, n = 3, min_lcs = 1)

  expect_true(all(c("from", "to", "year_from", "year_to") %in% names(h$edges)))
  expect_true(is.integer(h$edges$year_from))
  expect_true(is.integer(h$edges$year_to))
})

test_that("historiograph edges include citations from non-top documents", {
  d <- make_citation_data()
  ## Top-3 nodes: W1, W2, W3
  ## W4 is NOT in top-3 (lcs=0) but cites W2 and W3 which ARE in top-3
  h <- historiograph(d, n = 3, min_lcs = 1)

  ## W4→W2 and W4→W3 should appear (W4 is a citing doc, not a node)
  has_edge <- function(f, t) any(h$edges$from == f & h$edges$to == t)
  expect_true(has_edge("W4", "W2"))
  expect_true(has_edge("W4", "W3"))
  ## W3→W2 too
  expect_true(has_edge("W3", "W2"))
})

test_that("historiograph min_lcs filters correctly", {
  d <- make_citation_data()
  ## min_lcs = 2: only W1(2) and W2(2) qualify
  h <- historiograph(d, n = 30, min_lcs = 2)
  expect_equal(nrow(h$nodes), 2)
  expect_true(all(h$nodes$id %in% c("W1", "W2")))
})

test_that("historiograph returns empty list when min_lcs too high", {
  d <- make_citation_data()
  h <- historiograph(d, n = 10, min_lcs = 99)
  expect_equal(nrow(h$nodes), 0)
  expect_equal(nrow(h$edges), 0)
})

test_that("historiograph requires year column", {
  d <- make_citation_data()
  d$year <- NULL
  expect_error(historiograph(d))
})

test_that("historiograph edges are sorted by year_from", {
  d <- make_citation_data()
  h <- historiograph(d, n = 4, min_lcs = 0)
  if (nrow(h$edges) > 1) {
    expect_true(all(diff(h$edges$year_from) >= 0))
  }
})

#' Read Lens.org CSV export
#'
#' Parses a CSV file exported from Lens.org into a standardized bibliometric
#' data frame.
#'
#' @param file Path to a Lens.org CSV export file.
#' @param encoding Character. File encoding. Default `"UTF-8"`.
#'
#' @return A data frame in the standard bibnets format: `id`, `title`,
#'   `year`, `journal`, `doi`, `cited_by_count`, `abstract`, `type`,
#'   plus list-columns `authors`, `references`, and `keywords`.
#'
#' @export
#' @examples
#' f <- system.file("extdata", "lens_sample.csv", package = "bibnets")
#' data <- read_lens(f)
#' head(data[, c("id", "title", "year", "journal")])
read_lens <- function(file, encoding = "UTF-8") {
  check_file(file)

  raw <- utils::read.csv(file, stringsAsFactors = FALSE, fileEncoding = encoding,
                          check.names = FALSE)

  n <- nrow(raw)

  get_col <- function(names_vec, default = NA_character_) {
    for (nm in names_vec) {
      if (nm %in% names(raw)) return(raw[[nm]])
    }
    if (length(default) == n) default else rep(default, length.out = n)
  }

  id <- get_col(c("Lens ID", "ID"), paste0("LENS", seq_len(n)))

  result <- data.frame(
    id = id,
    title = get_col(c("Title")),
    year = as.integer(get_col(c("Publication Year", "Year of Publication"),
                               NA_integer_)),
    journal = get_col(c("Source Title", "Journal")),
    doi = get_col(c("DOI")),
    cited_by_count = as.integer(get_col(
      c("Citing Works Count", "Cited By Count", "Times Cited"), 0L
    )),
    abstract = get_col(c("Abstract")),
    type = get_col(c("Publication Type", "Document Type")),
    stringsAsFactors = FALSE
  )

  result$authors <- split_field(
    get_col(c("Authors", "Author/s")), sep = ";"
  )
  result$authors <- lapply(result$authors, standardize_authors)

  result$references <- split_field(
    get_col(c("References", "Cited Works")), sep = ";"
  )
  result$references <- lapply(result$references, standardize_refs)

  result$keywords <- split_field(
    get_col(c("Keywords", "MeSH Terms", "Fields of Study")), sep = ";"
  )
  result$keywords <- lapply(result$keywords, trimws)

  result
}

## tests/testthat/test-read-lens.R
## Coverage target: >=80% of R/read-lens.R

## ── helpers ──────────────────────────────────────────────────────────────────

## Build a minimal but realistic Lens.org CSV in a temp file.
## Column names are taken verbatim from R/read-lens.R get_col() calls.
make_lens_csv <- function(...) {
  header <- paste(
    "Lens ID", "Title", "Publication Year", "Source Title",
    "DOI", "Citing Works Count", "Abstract", "Publication Type",
    "Authors", "References", "Keywords",
    sep = ","
  )
  rows <- c(...)
  f <- tempfile(fileext = ".csv")
  writeLines(c(header, rows), f)
  f
}

## Escape a CSV field that may contain commas
q <- function(x) paste0('"', x, '"')

## ── standard columns ─────────────────────────────────────────────────────────

test_that("read_lens returns all standard bibnets columns", {
  row1 <- paste(
    "000-001", q("Network science"), "2021", q("Journal of Networks"),
    "10.1/net", "42", q("A study of networks."), "journal article",
    q("Smith, J.; Jones, M."), q("Doe, A. 2018; Roe, B. 2019"), q("networks; science"),
    sep = ","
  )
  f <- make_lens_csv(row1)
  d <- read_lens(f)

  expected_cols <- c("id", "title", "year", "journal", "doi",
                     "cited_by_count", "abstract", "type",
                     "authors", "references", "keywords")
  expect_true(all(expected_cols %in% names(d)))
})

## ── column types ─────────────────────────────────────────────────────────────

test_that("read_lens year is integer", {
  row1 <- paste(
    "000-001", q("A paper"), "2020", "Some Journal",
    "10.1/x", "5", q("Abstract here."), "journal article",
    q("Author A"), "", "",
    sep = ","
  )
  f <- make_lens_csv(row1)
  d <- read_lens(f)
  expect_type(d$year, "integer")
  expect_equal(d$year, 2020L)
})

test_that("read_lens cited_by_count is integer with no NAs", {
  row1 <- paste(
    "000-001", q("A paper"), "2020", "Some Journal",
    "10.1/x", "7", q("Abstract here."), "journal article",
    q("Author A"), "", "",
    sep = ","
  )
  f <- make_lens_csv(row1)
  d <- read_lens(f)
  expect_type(d$cited_by_count, "integer")
  expect_false(anyNA(d$cited_by_count))
  expect_equal(d$cited_by_count, 7L)
})

test_that("read_lens authors, references, keywords are list-columns", {
  row1 <- paste(
    "000-001", q("A paper"), "2020", "Some Journal",
    "10.1/x", "0", q("Abs."), "journal article",
    q("Smith, J.; Jones, M."), q("Ref A; Ref B"), q("kw1; kw2"),
    sep = ","
  )
  f <- make_lens_csv(row1)
  d <- read_lens(f)
  expect_true(is.list(d$authors))
  expect_true(is.list(d$references))
  expect_true(is.list(d$keywords))
})

## ── multi-value splitting ─────────────────────────────────────────────────────

test_that("read_lens splits authors on semicolon", {
  row1 <- paste(
    "000-001", q("A paper"), "2021", "Some Journal",
    "10.1/x", "3", q("Abstract."), "journal article",
    q("Smith, J.; Jones, M.; Doe, A."), "", "",
    sep = ","
  )
  f <- make_lens_csv(row1)
  d <- read_lens(f)
  expect_equal(length(d$authors[[1]]), 3L)
})

test_that("read_lens splits keywords on semicolon", {
  row1 <- paste(
    "000-001", q("A paper"), "2021", "Some Journal",
    "10.1/x", "0", q("Abstract."), "journal article",
    q("Author A"), "", q("network analysis; bibliometrics; citation"),
    sep = ","
  )
  f <- make_lens_csv(row1)
  d <- read_lens(f)
  expect_equal(length(d$keywords[[1]]), 3L)
})

test_that("read_lens splits references on semicolon", {
  row1 <- paste(
    "000-001", q("A paper"), "2021", "Some Journal",
    "10.1/x", "0", q("Abstract."), "journal article",
    q("Author A"), q("Ref A 2018; Ref B 2019; Ref C 2020"), "",
    sep = ","
  )
  f <- make_lens_csv(row1)
  d <- read_lens(f)
  expect_equal(length(d$references[[1]]), 3L)
})

## ── author standardization ────────────────────────────────────────────────────

test_that("read_lens uppercases author names", {
  row1 <- paste(
    "000-001", q("A paper"), "2021", "Some Journal",
    "10.1/x", "0", q("Abstract."), "journal article",
    q("Smith, Jane"), "", "",
    sep = ","
  )
  f <- make_lens_csv(row1)
  d <- read_lens(f)
  expect_equal(d$authors[[1]], "SMITH, JANE")
})

test_that("read_lens removes dots from author initials", {
  row1 <- paste(
    "000-001", q("A paper"), "2021", "Some Journal",
    "10.1/x", "0", q("Abstract."), "journal article",
    q("Smith, J.K."), "", "",
    sep = ","
  )
  f <- make_lens_csv(row1)
  d <- read_lens(f)
  expect_false(grepl("\\.", d$authors[[1]][1]))
})

## ── reference standardization ────────────────────────────────────────────────

test_that("read_lens uppercases references", {
  ## standardize_refs uppercases but does NOT strip dots (only standardize_authors does)
  row1 <- paste(
    "000-001", q("A paper"), "2021", "Some Journal",
    "10.1/x", "0", q("Abstract."), "journal article",
    "", q("doe, a. 2019"), "",
    sep = ","
  )
  f <- make_lens_csv(row1)
  d <- read_lens(f)
  expect_equal(d$references[[1]], "DOE, A. 2019")
})

## ── empty / missing fields ────────────────────────────────────────────────────

test_that("read_lens empty references field becomes empty list element", {
  row1 <- paste(
    "000-001", q("A paper"), "2021", "Some Journal",
    "10.1/x", "0", q("Abstract."), "journal article",
    q("Author A"), "", "",
    sep = ","
  )
  f <- make_lens_csv(row1)
  d <- read_lens(f)
  expect_equal(length(d$references[[1]]), 0L)
})

test_that("read_lens empty keywords field becomes empty list element", {
  row1 <- paste(
    "000-001", q("A paper"), "2021", "Some Journal",
    "10.1/x", "0", q("Abstract."), "journal article",
    q("Author A"), "", "",
    sep = ","
  )
  f <- make_lens_csv(row1)
  d <- read_lens(f)
  expect_equal(length(d$keywords[[1]]), 0L)
})

test_that("read_lens handles single-author record", {
  row1 <- paste(
    "000-002", q("Solo paper"), "2022", "Solo Journal",
    "10.2/solo", "1", q("Only one author."), "journal article",
    q("Lone, A."), "", q("solo"),
    sep = ","
  )
  f <- make_lens_csv(row1)
  d <- read_lens(f)
  expect_equal(nrow(d), 1L)
  expect_equal(length(d$authors[[1]]), 1L)
})

## ── multiple rows ─────────────────────────────────────────────────────────────

test_that("read_lens returns correct row count for multiple records", {
  row1 <- paste("000-001", q("Paper 1"), "2020", "Journal A",
                "10.1/a", "10", q("Abs 1"), "journal article",
                q("Alpha, A."), "", q("kw1"), sep = ",")
  row2 <- paste("000-002", q("Paper 2"), "2021", "Journal B",
                "10.1/b", "5",  q("Abs 2"), "journal article",
                q("Beta, B.; Gamma, G."), q("Ref X; Ref Y"), q("kw2; kw3"),
                sep = ",")
  row3 <- paste("000-003", q("Paper 3"), "2022", "Journal C",
                "", "0", q("Abs 3"), "conference paper",
                "", "", "",
                sep = ",")
  f <- make_lens_csv(row1, row2, row3)
  d <- read_lens(f)
  expect_equal(nrow(d), 3L)
})

test_that("read_lens preserves Lens ID values", {
  row1 <- paste("LNS-001", q("Paper 1"), "2020", "Journal A",
                "10.1/a", "10", q("Abs 1"), "journal article",
                q("Alpha, A."), "", q("kw1"), sep = ",")
  row2 <- paste("LNS-002", q("Paper 2"), "2021", "Journal B",
                "10.1/b", "5",  q("Abs 2"), "journal article",
                q("Beta, B."), "", "",
                sep = ",")
  f <- make_lens_csv(row1, row2)
  d <- read_lens(f)
  expect_equal(d$id, c("LNS-001", "LNS-002"))
})

## ── non-ASCII characters ──────────────────────────────────────────────────────

test_that("read_lens handles non-ASCII characters in title and abstract", {
  row1 <- paste(
    "000-005", q("Réseau éducatif"), "2023", "Revue Francophone",
    "10.5/fr", "2", q("Une étude sur les réseaux."), "journal article",
    q("Dupont, Jean-Pierre"), "", q("réseaux"),
    sep = ","
  )
  f <- make_lens_csv(row1)
  d <- read_lens(f)
  expect_equal(nrow(d), 1L)
  expect_true(grepl("é", d$title, fixed = TRUE))
})

## ── alternate column name fallbacks ──────────────────────────────────────────

test_that("read_lens accepts 'Author/s' instead of 'Authors'", {
  ## Use the alternate Lens column name
  header <- paste(
    "Lens ID", "Title", "Publication Year", "Source Title",
    "DOI", "Citing Works Count", "Abstract", "Publication Type",
    "Author/s", "References", "Keywords",
    sep = ","
  )
  row1 <- paste(
    "000-010", q("Alt author paper"), "2021", "Alt Journal",
    "10.1/alt", "3", q("Abstract alt."), "journal article",
    q("Alt, A.; Alt, B."), "", "",
    sep = ","
  )
  f <- tempfile(fileext = ".csv")
  writeLines(c(header, row1), f)
  d <- read_lens(f)
  expect_equal(length(d$authors[[1]]), 2L)
})

test_that("read_lens accepts 'Cited Works' instead of 'References'", {
  header <- paste(
    "Lens ID", "Title", "Publication Year", "Source Title",
    "DOI", "Citing Works Count", "Abstract", "Publication Type",
    "Authors", "Cited Works", "Keywords",
    sep = ","
  )
  row1 <- paste(
    "000-011", q("Cited works test"), "2020", "Test Journal",
    "10.1/cw", "1", q("Abstract cw."), "journal article",
    q("Test, T."), q("Ref One; Ref Two"), "",
    sep = ","
  )
  f <- tempfile(fileext = ".csv")
  writeLines(c(header, row1), f)
  d <- read_lens(f)
  expect_equal(length(d$references[[1]]), 2L)
})

test_that("read_lens accepts 'MeSH Terms' instead of 'Keywords'", {
  header <- paste(
    "Lens ID", "Title", "Publication Year", "Source Title",
    "DOI", "Citing Works Count", "Abstract", "Publication Type",
    "Authors", "References", "MeSH Terms",
    sep = ","
  )
  row1 <- paste(
    "000-012", q("MeSH test"), "2019", "Medical Journal",
    "10.1/med", "8", q("Abstract med."), "journal article",
    q("Med, M."), "", q("brain; neuron; cortex"),
    sep = ","
  )
  f <- tempfile(fileext = ".csv")
  writeLines(c(header, row1), f)
  d <- read_lens(f)
  expect_equal(length(d$keywords[[1]]), 3L)
})

test_that("read_lens accepts 'Fields of Study' instead of 'Keywords'", {
  header <- paste(
    "Lens ID", "Title", "Publication Year", "Source Title",
    "DOI", "Citing Works Count", "Abstract", "Publication Type",
    "Authors", "References", "Fields of Study",
    sep = ","
  )
  row1 <- paste(
    "000-013", q("FoS test"), "2022", "Science Journal",
    "10.1/fos", "4", q("Abstract fos."), "journal article",
    q("Fos, F."), "", q("physics; mathematics"),
    sep = ","
  )
  f <- tempfile(fileext = ".csv")
  writeLines(c(header, row1), f)
  d <- read_lens(f)
  expect_equal(length(d$keywords[[1]]), 2L)
})

test_that("read_lens accepts 'Year of Publication' instead of 'Publication Year'", {
  header <- paste(
    "Lens ID", "Title", "Year of Publication", "Source Title",
    "DOI", "Citing Works Count", "Abstract", "Publication Type",
    "Authors", "References", "Keywords",
    sep = ","
  )
  row1 <- paste(
    "000-014", q("Year test"), "2018", "Old Journal",
    "10.1/yr", "0", q("Abstract yr."), "journal article",
    q("Old, O."), "", q("history"),
    sep = ","
  )
  f <- tempfile(fileext = ".csv")
  writeLines(c(header, row1), f)
  d <- read_lens(f)
  expect_equal(d$year, 2018L)
})

test_that("read_lens accepts 'Document Type' instead of 'Publication Type'", {
  header <- paste(
    "Lens ID", "Title", "Publication Year", "Source Title",
    "DOI", "Citing Works Count", "Abstract", "Document Type",
    "Authors", "References", "Keywords",
    sep = ","
  )
  row1 <- paste(
    "000-015", q("DocType test"), "2023", "Doc Journal",
    "10.1/dt", "2", q("Abstract dt."), "conference paper",
    q("Doc, D."), "", q("doc"),
    sep = ","
  )
  f <- tempfile(fileext = ".csv")
  writeLines(c(header, row1), f)
  d <- read_lens(f)
  expect_equal(d$type, "conference paper")
})

test_that("read_lens accepts 'Cited By Count' instead of 'Citing Works Count'", {
  header <- paste(
    "Lens ID", "Title", "Publication Year", "Source Title",
    "DOI", "Cited By Count", "Abstract", "Publication Type",
    "Authors", "References", "Keywords",
    sep = ","
  )
  row1 <- paste(
    "000-016", q("Count test"), "2021", "Count Journal",
    "10.1/ct", "99", q("Abstract ct."), "journal article",
    q("Count, C."), "", q("counting"),
    sep = ","
  )
  f <- tempfile(fileext = ".csv")
  writeLines(c(header, row1), f)
  d <- read_lens(f)
  expect_equal(d$cited_by_count, 99L)
})

test_that("read_lens uses synthetic Lens ID when 'ID' column is present but no 'Lens ID'", {
  header <- paste(
    "ID", "Title", "Publication Year", "Source Title",
    "DOI", "Citing Works Count", "Abstract", "Publication Type",
    "Authors", "References", "Keywords",
    sep = ","
  )
  row1 <- paste(
    "ID-999", q("ID fallback"), "2020", "Fallback Journal",
    "10.1/fb", "0", q("Abstract fb."), "journal article",
    q("Fallback, F."), "", "",
    sep = ","
  )
  f <- tempfile(fileext = ".csv")
  writeLines(c(header, row1), f)
  d <- read_lens(f)
  expect_equal(d$id, "ID-999")
})

## ── missing columns fallback to NA ────────────────────────────────────────────

test_that("read_lens falls back to NA for missing optional columns", {
  ## Minimal CSV: only Lens ID and Title; everything else absent
  f <- tempfile(fileext = ".csv")
  writeLines(c(
    "Lens ID,Title",
    "000-020,Minimal paper"
  ), f)
  d <- read_lens(f)
  expect_equal(nrow(d), 1L)
  expect_true(is.na(d$doi))
  expect_true(is.na(d$abstract))
  expect_true(is.na(d$journal))
})

test_that("read_lens assigns synthetic LENS-prefixed IDs when no ID column present", {
  f <- tempfile(fileext = ".csv")
  writeLines(c(
    "Title,Publication Year",
    "Solo Paper,2021"
  ), f)
  d <- read_lens(f)
  expect_equal(d$id[[1]], "LENS1")
  expect_true(grepl("^LENS", d$id[[1]]))
})

test_that("read_lens synthetic IDs do not duplicate rows for multi-row files (regression)", {
  ## Regression for previously-observed n^2 row inflation when neither 'Lens ID'
  ## nor 'ID' columns existed and the default ID vector was passed to rep(default, n).
  f <- tempfile(fileext = ".csv")
  writeLines(c(
    "Title,Publication Year",
    "Paper A,2020",
    "Paper B,2021",
    "Paper C,2022"
  ), f)
  d <- read_lens(f)
  expect_equal(nrow(d), 3L)
  expect_equal(d$id, c("LENS1", "LENS2", "LENS3"))
})

## ── error on missing file ─────────────────────────────────────────────────────

test_that("read_lens errors on non-existent file", {
  expect_error(read_lens("no_such_file_lens.csv"), "File not found")
})

## ── read_biblio auto-detection ────────────────────────────────────────────────

test_that("read_biblio auto-detects lens format via 'Lens ID' header", {
  row1 <- paste("LNS-100", q("Auto-detect paper"), "2021", "Some Journal",
                "10.1/ad", "2", q("Abstract."), "journal article",
                q("Smith, A."), "", q("detection"),
                sep = ",")
  f <- make_lens_csv(row1)
  d <- read_biblio(f)
  expect_equal(nrow(d), 1L)
  expect_true(is.list(d$authors))
})

test_that("read_biblio with format='lens' works explicitly", {
  row1 <- paste("LNS-200", q("Explicit lens"), "2022", "Explicit Journal",
                "10.1/ex", "5", q("Abstract."), "journal article",
                q("Explicit, E."), "", q("explicit"),
                sep = ",")
  f <- make_lens_csv(row1)
  d <- read_biblio(f, format = "lens")
  expect_equal(nrow(d), 1L)
  expect_equal(d$id, "LNS-200")
})

#' Read Dimensions CSV export
#'
#' Parses a CSV file exported from Dimensions into a standardized
#' bibliometric data frame.
#'
#' @param file Path to a Dimensions CSV export file.
#' @param encoding Character. File encoding. Default `"UTF-8"`.
#'
#' @return A data frame in the standard bibnets format: `id`, `title`,
#'   `year`, `journal`, `doi`, `cited_by_count`, `abstract`, `type`,
#'   plus list-columns `authors`, `references`, and `keywords`.
#'   Dimensions-specific extras: `affiliations` (list-column),
#'   `countries` (list-column).
#'
#' @export
#' @examples
#' f <- system.file("extdata", "dimensions_sample.csv", package = "bibnets")
#' data <- read_dimensions(f)
#' head(data[, c("id", "title", "year", "journal")])
read_dimensions <- function(file, encoding = "UTF-8") {
  check_file(file)

  ## Dimensions CSVs include a one-line metadata header ("About the data: ...")
  ## before the actual column-name row. Detect and skip it.
  first_line <- readLines(file, n = 1L, encoding = encoding, warn = FALSE)
  skip_rows <- if (grepl("^\"?About the data", first_line)) 1L else 0L

  raw <- utils::read.csv(file, stringsAsFactors = FALSE, fileEncoding = encoding,
                          check.names = FALSE, skip = skip_rows)

  col_map <- list(
    id       = c("Publication ID", "Dimensions URL"),
    title    = c("Title"),
    year     = c("PubYear", "Publication Year", "Year"),
    journal  = c("Source title", "Source Title",
                  "Source title/Anthology title"),
    doi      = c("DOI"),
    cited_by = c("Times cited", "Times Cited", "Citation Count"),
    abstract = c("Abstract"),
    type     = c("Publication Type", "Document Type"),
    authors  = c("Authors"),
    refs     = c("Cited references", "Cited References", "References"),
    affiliations = c("Authors Affiliations - Name of Research organization",
                      "Authors Affiliations Name of Research organization",
                      "Research Organizations - standardized"),
    countries = c("Authors Affiliations - Country of Research organization",
                   "Authors Affiliations Country of Research organization",
                   "Countries of Research organization")
  )

  lower_names <- tolower(names(raw))
  find_col <- function(candidates) {
    idx <- match(tolower(candidates), lower_names)
    idx <- idx[!is.na(idx)]
    if (length(idx) == 0L) return(NA_character_)
    names(raw)[idx[1L]]
  }

  get_col <- function(candidates, default = NA_character_) {
    col_name <- find_col(candidates)
    if (is.na(col_name)) return(rep(default, nrow(raw)))
    raw[[col_name]]
  }

  ## Build ID
  id_raw <- get_col(col_map$id, NA_character_)
  id <- ifelse(is.na(id_raw) | nchar(id_raw) == 0,
               paste0("DIM", seq_len(nrow(raw))),
               id_raw)

  result <- data.frame(
    id = id,
    title = get_col(col_map$title),
    year = as.integer(get_col(col_map$year, NA_integer_)),
    journal = get_col(col_map$journal),
    doi = get_col(col_map$doi),
    cited_by_count = as.integer(get_col(col_map$cited_by, 0L)),
    abstract = get_col(col_map$abstract),
    type = get_col(col_map$type),
    stringsAsFactors = FALSE
  )

  ## Authors: semicolon-delimited
  result$authors <- lapply(
    split_field(get_col(col_map$authors), sep = ";"),
    standardize_authors
  )

  ## References: Dimensions uses semicolons before brackets or just semicolons
  refs_raw <- get_col(col_map$refs)
  result$references <- lapply(refs_raw, function(r) {
    if (is.na(r) || nchar(trimws(r)) == 0) return(character(0))
    r <- gsub("\\[([^]]+)\\]", "\\1", r)
    standardize_refs(strsplit(r, ";")[[1]])
  })

  ## Keywords: Dimensions may have "Fields of Study" or "Research Categories"
  kw_raw <- get_col(c("Fields of Study", "Keywords", "RCDC Categories",
                        "Research Categories"))
  result$keywords <- split_field(kw_raw, sep = ";")

  ## Source-specific extras — both are list-columns so institution_network()
  ## and country_network() can use them directly
  result$affiliations <- split_field(get_col(col_map$affiliations), sep = ";")
  result$countries <- split_field(get_col(col_map$countries), sep = ";")

  result
}

---
title: "Reading bibliometric data into bibnets"
output: rmarkdown::html_vignette
vignette: >
  %\VignetteIndexEntry{Reading bibliometric data into bibnets}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

```{r, include = FALSE}
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>"
)
```

```{r setup}
library(bibnets)
```

## 1. Introduction and the standard schema

Each network builder in `bibnets` (`author_network()`, `keyword_network()`,
`reference_network()`, `document_network()`, `source_network()`,
`country_network()`, `institution_network()`, `conetwork()`) requires a
data frame with a fixed set of columns and a small number of list-columns
holding multi-valued fields. The readers convert source-specific exports
(Scopus CSV, Web of Science plaintext, OpenAlex flat or nested,
Dimensions, Lens.org, BibTeX, RIS, Crossref) into this common
representation.

The standard schema returned by every reader is:

| Column | Type | Meaning |
|---|---|---|
| `id` | chr | Document identifier (EID, OpenAlex W-ID, DOI, etc.) |
| `title` | chr | Document title |
| `year` | int | Publication year |
| `journal` | chr | Source / journal / venue name |
| `doi` | chr | DOI without the `https://doi.org/` prefix |
| `cited_by_count` | int | Citations received (as reported by source) |
| `abstract` | chr | Abstract text; `NA` for sources that do not expose it |
| `type` | chr | Document type (article, review, book-chapter, ...) |
| `authors` | list | Character vector of author names per row |
| `references` | list | Character vector of cited references per row |
| `keywords` | list | Character vector of keywords per row |

Source-specific extras (e.g. `index_keywords`, `keywords_plus`,
`affiliations`, `countries`, `language`) follow the standard columns.
The contract that downstream functions such as `build_bipartite()` and
`author_network()` rely on is that each multi-valued field is stored as a
list-column whose elements are character vectors.

This vignette covers all nine readers, the `read_biblio()` entry point,
the generic-CSV path, the `split_field()` helper, and the manual
construction of bibnets-compatible data frames.

## 2. `read_biblio()`

`read_biblio()` accepts a single file, a vector of file paths, or a
directory. When `format = "auto"` (the default) it detects the format
from the contents of the file:

```{r read-biblio-signature, eval = FALSE}
data <- read_biblio("export.csv")          # auto-detect format
data <- read_biblio("scopus_dir/")         # entire directory, rbind'd
data <- read_biblio(c("a.csv", "b.csv"))   # multiple files, rbind'd
data <- read_biblio("file.csv", format = "scopus")   # force a format
```

When given a directory, `read_biblio()` collects every `.csv`, `.txt`,
`.bib`, `.ris`, `.xls`, and `.xlsx` file in it, reads each one, and
combines the results with `rbind()`. For more than one file a summary
message is emitted:

```
Read 3 files: 1247 rows total
```

Format detection is performed on the first non-empty line of the file:

- BibTeX: line 1 begins with `@`
- RIS: line 1 begins with `TY  -`
- Web of Science plaintext: line 1 begins with `FN ` or `PT `
- CSV-based: detection inspects the header row. When the first line
  matches the Dimensions preamble (`"About the data: ..."`), line 2 is
  used instead. Header tokens determine the format: `eid` for Scopus,
  `lens id` for Lens.org, `publication id` or `dimensions url` for
  Dimensions, `authorships.author.display_name` for the OpenAlex flat
  CSV.

If detection fails, `read_biblio()` raises an error that lists the
supported formats and indicates how to pass `format` explicitly or use
`format = "generic"` with `actors`.

Two readers are not dispatched by `read_biblio()`:

- `read_openalex()` accepts an in-memory tibble from
  `openalexR::oa_fetch()`, not a file path.
- `read_crossref()` accepts the `data` element of `rcrossref::cr_works()`.

Both take R objects rather than files and are called directly.

## 3. Worked example — OpenAlex flat CSV

The package includes a 30-row OpenAlex flat CSV at
`inst/extdata/openalex_works.csv`, corresponding to the export produced
by downloading "Works" results from the OpenAlex web interface.
Multi-valued fields use `|` as the delimiter.

```{r openalex-csv-demo}
f <- system.file("extdata", "openalex_works.csv", package = "bibnets")
oa <- read_openalex_csv(f)
str(oa, max.level = 1)
head(oa[, c("id", "title", "year", "journal", "type")], 5)
```

The list-columns:

```{r openalex-csv-lists}
oa$authors[[1]]
oa$affiliations[[1]]
oa$countries[[1]]
```

Two limitations of the flat export should be noted:

```{r openalex-csv-limits}
all(vapply(oa$references, function(x) length(x) == 0 || all(is.na(x)), logical(1)))
all(is.na(oa$abstract))
```

`references` is empty and `abstract` is `NA` because these fields are
not included in the OpenAlex web download. Cited references and
abstracts from OpenAlex are obtained via `openalexR::oa_fetch()` and
processed with `read_openalex()` (described in section 6).

The remaining fields support several network constructions that do not
require references — co-authorship, country, institution, keyword,
source, and document networks:

```{r openalex-csv-networks}
co <- country_network(oa, counting = "fractional")
head(co, 5)
```

## 4. Scopus

```{r scopus-call, eval = FALSE}
sc <- read_scopus("scopus.csv")
```

`read_scopus()` ingests the standard Scopus CSV export (`File -> Export ->
CSV` from the Scopus search UI). Mappings from Scopus columns to the
bibnets schema:

| Scopus column | Standard column |
|---|---|
| `EID` (or `Article No.`) | `id` |
| `Title` | `title` |
| `Year` | `year` |
| `Source title` | `journal` |
| `DOI` | `doi` (prefix stripped) |
| `Cited by` | `cited_by_count` |
| `Abstract` | `abstract` |
| `Document Type` | `type` |
| `Authors` (`;`-delimited) | `authors` (list) |
| `References` (`;`-delimited) | `references` (list) |
| `Author Keywords` (`;`-delimited) | `keywords` (list) |
| `Index Keywords` (`;`-delimited) | `index_keywords` (list, extra) |
| `Affiliations` (`;`-delimited) | `affiliations` (list, extra) |
| `Language of Original Document` | `language` (extra) |

Scopus stores each cited reference as one semicolon-delimited string in a
single cell. `read_scopus()` splits on `;` and applies
`standardize_refs()` to each entry: uppercasing, whitespace
normalisation, and removal of a trailing DOI where present. References
differing only in case or trailing DOI then resolve to the same node in
co-citation and reference networks.

## 5. Web of Science

WoS exports come in two shapes:

```{r wos-call, eval = FALSE}
wos1 <- read_wos("savedrecs.txt")                       # plaintext (default)
wos2 <- read_wos("savedrecs.tsv", format = "tab")       # tab-delimited
```

The plaintext format is a tagged record syntax. Each record begins with a
`PT` (publication type) tag and ends with `ER` (end record). Within the
record, every field is introduced by a 2-letter tag at the start of a
line, with continuation lines indented:

| Tag | Field |
|---|---|
| `AU` | Authors (one per line) |
| `TI` | Title |
| `SO` | Source / journal |
| `PY` | Year |
| `DI` | DOI |
| `TC` | Times cited |
| `AB` | Abstract |
| `DT` | Document type |
| `DE` | Author keywords |
| `ID` | Keywords plus (extra: `keywords_plus`) |
| `CR` | Cited references (one per line) |

`read_wos()` walks the file, splitting on `ER` boundaries, and emits one
row per record. The tab-delimited variant carries the same fields in a
flat CSV-like grid. Either way the output schema is identical.

## 6. OpenAlex — two paths

OpenAlex ships data through two routes that bibnets supports separately.

### Path A: in-memory tibble from `openalexR`

This path is used when references and abstracts are required.
`openalexR::oa_fetch()` returns a nested tibble with `author`,
`referenced_works`, `concepts`, and `keywords` list-columns;
`read_openalex()` converts it to the standard schema:

```{r openalex-fetch, eval = FALSE}
library(openalexR)
raw  <- oa_fetch(entity = "works", search = "learning analytics", per_page = 200)
data <- read_openalex(raw)
```

References are returned as OpenAlex Work IDs (e.g. `W2769342982`) rather
than formatted citation strings. The IDs are stable identifiers suitable
for co-citation and direct-citation networks; visualisations that need
human-readable labels can join the IDs back to titles in a separate step.

### Path B: flat CSV

The `read_openalex_csv()` reader, demonstrated in section 3, applies to
the file format produced by the OpenAlex web interface. References and
abstracts are not present in this format.

## 7. Dimensions

```{r dimensions-call, eval = FALSE}
dm <- read_dimensions("dimensions_export.csv")
```

The Dimensions CSV begins with a metadata row of the form

```
"About the data: This export was generated on YYYY-MM-DD ..."
```

before the column header. `read_dimensions()` detects this preamble and
skips it. If the line has been removed (for example, by manual editing
of the file), the reader continues to function because it identifies the
column row by the Dimensions header tokens `Publication ID` and
`Dimensions URL`.

Extras returned: `affiliations` and `countries` as list-columns,
analogous to the OpenAlex schema.

## 8. Lens.org

```{r lens-call, eval = FALSE}
ln <- read_lens("lens_export.csv")
```

Key Lens columns and how they map:

| Lens column | Standard column |
|---|---|
| `Lens ID` | `id` |
| `Title` | `title` |
| `Publication Year` | `year` |
| `Source Title` | `journal` |
| `DOI` | `doi` |
| `Cited by Count` | `cited_by_count` |
| `Abstract` | `abstract` |
| `Publication Type` | `type` |
| `Author/s` | `authors` (list) |
| `Reference Identifiers` | `references` (list) |
| `Keywords` | `keywords` (list) |

## 9. BibTeX & RIS

```{r bibtex-ris-call, eval = FALSE}
bt <- read_bibtex("library.bib")
ri <- read_ris("savedrecs.ris")
```

`read_bibtex()` parses `@type{key, field = {value}, ...}` blocks.
`read_ris()` parses tagged `TY  - ... ER  -` blocks; the structure is
equivalent to WoS plaintext, but with a different tag dictionary.

Standard BibTeX and RIS do not contain cited-reference data, so the
`references` column in the resulting data frame is empty on every row.
These formats are sufficient for co-authorship and keyword co-occurrence
networks. For co-citation, coupling, or direct citation networks, the
appropriate sources are Scopus, Web of Science, OpenAlex (via
`oa_fetch()`), Dimensions, Lens, or Crossref.

## 10. Crossref via rcrossref

```{r crossref-call, eval = FALSE}
library(rcrossref)
raw  <- cr_works(query = "graph neural networks", limit = 100)
data <- read_crossref(raw$data)
```

`read_crossref()` accepts the `data` element of the `cr_works()` result
(a data frame, not the wrapping list). The function handles the two
field-naming variants Crossref returns (`container.title` vs
`container-title`; `is.referenced.by.count` vs
`is-referenced-by-count`) and maps both to the standard schema.

## 11. Generic CSV — `read_biblio(format = "generic", ...)`

For CSV files that do not match any of the recognised signatures
(in-house exports, custom dumps, public datasets), the generic path
provides explicit column-name mapping. The identifier column is named
via `id`; columns to be treated as list-columns are named via `actors`.
`sep` is the delimiter applied inside those cells.

Hypothetical call:

```{r generic-call, eval = FALSE}
data <- read_biblio(
  "my_data.csv",
  format  = "generic",
  id      = "doc_id",
  actors  = c("Authors", "Keywords"),
  sep     = ";"
)
```

Demonstrated on the bundled OpenAlex CSV (which uses `|` as the delimiter):

```{r generic-demo}
f <- system.file("extdata", "openalex_works.csv", package = "bibnets")
generic <- read_biblio(
  f,
  format = "generic",
  id     = "id",
  actors = c("authorships.author.display_name", "primary_topic.display_name"),
  sep    = "|"
)
names(generic)[1:6]
generic$authorships.author.display_name[[1]]
```

The named `id` column is copied to a top-level `id`. Each column listed
in `actors` is split on `sep` and stored as a list-column. Other columns
are retained unchanged. The resulting frame is therefore *not* in the
standard schema; it is a wider source-specific table. Network
constructors can either be pointed at the relevant columns directly, or
the frame can be post-processed into the standard schema.

## 12. Building data manually

When data does not come from any of the supported sources, a
bibnets-compatible data frame can be constructed directly. The
requirement is: standard scalar columns are character or integer;
multi-valued fields are list-columns whose elements are character
vectors.

```{r manual-build}
df <- data.frame(
  id    = c("p1", "p2", "p3"),
  title = c("Paper A", "Paper B", "Paper C"),
  year  = c(2020L, 2021L, 2022L),
  stringsAsFactors = FALSE
)
df$authors <- list(
  c("ALICE", "BOB"),
  c("BOB", "CAROL"),
  c("ALICE", "CAROL", "DAVE")
)
df$references <- list(
  c("R1", "R2"),
  c("R1", "R3"),
  c("R2", "R3", "R4")
)
df$keywords <- list(
  c("graph", "network"),
  c("network", "embedding"),
  c("graph", "embedding", "neural")
)

author_network(df, "collaboration")
keyword_network(df)
reference_network(df)
```

`build_bipartite()` applies `toupper(trimws(...))` to every entity label
before constructing the sparse matrix, so `"graph"`, `"Graph"`, and
`"GRAPH"` are mapped to the same node `"GRAPH"`. Tests or comparisons
that reference node names should use uppercase strings.

## 13. The `split_field()` helper

`split_field()` converts a character column with semicolon-delimited (or
otherwise delimited) values into a list-column without going through
`read_biblio(format = "generic")`:

```{r split-field-demo}
split_field(c("Alice; Bob; Carol", "Dave; Eve"))
split_field(c("a|b|c", "d|e"), sep = "|")
```

This is the same operation that `read_scopus()` and the other readers
apply internally to multi-valued columns; it is exported for use in
custom pipelines.

## 14. Combining data from multiple sources

Different readers expose different extras: WoS provides `keywords_plus`,
Scopus provides `index_keywords`, OpenAlex provides `countries`. To
combine sources, restrict each frame to the standard columns and bind:

```{r combine-sources}
common <- c("id", "title", "year", "journal", "doi", "cited_by_count",
            "abstract", "type", "authors", "references", "keywords")

data(biblio_data)
b1 <- biblio_data
b2 <- biblio_data
b2$id <- paste0(b2$id, "_dup")

cols <- intersect(common, names(b1))
combined <- rbind(b1[, cols], b2[, cols])
nrow(combined)
```

Two practical notes:

1. When document IDs overlap across sources (which occurs when Scopus and
   WoS both index the same article), prefixing the IDs as shown
   prevents duplicate documents from inflating co-occurrence counts.
2. Source-specific extras (e.g. WoS `keywords_plus`) should be retained
   on the per-source frame and merged selectively rather than coerced
   into the combined frame.

## 15. Inspecting and sanity-checking

After reading, basic checks on the list-column sizes and the scalar
columns help detect silent corruption. Empty list-columns and
out-of-range years are common indicators that an export is incomplete.

```{r sanity-check}
data(scopus_quantum_cloud)
sc <- scopus_quantum_cloud

range(lengths(sc$authors))
range(lengths(sc$references))
range(lengths(sc$keywords))

head(sort(table(sc$journal), decreasing = TRUE), 5)
range(sc$year, na.rm = TRUE)
table(sc$type)
```

Indicators to check:

- A `lengths()` of `0` on every row of `references` for a Scopus or WoS
  file indicates that the export did not include the references column.
  Re-export from the source with the references field selected.
- A year of `0` or `NA` indicates an empty source field.
- A single dominant document type (e.g. only `"article"`) is expected
  for filtered searches; broader mixes are expected for thematic
  searches.

## 16. Troubleshooting

| Symptom | Cause | Fix |
|---|---|---|
| `Could not detect file format` | First line doesn't match any signature | Pass `format = "scopus"` (etc.) explicitly, or use `format = "generic"` with `actors` |
| Empty `references` list on every row | BibTeX/RIS or OpenAlex flat CSV — these don't carry citations | Use Scopus/WoS, OpenAlex via `oa_fetch()`, Dimensions, Lens, or Crossref |
| `Invalid multibyte string` on read | Wrong encoding | Most readers accept `encoding = "latin1"`; pass it through `read_biblio(..., encoding = "latin1")` |
| Author names look like `LASTNAME, F.J.` not `FJ LASTNAME` | Default is `flip_names = FALSE` | The reader returns names as-is from the source. Cluster them by string match downstream, or pass `flip_names = TRUE` if all names follow `Last, First` |
| Dimensions file silently fails | "About the data" preamble removed and column header edited | `read_dimensions()` detects the standard preamble and falls back to header-token detection; the failure mode requires the column header itself to have been edited |
| Co-authorship network contains duplicate nodes (e.g. `"Alice"` and `"ALICE"`) | Mixed casing in the source | The standard readers and `build_bipartite()` apply `toupper(trimws(...))` to entity labels. Manually constructed frames should apply the same normalisation |

## Further reading

- The companion vignette, `vignette("bibnets")`, covers network
  construction on the in-package datasets.
- All network builders (`author_network()`, `keyword_network()`,
  `reference_network()`, `document_network()`, `source_network()`,
  `country_network()`, `institution_network()`, `conetwork()`) accept
  the same set of arguments (`type`, `counting`, `similarity`,
  `threshold`, `top_n`, `format`), so switching between network types
  on data already in the standard schema requires only a function-name
  change.

---
title: "Getting Started with bibnets"
output: rmarkdown::html_vignette
vignette: >
  %\VignetteIndexEntry{Getting Started with bibnets}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

```{r, include = FALSE}
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>"
)
```

```{r setup}
library(bibnets)
```

## What bibnets Builds

Bibliometric data usually arrives as a table of papers. Each paper has fields
such as authors, references, keywords, countries, institutions, source title,
and year. Most bibliometric networks are projections of those fields.

The core idea is simple:

1. Build a sparse `papers x entities` matrix.
2. Weight the matrix with a counting method.
3. Multiply the matrix to obtain entity-entity or paper-paper links.
4. Return a standard edge list.

Internally, this is the package pipeline:

```
build_bipartite()
  -> apply_counting()
  -> multiply_bipartite()
  -> as_bibnets_network()
```

The exported builders wrap that pipeline for common bibliometric questions:

| Function | Nodes | Link means |
|---|---|---|
| `author_network()` | authors | co-authorship, author coupling, or author co-citation |
| `reference_network()` | cited references | references are cited together |
| `document_network()` | documents | shared references, shared citers, or direct citation |
| `keyword_network()` | keywords | keywords appear together in papers |
| `source_network()` | journals/sources | sources share references or are co-cited |
| `country_network()` | countries | countries collaborate or share references |
| `institution_network()` | institutions | institutions collaborate or share references |
| `conetwork()` | any field | entities co-occur or share values of another field |
| `local_citations()` | documents | local citation counts inside the corpus |
| `historiograph()` | documents | directed citation history among locally cited papers |
| `temporal_network()` | any builder's nodes | the same network repeated over time windows |

Every network builder returns a `bibnets_network`: a data frame with columns
`from`, `to`, `weight`, and `count`.

`count` is the raw binary co-occurrence. `weight` is the analytical weight after
counting and optional similarity normalization.

## Data Used in This Vignette

The package includes small and medium example datasets:

```{r}
data(biblio_data)
data(scopus_quantum_cloud)
data(open_alex_gold_open_access_learning_analytics)

small <- biblio_data
sc <- scopus_quantum_cloud
oa <- open_alex_gold_open_access_learning_analytics

nrow(small)
nrow(sc)
nrow(oa)
```

`biblio_data` is a tiny synthetic dataset. `scopus_quantum_cloud` contains 499
Scopus records. `open_alex_gold_open_access_learning_analytics` contains 1,508
OpenAlex records with authors, countries, institutions, and primary topics.

## Reading Your Own Data

For files, use `read_biblio()`:

```{r, eval = FALSE}
data <- read_biblio("export.csv")
data <- read_biblio("folder_with_exports/")
data <- read_biblio(c("part_1.csv", "part_2.csv"))
```

`read_biblio()` detects common formats from file content. You can also call a
reader directly:

```{r, eval = FALSE}
read_scopus("scopus.csv")
read_wos("savedrecs.txt")
read_openalex_csv("openalex_works.csv")
read_dimensions("dimensions.csv")
read_lens("lens.csv")
read_bibtex("library.bib")
read_ris("library.ris")
```

For a custom CSV, specify the identifier and the columns that should be split
into list-columns:

```{r, eval = FALSE}
data <- read_biblio(
  "custom.csv",
  format = "generic",
  id = "paper_id",
  actors = c("Authors", "Keywords"),
  sep = ";"
)
```

Readers return a common schema where possible:

```{r}
names(sc)[1:12]
```

The most important columns for network construction are:

- `id`: the document identifier.
- `authors`: a list-column of author names.
- `references`: a list-column of cited references or cited work IDs.
- `keywords`: a list-column of author/index/topic keywords.
- `year`: the time variable used by `temporal_network()`.

Source-specific columns such as `countries`, `affiliations`, `index_keywords`,
and `keywords_plus` are preserved when available.

## Author Collaboration

The simplest author network links two authors when they appear on the same
paper:

```{r}
authors_full <- author_network(oa, type = "collaboration")
head(authors_full, 5)
```

The printed result has the standard schema:

```{r}
summary(authors_full)
```

Use `min_occur` to remove very rare authors before projection:

```{r}
authors_core <- author_network(oa, "collaboration", min_occur = 2)
nrow(authors_full)
nrow(authors_core)
```

## Counting Methods

Counting determines how much a paper contributes to edge weights.

Full counting gives every observed co-occurrence a weight of 1:

```{r}
head(author_network(small, "collaboration", counting = "full"), 5)
```

Fractional counting reduces the influence of long author lists:

```{r}
head(author_network(small, "collaboration", counting = "fractional"), 5)
```

Harmonic counting gives more credit to earlier byline positions while keeping
the paper's total credit normalized:

```{r}
head(author_network(small, "collaboration", counting = "harmonic"), 5)
```

First-last counting is useful only when the field's authorship conventions make
both first and last positions meaningful:

```{r}
head(author_network(small, "collaboration", counting = "first_last"), 5)
```

The correct method depends on the claim being made. Use `full` when the
question is about observed collaboration events. Use `fractional` when papers
with many entities should not dominate. Use position-dependent methods only
when author order is analytically meaningful.

## Attention-Style Position Weights

The `attention` argument applies a smooth position profile. It is separate from
`counting` and is available for author, keyword, country, and institution
networks.

```{r}
lead <- author_network(small, attention = "lead")
last <- author_network(small, attention = "last")

head(lead, 5)
head(last, 5)
```

The four profiles are:

| `attention` | Highest weight |
|---|---|
| `"lead"` | first position |
| `"last"` | last position |
| `"proximity"` | middle positions |
| `"circular"` | first and last positions |

Use attention weighting when the analysis needs a transparent positional
assumption rather than a named bibliometric counting convention.

## Reference Co-citation

Co-citation links two cited references when they are cited together by at least
one paper:

```{r}
refs <- reference_network(sc, min_occur = 2)
head(refs, 5)
```

Co-citation is a column-mode projection of the `papers x references` matrix.
The nodes are references; the links come from papers that cite both references.

Similarity normalization can reduce the advantage of very frequently cited
references:

```{r}
refs_cos <- reference_network(sc, min_occur = 2, similarity = "cosine")
head(refs_cos, 5)
```

## Document Coupling and Citation

Bibliographic coupling links two documents when they share cited references:

```{r}
coupled_docs <- document_network(sc, type = "coupling", similarity = "cosine")
head(coupled_docs, 5)
```

Direct citation is different. It keeps direction: `from` is the citing document
and `to` is the cited document, but only when both documents are inside the same
corpus.

```{r}
direct_docs <- document_network(sc, type = "citation")
head(direct_docs, 5)
```

Many exported datasets cite external works that are not themselves rows in the
dataset. Those external citations support co-citation and coupling, but they do
not become direct-citation edges unless the cited work is also present in `id`.

## Keyword Co-occurrence

Keyword networks are often the quickest way to inspect a corpus thematically:

```{r}
kw <- keyword_network(sc, min_occur = 2)
head(kw, 5)
```

Entity labels are trimmed and uppercased during matrix construction. This means
that `machine learning`, `Machine Learning`, and ` MACHINE LEARNING ` resolve
to the same node.

Association strength is commonly useful for co-occurrence maps because it
downweights pairs that are common only because both keywords are individually
frequent:

```{r}
kw_assoc <- keyword_network(sc, min_occur = 2, similarity = "association")
head(kw_assoc, 5)
```

## Countries, Institutions, and Sources

OpenAlex-style data often contains country and institution list-columns:

```{r}
country_edges <- country_network(oa, counting = "fractional")
head(country_edges, 5)

inst_edges <- institution_network(oa, counting = "fractional", min_occur = 2)
head(inst_edges, 5)
```

Source networks use `journal` as the entity field. Coupling links sources that
cite the same references:

```{r}
source_edges <- source_network(sc, type = "coupling", min_occur = 2)
head(source_edges, 5)
```

For source, country, institution, and author coupling, `min_occur` is applied
to the aggregated entity before building the coupling network.

## Generic Co-networks

Use `conetwork()` when you want a projection not covered by a dedicated helper.

One-field use:

```{r}
head(conetwork(sc, "keywords", min_occur = 2), 5)
```

Two-field use:

```{r}
head(conetwork(sc, "authors", by = "keywords", min_occur = 2), 5)
```

The second example links authors through shared keywords. This is not a
co-authorship network; it is a thematic-similarity network between authors.

Delimited character columns are split automatically:

```{r}
toy <- data.frame(
  id = c("P1", "P2", "P3"),
  tags = c("methods; networks", "networks; R", "methods; R")
)

conetwork(toy, "tags")
```

## Normalization

The same raw counts can support different similarity scores:

```{r}
none <- keyword_network(sc, min_occur = 2, similarity = "none")
cos  <- keyword_network(sc, min_occur = 2, similarity = "cosine")

head(none[, c("from", "to", "weight", "count")], 3)
head(cos[, c("from", "to", "weight", "count")], 3)
```

Notice that `count` is unchanged. The `weight` column changes because
normalization is applied after raw co-occurrence has been counted.

Available methods are:

```{r}
normalize(to_matrix(keyword_network(small)), "cosine")
```

In practice:

- Use `similarity = "none"` for raw weighted counts.
- Use `similarity = "cosine"` for interpretable overlap scaled by marginal
  size.
- Use `similarity = "association"` when the goal is to emphasize pairs that
  co-occur more than expected from their individual frequencies.
- Use `jaccard`, `inclusion`, or `equivalence` when those coefficients match a
  downstream method or established reporting convention.

## Reducing Large Networks

Dense co-occurrence networks can be hard to inspect. `bibnets` provides three
different reduction strategies.

```{r}
edges <- author_network(oa, "collaboration")

nrow(edges)
nrow(prune(edges, threshold = 2))
nrow(prune(edges, top_n = 5))
nrow(filter_top(edges, n = 50))
```

`prune(threshold = x)` keeps edges with weight at least `x`.
`prune(top_n = k)` keeps locally strong edges for each endpoint.
`filter_top(n = k)` first selects the most connected nodes, then keeps edges
among them.

`backbone()` applies the disparity filter for multiscale weighted networks:

```{r}
bb <- backbone(edges, alpha = 0.05)
nrow(bb)
head(bb, 5)
```

The disparity filter asks whether an edge is unusually strong relative to at
least one endpoint's local strength distribution. This is different from a
global weight cutoff and can preserve meaningful edges attached to smaller
nodes.

## Temporal Networks

`temporal_network()` runs any network builder over time windows:

```{r}
tn <- temporal_network(oa, author_network, "collaboration", window = 3)
names(tn)
```

Fixed windows are non-overlapping. Sliding windows overlap:

```{r}
tn_slide <- temporal_network(
  oa,
  author_network,
  "collaboration",
  window = 3,
  step = 1,
  strategy = "sliding"
)

names(tn_slide)
```

Cumulative windows always start at the first observed year and grow forward:

```{r}
tn_cum <- temporal_network(
  oa,
  author_network,
  "collaboration",
  window = 3,
  strategy = "cumulative"
)

names(tn_cum)
```

Each returned edge list has a `window` column. Windows with fewer than two
records or no surviving edges are omitted. If a builder errors inside a window,
`temporal_network()` reports a warning with the window label.

## Local Citations and Historiographs

`local_citations()` counts how often each document is cited by other documents
inside the same dataset:

```{r}
lcs <- local_citations(sc)
head(lcs, 5)
```

`historiograph()` builds a directed citation graph among the top locally cited
documents:

```{r}
h <- historiograph(sc, n = 10)
h$nodes
head(h$edges, 5)
```

This requires reference strings or IDs to match document IDs in the same data
frame. If the cited works are external to the corpus, local citation counts will
be low or zero even when global citation counts are high.

## Exporting Results

The default edge list is already useful for many tools:

```{r}
edges <- keyword_network(sc, min_occur = 2)
head(edges, 5)
```

Convert to a sparse matrix:

```{r}
m <- to_matrix(edges)
m[1:4, 1:4]
```

Prepare Gephi tables:

```{r}
gephi <- to_gephi(edges)
head(gephi$nodes, 3)
head(gephi$edges, 3)
```

Write GraphML without adding an XML dependency:

```{r}
xml <- to_graphml(edges)
cat(substr(xml, 1, 300))
```

Optional graph objects are available when the suggested packages are installed:

```{r, eval = FALSE}
if (requireNamespace("igraph", quietly = TRUE)) {
  g <- to_igraph(edges)
}

if (requireNamespace("tidygraph", quietly = TRUE)) {
  tg <- to_tbl_graph(edges)
}

if (requireNamespace("cograph", quietly = TRUE)) {
  cg <- to_cograph(edges)
}
```

## Interpreting a `bibnets_network`

The object stores construction metadata as attributes:

```{r}
edges <- author_network(oa, "collaboration", counting = "harmonic")

attr(edges, "network_type")
attr(edges, "counting")
attr(edges, "similarity")
```

The `print()` method reports the network type, node count, edge count, counting
method, and similarity method. `summary()` reports basic network and weight
summaries:

```{r}
summary(edges)
```

These attributes are meant to make downstream output easier to audit. A saved
edge list should still say how it was produced.

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/read-biblio.R
\name{detect_format}
\alias{detect_format}
\title{Detect bibliometric file format}
\usage{
detect_format(file)
}
\arguments{
\item{file}{Path to file.}
}
\value{
Character: format name or \code{"unknown"}.
}
\description{
Detect bibliometric file format
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/read-biblio.R
\name{read_single_biblio}
\alias{read_single_biblio}
\title{Read a single bibliometric file}
\usage{
read_single_biblio(file, format, id, actors, sep, ...)
}
\description{
Read a single bibliometric file
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/converters.R
\name{to_matrix}
\alias{to_matrix}
\title{Convert edge data frame to adjacency matrix}
\usage{
to_matrix(edges, symmetric = TRUE)
}
\arguments{
\item{edges}{A data frame with \code{from}, \code{to}, \code{weight} columns.}

\item{symmetric}{Logical. If \code{TRUE} (default), produces a symmetric matrix.}
}
\value{
A sparse Matrix.
}
\description{
Convert edge data frame to adjacency matrix
}
\examples{
data(biblio_data)
edges <- reference_network(biblio_data, min_occur = 2)
to_matrix(edges)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/read-openalex.R
\name{read_openalex}
\alias{read_openalex}
\title{Convert OpenAlex data to bibnets format}
\usage{
read_openalex(data)
}
\arguments{
\item{data}{A data frame from \code{oa_fetch(entity = "works", ...)}. Must
contain at least an \code{id} column. Common columns include \code{display_name},
\code{publication_year}, \code{so}, \code{doi}, \code{cited_by_count}, \code{referenced_works},
\code{ab}, and \code{author} (nested).}
}
\value{
A data frame in the standard bibnets format: \code{id}, \code{title},
\code{year}, \code{journal}, \code{doi}, \code{cited_by_count}, \code{abstract}, \code{type},
plus list-columns \code{authors}, \code{references}, and \code{keywords}.
}
\description{
Takes the output of \code{\link[openalexR:oa_fetch]{openalexR::oa_fetch()}} (a tibble/data frame of works)
and converts it to the standardized bibnets format with list-columns.
}
\examples{
# Construct a minimal data frame matching the structure returned by
# openalexR::oa_fetch(entity = "works", ...). In practice, pass the
# result of oa_fetch() directly.
raw <- data.frame(
  id = c("W123", "W456"),
  display_name = c("First paper", "Second paper"),
  publication_year = c(2022L, 2021L),
  so = c("Journal A", "Journal B"),
  doi = c("https://doi.org/10.1/a", "https://doi.org/10.2/b"),
  cited_by_count = c(5L, 12L),
  stringsAsFactors = FALSE
)
raw$author <- list(
  data.frame(au_display_name = c("Smith J", "Jones A"),
             stringsAsFactors = FALSE),
  data.frame(au_display_name = "Davis M", stringsAsFactors = FALSE)
)
raw$referenced_works <- list(c("W100", "W200"), "W123")
data <- read_openalex(raw)
head(data[, c("id", "title", "year", "journal", "doi")])
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utils.R
\name{standardize_authors}
\alias{standardize_authors}
\title{Standardize author names}
\usage{
standardize_authors(x, flip_names = FALSE)
}
\arguments{
\item{x}{Character vector of author names.}

\item{flip_names}{Logical. If \code{TRUE}, names in \verb{Last, First} format are
reordered to \verb{First Last}. Off by default — enable only when all names
in \code{x} reliably follow the \verb{Last, First} convention.}
}
\value{
Character vector, uppercased and cleaned.
}
\description{
Uppercase, whitespace normalisation, and dot removal from initials
(\code{F.J.} → \code{FJ}). Name order and format are preserved — consistent with
how bibliometrix handles multi-source data.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/document-network.R
\name{document_network}
\alias{document_network}
\title{Build a document network}
\usage{
document_network(
  data,
  type = "coupling",
  counting = "full",
  similarity = "none",
  threshold = 0,
  min_occur = 1L,
  top_n = NULL,
  self_loops = FALSE,
  deduplicate = TRUE,
  format = "edgelist"
)
}
\arguments{
\item{data}{A data frame with \code{id} and \code{references} (list-column).}

\item{type}{Character. Relationship type:
\describe{
\item{\code{"coupling"}}{Bibliographic coupling: documents linked when they
share cited references.}
\item{\code{"citation"}}{Direct citation: directed edges from citing to
cited documents (internal citations only).}
\item{\code{"co_citation"}}{Co-citation: documents linked when they are
cited together by other documents in the dataset.}
\item{\code{"equivalence"}}{Profile similarity of reference vectors.}
}}

\item{counting}{Character. Counting method. Default \code{"full"}.
Position-dependent methods are not applicable to document networks.}

\item{similarity}{Character. Similarity measure. Default \code{"none"}.}

\item{threshold}{Numeric. Minimum edge weight. Default 0.}

\item{min_occur}{Integer. Minimum reference frequency. Default 1.}

\item{top_n}{Integer or NULL. Return only the top n edges by weight.
Default NULL (all edges).}

\item{self_loops}{Logical. If \code{TRUE}, include self-loops (an entity linked
to itself). Default \code{FALSE}.}

\item{deduplicate}{Logical. If \code{TRUE} (default), each \verb{(paper, entity)}
pair is counted at most once — duplicate entries in the source data
(e.g., the same author listed twice on a paper) are treated as one
occurrence. Set to \code{FALSE} to count every raw occurrence.}

\item{format}{Character. Output format:
\describe{
\item{\code{"edgelist"}}{Default. A \code{bibnets_network} data frame with
columns \code{from}, \code{to}, \code{weight}, \code{count}.}
\item{\code{"gephi"}}{Gephi-ready data frame: \code{Source}, \code{Target},
\code{Weight}, \code{Count}, \code{Type}.}
\item{\code{"igraph"}}{An igraph graph object (requires igraph).}
\item{\code{"cograph"}}{A cograph_network object (requires cograph).}
\item{\code{"matrix"}}{A sparse adjacency matrix.}
}}
}
\value{
Depends on \code{format}. For \code{type = "citation"}, edges are directed
(from = citing, to = cited) with \code{weight} and \code{count} both 1.
}
\description{
Constructs a network between documents (papers) in the dataset.
}
\examples{
data(biblio_data)
document_network(biblio_data, "coupling")
document_network(biblio_data, "coupling", counting = "strength")
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/co-network.R
\name{build_by_network}
\alias{build_by_network}
\title{Build a network where entities share values from another field}
\usage{
build_by_network(
  data,
  field,
  by,
  counting,
  similarity,
  threshold,
  min_occur,
  top_n = NULL,
  self_loops = FALSE,
  deduplicate = TRUE
)
}
\description{
Build a network where entities share values from another field
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/read-biblio.R
\name{resolve_paths}
\alias{resolve_paths}
\title{Resolve file paths from a file, vector of files, or directory}
\usage{
resolve_paths(path)
}
\description{
Resolve file paths from a file, vector of files, or directory
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/converters.R
\name{to_igraph}
\alias{to_igraph}
\title{Convert edge data frame to igraph}
\usage{
to_igraph(edges, directed = FALSE)
}
\arguments{
\item{edges}{A data frame with at least \code{from}, \code{to}, \code{weight} columns,
as returned by any network function in bibnets.}

\item{directed}{Logical. Default \code{FALSE}.}
}
\value{
An igraph graph object.
}
\description{
Convert edge data frame to igraph
}
\examples{
\dontshow{if (requireNamespace("igraph", quietly = TRUE)) withAutoprint(\{ # examplesIf}
data(biblio_data)
edges <- author_network(biblio_data, "collaboration")
g <- to_igraph(edges)
\dontshow{\}) # examplesIf}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data.R
\docType{data}
\name{biblio_data}
\alias{biblio_data}
\title{Example bibliometric dataset}
\format{
A data frame with 10 rows and 9 columns:
\describe{
\item{id}{Unique document identifier (W1--W10).}
\item{title}{Document title.}
\item{year}{Publication year (2018--2022).}
\item{journal}{Source journal (Scientometrics, Journal of Informetrics,
JASIST, Quantitative Science Studies).}
\item{doi}{DOI string.}
\item{cited_by_count}{Times cited.}
\item{authors}{List-column of author name strings (6 unique authors).}
\item{references}{List-column of cited reference IDs (10 unique refs,
R1--R10). Each paper cites exactly 4 references.}
\item{keywords}{List-column of keyword strings (24 unique keywords).
Each paper has 3 keywords.}
}
}
\usage{
biblio_data
}
\description{
A small synthetic dataset of 10 scholarly papers with overlapping authors,
references, and keywords. Designed for testing and demonstrating all
network construction functions in bibnets.
}
\examples{
data(biblio_data)
reference_network(biblio_data)
document_network(biblio_data, "coupling")
author_network(biblio_data, "collaboration")
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/backbone.R
\name{backbone}
\alias{backbone}
\title{Extract network backbone using the disparity filter}
\usage{
backbone(edges, alpha = 0.05)
}
\arguments{
\item{edges}{A data frame with at least columns \code{from}, \code{to}, and \code{weight}.
Must be an undirected edge list (each pair appears once).}

\item{alpha}{Numeric. Significance threshold in (0, 1). Default \code{0.05}.}
}
\value{
The filtered edge data frame with an added \code{alpha} column (the
minimum alpha from the two endpoints).
}
\description{
Applies the disparity filter to a weighted edge list. For each edge, it
computes an alpha (p-value) from both endpoints and keeps the edge if it
is statistically significant from at least one endpoint.
}
\details{
The null model asks: given that node \eqn{i} has total strength \eqn{s_i}
distributed uniformly across \eqn{k_i} edges, what is the probability that
a single edge weight is as large as \eqn{w_{ij}}? The answer is
\deqn{\alpha_{ij} = \left(1 - \frac{w_{ij}}{s_i}\right)^{k_i - 1}}

An edge is retained if \eqn{\min(\alpha_{ij}, \alpha_{ji}) < \alpha}.
Nodes with only one edge always have \eqn{\alpha = 0} and are always kept.
}
\examples{
edges <- data.frame(
  from   = c("A", "A", "A", "B", "C"),
  to     = c("B", "C", "D", "C", "D"),
  weight = c(10,   1,   1,   8,   1)
)
backbone(edges, alpha = 0.05)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/author-network.R
\name{author_network}
\alias{author_network}
\title{Build an author network}
\usage{
author_network(
  data,
  type = "collaboration",
  counting = "full",
  similarity = "none",
  threshold = 0,
  min_occur = 1L,
  position_weights = c(1, 0.8, 0.6, 0.4),
  first_last_weight = 2,
  attention = NULL,
  top_n = NULL,
  self_loops = FALSE,
  deduplicate = TRUE,
  format = "edgelist"
)
}
\arguments{
\item{data}{A data frame with at least \code{id} and \code{authors} (list-column,
order preserved). For coupling/co-citation, also needs \code{references}.}

\item{type}{Character. Relationship type:
\describe{
\item{\code{"collaboration"}}{Co-authorship: authors linked when they
co-author a publication.}
\item{\code{"coupling"}}{Bibliographic coupling aggregated at author level:
authors linked when they cite the same references.}
\item{\code{"co_citation"}}{Author co-citation: authors linked when they
are cited together by the same paper. Requires a
\code{cited_first_authors} list-column.}
\item{\code{"equivalence"}}{Profile similarity: cosine similarity of
authors' full collaboration/citation profiles.}
}}

\item{counting}{Character. Counting method. Position-independent methods
(\code{"full"}, \code{"fractional"}, \code{"paper"}, \code{"strength"}) work for all types.
Position-dependent methods (\code{"harmonic"}, \code{"arithmetic"}, \code{"geometric"},
\code{"adaptive_geometric"}, \code{"golden"}, \code{"first"}, \code{"last"},
\code{"first_last"}, \code{"position_weighted"}) are available for
\code{type = "collaboration"}.}

\item{similarity}{Character. Similarity measure: \code{"none"}, \code{"association"},
\code{"cosine"}, \code{"jaccard"}, \code{"inclusion"}, \code{"equivalence"}.}

\item{threshold}{Numeric. Minimum edge weight. Default 0.}

\item{min_occur}{Integer. Minimum number of papers for an author to be
included. Default 1.}

\item{position_weights}{Numeric vector. Custom weights for
\code{counting = "position_weighted"}. Default \code{c(1, 0.8, 0.6, 0.4)}.}

\item{first_last_weight}{Numeric. Multiplier for \code{counting = "first_last"}.
Default 2.}

\item{attention}{Character or NULL. Attention-based weighting independent of
\code{type} and \code{counting}. One of \code{"proximity"} (center authors weighted most),
\code{"lead"} (first author dominates, quadratic drop), \code{"last"} (last author
dominates, quadratic rise), \code{"circular"} (first and last both prominent).
Default \code{NULL} (disabled).}

\item{top_n}{Integer or NULL. Return only the top n edges by weight.
Default NULL (all edges).}

\item{self_loops}{Logical. If \code{TRUE}, include self-loops (an entity linked
to itself). Default \code{FALSE}.}

\item{deduplicate}{Logical. If \code{TRUE} (default), each \verb{(paper, entity)}
pair is counted at most once — duplicate entries in the source data
(e.g., the same author listed twice on a paper) are treated as one
occurrence. Set to \code{FALSE} to count every raw occurrence.}

\item{format}{Character. Output format:
\describe{
\item{\code{"edgelist"}}{Default. A \code{bibnets_network} data frame with
columns \code{from}, \code{to}, \code{weight}, \code{count}.}
\item{\code{"gephi"}}{Gephi-ready data frame: \code{Source}, \code{Target},
\code{Weight}, \code{Count}, \code{Type}.}
\item{\code{"igraph"}}{An igraph graph object (requires igraph).}
\item{\code{"cograph"}}{A cograph_network object (requires cograph).}
\item{\code{"matrix"}}{A sparse adjacency matrix.}
}}
}
\value{
Depends on \code{format}: a \code{bibnets_network} data frame (default),
a Gephi-ready data frame, an igraph graph, a cograph_network, or a
sparse matrix.
}
\description{
Constructs a network between authors using one of four relationship types
and any of 13 counting methods, including 9 position-dependent methods
that respect author byline order.
}
\examples{
data(biblio_data)
author_network(biblio_data, "collaboration")
author_network(biblio_data, "collaboration", counting = "harmonic")
author_network(biblio_data, "collaboration", counting = "geometric",
               similarity = "association")
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/read-biblio.R
\name{read_biblio}
\alias{read_biblio}
\title{Read bibliometric data}
\usage{
read_biblio(path, format = "auto", id = NULL, actors = NULL, sep = ";", ...)
}
\arguments{
\item{path}{Character. Path to a file, a vector of file paths, or a
directory containing export files.}

\item{format}{Character. File format:
\describe{
\item{\code{"auto"}}{Default. Auto-detect from file content.}
\item{\code{"scopus"}}{Scopus CSV.}
\item{\code{"wos"}}{Web of Science plaintext.}
\item{\code{"wos_tab"}}{Web of Science tab-delimited.}
\item{\code{"bibtex"}}{BibTeX .bib file.}
\item{\code{"ris"}}{RIS file.}
\item{\code{"dimensions"}}{Dimensions CSV.}
\item{\code{"lens"}}{Lens.org CSV.}
\item{\code{"openalex_csv"}}{Flat OpenAlex CSV export (pipe-delimited fields).}
\item{\code{"generic"}}{Any CSV. Use \code{id} and \code{actors} to specify columns.}
}}

\item{id}{Character. Column name for document identifier. Only used
when \code{format = "generic"}. Default \code{NULL} (uses row numbers).}

\item{actors}{Character vector. Column names to split into list-columns.
Only used when \code{format = "generic"}.}

\item{sep}{Character. Delimiter for splitting actor columns. Default \code{";"}.}

\item{...}{Additional arguments passed to the format-specific reader.}
}
\value{
A data frame.
}
\description{
Universal reader that handles files, folders, format detection, and
generic CSV input. Accepts a single file, multiple files, or a directory.
}
\examples{
# Auto-detect format from file content (here: a bundled OpenAlex CSV)
f <- system.file("extdata", "openalex_works.csv", package = "bibnets")
data <- read_biblio(f)
head(data[, c("id", "title", "year", "journal")])

# Read multiple files at once; auto-detects each format
f_scopus <- system.file("extdata", "scopus_sample.csv", package = "bibnets")
f_wos    <- system.file("extdata", "wos_sample.txt",  package = "bibnets")
combined <- read_biblio(c(f_scopus, f_wos))
head(combined[, c("id", "title", "year", "journal")])

# Read every supported export in a directory (here: the bundled extdata)
folder <- system.file("extdata", package = "bibnets")
all_data <- read_biblio(folder)
nrow(all_data)

# Generic CSV: point read_biblio at any CSV and name the list-column fields
tmp <- tempfile(fileext = ".csv")
write.csv(data.frame(
  doc_id  = c("a", "b"),
  Authors = c("Smith J; Jones A", "Davis M"),
  Keywords = c("networks; bibliometrics", "analytics")
), tmp, row.names = FALSE)
generic <- read_biblio(tmp, format = "generic",
                       id = "doc_id",
                       actors = c("Authors", "Keywords"),
                       sep = ";")
head(generic)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/methods.R
\name{summary.bibnets_network}
\alias{summary.bibnets_network}
\title{Summarise a bibnets network}
\usage{
\method{summary}{bibnets_network}(object, ...)
}
\arguments{
\item{object}{A \code{bibnets_network} data frame.}

\item{...}{Ignored.}
}
\value{
Invisibly returns \code{object}.
}
\description{
Summarise a bibnets network
}
\examples{
data(biblio_data)
edges <- author_network(biblio_data, "collaboration")
summary(edges)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/temporal-network.R
\name{build_windows}
\alias{build_windows}
\title{Build time window definitions}
\usage{
build_windows(min_time, max_time, window, step, strategy)
}
\description{
Build time window definitions
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/prune.R
\name{prune}
\alias{prune}
\title{Prune a weighted edge list}
\usage{
prune(edges, threshold = NULL, top_n = NULL)
}
\arguments{
\item{edges}{A data frame with at least columns \code{from}, \code{to}, and \code{weight}.}

\item{threshold}{Numeric. Keep only edges with \code{weight >= threshold}.}

\item{top_n}{Integer. For each node, keep only its \code{top_n} strongest edges.
An edge is kept if it is in the top \code{top_n} for \emph{either} endpoint.}
}
\value{
The filtered edge data frame (same columns as input).
}
\description{
Reduces a weighted edge list by removing weak or excess edges.
}
\examples{
edges <- data.frame(
  from   = c("A","A","A","B","B","C"),
  to     = c("B","C","D","C","D","D"),
  weight = c(5,  1,  2,  4,  1,  3)
)

# Keep only edges with weight >= 3
prune(edges, threshold = 3)

# Keep the 2 strongest edges per node
prune(edges, top_n = 2)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/co-network.R
\name{conetwork}
\alias{conetwork}
\title{Build a co-occurrence network from any field}
\usage{
conetwork(
  data,
  field,
  by = NULL,
  sep = ";",
  counting = "full",
  similarity = "none",
  threshold = 0,
  min_occur = 1L,
  top_n = NULL,
  self_loops = FALSE,
  deduplicate = TRUE,
  format = "edgelist"
)
}
\arguments{
\item{data}{A data frame with column \code{id} and the specified field(s).}

\item{field}{Character. The entity field — determines what the nodes are.}

\item{by}{Character or \code{NULL}. What links the nodes. If \code{NULL} (default),
entities are linked by co-occurring in the same document. If specified,
entities are linked when they share values from the \code{by} field.}

\item{sep}{Character or \code{NULL}. Delimiter for splitting character columns.
Default \code{";"}. Set to \code{NULL} if columns are already list-columns.}

\item{counting}{Character. Counting method. Default \code{"full"}.}

\item{similarity}{Character. Normalization method. Default \code{"none"}.}

\item{threshold}{Numeric. Minimum edge weight. Default 0.}

\item{min_occur}{Integer. Minimum entity frequency. Default 1.}

\item{top_n}{Integer or NULL. Return only the top n edges by weight.
Default NULL (all edges).}

\item{self_loops}{Logical. If \code{TRUE}, include self-loops (an entity linked
to itself). Default \code{FALSE}.}

\item{deduplicate}{Logical. If \code{TRUE} (default), each \verb{(paper, entity)}
pair is counted at most once — duplicate entries in the source data
(e.g., the same author listed twice on a paper) are treated as one
occurrence. Set to \code{FALSE} to count every raw occurrence.}

\item{format}{Character. Output format:
\describe{
\item{\code{"edgelist"}}{Default. A \code{bibnets_network} data frame with
columns \code{from}, \code{to}, \code{weight}, \code{count}.}
\item{\code{"gephi"}}{Gephi-ready data frame: \code{Source}, \code{Target},
\code{Weight}, \code{Count}, \code{Type}.}
\item{\code{"igraph"}}{An igraph graph object (requires igraph).}
\item{\code{"cograph"}}{A cograph_network object (requires cograph).}
\item{\code{"matrix"}}{A sparse adjacency matrix.}
}}
}
\value{
Depends on \code{format}: a \code{bibnets_network} data frame (default),
a Gephi-ready data frame, an igraph graph, a cograph_network, or a
sparse matrix.
}
\description{
With one field, entities are linked when they co-occur in the same
document. With \code{by}, entities are linked when they share values of the
\code{by} field across documents.
}
\details{
Fields can be list-columns (already split) or character columns with
delimiters (auto-split via \code{sep}).
}
\examples{
data(biblio_data)

# Co-occurrence: keywords appearing in the same document
conetwork(biblio_data, "keywords")

# Authors linked by shared keywords
conetwork(biblio_data, "authors", by = "keywords")

# Keywords linked by shared authors
conetwork(biblio_data, "keywords", by = "authors")

# Journals linked by shared references (= journal coupling)
conetwork(biblio_data, "journal", by = "references", similarity = "cosine")

# Auto-splits semicolon-delimited string columns
d <- data.frame(id = 1:3, tags = c("ml; dl; nlp", "ml; cv", "dl; cv"))
conetwork(d, "tags")
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/read-bibtex.R
\name{read_bibtex}
\alias{read_bibtex}
\title{Read a BibTeX file}
\usage{
read_bibtex(file, encoding = "UTF-8")
}
\arguments{
\item{file}{Path to a \code{.bib} file.}

\item{encoding}{Character. File encoding. Default \code{"UTF-8"}.}
}
\value{
A data frame in the standard bibnets format: \code{id}, \code{title},
\code{year}, \code{journal}, \code{doi}, \code{cited_by_count}, \code{abstract}, \code{type},
plus list-columns \code{authors}, \code{references} (typically empty for
BibTeX), and \code{keywords}.
}
\description{
Parses a \code{.bib} file into a standardized bibliometric data frame.
Note: standard BibTeX does not contain cited references, so the
\code{references} column will be empty unless the file includes a
non-standard \code{cited-references} or \code{note} field with reference data.
}
\examples{
# Write a minimal BibTeX entry to a temp file, then read it
bib <- '@article{smith2020,
  title  = {Bibliometric networks},
  author = {Smith, J. and Jones, K.},
  journal = {Test Journal},
  year   = {2020},
  doi    = {10.1000/test}
}'
f <- tempfile(fileext = ".bib")
writeLines(bib, f)
data <- read_bibtex(f)
data[, c("id", "title", "year", "journal", "doi")]
unlink(f)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/filter.R
\name{filter_top}
\alias{filter_top}
\title{Filter edges to top-n nodes}
\usage{
filter_top(edges, n)
}
\arguments{
\item{edges}{A data frame with at least \code{from}, \code{to}, \code{weight} columns.}

\item{n}{Integer. Number of top nodes to keep.}
}
\value{
A filtered data frame with edges among the top \code{n} nodes.
}
\description{
Keeps only edges between the most frequent nodes. Node frequency is
determined by how many edges each node participates in.
}
\examples{
data(biblio_data)
edges <- author_network(biblio_data, "collaboration")

# Keep only edges among the top 3 most connected authors
filter_top(edges, 3)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/historiograph.R
\name{historiograph}
\alias{historiograph}
\title{Build a historiograph (chronological citation network)}
\usage{
historiograph(data, n = 30, min_lcs = 1)
}
\arguments{
\item{data}{A data frame with \code{id}, \code{references} (list-column), and
\code{year}. Optionally \code{title}, \code{journal}, \code{doi}, \code{cited_by_count}.}

\item{n}{Integer. Number of top locally cited documents to include.
Default 30.}

\item{min_lcs}{Integer. Minimum local citation score for inclusion.
Default 1.}
}
\value{
A list with:
\describe{
\item{\verb{$nodes}}{Data frame of included documents with \code{id}, \code{lcs},
\code{gcs}, \code{year}, \code{title}, \code{journal}, \code{doi}.}
\item{\verb{$edges}}{Data frame of directed citation edges with \code{from}
(citing), \code{to} (cited), \code{year_from}, \code{year_to}.}
}
}
\description{
Constructs a Garfield-style historiograph: a directed citation network
among the most locally cited documents, laid out chronologically.
}
\examples{
data(biblio_data)
h <- historiograph(biblio_data, n = 5)
h$nodes
h$edges
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/edgelist.R
\name{mat_to_edgelist}
\alias{mat_to_edgelist}
\title{Convert a sparse square matrix to a tidy edge list}
\usage{
mat_to_edgelist(A, directed = FALSE)
}
\arguments{
\item{A}{A square sparse or dense matrix.}

\item{directed}{Logical. If \code{FALSE} (default), returns only upper-triangle
entries. If \code{TRUE}, returns all non-zero off-diagonal entries.}
}
\value{
A data frame with columns \code{from}, \code{to}, \code{weight}, sorted by
descending weight.
}
\description{
Extracts non-zero entries directly from the sparse representation —
no dense matrix allocation. For undirected networks, only the upper
triangle is returned.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utils.R
\name{split_field}
\alias{split_field}
\title{Parse semicolon-delimited strings into list-column}
\usage{
split_field(x, sep = ";")
}
\arguments{
\item{x}{A character vector of semicolon-delimited strings.}

\item{sep}{Character. Delimiter. Default \code{";"}.}
}
\value{
A list of character vectors.
}
\description{
Splits semicolon-separated strings (common in Scopus/WoS exports) into
character vectors, trimming whitespace.
}
\examples{
split_field(c("Alice; Bob; Carol", "Dave; Eve"))
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/normalize.R
\name{normalize}
\alias{normalize}
\title{Normalize a co-occurrence matrix}
\usage{
normalize(A, method = "none")
}
\arguments{
\item{A}{A square symmetric matrix (dense or sparse) representing
co-occurrence counts.}

\item{method}{Character. Normalization method:
\describe{
\item{\code{"none"}}{No normalization. Returns raw co-occurrence counts.}
\item{\code{"association"}}{Association strength (probabilistic affinity
index). \eqn{s_{ij} = c_{ij} / (w_i \cdot w_j)}.
Often recommended as the best normalization for co-occurrence data.}
\item{\code{"cosine"}}{Salton's cosine.
\eqn{s_{ij} = c_{ij} / \sqrt{w_i \cdot w_j}}.}
\item{\code{"jaccard"}}{Jaccard index.
\eqn{s_{ij} = c_{ij} / (w_i + w_j - c_{ij})}.}
\item{\code{"inclusion"}}{Inclusion index (Simpson coefficient).
\eqn{s_{ij} = c_{ij} / \min(w_i, w_j)}.}
\item{\code{"equivalence"}}{Equivalence index (Salton's cosine squared).
\eqn{s_{ij} = c_{ij}^2 / (w_i \cdot w_j)}.}
}}
}
\value{
A normalized sparse matrix of the same dimensions.
}
\description{
Applies a similarity normalization to a square co-occurrence matrix.
The diagonal of the input matrix is used as the total occurrence count
for each item. Operates entirely in sparse representation.
}
\examples{
# Create a small co-occurrence matrix
A <- matrix(c(10, 3, 1, 3, 8, 2, 1, 2, 5), nrow = 3,
            dimnames = list(c("a", "b", "c"), c("a", "b", "c")))
normalize(A, "association")
normalize(A, "cosine")
normalize(A, "jaccard")
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utils.R
\name{aggregate_by_entity}
\alias{aggregate_by_entity}
\title{Aggregate multi-valued fields by an entity}
\usage{
aggregate_by_entity(data, entity_field, value_field, min_freq = 1L)
}
\arguments{
\item{data}{A data frame with \code{id} and the specified columns.}

\item{entity_field}{Character. Name of the entity column. If it is a scalar
column (e.g., \code{"journal"}), each document belongs to one entity. If it
is a list-column (e.g., \code{"authors"}), each document may belong to
multiple entities.}

\item{value_field}{Character. Name of the list-column to aggregate
(e.g., \code{"references"}).}

\item{min_freq}{Integer. Minimum number of papers per entity. Default 1.}
}
\value{
A data frame with columns \code{id} (entity name) and \code{value_field}
(list-column of pooled values, with duplicates preserved).
}
\description{
Groups documents by a single-valued or list-column entity (e.g., author,
journal) and pools all values from another list-column (e.g., references,
keywords) across documents belonging to that entity.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/bipartite.R
\name{build_bipartite}
\alias{build_bipartite}
\title{Build a bipartite incidence matrix from bibliometric data}
\usage{
build_bipartite(data, field, min_freq = 1L, deduplicate = TRUE)
}
\arguments{
\item{data}{A data frame with at least columns \code{id} and the field specified.}

\item{field}{Character. Name of the list-column containing entities
(e.g., \code{"authors"}, \code{"references"}, \code{"keywords"}).}

\item{min_freq}{Integer. Minimum number of occurrences for an entity to be
included. Default 1 (no filtering).}
}
\value{
A sparse \code{dgCMatrix} with rows = works (named by \code{id}) and
columns = unique entities.
}
\description{
Constructs a sparse works x entities two-mode matrix from a data frame
with a list-column. This is the core engine behind all network construction
functions.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/multiply.R
\name{multiply_bipartite}
\alias{multiply_bipartite}
\title{Construct a co-occurrence network via two-mode multiplication}
\usage{
multiply_bipartite(
  B,
  mode = "columns",
  similarity = "none",
  threshold = 0,
  top_n = NULL,
  self_loops = FALSE
)
}
\arguments{
\item{B}{A sparse bipartite matrix (works x entities), already weighted
by the counting method.}

\item{mode}{Character. \code{"columns"} for column-mode co-occurrence
(e.g., co-citation), \code{"rows"} for row-mode (e.g., coupling).}

\item{similarity}{Character. Normalization method (see \code{\link[=normalize]{normalize()}}).}

\item{threshold}{Numeric. Minimum edge weight to retain.}

\item{top_n}{Integer or \code{NULL}. If specified, keep only the top \code{n}
most frequent nodes and return all edges among them.}
}
\value{
A data frame with columns \code{from}, \code{to}, \code{weight}, \code{count}.
}
\description{
The unified engine for all bibliometric networks. Operates entirely in
sparse representation — never allocates a dense n x n matrix.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/read-biblio.R
\name{align_biblio_columns}
\alias{align_biblio_columns}
\title{Align columns before row-binding bibliographic files}
\usage{
align_biblio_columns(dfs)
}
\description{
Align columns before row-binding bibliographic files
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/counting.R
\name{author_weights}
\alias{author_weights}
\title{Compute positional author weights for a single paper}
\usage{
author_weights(
  n,
  counting = "fractional",
  position_weights = c(1, 0.8, 0.6, 0.4),
  first_last_weight = 2
)
}
\arguments{
\item{n}{Integer. Number of authors.}

\item{counting}{Character. Counting method.}

\item{position_weights}{Numeric vector. Custom weights for
\code{counting = "position_weighted"}. Default \code{c(1, 0.8, 0.6, 0.4)}.}

\item{first_last_weight}{Numeric. Multiplier for first/last authors
when \code{counting = "first_last"}. Default 2.}
}
\value{
Numeric vector of length \code{n}, summing to 1.
}
\description{
Given the number of authors and their positions, returns a weight vector
for positional counting methods. All methods normalize weights to sum to 1
per paper. Position-independent methods (fractional, paper, strength) are
handled by \code{\link[=apply_counting]{apply_counting()}} instead.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/historiograph.R
\name{local_citations}
\alias{local_citations}
\title{Compute local citation scores}
\usage{
local_citations(data)
}
\arguments{
\item{data}{A data frame with \code{id} and \code{references} (list-column).
Optionally \code{year}, \code{title}, \code{journal}, \code{doi}, \code{cited_by_count}.}
}
\value{
A data frame with columns:
\describe{
\item{\code{id}}{Document identifier.}
\item{\code{lcs}}{Local Citation Score: times cited within the dataset.}
\item{\code{gcs}}{Global Citation Score: \code{cited_by_count} if available.}
}
Plus any metadata columns present in the input (\code{title}, \code{year},
\code{journal}, \code{doi}).
}
\description{
Counts how many times each document is cited by other documents
within the dataset.
}
\examples{
data(biblio_data)
local_citations(biblio_data)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/bipartite.R
\name{build_bipartite_long}
\alias{build_bipartite_long}
\title{Build bipartite matrix from a long-format edge table}
\usage{
build_bipartite_long(edges, min_freq = 1L)
}
\arguments{
\item{edges}{A data frame with columns \code{source} (work id) and \code{target}
(entity id).}

\item{min_freq}{Integer. Minimum entity frequency. Default 1.}
}
\value{
A sparse \code{dgCMatrix}.
}
\description{
Alternative constructor when data is already in long form (e.g., a two-column
data frame of document-reference pairs).
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/read-scopus.R
\name{read_scopus}
\alias{read_scopus}
\title{Read Scopus CSV export}
\usage{
read_scopus(file, encoding = "UTF-8")
}
\arguments{
\item{file}{Path to a Scopus CSV export file.}

\item{encoding}{Character. File encoding. Default \code{"UTF-8"}.}
}
\value{
A data frame in the standard bibnets format: \code{id}, \code{title},
\code{year}, \code{journal}, \code{doi}, \code{cited_by_count}, \code{abstract}, \code{type},
plus list-columns \code{authors}, \code{references}, and \code{keywords}.
Scopus-specific extras: \code{index_keywords} (list-column),
\code{affiliations} (character), \code{language} (character).
}
\description{
Parses a CSV file exported from Scopus into a standardized bibliometric
data frame with list-columns for multi-valued fields.
}
\examples{
f <- system.file("extdata", "scopus_sample.csv", package = "bibnets")
data <- read_scopus(f)
head(data[, c("id", "title", "year", "journal")])
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/read-biblio.R
\name{read_generic}
\alias{read_generic}
\title{Read a generic CSV with user-specified columns}
\usage{
read_generic(file, id = NULL, actors = NULL, sep = ";")
}
\description{
Read a generic CSV with user-specified columns
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/read-crossref.R
\name{read_crossref}
\alias{read_crossref}
\title{Convert Crossref API data to bibnets format}
\usage{
read_crossref(data)
}
\arguments{
\item{data}{A data frame from \code{cr_works(...)$data}.}
}
\value{
A data frame in the standard bibnets format: \code{id}, \code{title},
\code{year}, \code{journal}, \code{doi}, \code{cited_by_count}, \code{abstract}, \code{type},
plus list-columns \code{authors}, \code{references}, and \code{keywords}.
}
\description{
Takes the output of \code{rcrossref::cr_works()} (the \verb{$data} tibble/data frame)
and converts it to the standardized bibnets format.
}
\examples{
# Construct a minimal data frame matching the structure of
# rcrossref::cr_works(...)$data. In practice, pass that data frame directly.
raw <- data.frame(
  doi = c("10.1/a", "10.2/b"),
  title = c("First paper", "Second paper"),
  issued = c("2022-01-01", "2021-06-15"),
  container.title = c("Journal A", "Journal B"),
  is.referenced.by.count = c("3", "9"),
  type = c("journal-article", "journal-article"),
  stringsAsFactors = FALSE
)
raw$author <- list(
  data.frame(given = c("Jane", "Anne"),
             family = c("Smith", "Jones"),
             stringsAsFactors = FALSE),
  data.frame(given = "Mark", family = "Davis", stringsAsFactors = FALSE)
)
data <- read_crossref(raw)
head(data[, c("id", "title", "year", "journal")])
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/counting.R
\name{position_independent_counts}
\alias{position_independent_counts}
\title{List of all available counting methods}
\usage{
position_independent_counts()
}
\value{
Character vector of method names.
}
\description{
List of all available counting methods
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data.R
\docType{data}
\name{open_alex_gold_open_access_learning_analytics}
\alias{open_alex_gold_open_access_learning_analytics}
\title{OpenAlex Gold Open Access Learning Analytics dataset}
\format{
A data frame with 1,508 rows and 11 columns:
\describe{
\item{id}{OpenAlex work ID (e.g. \code{"W2769342982"}).}
\item{title}{Work title.}
\item{year}{Publication year (integer).}
\item{journal}{Source name, or \code{NA} if not available.}
\item{doi}{DOI string without the \code{https://doi.org/} prefix,
or \code{NA}.}
\item{cited_by_count}{Number of citing works as recorded in OpenAlex.}
\item{type}{Work type (\code{"article"}, \code{"review"},
\code{"preprint"}, \code{"book-chapter"}, etc.).}
\item{authors}{List-column of author display names (pipe-split from
the OpenAlex flat export; one name per authorship slot).}
\item{keywords}{List-column with one element: the primary OpenAlex
topic for the work (e.g. \code{"Online Learning and Analytics"}).}
\item{affiliations}{List-column of institution display names (one
entry per authorship–institution pair).}
\item{countries}{List-column of two-letter ISO country codes (one
entry per authorship–institution pair).}
}
}
\source{
OpenAlex \url{https://openalex.org}, CC0 licence.
}
\usage{
open_alex_gold_open_access_learning_analytics
}
\description{
A corpus of 1,508 gold open-access scholarly works on learning analytics,
retrieved from OpenAlex (CC0 licence). All records have a verified title,
publication year, and at least one author. Journal names are present for
works published in a named source; preprints and book chapters may have
\code{NA} in \code{journal}.
}
\examples{
data(open_alex_gold_open_access_learning_analytics)
d <- open_alex_gold_open_access_learning_analytics
author_network(d, "collaboration")
country_network(d, "collaboration")
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/converters.R
\name{to_gephi}
\alias{to_gephi}
\title{Export to Gephi node and edge tables}
\usage{
to_gephi(edges, nodes = NULL, file = NULL, directed = FALSE)
}
\arguments{
\item{edges}{A data frame with at least \code{from}, \code{to}, \code{weight} columns.}

\item{nodes}{Optional data frame of node attributes. Must contain an \code{id}
column. All other columns are included as Gephi node attributes.}

\item{file}{Optional directory path. If supplied, writes \code{nodes.csv} and
\code{edges.csv} into that directory. If \code{NULL} (default), returns a list.}

\item{directed}{Logical. Sets the \code{Type} column. Default \code{FALSE}.}
}
\value{
If \code{file = NULL}: a list with \verb{$nodes} and \verb{$edges} data frames.
If \code{file} is a directory path: writes two CSV files invisibly and returns
the file paths.
}
\description{
Converts a bibnets edge list (and optional node table) to the CSV format
expected by Gephi's Data Laboratory. Column names are remapped to Gephi
conventions (\code{Source}, \code{Target}, \code{Weight}, \code{Id}, \code{Label}).
}
\examples{
data(biblio_data)
edges <- author_network(biblio_data, "collaboration")
gephi <- to_gephi(edges)
head(gephi$edges)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/read-openalex.R
\name{read_openalex_csv}
\alias{read_openalex_csv}
\title{Read a flat OpenAlex CSV export}
\usage{
read_openalex_csv(file, sep = "|")
}
\arguments{
\item{file}{Path to the CSV file.}

\item{sep}{Character. Delimiter for multi-value fields. Default \code{"|"}.}
}
\value{
A data frame in the standard bibnets format: \code{id}, \code{title},
\code{year}, \code{journal}, \code{doi}, \code{cited_by_count}, \code{abstract}, \code{type},
plus list-columns \code{authors}, \code{references}, \code{keywords}, \code{affiliations},
\code{countries}. \code{abstract} and \code{references} are always \code{NA} / empty
(not available in the flat export).
}
\description{
Reads the flat CSV format downloaded directly from the OpenAlex website
(\code{openalex.org/works} exports). Multi-value fields are pipe-delimited (\code{|}).
This is distinct from the nested tibble produced by \code{\link[openalexR:oa_fetch]{openalexR::oa_fetch()}},
which is handled by \code{\link[=read_openalex]{read_openalex()}}.
}
\examples{
f <- system.file("extdata", "openalex_works.csv", package = "bibnets")
data <- read_openalex_csv(f)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/counting.R
\name{build_author_bipartite}
\alias{build_author_bipartite}
\title{Build a weighted bipartite matrix using positional counting}
\usage{
build_author_bipartite(
  data,
  field = "authors",
  counting = "full",
  position_weights = c(1, 0.8, 0.6, 0.4),
  first_last_weight = 2,
  deduplicate = TRUE
)
}
\arguments{
\item{data}{A data frame with \code{id} and \code{authors} (list-column where
author order is preserved).}

\item{counting}{Character. Counting method.}

\item{position_weights}{Numeric vector for \code{counting = "position_weighted"}.}

\item{first_last_weight}{Numeric for \code{counting = "first_last"}.}
}
\value{
A sparse weighted bipartite matrix (works x authors).
}
\description{
For position-dependent counting methods, this replaces the binary bipartite
matrix entries with positional weights derived from author order.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/temporal-network.R
\name{temporal_network}
\alias{temporal_network}
\title{Build time-windowed networks}
\usage{
temporal_network(
  data,
  network_fun,
  ...,
  window = 3,
  step = NULL,
  strategy = "fixed",
  time_col = "year"
)
}
\arguments{
\item{data}{A data frame with a numeric time column.}

\item{network_fun}{Function or character string naming a network function
(e.g., \code{author_network}, \code{"reference_network"}, \code{conetwork}).}

\item{...}{Additional arguments passed to \code{network_fun} (e.g., \code{type},
\code{counting}, \code{similarity}, \code{threshold}, \code{top_n}).}

\item{window}{Integer. Width of each time window in units of the time
column (years, months, quarters, etc.). Default 3.}

\item{step}{Integer or \code{NULL}. Step size between windows. Default \code{NULL}
(equals \code{window} for fixed, 1 for sliding).}

\item{strategy}{Character. Time window strategy:
\describe{
\item{\code{"fixed"}}{Disjoint non-overlapping windows (default).}
\item{\code{"sliding"}}{Overlapping windows advancing by \code{step} units.}
\item{\code{"cumulative"}}{Each window starts at the earliest value and
extends further.}
}}

\item{time_col}{Character. Name of the column containing the time variable.
Default \code{"year"}. Works with any numeric time unit: years, months,
quarters, semesters, weeks, etc. (e.g., \code{"month"}, \code{"quarter"}, \code{"time"}).}
}
\value{
A named list of data frames (edge lists). Names are window
labels like \code{"2018-2020"}.
}
\description{
Splits data by time windows and builds a separate network for each
window using any network function.
}
\examples{
data(biblio_data)

# Fixed 3-year windows
temporal_network(biblio_data, author_network, "collaboration")

# Sliding window
temporal_network(biblio_data, author_network, "collaboration",
                 window = 2, strategy = "sliding")

# Cumulative
temporal_network(biblio_data, reference_network,
                 threshold = 0, strategy = "cumulative", window = 2)

# With string name
temporal_network(biblio_data, "keyword_network", window = 3)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/converters.R
\name{to_graphml}
\alias{to_graphml}
\title{Export to GraphML}
\usage{
to_graphml(edges, nodes = NULL, file = NULL, directed = FALSE)
}
\arguments{
\item{edges}{A data frame with at least \code{from}, \code{to}, \code{weight} columns.}

\item{nodes}{Optional data frame of node attributes with an \code{id} column.}

\item{file}{File path to write. If \code{NULL} (default), returns the GraphML
as a character string.}

\item{directed}{Logical. Default \code{FALSE}.}
}
\value{
If \code{file = NULL}: GraphML as a character string. Otherwise writes
the file and returns the path invisibly.
}
\description{
Writes a bibnets edge list (and optional node attributes) to a GraphML
file using pure base R — no XML package required.
}
\examples{
data(biblio_data)
edges <- keyword_network(biblio_data)
xml <- to_graphml(edges)
cat(substr(xml, 1, 300))
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/read-wos.R
\name{read_wos}
\alias{read_wos}
\title{Read Web of Science plaintext or tab-delimited export}
\usage{
read_wos(file, format = "plaintext")
}
\arguments{
\item{file}{Path to a WoS export file (.txt).}

\item{format}{Character. \code{"plaintext"} (default) for WoS tagged format,
or \code{"tab"} for tab-delimited export.}
}
\value{
A data frame in the standard bibnets format: \code{id}, \code{title},
\code{year}, \code{journal}, \code{doi}, \code{cited_by_count}, \code{abstract}, \code{type},
plus list-columns \code{authors}, \code{references}, and \code{keywords}.
WoS-specific extra: \code{keywords_plus} (list-column).
}
\description{
Parses a Web of Science export file (plaintext or tab-delimited) into a
standardized bibliometric data frame.
}
\examples{
f <- system.file("extdata", "wos_sample.txt", package = "bibnets")
data <- read_wos(f)
head(data[, c("id", "title", "year", "journal")])
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/counting.R
\name{apply_counting}
\alias{apply_counting}
\title{Apply counting weights to a generic bipartite matrix}
\usage{
apply_counting(B, counting = "full", network_type = "symmetric")
}
\arguments{
\item{B}{A sparse binary bipartite matrix (works x entities).}

\item{counting}{Character. One of \code{"full"}, \code{"fractional"}, \code{"paper"},
\code{"strength"}.}

\item{network_type}{Character. \code{"symmetric"} or \code{"coupling"}.}
}
\value{
A weighted sparse matrix.
}
\description{
For position-independent counting of non-author fields (references,
keywords, etc.). Modifies the bipartite matrix row weights.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/methods.R
\name{print.bibnets_network}
\alias{print.bibnets_network}
\title{Print a bibnets network edge list}
\usage{
\method{print}{bibnets_network}(x, n = 10L, ...)
}
\arguments{
\item{x}{A \code{bibnets_network} data frame.}

\item{n}{Integer. Number of rows to show. Default 10.}

\item{...}{Ignored.}
}
\value{
Invisibly returns \code{x}.
}
\description{
Print a bibnets network edge list
}
\examples{
data(biblio_data)
edges <- author_network(biblio_data, "collaboration")
print(edges)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/read-ris.R
\name{read_ris}
\alias{read_ris}
\title{Read an RIS file}
\usage{
read_ris(file, encoding = "UTF-8")
}
\arguments{
\item{file}{Path to a \code{.ris} file.}

\item{encoding}{Character. File encoding. Default \code{"UTF-8"}.}
}
\value{
A data frame in the standard bibnets format: \code{id}, \code{title},
\code{year}, \code{journal}, \code{doi}, \code{cited_by_count}, \code{abstract}, \code{type},
plus list-columns \code{authors}, \code{references} (typically empty for
RIS), and \code{keywords}.
}
\description{
Parses a \code{.ris} file into a standardized bibliometric data frame.
Like BibTeX, standard RIS does not include cited references.
}
\examples{
# Write a minimal RIS record to a temp file, then read it
ris <- "TY  - JOUR
AU  - Smith, J.
AU  - Jones, K.
TI  - Bibliometric networks
JO  - Test Journal
PY  - 2020
DO  - 10.1000/test
ER  - "
f <- tempfile(fileext = ".ris")
writeLines(ris, f)
data <- read_ris(f)
data[, c("id", "title", "year", "journal", "doi")]
unlink(f)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/converters.R
\name{to_cograph}
\alias{to_cograph}
\title{Prepare network for cograph::splot()}
\usage{
to_cograph(edges, nodes = NULL, directed = FALSE)
}
\arguments{
\item{edges}{A data frame with at least \code{from}, \code{to}, \code{weight} columns.}

\item{nodes}{Optional data frame of node attributes with an \code{id} column
(e.g., output of \code{\link[=local_citations]{local_citations()}}). All columns are merged into the
\code{cograph_network$nodes} table and become available as aesthetic mappings.}

\item{directed}{Logical. Default \code{FALSE}.}
}
\value{
A \code{cograph_network} object (S3 list with \verb{$nodes} and \verb{$edges}).
}
\description{
Converts a bibnets edge list to a \code{cograph_network} object by calling
\code{cograph::as_cograph()}. Optionally merges node metadata (e.g., from
\code{\link[=local_citations]{local_citations()}}) into the network's node table so attributes like
\code{lcs} or \code{year} can be used directly in \code{splot()} aesthetic parameters
(e.g., \code{node_size = "lcs"}).
}
\details{
Note: bibnets edge lists (\code{from}, \code{to}, \code{weight}) are accepted directly
by \code{cograph::splot()} without conversion. This function is only needed
when you want to attach node-level metadata.
}
\examples{
\dontshow{if (requireNamespace("cograph", quietly = TRUE)) withAutoprint(\{ # examplesIf}
data(biblio_data)

# Without metadata: splot() accepts bibnets edges directly
edges <- author_network(biblio_data, "collaboration")

# With metadata: document network + local citation scores as node size
edges <- document_network(biblio_data, type = "coupling")
nodes <- local_citations(biblio_data)   # keyed by document id
net   <- to_cograph(edges, nodes = nodes)
\dontshow{\}) # examplesIf}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/edgelist.R
\name{edgelist_to_mat}
\alias{edgelist_to_mat}
\title{Convert edge list to a square sparse matrix}
\usage{
edgelist_to_mat(edges, nodes = NULL, symmetric = TRUE)
}
\arguments{
\item{edges}{A data frame with columns \code{from}, \code{to}, \code{weight}.}

\item{nodes}{Optional character vector of node names. If \code{NULL},
derived from the edge list.}

\item{symmetric}{Logical. If \code{TRUE} (default), the matrix is made
symmetric.}
}
\value{
A sparse \code{dgCMatrix}.
}
\description{
Convert edge list to a square sparse matrix
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data.R
\docType{data}
\name{scopus_quantum_cloud}
\alias{scopus_quantum_cloud}
\title{Scopus dataset — Green Cloud Computing and Quantization (2020–2025)}
\format{
A data frame with 499 rows and 12 columns:
\describe{
\item{id}{Scopus EID.}
\item{title}{Work title.}
\item{year}{Publication year (integer).}
\item{journal}{Source title.}
\item{doi}{DOI string without the \code{https://doi.org/} prefix.}
\item{cited_by_count}{Times cited in Scopus.}
\item{abstract}{Abstract text.}
\item{type}{Document type (\code{"Article"}, \code{"Review"}, etc.).}
\item{authors}{List-column of author name strings.}
\item{references}{List-column of cited reference strings.}
\item{keywords}{List-column of author keywords.}
\item{affiliations}{List-column of affiliation strings.}
}
}
\source{
Scopus bibliometric export. Dataset archived at
\doi{10.5281/zenodo.17142636} (CC BY 4.0).
}
\usage{
scopus_quantum_cloud
}
\description{
First 500 records from a Scopus bibliometric export on the intersection of
green cloud computing and quantization, covering 2020–2025. Includes full
references, author keywords, index keywords, and affiliations.
}
\examples{
data(scopus_quantum_cloud)
author_network(scopus_quantum_cloud, "collaboration")
keyword_network(scopus_quantum_cloud)
document_network(scopus_quantum_cloud, "coupling", similarity = "cosine")
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/co-network.R
\name{ensure_list_column}
\alias{ensure_list_column}
\title{Ensure a column is a list-column, splitting if needed}
\usage{
ensure_list_column(data, field, sep = ";")
}
\description{
Ensure a column is a list-column, splitting if needed
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/source-network.R
\name{source_network}
\alias{source_network}
\title{Build a source (journal) network}
\usage{
source_network(
  data,
  type = "coupling",
  counting = "full",
  similarity = "none",
  threshold = 0,
  min_occur = 1L,
  top_n = NULL,
  self_loops = FALSE,
  deduplicate = TRUE,
  format = "edgelist"
)
}
\arguments{
\item{data}{A data frame with \code{id} and \code{journal} (character column).
For coupling, also needs \code{references}. For co-citation, needs a
\code{cited_journals} list-column.}

\item{type}{Character. \code{"coupling"} (default), \code{"co_citation"}, or
\code{"equivalence"}.}

\item{counting}{Character. Counting method. Default \code{"full"}.}

\item{similarity}{Character. Similarity measure. Default \code{"none"}.}

\item{threshold}{Numeric. Minimum edge weight. Default 0.}

\item{min_occur}{Integer. Minimum papers per source. Default 1.}

\item{top_n}{Integer or NULL. Return only the top n edges by weight.
Default NULL (all edges).}

\item{self_loops}{Logical. If \code{TRUE}, include self-loops (an entity linked
to itself). Default \code{FALSE}.}

\item{deduplicate}{Logical. If \code{TRUE} (default), each \verb{(paper, entity)}
pair is counted at most once — duplicate entries in the source data
(e.g., the same author listed twice on a paper) are treated as one
occurrence. Set to \code{FALSE} to count every raw occurrence.}

\item{format}{Character. Output format:
\describe{
\item{\code{"edgelist"}}{Default. A \code{bibnets_network} data frame with
columns \code{from}, \code{to}, \code{weight}, \code{count}.}
\item{\code{"gephi"}}{Gephi-ready data frame: \code{Source}, \code{Target},
\code{Weight}, \code{Count}, \code{Type}.}
\item{\code{"igraph"}}{An igraph graph object (requires igraph).}
\item{\code{"cograph"}}{A cograph_network object (requires cograph).}
\item{\code{"matrix"}}{A sparse adjacency matrix.}
}}
}
\value{
Depends on \code{format}: a \code{bibnets_network} data frame (default),
a Gephi-ready data frame, an igraph graph, a cograph_network, or a
sparse matrix.
}
\description{
Constructs a network between publication sources (journals, book series).
}
\examples{
data(biblio_data)
source_network(biblio_data, "coupling")
}
